Package: geosapi
Type: Package
Title: GeoServer REST API R Interface
Version: 0.8-1
Date: 2026-07-22
Authors@R: c(person("Emmanuel", "Blondel", role = c("aut", "cre"), email = "emmanuel.blondel1@gmail.com", comment = c(ORCID = "0000-0002-5870-5762")))
Maintainer: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
Description: Provides an R interface to the GeoServer REST API, allowing to upload 
 and publish data in a GeoServer web-application and expose data to OGC Web-Services. 
 The package currently supports all CRUD (Create,Read,Update,Delete) operations
 on GeoServer workspaces, namespaces, datastores (stores of vector data), featuretypes,
 layers, styles, as well as vector data upload operations. For more information about 
 the GeoServer REST API, see <https://docs.geoserver.org/main/en/user/rest/>.
Depends: R (>= 3.1.0)
Imports: R6, cli, openssl, httr, xml2, magrittr, keyring, readr
Suggests: testthat, waldo, roxygen2, shiny, knitr, markdown
License: MIT + file LICENSE
URL: https://github.com/eblondel/geosapi,
        https://eblondel.github.io/geosapi/, https://geoserver.org/
BugReports: https://github.com/eblondel/geosapi/issues
LazyLoad: yes
RoxygenNote: 7.3.3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-07-22 07:28:04 UTC; emman
Author: Emmanuel Blondel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5870-5762>)
Repository: CRAN
Date/Publication: 2026-07-22 08:20:02 UTC

# Generated by roxygen2: do not edit by hand

export(GSAbstractCoverageStore)
export(GSAbstractDBDataStore)
export(GSAbstractDataStore)
export(GSAbstractStore)
export(GSAccessControlListManager)
export(GSArcGridCoverageStore)
export(GSCoverage)
export(GSCoverageBand)
export(GSCoverageStoreManager)
export(GSCoverageView)
export(GSDataStoreManager)
export(GSDimension)
export(GSFeatureDimension)
export(GSFeatureType)
export(GSGeoPackageDataStore)
export(GSGeoTIFFCoverageStore)
export(GSImageMosaicCoverageStore)
export(GSInputCoverageBand)
export(GSLayer)
export(GSLayerGroup)
export(GSLayerManager)
export(GSLayerRule)
export(GSManager)
export(GSMetadataLink)
export(GSMonitorManager)
export(GSNamespace)
export(GSNamespaceManager)
export(GSOracleNGDataStore)
export(GSPostGISDataStore)
export(GSPublishable)
export(GSRESTEntrySet)
export(GSRESTResource)
export(GSResource)
export(GSRestRule)
export(GSRule)
export(GSServiceManager)
export(GSServiceRule)
export(GSServiceSettings)
export(GSShapefileDataStore)
export(GSShapefileDirectoryDataStore)
export(GSShinyMonitor)
export(GSStyle)
export(GSStyleManager)
export(GSUtils)
export(GSVersion)
export(GSVirtualTable)
export(GSVirtualTableGeometry)
export(GSVirtualTableParameter)
export(GSWorkspace)
export(GSWorkspaceManager)
export(GSWorkspaceSettings)
export(GSWorldImageCoverageStore)
import(cli)
import(httr)
import(keyring)
import(magrittr)
import(xml2)
importFrom(R6,R6Class)
importFrom(openssl,base64_encode)
importFrom(readr,read_csv)
importFrom(readr,write_csv)

**geosapi**
===========

[![Build Status](https://github.com/eblondel/geosapi/actions/workflows/r-cmd-check.yml/badge.svg?branch=master)](https://github.com/eblondel/geosapi/actions/workflows/r-cmd-check.yml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/geosapi)](https://cran.r-project.org/package=geosapi)
[![cran checks](https://badges.cranchecks.info/worst/geosapi.svg)](https://cran.r-project.org/web/checks/check_results_geosapi.html)
[![Github_Status_Badge](https://img.shields.io/badge/Github-0.8--1-blue.svg)](https://github.com/eblondel/geosapi)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.1184895.svg)](https://doi.org/10.5281/zenodo.1184895)

``geosapi``: GeoServer REST API R Interface

### Citation

We thank in advance people that use ``geosapi`` for citing it in their work / publication(s). For this, please use the citation provided at this link [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.1184895.svg)](https://doi.org/10.5281/zenodo.1184895)

### User Manual

A user manual is available at [https://github.com/eblondel/geosapi/wiki](https://github.com/eblondel/geosapi/wiki)

The R package is also documented through its [website](https://eblondel.github.io/geosapi/)

### Sponsors

Many thanks to the following organizations that have provided fundings for strenghtening the ``geosapi`` package:

<a href="https://unepgrid.ch/en"><img height=100 width=200 src="https://unepgrid.ch/themes/grid/assets/dist/images/grid-logo-black-en.svg"></a><a href="https://www.fao.org/home/en/"><img height=100 width=200 src="https://www.fao.org/fileadmin/templates/family-farming-decade/images/FAO-IFAD-Logos/FAO-Logo-EN.svg"></a>
<a href="http://www.cnrs.fr"><img src="http://www.cnrs.fr/themes/custom/cnrs/logo.svg" height=100 width=100/></a>
<a href="https://letg.cnrs.fr"><img height=100 width=50 src="https://letg.cnrs.fr/wp-content/uploads/2023/05/logo_letg.png"></a>

#' Geoserver REST API REST Resource interface
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer REST resource interface
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSRESTResource <- R6Class("GSRESTResource",
  public = list(
    
    #'@field rootName root name
    rootName = NA,
    
    #'@field attrs attrs
    attrs = list(),
    
    #'@description Initializes an object of class \link{GSRESTResource}
    #'@param xml object of class \link[xml2]{xml_node-class}
    #'@param rootName root name
    initialize = function(xml, rootName){
      if(missing(rootName) | is.null(rootName)){
        stop("No root name specified for GSRESTResource")
      }
      self$rootName = rootName
    },
    
    #'@description Decodes from XML. Abstract method to be implemented by sub-classes
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      stop("Unimplemented XML 'decode' method") 
    },
    
    #'@description Encodes as XML
    #'@return an object of class \link[xml2]{xml_node-class}
    encode = function(){
      #Generic XML encoder
      rootXML <- xml2::xml_new_root(self$rootName)
      if(is(self, "GSPublishable")){
        xml2::xml_attrs(rootXML) <- c(type = self$attr_type)
      }
      
      #list of fields to encode as XML
      fields <- rev(names(self))
      fields <- fields[!sapply(fields, function(x){
        (class(self[[x]])[1] %in% c("environment", "function")) ||
        (x %in% c("rootName", "full", "attr_type", "attrs"))
      })]
      
      if(is(self, "GSRESTEntrySet")){
        items <- self$entryset
        itemNb <- length(items)
        if(itemNb > 0){
          for(i in 1:itemNb){
            itemName <- names(items)[i]
            
            item <- xml2::xml_new_root("entry")
            xml2::xml_attrs(item) = c(key = itemName)
            
            itemValue <- items[[itemName]]
            if(is.logical(itemValue)){
              itemValue <- tolower(as.character(itemValue))
            }
            if(is(itemValue, "GSRESTResource")){
              itemValue <- itemValue$encode()
              item %>% xml2::xml_add_child(itemValue)
            }else{
              xml2::xml_text(item) <- itemValue
            }
            rootXML %>% xml2::xml_add_child(item)
          }
        }
      }else{
        for(field in fields){
          fieldObj <- self[[field]]
          if(!is.null(fieldObj)){
            if(is.logical(fieldObj)){
              fieldObj <- tolower(as.character(fieldObj))
            }
            
            if(any(class(fieldObj) == "list")){
              if(is(self, "GSVirtualTable")){
                itemsXML <- rootXML
              }else{
                itemsXML <- xml2::xml_new_root(field)
              }
              items <- fieldObj
              itemNames <- names(items)
              if(length(items) > 0){
                for(i in 1:length(items)){
                  item <- items[[i]]
                  itemName <- itemNames[i]
                  if(any(class(item) == "R6")){
                    itemXML <- item$encode()
                    itemsXML %>% xml2::xml_add_child(itemXML)
                  }else{
                    if(is.null(itemNames)){
                      itemXML <- xml2::xml_new_root("string")
                      if(!is.na(item)) xml2::xml_text(itemXML) <- as.character(item)
                      itemsXML %>% xml2::xml_add_child(itemXML)
                    }else{
                      itemXML <- xml2::xml_new_root(itemName)
                      if(!is.na(item)) xml2::xml_text(itemXML) <- as.character(item)
                      itemsXML %>% xml2::xml_add_child(itemXML)
                    }
                  }
                }
              }
              if(!is(self, "GSVirtualTable")) rootXML %>% xml2::xml_add_child(itemsXML)
            }else{
              if(any(class(fieldObj) == "R6")){
                rootXML %>% xml2::xml_add_child(fieldObj$encode())
              }else{
                itemXML <- xml2::xml_new_root(field)
                if(!is.na(fieldObj)) xml2::xml_text(itemXML) <- as.character(fieldObj)
                rootXML %>% xml2::xml_add_child(itemXML)
              }
            }
          }
        }
      }
      
      #XML attribute support
      if(length(self$attrs)>0){
        xml2::xml_attrs(rootXML) <- self$attrs
      }
      
      return(rootXML)
    },
    
    #'@description Provides a custom print output (as tree) of the current class
    #'@param ... args
    #'@param depth class nesting depth
    print = function(..., depth = 1){
      #list of fields to encode as XML
      fields <- rev(names(self))
      
      #fields
      fields <- fields[!sapply(fields, function(x){
        (class(self[[x]])[1] %in% c("environment", "function")) ||
          (x %in% c("rootName", "full", "attr_type"))
      })]
      
      cat(crayon::white(paste0("<", crayon::underline(self$getClassName()), ">")))
      
      for(field in fields){
        fieldObj <- self[[field]]
        
        print_attrs <- function(obj){
          paste(
            sapply(names(obj$attrs), function(attrName){
              paste0( crayon::magenta(attrName,"=",sep=""), crayon::green(obj$attrs[[attrName]]))
            }
            ), 
            collapse=",")
        }
        
        #user values management
        shift <- "...."
        if(!is.null(fieldObj)){
          if(is(fieldObj, "GSRESTResource")){
            attrs_str <- ""
            if(length(fieldObj$attrs)>0){
              attrs <- print_attrs(fieldObj)
              attrs_str <- paste0(" [",attrs,"] ")
            }
            cat(paste0("\n", paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), " ", attrs_str))
            fieldObj$print(depth = depth+1)
          }else if(is(fieldObj, "list")){
            if(is.null(names(fieldObj))){
              for(item in fieldObj){
                if(is(item, "GSRESTResource")){
                  attrs_str <- ""
                  if(length(item$attrs)>0){
                    attrs <- print_attrs(item)
                    attrs_str <- paste0(" [",attrs,"] ")
                  }
                  cat(paste0("\n", paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), " ", attrs_str))
                  item$print(depth = depth+1)
                }else if(is(item, "matrix")){
                  m <- paste(apply(item, 1L, function(x){
                    x <- lapply(x, function(el){
                      if(is.na(suppressWarnings(as.numeric(el))) & !all(sapply(item,class)=="character")){
                        el <- paste0("\"",el,"\"")
                      }else{
                        if(!is.na(suppressWarnings(as.numeric(el)))){
                          el <- as.numeric(el)
                        }
                      }
                      return(el)
                    })
                    return(paste(x, collapse = " "))
                  }), collapse = " ")
                  cat(paste0("\n",paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), ": ", crayon::bgWhite(m)))
                }else{
                  cat(paste0("\n", paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), ": ", crayon::bgWhite(item)))
                }
              }
            }else{
              cat(paste0("\n", paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field)))
              for(itemName in names(fieldObj)){
                item = fieldObj[[itemName]]
                if(is(item, "GSRESTResource")){
                  attrs_str <- ""
                  if(length(item$attrs)>0){
                    attrs <- print_attrs(item)
                    attrs_str <- paste0(" [",attrs,"] ")
                  }
                  cat(paste0("\n", paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), " ", attrs_str))
                  item$print(depth = depth+1)
                }else{
                  cat(paste0("\n", paste(rep(shift, depth+1), collapse=""),"|-- ", crayon::italic(itemName), ": ", crayon::bgWhite(fieldObj[[itemName]])))
                }
              }
            }
          }else if (is(fieldObj,"matrix")){
            m <- paste(apply(fieldObj, 1L, function(x){
              x <- lapply(x, function(el){
                if(is.na(suppressWarnings(as.numeric(el)))& !all(sapply(fieldObj,class)=="character")){
                  el <- paste0("\"",el,"\"")
                }else{
                  if(!is.na(suppressWarnings(as.numeric(el)))){
                    el <- as.numeric(el)
                  }
                }
                return(el)
              })
              return(paste(x, collapse = " "))
            }), collapse = " ")
            cat(paste0("\n",paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), ": ", crayon::bgWhite(m)))
          }else{
            cat(paste0("\n",paste(rep(shift, depth), collapse=""),"|-- ", crayon::italic(field), ": ", crayon::bgWhite(fieldObj)))
          }
        }
      }
      invisible(self)
    },
    
    #'@description Get class name
    #'@return an object of class \code{character}
    getClassName = function(){
      return(class(self)[1])
    }
  )                     
)

# test_GSVirtualTableParameter.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSVirtualTableParameter.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSVirtualTableParameter")

test_that("virtual table parameter encoding/decoding",{
  vtp <- GSVirtualTableParameter$new(name = "fieldname", defaultValue = "default_value",
                                    regexpValidator = "^[\\w\\d\\s]+$")
  expect_is(vtp, "GSVirtualTableParameter")
  expect_equal(vtp$name, "fieldname")
  expect_equal(vtp$defaultValue, "default_value")
  expect_equal(vtp$regexpValidator, "^[\\w\\d\\s]+$")
  
  vtpXML <- vtp$encode()
  expect_is(vtpXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  vtp2 <- GSVirtualTableParameter$new(xml = vtpXML)
  vtp2XML <- vtp2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::compare(vtpXML, vtp2XML)
  
})

# test_GSVirtualTableGeometry.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSVirtualTableGeometry.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSVirtualTableGeometry")

test_that("virtual table geometry encoding/decoding",{
  vtg <- GSVirtualTableGeometry$new(name = "the_geom", type = "MultiPolygon", srid = "4326")
  expect_is(vtg, "GSVirtualTableGeometry")
  expect_equal(vtg$name, "the_geom")
  expect_equal(vtg$type, "MultiPolygon")
  expect_equal(vtg$srid, "4326")
  
  vtgXML <- vtg$encode()
  expect_is(vtgXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  vtg2 <- GSVirtualTableGeometry$new(xml = vtgXML)
  vtg2XML <- vtg2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(vtgXML, vtg2XML))==0)
  
})

# test_GSMetadataLink.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSMetadataLink.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSMetadataLink")

test_that("metadataLink encoding/decoding",{
  
  md = GSMetadataLink$new(
    type = "text/xml",
    metadataType = "TC211",
    content = "http://someuri"
  )
  expect_is(md, "GSMetadataLink")
  expect_equal(md$type, "text/xml")
  expect_equal(md$metadataType, "TC211")
  expect_equal(md$content, "http://someuri")
  
  #encoding to XML
  mdXML <- md$encode()
  expect_is(mdXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  md2 <- GSMetadataLink$new(xml = mdXML)
  md2XML <- md2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(mdXML, md2XML))==0)
  
})

#' Geoserver REST API GSVirtualTableParameter
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api virtualTable
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer virtual table parameter
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSVirtualTableParameter$new(name = "fieldname", defaultValue = "default_value",
#'                             regexpValidator = "someregexp")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSVirtualTableParameter <- R6Class("GSVirtualTableParameter",
  inherit = GSRESTResource,                    
  public = list(
    #' @field name parameter name
    name = NA,
    #' @field defaultValue parameter default value
    defaultValue = NA,
    #' @field regexpValidator parameter regexp validator
    regexpValidator = NA,
    
    #'@description Initializes an object of class \link{GSVirtualTableParameter}
    #'@param xml object of class \link[xml2]{xml_node-class}
    #'@param name name
    #'@param defaultValue default value
    #'@param regexpValidator regexp validator
    initialize = function(xml = NULL, name, defaultValue, regexpValidator){
      super$initialize(rootName = "parameter")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }else{
        self$name = name
        self$defaultValue = defaultValue
        self$regexpValidator = regexpValidator
      }
    },
    
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      self$name <- xml2::xml_find_first(xml, "//name") %>% xml2::xml_text()
      self$defaultValue <- xml2::xml_find_first(xml, "//defaultValue") %>% xml2::xml_text()
      self$regexpValidator <- xml2::xml_find_first(xml, "//regexpValidator") %>% xml2::xml_text()
    }
    
  )                     
)

#' Geoserver REST API CoverageStore Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api CoverageStore
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
#' CoverageStores (i.e. stores of coverage data)
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSCoverageStoreManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#'  }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSCoverageStoreManager <- R6Class("GSCoverageStoreManager",
  inherit = GSManager,
  
  public = list(   
    
    #CoverageStore generic CRUD methods
    #===========================================================================
    
    #'@description Get the list of available coverage stores. Returns an object of class \code{list}
    #'    giving items of class \code{\link{GSAbstractCoverageStore}}
    #'@param ws workspace name
    #'@return the list of coverage stores
    getCoverageStores = function(ws){
      msg = sprintf("Fetching list of coverage stores in workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/coveragestores.xml", ws),
        verbose = self$verbose.debug)
      covList <- NULL
      if(status_code(req) == 200){
        covXML <- GSUtils$parseResponseXML(req)
        covXMLList <- as(xml2::xml_find_all(covXML, "//coverageStore"), "list")
        covList <- lapply(covXMLList, function(xml){
          covType <- xml2::xml_find_first(xml, "//type") %>% xml2::xml_text()
          coverageStore <- switch(covType,
            "GeoTIFF" = GSGeoTIFFCoverageStore$new(xml = xml),
            "WorldImage" = GSWorldImageCoverageStore$new(xml = xml),
            "ImageMosaic" = GSImageMosaicCoverageStore$new(xml = xml),
            "ArcGrid" = GSArcGridCoverageStore$new(xml = xml),
            GSAbstractCoverageStore$new(xml = xml)
          )
          return(coverageStore)
        })
        msg = sprintf("Successfully fetched %s coverage stores!", length(covList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of datastores"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(covList)
    },
    
    #'@description Get the list of available coverage store names. Returns an vector of class \code{character}
    #'@param ws workspace name
    #'@return the list of coverage store names, as \code{character}
    getCoverageStoreNames = function(ws){
      covList <- sapply(self$getCoverageStores(ws), function(x){x$name})
      return(covList)
    },
    
    #'@description Get an object of class \code{\link{GSAbstractDataStore}} given a workspace and coverage store names.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@return the coverage store
    getCoverageStore = function(ws, cs){
      msg = sprintf("Fetching coverage store '%s' in workspace '%s'", cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/coveragestores/%s.xml", ws, cs),
        verbose = self$verbose.debug)
      coverageStore <- NULL
      if(status_code(req) == 200){
        covXML <- GSUtils$parseResponseXML(req)
        covType <-  xml2::xml_find_first(covXML, "//type") %>% xml2::xml_text()
        coverageStore <- switch(covType,
          "GeoTIFF" = GSGeoTIFFCoverageStore$new(xml = covXML),
          "WorldImage" = GSWorldImageCoverageStore$new(xml = covXML),
          "ImageMosaic" = GSImageMosaicCoverageStore$new(xml = covXML),
          "ArcGrid" = GSArcGridCoverageStore$new(xml = covXML),
          GSAbstractCoverageStore$new(xml = covXML)
        )
        msg = "Successfully fetched coverage store!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching coverage store"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(coverageStore)
    },

    #'@description  Creates a new coverage store given a workspace, coverage store name. Abstract method used in below format-specific
    #'    methods to create coverage stores.
    #'@param ws workspace name
    #'@param coverageStore coverage store object
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createCoverageStore = function(ws, coverageStore){
      msg = sprintf("Creating coverage store '%s' in workspace '%s'", coverageStore$name, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      if(is.null(coverageStore$workspace)) coverageStore$workspace <- ws
      req <- GSUtils$POST(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/coveragestores.xml", ws),
        content = GSUtils$getPayloadXML(coverageStore),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfully created coverage store!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating coverage store"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
    },
    
    #'@description  Updates a coverage store given a workspace, coverage store name. Abstract method used in below format-specific
    #'    methods to create coverage stores.
    #'@param ws workspace name
    #'@param coverageStore coverage store object
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateCoverageStore = function(ws, coverageStore){
      if(is.null(coverageStore$workspace)) coverageStore$workspace <- ws
      updated <- FALSE
      msg = sprintf("Updating coverage store '%s' in workspace '%s'", coverageStore$name, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user, pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/coveragestores/%s.xml", ws, coverageStore$name),
        content = GSUtils$getPayloadXML(coverageStore),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated coverage store!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating coverage store"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a coverage store given a workspace and an object of class \code{\link{GSAbstractCoverageStore}}.
    #'    By defaut, the option \code{recurse} is set to FALSE, ie datastore layers are not removed.
    #'    To remove all coverage store layers, set this option to TRUE. The \code{purge} parameter is used 
    #'    to customize the delete of files on disk (in case the underlying reader implements a delete method). 
    #'    It can take one of the three values: none, metadata, all. For more details see \url{https://docs.geoserver.org/main/en/user/rest/api/coveragestores/#rest_api_coveragestores_purge}
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param recurse recurse
    #'@param purge purge
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteCoverageStore = function(ws, cs, recurse = FALSE, purge = NULL){
      msg = sprintf("Deleting coverage store '%s' in workspace '%s'", cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/workspaces/%s/coveragestores/%s.xml", ws, cs)
      if(recurse) path <- paste0(path, "?recurse=true")
      if(!is.null(purge)){
        allowedPurgeValues <- c("none","metadata","all")
        if(!(purge %in% allowedPurgeValues)){
          err = sprintf("Purge value should be among allowed purge values [%s]",
                        paste(allowedPurgeValues, collapse=","))
          cli::cli_alert_danger(err)
          stop(err)
        }
        path <- paste0(path, ifelse(recurse,"&","?"), "purge=", purge)
      }
      req <- GSUtils$DELETE(self$getUrl(), private$user, private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully deleted coverage store!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting coverage store"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)  
    },
    
    #CoverageStore coverages CRUD methods
    #===========================================================================
    
    #'@description Get the list of available coverages for given workspace and coverage store.
    #'    Returns an object of class \code{list} giving items of class \code{\link{GSCoverage}}
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@return the list of \link{GSCoverage}
    getCoverages = function(ws, cs){
      msg = sprintf("Fetching coverages for coverage store '%s' in workspace '%s'", cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/coveragestores/%s/coverages.xml", ws, cs),
        verbose = self$verbose.debug)
      covList <- NULL
      if(status_code(req) == 200){
        covXML <- GSUtils$parseResponseXML(req)
        covXMLList <- as(xml2::xml_find_all(covXML, "//coverages/coverage"), "list")
        covList <- lapply(covXMLList, GSCoverage$new)
        msg = sprintf("Successfully fetched %s coverages!", length(covList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of coverages"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(covList)
    },
    
    #'@description Get the list of available coverage names for given workspace and coverage store.
    #'    Returns an object of class \code{list} giving items of class \code{\link{GSCoverage}}
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@return the list of coverage names
    getCoverageNames = function(ws, cs){
      covList <- sapply(self$getCoverages(ws, cs), function(x){x$name})
      return(covList)
    },
    
    #'@description Get coverage
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param cv coverage name
    getCoverage = function(ws, cs, cv){
      msg = sprintf("Fetching coverage '%s' in coverage store '%s' (workspace '%s')", cv, cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, 
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/coveragestores/%s/coverages/%s.xml", ws, cs, cv),
        verbose = self$verbose.debug)
      coverage <- NULL
      if(status_code(req) == 200){
        covXML <- GSUtils$parseResponseXML(req)
        coverage <- GSCoverage$new(xml = covXML)
        msg = "Successfully fetched coverage!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching coverage"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(coverage)
    },
    
    #'@description  Creates a new coverage given a workspace, coverage store names and an object of class \code{\link{GSCoverage}}
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param coverage object of class \link{GSCoverage}
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createCoverage = function(ws, cs, coverage){
      msg = sprintf("Creating coverage '%s' in coverage store '%s' (workspace '%s')", coverage$name, cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      req <- GSUtils$POST(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/coveragestores/%s/coverages.xml", ws, cs),
        content = GSUtils$getPayloadXML(coverage),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfully created coverage!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating coverage"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description  Updates a coverage given a workspace, coverage store names and an object of class \code{\link{GSCoverage}}
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param coverage object of class \link{GSCoverage}
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateCoverage = function(ws, cs, coverage){
      msg = sprintf("Updating coverage '%s' in coverage store '%s' (workspace '%s')", coverage$name, cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      updated <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user, 
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/coveragestores/%s/coverages/%s.xml",
                       ws, cs, coverage$name),
        content = GSUtils$getPayloadXML(coverage),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated coverage!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating coverage"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a coverage given a workspace, coverage store names, and an object of 
    #'    class \code{\link{GSCoverage}}. By defaut, the option \code{recurse} is 
    #'    set to FALSE, ie coverage layers are not removed.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param cv coverage name
    #'@param recurse recurse
    deleteCoverage = function(ws, cs, cv, recurse = FALSE){
      msg = sprintf("Deleting coverage '%s' in coverage '%s' (workspace '%s')", cv, cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/workspaces/%s/coveragestores/%s/coverages/%s.xml", ws, cs, cv)
      if(recurse) path <- paste0(path, "?recurse=true")
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfuly deleted coverage!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting coverage"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)  
    },
    
    #Upload methods
    #===========================================================================
    
    #'@description Abstract method to upload a coverage file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{extension}
    #'    corresponds to the format/type of coverage to be uploaded (among values 'geotiff', 'worldimage', 'arcgrid', or 'imagemosaic'). 
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param endpoint endpoint. Default is "file"
    #'@param extension extension
    #'@param filename filename
    #'@param configure configure. Default is "first"
    #'@param update update. Default is "append"
    #'@param contentType content type
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadCoverage = function(ws, cs,
                                   endpoint = "file", extension, filename,
                                   configure = "first", update = "append",
                                   contentType){
      msg = sprintf("Uploading %s coverage in new datastore '%s' (workspace '%s')",
                    toupper(extension), cs, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      
      uploaded <- FALSE
      
      supportedEndpoints <- c("file","url","external")
      if(!(endpoint %in% supportedEndpoints)){
        err = sprintf("Unsupported endpoint '%s'. Possible values: [%s]",
                      endpoint, paste0(supportedEndpoints, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      supportedExtensions <- c("geotiff", "worldimage", "imagemosaic", "arcgrid")
      if(!(extension %in% supportedExtensions)){
        err = sprintf("Unsupported extension '%s'. Possible values: [%s]",
                      extension, paste0(supportedExtensions, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      supportedConfigurations <- c("first", "none", "all")
      if(!(configure %in% supportedConfigurations)){
        err = sprintf("Unsupported configure parameter '%s'. Possible values: [%s]",
                      configure, paste0(supportedConfigurations, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      supportedUpdates <- c("append","overwrite")
      if(!(update %in% supportedUpdates)){
        err = sprintf("Unsupported update parameter '%s'. Possible values: [%s]",
                      update, paste0(supportedUpdates, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user, pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/coveragestores/%s/%s.%s?configure=%s&update=%s",
                       ws, cs, endpoint, extension, configure, update),
        content = NULL,
        filename = filename,
        contentType = contentType,
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfull coverage upload!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        uploaded = TRUE
      }
      if(!uploaded){
        err = sprintf("Error while uploading coverage: %s", http_status(req)$message)
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(uploaded)
    },
    
    #'@description Uploads a GeoTIFF file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
    #'    \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the GeoTIFF file to
    #'    upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param endpoint endpoint. Default is "file"
    #'@param filename filename
    #'@param configure configure. Default is "first"
    #'@param update update. Default is "append"
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise   
    uploadGeoTIFF = function(ws, cs,
                             endpoint = "file", filename,
                             configure = "first", update = "append"){
      return(self$uploadCoverage(
        ws = ws, cs = cs, 
        endpoint = endpoint, extension = "geotiff", filename = filename,
        configure = configure, update = update, 
        contentType = "text/plain"
      ))
    },
    
    #'@description Uploads a WorldImage file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
    #'    \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the zipped file to
    #'    upload and set for the newly created datastore. It is assumed the zip archive contains the .prj file to set the SRS. 
    #'    The \code{configure} parameter can take a value among values  \code{"none"} (indicates to configure only the datastore 
    #'    but no layer configuration) or \code{"first"} (configure both datastore and layer). The \code{update} defines the strategy
    #'    for the upload: \code{"append"} (default value) for the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param endpoint endpoint. Default is "file"
    #'@param filename filename
    #'@param configure configure. Default is "first"
    #'@param update update. Default is "append"
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise 
    uploadWorldImage = function(ws, cs,
                                endpoint = "file", filename,
                                configure = "first", update = "append"){
      return(self$uploadCoverage(
        ws = ws, cs = cs, 
        endpoint = endpoint, extension = "worldimage", filename = filename,
        configure = configure, update = update, 
        contentType = "application/zip"
      ))
    },
    
    #'@description Uploads an ArcGrid file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
    #'    \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the ArcGrid file to
    #'    upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param endpoint endpoint. Default is "file"
    #'@param filename filename
    #'@param configure configure. Default is "first"
    #'@param update update. Default is "append"
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise 
    uploadArcGrid = function(ws, cs,
                             endpoint = "file", filename,
                             configure = "first", update = "append"){
      return(self$uploadCoverage(
        ws = ws, cs = cs, 
        endpoint = endpoint, extension = "arcgrid", filename = filename,
        configure = configure, update = update, 
        contentType = "text/plain"
      ))
    },
    
    #'@description Uploads an ImageMosaic file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
    #'    \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the ImageMosaic file to
    #'    upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param cs coverage store name
    #'@param endpoint endpoint. Default is "file"
    #'@param filename filename
    #'@param configure configure. Default is "first"
    #'@param update update. Default is "append"
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise 
    uploadImageMosaic = function(ws, cs,
                                 endpoint = "file", filename,
                                 configure = "first", update = "append"){
      return(self$uploadCoverage(
        ws = ws, cs = cs, 
        endpoint = endpoint, extension = "imagemosaic", filename = filename,
        configure = configure, update = update, 
        contentType = "application/zip"
      ))
    }
    
  )
                              
)

#' Geoserver REST API ShapeFileDirectoryDataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore ESRI shapefile directory
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer Shapefile directory dataStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSShapefileDirectoryDataStore$new(name = "ds", description = "des",
#'                          enabled = TRUE, url = "file://data")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSShapefileDirectoryDataStore <- R6Class("GSShapefileDirectoryDataStore",
    inherit = GSShapefileDataStore,
    private = list(
      TYPE = "Directory of spatial files (shapefiles)"
    ),
    public = list(
      
      #'@description initializes a shapefile directory data store
      #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
      #'@param name coverage store name
      #'@param description coverage store description
      #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
      #'@param url url
      initialize = function(xml = NULL, name = NULL, description = "",
                            enabled = TRUE, url){
        if(missing(xml)) xml <- NULL
        super$initialize(xml = xml, name = name,
                         description = description,
                         enabled = enabled, url = url)
        self$setType(private$TYPE)
      },
      
      #'@description Set the spatial files data URL
      #'@param url url
      setUrl = function(url){
        self$setConnectionParameter("url", url)
      },
      
      #'@description Set the charset used for DBF file. 
      #'@param charset charset. Default value is 'ISO-8859-1'
      setCharset = function(charset = "ISO-8859-1"){
        self$setConnectionParameter("charset", charset)
      },
      
      #'@description Set the 'Create Spatial Index' option
      #'@param create create. Default is \code{TRUE}
      setCreateSpatialIndex = function(create = TRUE){
        self$setConnectionParameter("create spatial index", create)
      },
      
      #'@description Set the 'Memory Mapped Buffer' option
      #'@param buffer buffer. Default is \code{FALSE}
      setMemoryMappedBuffer = function(buffer = FALSE){
        self$setConnectionParameter("memory mapped buffer", buffer)
      },
      
      #'@description Set the 'Cache & Reuse Memory Maps' option.
      #'@param maps maps. Default is \code{TRUE}
      setCacheReuseMemoryMaps = function(maps = TRUE){
        self$setConnectionParameter("cache and reuse memory maps", maps)
      },
      
      #'@description Set default connection parameters
      setDefautConnectionParameters = function(){
        self$setCharset()
        self$setCreateSpatialIndex()
        self$setMemoryMappedBuffer()
        self$setCacheReuseMemoryMaps()
      }
    )                     
)

#' Geoserver REST API GeoPackageDataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore GeoPackage
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer GeoPackage dataStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' ds <- GSGeoPackageDataStore$new(
#'  name = "ds", description = "des", 
#'  enabled = TRUE, database = NULL
#' )
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSGeoPackageDataStore <- R6Class("GSGeoPackageDataStore",
  inherit = GSAbstractDBDataStore,
  private = list(
    TYPE = "GeoPackage",
    DBTYPE = "geopkg"
  ),
  public = list(
    
    #'@description initializes an GeoPackage data store
    #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
    #'@param name coverage store name
    #'@param description coverage store description
    #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
    #'@param database database
    initialize = function(xml = NULL, name = NULL, 
                          description = "", enabled = TRUE,
                          database = NULL){
      super$initialize(xml = xml, type = private$TYPE, dbType = private$DBTYPE,
                       name = name, description = description,
                       enabled = enabled)
      if(!is.null(database)) super$setDatabase(database)
      super$setMinConnections(NULL)
      super$setMaxConnections(NULL)
      super$setConnectionTimeout(NULL)
      super$setFetchSize(NULL)
      super$setLooseBBox(NULL)
      super$setPreparedStatements(NULL)
      super$setMaxOpenPreparedStatements(NULL)
      super$setEstimatedExtends(NULL)
    }
   )                     
)

#' Geoserver REST API Namespace Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api namespace
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing the namespaces
#'  of a GeoServer instance.
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSNamespaceManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSNamespaceManager <- R6Class("GSNamespaceManager",
  inherit = GSManager,
  
  public = list(
    
    #'@description Get the list of available namespace. Re
    #'@return an object of class \code{list} containing items of class \code{\link{GSNamespace}}
    getNamespaces = function(){
      msg = "Fetching list of namespaces"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         "/namespaces.xml", self$verbose.debug)
      nsList <- NULL
      if(status_code(req) == 200){
        nsXML <- GSUtils$parseResponseXML(req)
        nsXMLList <- as(xml2::xml_find_all(nsXML, "//namespace"), "list")
        nsList <- lapply(nsXMLList, GSNamespace$new)
        msg = sprintf("Successfully fetched %s namespaces", length(nsList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of namespaces"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(nsList)
    },
    
    #'@description Get the list of available namespace names.
    #'@return a vector of class \code{character}
    getNamespaceNames = function(){
      nsList <- sapply(self$getNamespaces(), function(x){x$name})
      return(nsList)
    },
    
    #'@description Get a \code{\link{GSNamespace}} object given a namespace name.
    #'@param ns namespace
    #'@return an object of class \link{GSNamespace}
    getNamespace = function(ns){
      msg = sprintf("Fetching workspace '%s'", ns)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         sprintf("/namespaces/%s.xml", ns), self$verbose.debug)
      namespace <- NULL
      if(status_code(req) == 200){
        nsXML <- GSUtils$parseResponseXML(req)
        namespace <- GSNamespace$new(xml = nsXML)
        msg = "Successfully fetched namespace!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching namespace"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(namespace)
    },
    
    #'@description  Creates a GeoServer namespace given a prefix, and an optional URI. 
    #'@param prefix prefix
    #'@param uri uri
    #'@return \code{TRUE} if the namespace has been successfully created, \code{FALSE} otherwise
    createNamespace = function(prefix, uri){
      msg = sprintf("Creating namespace '%s'", prefix)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      namespace <- GSNamespace$new(prefix = prefix, uri = uri)

      req <- GSUtils$POST(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = "/namespaces",
        content = GSUtils$getPayloadXML(namespace),
        contentType = "text/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfully created namespace!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating namespace"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description  Updates a GeoServer namespace given a prefix, and an optional URI. 
    #'@param prefix prefix
    #'@param uri uri
    #'@return \code{TRUE} if the namespace has been successfully updated, \code{FALSE} otherwise
    updateNamespace = function(prefix, uri){
      msg = sprintf("Updating namespace '%s'", prefix)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      updated <- FALSE
      namespace <- GSNamespace$new(prefix = prefix, uri = uri)
      
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/namespaces/%s.xml", prefix),
        content = GSUtils$getPayloadXML(namespace),
        contentType = "application/xml",
        self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated namespace!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating namespace"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a GeoServer namespace given a name. 
    #'@param name name
    #'@param recurse recurse
    #'@return \code{TRUE} if the namespace has been successfully deleted, \code{FALSE} otherwise
    deleteNamespace = function(name, recurse = FALSE){
      msg = sprintf("Deleting namespace '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/namespaces/%s", name)
      if(recurse) path <- paste0(path, "?recurse=true")
      #TODO hack for style removing (not managed by REST API)
      
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully deleted namespace!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting namespace"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    }   
    
  )
  
)

# test_GSNamespace.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSNamespace*.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSNamespace")

test_that("READ namespace",{
  ns <- gsman$getNamespace("topp")
  expect_is(ns, "GSNamespace")
  expect_true(all(c("name", "prefix", "uri") %in% names(ns)))
  expect_equal(ns$name, "topp")
  expect_equal(ns$prefix, "topp")
  expect_equal(ns$uri, "http://www.openplans.org/topp")
})

test_that("READ namespaces",{
  nslist <- gsman$getNamespaces()
  expect_true(all(sapply(nslist, function(x){class(x)[1] == "GSNamespace"})))
})

test_that("CREATE namespace",{
  created <- gsman$createNamespace("geosapi", "http://www.my.org/geosapi")
  expect_true(created)
  ns <- gsman$getNamespace("geosapi")
  expect_equal(ns$name, "geosapi")
  expect_equal(ns$prefix, "geosapi")
  expect_equal(ns$uri, "http://www.my.org/geosapi")
})

test_that("UPDATE namespace",{
  updated <- gsman$updateNamespace("geosapi", "http://www.my.org/geosapi2")
  expect_true(updated)
  ns <- gsman$getNamespace("geosapi")
  expect_equal(ns$name, "geosapi")
  expect_equal(ns$prefix, "geosapi")
  expect_equal(ns$uri, "http://www.my.org/geosapi2")
})

test_that("DELETE namespace",{
  deleted <- gsman$deleteNamespace("geosapi")
  expect_true(deleted)
  ns <- gsman$getNamespace("geosapi")
  expect_is(ns, "NULL")
})

#' Geoserver REST API Resource
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSLayer
#' @title A GeoServer layer resource
#' @description This class models a GeoServer layer. This class is to be
#' used for published resource (feature type or coverage).
#' @keywords geoserver rest api resource featureType coverage layer
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer layer
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   lyr <- GSLayer$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSLayer <- R6Class("GSLayer",
  inherit = GSRESTResource,
  
  public = list(
    #'@field full full
    full = TRUE,
    #'@field name name
    name = NULL,
    #'@field path path
    path = NULL,
    #'@field defaultStyle default style
    defaultStyle = list(),
    #'@field styles styles
    styles = list(),
    #'@field enabled enabled
    enabled = TRUE,
    #'@field queryable queryable
    queryable = TRUE,
    #'@field advertised advertised
    advertised = TRUE,
    
    #'@description Initializes an object of class \link{GSLayer}
    #'@param xml object of class \link[xml2]{xml_node-class}
    initialize = function(xml = NULL){
      super$initialize(rootName = "layer")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }
    },
   
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      names <- xml2::xml_find_first(xml, "//name")
      self$name <- xml2::xml_text(names)
      defaultStyle <- xml2::xml_find_first(xml, "//defaultStyle/name")
      if(length(defaultStyle)==0) self$full <- FALSE
      
      if(self$full){
        self$setDefaultStyle(xml2::xml_text(defaultStyle))
        
        styles <- as(xml2::xml_find_all(xml, "//styles/style"), "list")
        if(length(styles)>0){
          styles <- lapply(styles, GSStyle$new)
          self$setStyles(styles)
        }
        
        paths <- xml2::xml_find_first(xml, "//path")
        if(length(paths)>0) self$path = xml2::xml_text(paths)
        enabled <-xml2::xml_find_first(xml, "//enabled")
        if(length(enabled)>0) self$enabled <- as.logical(xml2::xml_text(enabled))
        queryable <- xml2::xml_find_first(xml, "//queryable")
        if(length(queryable)>0) self$queryable <- as.logical(xml2::xml_text(queryable))
        advertised <- xml2::xml_find_first(xml, "//advertised")
        if(length(advertised)>0) self$advertised <- as.logical(xml2::xml_text(advertised))
      }
    },
    
    #'@description Set name
    #'@param name name
    setName = function(name){
      self$name = name
    },
    
    #'@description Set path
    #'@param path path
    setPath = function(path){
      self$path = path
    },
    
    #'@description Set enabled
    #'@param enabled enabled
    setEnabled = function(enabled){
      self$enabled = enabled
    },
    
    #'@description Set queryable
    #'@param queryable queryable
    setQueryable = function(queryable){
      self$queryable = queryable
    },
    
    #'@description Set advertised
    #'@param advertised advertised
    setAdvertised = function(advertised){
      self$advertised = advertised
    },
    
    #'@description Set default style
    #'@param style object o class \link{GSStyle} or \code{character}
    setDefaultStyle = function(style){
      if(any(class(style)=="GSStyle")) style <- style$name
      self$defaultStyle[["name"]] <- style
    },
    
    #'@description Set styles
    #'@param styles styles
    setStyles = function(styles){
      if(!is.list(styles)) styles = list(styles)
      self$styles = styles
      return(TRUE)
    },
    
    #'@description Adds style
    #'@param style style, object o class \link{GSStyle} or \code{character}
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addStyle = function(style){
      if(is(style, "character")) style <- GSStyle$new(xml=NULL, name = style)
      startNb = length(self$styles)
      availableStyles <- sapply(self$styles, function(x){x$name})
      if(length(which(availableStyles == style$name)) == 0){
        self$styles = c(self$styles, style)
      }
      endNb = length(self$styles)
      return(endNb == startNb+1)
    },
    
    #'@description Deletes style
    #'@param style style, object o class \link{GSStyle} or \code{character}
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    delStyle = function(style){
      if(is(style, "character")) style <- GSStyle$new(xml=NULL, name = style)
      startNb = length(self$styles)
      availableStyles <- sapply(self$styles, function(x){x$name})
      self$styles = self$styles[which(availableStyles != style$name)]
      endNb = length(self$styles)
      return(endNb == startNb-1)
    }
   
  )                       
)

#' Geoserver REST API Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @importFrom openssl base64_encode
#' @import httr
#' @import xml2
#' @import keyring
#' @import cli
#' @import magrittr
#' @importFrom readr read_csv
#' @importFrom readr write_csv
#' @export
#' @keywords geoserver rest api
#' @return Object of \code{\link[R6]{R6Class}} with methods for communication with
#' the REST API of a GeoServer instance.
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
GSManager <- R6Class("GSManager",
  lock_objects = FALSE,

  private = list(
    keyring_backend = NULL,
    keyring_service = NULL,
    user = NA
  ),
                     
  public = list(
    
    #' @field verbose.info if geosapi logs have to be printed
    verbose.info = FALSE,
    #' @field verbose.debug if curl logs have to be printed
    verbose.debug = FALSE,
    #' @field loggerType the type of logger
    loggerType = NULL,
    
    #'@description Prints a log message
    #'@param type type of log, "INFO", "WARN", "ERROR"
    #'@param text text
    logger = function(type, text){
      if(self$verbose.info){
        cat(sprintf("[geosapi][%s] %s \n", type, text))
      }
    },
    
    #'@description Prints an INFO log message
    #'@param text text
    INFO = function(text){self$logger("INFO", text)},
    
    #'@description Prints an WARN log message
    #'@param text text
    WARN = function(text){self$logger("WARN", text)},
    
    #'@description Prints an ERROR log message
    #'@param text text
    ERROR = function(text){self$logger("ERROR", text)},
    
    #' @field url the Base url of GeoServer
    url = NA,
    #' @field version the version of Geoserver. Handled as \code{GSVersion} object
    version = NULL,
    
    #'@description This method is used to instantiate a GSManager with the \code{url} of the
    #'    GeoServer and credentials to authenticate (\code{user}/\code{pwd}). 
    #'    
    #'    By default, the \code{logger} argument will be set to \code{NULL} (no logger). 
    #'    This argument accepts two possible values: \code{INFO}: to print only geosapi logs,
    #'    \code{DEBUG}: to print geosapi and CURL logs.
    #'    
    #'    The \code{keyring_backend} can be set to use a different backend for storing 
    #'    the Geoserver user password with \pkg{keyring} (Default value is 'env').
    #'@param url url
    #'@param user user
    #'@param pwd pwd
    #'@param logger logger
    #'@param keyring_backend keyring backend. Default is 'env'
    initialize = function(url, user, pwd, logger = NULL,
                          keyring_backend = 'env'){
      
      if(!keyring_backend %in% names(keyring:::known_backends)){
        errMsg <- sprintf("Backend '%s' is not a known keyring backend!", keyring_backend)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      private$keyring_backend <- keyring:::known_backends[[keyring_backend]]$new()
      private$keyring_service <- paste0("geosapi@", url)
      
      #logger
      if(!missing(logger)){
        if(!is.null(logger)){
          self$loggerType <- toupper(logger)
          if(!(self$loggerType %in% c("INFO","DEBUG"))){
            stop(sprintf("Unknown logger type '%s", logger))
          }
          if(self$loggerType == "INFO"){
            self$verbose.info = TRUE
          }else if(self$loggerType == "DEBUG"){
            self$verbose.info = TRUE
            self$verbose.debug = TRUE
          }
        }
      }
      
      #baseUrl
      if(missing(url)) stop("Please provide the GeoServer base URL")
      baseUrl = url
      if(!grepl("/rest", baseUrl)){
        if(grepl("/$", baseUrl)){
          baseUrl = paste0(baseUrl, "rest")
        }else{
          baseUrl = paste(baseUrl, "rest", sep = "/")
        }
      }
      self$url = baseUrl
      private$user = user
      private$keyring_backend$set_with_value(private$keyring_service, username = user, password = pwd)
      
      #try to connect
      if(self$getClassName() == "GSManager"){
        
        #test connection
        self$connect()
      
        #inherit managers methods (experimenting)
        list_of_classes <- ls(getNamespaceInfo("geosapi", "exports"))
        supportedManagers <- list_of_classes[regexpr("GS.+Manager", list_of_classes)>0]
        for(manager in supportedManagers){
          class <- eval(parse(text = paste0("geosapi::", manager)))
          man <- class$new(baseUrl, user, pwd, logger)          
          list_of_methods <- rev(names(man))
          for(method in list_of_methods){
            methodObj <- man[[method]]
            if(!(method %in% names(self)) && is(methodObj,"function")){
              self[[method]] <- methodObj
              environment(self[[method]]) <- environment(self$connect)
            } 
          }
        }
        
        #inherit GeoServer version
        self$version <- GSVersion$new(baseUrl, user, pwd)
      }
    
      invisible(self)
      
    },
    
    #'@description Get URL
    #'@return the Geoserver URL
    getUrl = function(){
      return(self$url)
    },
    
    #'@description Connects to geoServer
    #'@return \code{TRUE} if connected, raises an error otherwise
    connect = function(){
      req <- GSUtils$GET(
        url = self$getUrl(), 
        user = private$user, 
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user), 
        path = "",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 401){
        err <- "Impossible to connect to GeoServer: Wrong credentials"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      if(status_code(req) == 404){
        err <- "Impossible to connect to GeoServer: Incorrect URL or GeoServer temporarily unavailable"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      if(status_code(req) != 200){
        err <- sprintf("Impossible to connect to Geoserver: Unexpected error (status code %s)", status_code(req))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }else{
        msg = "Successfully connected to GeoServer!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }
      return(TRUE)
    },
   
    #'@description Reloads the GeoServer catalog
    #'@return \code{TRUE} if reloaded, \code{FALSE} otherwise
    reload = function(){
      msg = "Reloading GeoServer catalog"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      reloaded <- FALSE
      req <- GSUtils$POST(self$getUrl(), private$user, 
                          private$keyring_backend$get(service = private$keyring_service, username = private$user), 
                          "/reload",
                          content = NULL, contentType = "text/plain",
                          self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully reloaded GeoServer catalog!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        reloaded <- TRUE
      }else{
        err = "Error while reloading the GeoServer catalog"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(reloaded)
    },
    
    #'@description Get system status
    #'@return an object of class \code{data.frame} given the date time and metrics value
    getSystemStatus = function(){
      msg = "Get system status"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      datetime <- Sys.time()
      req <- GSUtils$GET(self$getUrl(), private$user, 
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         "/about/system-status",
                         contentType = "application/json",
                         self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully fetched system status"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching system status"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      content <- httr::content(req)
      status <- list(
        raw = do.call("rbind", lapply(content$metrics$metric, function(metric){
          met <- data.frame(metric, stringsAsFactors = FALSE)
          return(met)
        }))
      )
      status$values = {
        df <- cbind(time = datetime, data.frame(t(status$raw$value), stringsAsFactors = F))
        colnames(df) <- c("TIME", status$raw$identifier)
        df
      }

      return(status)
    },
    
    #'@description Monitors the Geoserver by launching a small shiny monitoring application
    #'@param file file where to store monitoring results
    #'@param append whether to append results to existing files
    #'@param sleep sleeping interval to trigger a system status call
    monitor = function(file = NULL, append = FALSE, sleep = 1){
      monitor <- GSShinyMonitor$new(manager = self, file = file, append = append, sleep = sleep)
      monitor$run()
    },

    #'@description Get class name
    #'@return the self class name, as \code{character}
    getClassName = function(){
      return(class(self)[1])
    },
    
    #Resources GS managers
    #---------------------------------------------------------------------------
    
    #'@description Get Workspace manager
    #'@return an object of class \link{GSWorkspaceManager}
    getWorkspaceManager = function(){
      return(GSWorkspaceManager$new(self$getUrl(), private$user, 
                                    private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                    self$loggerType))
    },
    
    #'@description Get Namespace manager
    #'@return an object of class \link{GSNamespaceManager}
    getNamespaceManager = function(){
      return(GSNamespaceManager$new(self$getUrl(), private$user, 
                                    private$keyring_backend$get(service = private$keyring_service, username = private$user)
                                    , self$loggerType))
    },
    
    #'@description Get Datastore manager
    #'@return an object of class \link{GSDataStoreManager}
    getDataStoreManager = function(){
      return(GSDataStoreManager$new(self$getUrl(), private$user,
                                    private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                    self$loggerType))
    },
    
    #'@description Get Coverage store manager
    #'@return an object of class \link{GSCoverageStoreManager}
    getCoverageStoreManager = function(){
      return(GSCoverageStoreManager$new(self$getUrl(), private$user,
                                    private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                    self$loggerType))
    },
    
    #'@description Get service manager
    #'@return an object of class \link{GSServiceManager}
    getServiceManager = function(){
      return(GSServiceManager$new(self$getUrl(), private$user,
                                private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                self$loggerType))      
    },
    
    #'@description Get style manager
    #'@return an object of class \link{GSStyleManager}
    getStyleManager = function(){
      return(GSStyleManager$new(self$getUrl(), private$user,
                                        private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                        self$loggerType))      
    }
    
  )
                     
)

#' Geoserver REST API Service Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api service
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer services
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSServiceManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#'  }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSServiceManager <- R6Class("GSServiceManager",
  inherit = GSManager,
  
  public = list(   
    
    #Settings generic CRUD methods
    #===========================================================================
    #'@description Get the service settings. To get the service settings for a specific workspace,
    #'    specify the workspace name as \code{ws} parameter, otherwise global settings are
    #'    retrieved.
    #'@param service service
    #'@param ws workspace name
    #'@return an object of class \link{GSServiceSettings}
    getServiceSettings = function(service, ws = NULL){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      restPath <- NULL
      service <- tolower(service)
      if(!is.null(ws)){
        self$INFO(sprintf("Fetching %s service settings in workspace '%s'", service, ws))
        restPath <- sprintf("/services/%s/workspaces/%s/settings.xml", service, ws)
      }else{
        self$INFO(sprintf("Fetching %s service global settings", service))
        restPath <- sprintf("/services/%s/settings.xml", service)
      }
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         restPath, verbose = self$verbose.debug)
      serviceSettings <- NULL
      if(status_code(req) == 200){
        settingsXML <- GSUtils$parseResponseXML(req)
        serviceSettings <- GSServiceSettings$new(xml = settingsXML, service = tolower(xml2::xml_name(xml2::as_xml_document(settingsXML))))
        msg = "Successfully fetched service settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching service settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(serviceSettings)
    },
    
    #'@description Get WMS settings. To get the WMS settings for a specific workspace,
    #'    specify the workspace name as \code{ws} parameter, otherwise global settings are
    #'    retrieved.
    #'@param ws workspace name
    #'@return an object of class \link{GSServiceSettings}
    getWmsSettings = function(ws = NULL){
      return(self$getServiceSettings(service = "WMS", ws = ws))
    },
    
    #'@description Get WFS settings. To get the WFS settings for a specific workspace,
    #'    specify the workspace name as \code{ws} parameter, otherwise global settings are
    #'    retrieved.
    #'@param ws workspace name
    #'@return an object of class \link{GSServiceSettings}
    getWfsSettings = function(ws = NULL){
      return(self$getServiceSettings(service = "WFS", ws = ws))
    },
    
    #'@description Get WCS settings. To get the WCS settings for a specific workspace,
    #'    specify the workspace name as \code{ws} parameter, otherwise global settings are
    #'    retrieved.
    #'@param ws workspace name
    #'@return an object of class \link{GSServiceSettings}
    getWcsSettings = function(ws = NULL){
      return(self$getServiceSettings(service = "WCS", ws = ws))
    },
    
    #'@description Updates the service settings with an object of class \code{GSServiceSettings}.
    #'    An optional workspace name \code{ws} can be specified to update service settings
    #'    applying to a workspace.
    #'@param serviceSettings serviceSettings object of class \link{GSServiceSettings}
    #'@param service service
    #'@param ws workspace name
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateServiceSettings = function(serviceSettings, service, ws = NULL){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      updated <- FALSE
      restPath <- NULL
      service <- tolower(service)
      if(!is.null(ws)){
        self$INFO(sprintf("Fetching %s service settings in workspace '%s'", service, ws))
        restPath <- sprintf("/services/%s/workspaces/%s/settings.xml", service, ws)
      }else{
        self$INFO(sprintf("Fetching %s service global settings", service))
        restPath <- sprintf("/services/%s/settings.xml", service)
      }
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user, 
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = restPath,
        content = GSUtils$getPayloadXML(serviceSettings),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated service settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating service settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes the service settings. This method is used internally by \pkg{geosapi} 
    #'    for disabling a service setting at workspace level.
    #'@param service service
    #'@param ws workspace name
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteServiceSettings = function(service, ws = NULL){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      deleted <- FALSE
      restPath <- NULL
      service <- tolower(service)
      if(!is.null(ws)){
        self$INFO(sprintf("Deleting %s service settings in workspace '%s'", service, ws))
        restPath <- sprintf("/services/%s/workspaces/%s/settings.xml", service, ws)
      }else{
        #normally this delete operation is not available
        self$INFO(sprintf("Deleting %s service global settings", service))
        restPath <- sprintf("/services/%s/settings.xml", service)
      }
      req <- GSUtils$DELETE(
        url = self$getUrl(), user = private$user, 
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = restPath, verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully deleted service settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleted service settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    },
    
    #'@description Updates the WMS settings with an object of class \code{GSServiceSettings}.
    #'    An optional workspace name \code{ws} can be specified to update WMS settings
    #'    applying to a workspace.
    #'@param serviceSettings service settings object of class \link{GSServiceSettings}
    #'@param ws workspace name
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    updateWmsSettings = function(serviceSettings, ws = NULL){
      return(self$updateServiceSettings(serviceSettings, service = "WMS", ws = ws))
    },
    
    #'@description Updates the WFS settings with an object of class \code{GSServiceSettings}.
    #'    An optional workspace name \code{ws} can be specified to update WFS settings
    #'    applying to a workspace.
    #'@param serviceSettings service settings object of class \link{GSServiceSettings}
    #'@param ws workspace name
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    updateWfsSettings = function(serviceSettings, ws = NULL){
      return(self$updateServiceSettings(serviceSettings, service = "WFS", ws = ws))
    },
    
    #'@description Updates the WCS settings with an object of class \code{GSServiceSettings}.
    #'    An optional workspace name \code{ws} can be specified to update WCS settings
    #'    applying to a workspace.
    #'@param serviceSettings service settings object of class \link{GSServiceSettings}
    #'@param ws workspace name
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    updateWcsSettings = function(serviceSettings, ws = NULL){
      return(self$updateServiceSettings(serviceSettings, service = "WCS", ws = ws))
    },
    
    #'@description Enables WMS service settings
    #'@param ws workspace name
    #'@return \code{TRUE} if enabled, \code{FALSE} otherwise
    enableWMS = function(ws = NULL){
      serviceSettings <- GSServiceSettings$new(service = "WMS")
      return(self$updateWmsSettings(serviceSettings, ws = ws))
    },
    
    #'@description Enables WFS service settings
    #'@param ws workspace name
    #'@return \code{TRUE} if enabled, \code{FALSE} otherwise
    enableWFS = function(ws = NULL){
      serviceSettings <- GSServiceSettings$new(service = "WFS")
      return(self$updateWfsSettings(serviceSettings, ws = ws))
    },
    
    #'@description Enables WCS service settings
    #'@param ws workspace name
    #'@return \code{TRUE} if enabled, \code{FALSE} otherwise
    enableWCS = function(ws = NULL){
      serviceSettings <- GSServiceSettings$new(service = "WCS")
      return(self$updateWcsSettings(serviceSettings, ws = ws))
    },
    
    #'@description Disables service settings
    #'@param service service
    #'@param ws workspace name
    #'@return \code{TRUE} if disabled, \code{FALSE} otherwise
    disableServiceSettings = function(service, ws = NULL){
      disabled <- FALSE
      if(is.null(ws)){
        #use update op for global service setting
        serviceSettings <- GSServiceSettings$new(service = service)
        serviceSettings$setEnabled(FALSE)
        disabled <- self$updateWmsSettings(serviceSettings, ws = ws)
      }else{
        #use delete op for workspace service setting
        disabled <- self$deleteServiceSettings(service = service, ws = ws)
      }
      return(disabled)
    },
    
    #'@description Disables WMS service settings
    #'@param ws workspace name
    #'@return \code{TRUE} if disabled, \code{FALSE} otherwise
    disableWMS = function(ws = NULL){
      return(self$disableServiceSettings(service = "WMS", ws = ws))
    },
    
    #'@description Disables WFS service settings
    #'@param ws workspace name
    #'@return \code{TRUE} if disabled, \code{FALSE} otherwise
    disableWFS = function(ws = NULL){
      return(self$disableServiceSettings(service = "WFS", ws = ws))
    },
    
    #'@description Disables WCS service settings
    #'@param ws workspace name
    #'@return \code{TRUE} if disabled, \code{FALSE} otherwise
    disableWCS = function(ws = NULL){
      return(self$disableServiceSettings(service = "WCS", ws = ws))
    }
    
  )
)

# test_GSServiceSettings.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSService*.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSServiceSettings")

gsman$createWorkspace("geosapiWS")
settings <- GSWorkspaceSettings$new()
created <- gsman$createWorkspaceSettings("geosapiWS", settings)

test_that("Enable/Disable WMS",{
  enabled <- gsman$enableWMS("geosapiWS")
  expect_true(enabled)
  wms_settings <- gsman$getWmsSettings("geosapiWS")
  expect_true(wms_settings$enabled)
  disabled <- gsman$disableWMS("geosapiWS")
  expect_true(disabled)
  wms_settings <- gsman$getWmsSettings("geosapiWS")
  expect_null(wms_settings)
})

test_that("Enable/Disable WFS",{
  enabled <- gsman$enableWFS("geosapiWS")
  expect_true(enabled)
  wfs_settings <- gsman$getWfsSettings("geosapiWS")
  expect_true(wfs_settings$enabled)
  disabled <- gsman$disableWFS("geosapiWS")
  expect_true(disabled)
  wfs_settings <- gsman$getWfsSettings("geosapiWS")
  expect_null(wfs_settings)
})

test_that("Enable/Disable WCS",{
  enabled <- gsman$enableWCS("geosapiWS")
  expect_true(enabled)
  wcs_settings <- gsman$getWcsSettings("geosapiWS")
  expect_true(wcs_settings$enabled)
  disabled <- gsman$disableWCS("geosapiWS")
  expect_true(disabled)
  wcs_settings <- gsman$getWcsSettings("geosapiWS")
  expect_null(wcs_settings)
})

gsman$deleteWorkspace("geosapiWS")

#' @name geosapi
#' @aliases geosapi-package
#' @aliases geosapi
#' 
#' @title GeoServer REST API R Interface
#' @description Provides an R interface to the GeoServer REST API, allowing to 
#' upload and publish data in a GeoServer web-application and expose data to OGC 
#' Web-Services. The package currently supports all CRUD (Create,Read,Update,Delete) 
#' operations on GeoServer workspaces, namespaces, datastores (stores of vector data), 
#' featuretypes, layers, styles, as well as vector data upload operations. For 
#' more information about the GeoServer REST API, see <https://docs.geoserver.org/main/en/user/rest/> .
#'  
#'@author Emmanuel Blondel \email{emmanuel.blondel1@@gmail.com}
#'
"_PACKAGE"

#' Geoserver REST API Store
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api store
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer store
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSAbstractStore <- R6Class("GSAbstractStore",
 inherit = GSRESTResource,
 private = list(
   store_type = NULL
 ),
 public = list(
   #'@field full whether store object is fully described
   full = FALSE,
   #' @field name store name
   name = NULL,
   #'@field enabled if the store is enabled or not
   enabled = NULL,
   #'@field description store description
   description = "",
   #' @field type store type
   type = NULL,
   #'@field workspace workspace name
   workspace = NULL,

   #'@description initializes an abstract store
   #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
   #'@param storeType store type
   #'@param type the type of coverage store
   #'@param name coverage store name
   #'@param description coverage store description
   #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
   initialize = function(xml = NULL, storeType, type = NULL,
                         name = NULL, description = "", enabled = TRUE){
     super$initialize(rootName = storeType)
     private$store_type = storeType
     if(!missing(xml) & !is.null(xml)){
       if(!any(class(xml) %in% c("xml_document","xml_node"))){
         stop("The argument 'xml' is not a valid XML object")
       }
       self$decode(xml)
     }else{
       if(is.null(name)) stop("The 'name' cannot be null")
       self$name = name
       self$description = description
       self$type = type
       self$enabled = enabled
       self$full <- TRUE
     }
   },
   
   #'@description Decodes store from XML
   #'@param xml object of class \link[xml2]{xml_node-class}
   decode = function(xml){
     xml = xml2::as_xml_document(xml)
     names <- xml2::xml_find_first(xml, sprintf("//%s/name", private$store_type))
     self$name <- xml2::xml_text(names)
     enabled <- xml2::xml_find_first(xml,"//enabled")
     self$full <- length(enabled) > 0
     if(self$full){
       self$enabled <- as.logical(xml2::xml_text(enabled))
       
       descriptionXML <- xml2::xml_find_first(xml,"//description")
       if(length(descriptionXML) > 0) self$description <- xml2::xml_text(descriptionXML)
       
       typeXML <- xml2::xml_find_first(xml,"//type")
       if(length(typeXML) > 0) self$type <- xml2::xml_text(typeXML)
     }
   },
   
   #'@description Set type
   #'@param type type
   setType = function(type){
     self$type = type
   },
   
   #'@description Set enabled
   #'@param enabled enabled
   setEnabled = function(enabled){
     self$enabled <- enabled
   },
   
   #'@description Set description
   #'@param description description
   setDescription = function(description){
     self$description = description
   }
 )                     
)

#' Geoserver REST API GSCoverageBand
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api coverageBand
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer coverage band
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSCoverageBand$new()
#' 

#'
#' @section Methods:
#' \describe{
#'  \item{\code{new(xml)}}{
#'    This method is used to instantiate a \code{GSCoverageBand}
#'  }
#'  \item{\code{decode(xml)}}{
#'    This method is used to decode a \code{GSCoverageBand} from XML
#'  }
#'  \item{\code{encode()}}{
#'    This method is used to encode a \code{GSCoverageBand} to XML
#'  }
#'  \item{\code{setDefinition(definition)}}{
#'    Sets the coverage band definition
#'  }
#'  \item{\code{setIndex(index)}}{
#'    Sets the coverage band index
#'  }
#'  \item{\code{setCompositionType}}{
#'    Sets the composition type. Only 'BAND_SELECT' is supported by GeoServer for now.
#'  }
#'  \item{\code{addInputBand(band)}}{
#'    Adds a input coverage band, object of class \code{GSInputCoverageBand}
#'  }
#'  \item{\code{delInputBand(band)}}{
#'    Removes a input coverage band, object of class \code{GSInputCoverageBand}
#'  }
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSCoverageBand <- R6Class("GSCoverageBand",
  inherit = GSRESTResource,                    
  public = list(
    
    #' @field inputCoverageBands list of input coverage bands
    inputCoverageBands = list(),
    #' @field definition coverage band definition
    definition = NULL,
    #' @field index coverage band index
    index = NULL,
    #' @field compositionType coverage band composition type
    compositionType = "BAND_SELECT",
    
    #'@description Initalizes a \link{GSCoverageBand}
    #'@param xml object of class \link[xml2]{xml_node-class}
    initialize = function(xml = NULL){
      super$initialize(rootName = "coverageBand")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }
    },
    
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      def <- xml2::xml_find_first(xml, "//definition")
      self$definition <- xml2::xml_text(def)
      
      idx <- xml2::xml_find_first(xml, "//index")
      self$index <- xml2::xml_text(idx)
      
      ct <- xml2::xml_find_first(xml, "//compositionType")
      self$compositionType <- xml2::xml_text(ct)
      
      bands <- as(xml2::xml_find_all(xml, "//inputCoverageBands/inputCoverageBand"),"list")
      if(length(bands)>0){
        for(band in bands){
          band <- GSInputCoverageBand$new(xml = band)
          self$addInputBand(band)
        }
      }   
    },
    
    #'@description Set name
    #'@param name name
    setName = function(name){
      self$name = name
    },
    
    #'@description Set definition
    #'@param definition definition
    setDefinition = function(definition){
      self$definition = definition
    },
    
    #'@description Set index
    #'@param index index
    setIndex = function(index){
      self$index = index
    },
    
    #'@description Set composition type
    #'@param compositionType composition type
    setCompositionType = function(compositionType){
      self$compositionType = compositionType
    },
    
    #'@description Adds an input band
    #'@param band object of class \link{GSInputCoverageBand}
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addInputBand = function(band){
      if(!is(band, "GSInputCoverageBand")){
        stop("The 'band' object object should be of class 'GSInputCoverageBand'")
      }
      startNb = length(self$inputCoverageBands)
      availableInputBands <- sapply(self$inputCoverageBands, function(x){x$band})
      if(length(which(availableInputBands == band$band)) == 0){
        self$inputCoverageBands = c(self$inputCoverageBands, band)
      }
      endNb = length(self$inputCoverageBands)
      return(endNb == startNb+1)
    },
    
    #'@description Deletes an input band
    #'@param band object of class \link{GSInputCoverageBand}
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    delInputBand = function(band){
      if(!is(band, "GSInputCoverageBand")){
        stop("The 'band' object object should be of class 'GSInputCoverageBand'")
      }
      startNb = length(self$inputCoverageBands)
      availableInputBands <- sapply(self$inputCoverageBands, function(x){x$band})
      self$inputCoverageBands = self$inputCoverageBands[which(availableInputBands != band$band)]
      endNb = length(self$inputCoverageBands)
      return(endNb == startNb-1)
    }
    
  )                     
)

#' Geoserver REST API ArcGridCoverageStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api CoverageStore ArcGrid
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer ArcGrid CoverageStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSArcGridCoverageStore <- R6Class("GSArcGridCoverageStore",
  inherit = GSAbstractCoverageStore,
  private = list(
    TYPE = "ArcGrid"
  ),
  public = list(
   #'@field url url
   url = NULL,
   
   #'@description initializes an abstract ArcGrid coverage store
   #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
   #'@param name coverage store name
   #'@param description coverage store description
   #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
   #'@param url url
   initialize = function(xml = NULL, name = NULL, description = "", enabled = TRUE, url = NULL){
     super$initialize(xml = xml, type = private$TYPE, 
                      name = name, description = description, enabled = enabled, url = url)
   }
  )                     
)

#' Geoserver REST API Dimension
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSDimension
#' @title A GeoServer dimension
#' @description This class models a GeoServer resource dimension.
#' @keywords geoserver rest api resource dimension
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer dimension
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   dim <- GSDimension$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSDimension <- R6Class("GSDimension",
  inherit = GSRESTResource,
  
  public = list(
    #' @field enabled true/false
    enabled = TRUE,
    #' @field presentation dimension presentation
    presentation = NULL,
    #' @field resolution dimension resolution
    resolution = NULL,
    #' @field units dimension units
    units = NULL,
    #' @field unitSymbol dimension unitsSymbol
    unitSymbol = NULL,
    
    #'@description Initializes an object of class \link{GSDimension}
    #'@param xml object of class \link[xml2]{xml_node-class}
    initialize = function(xml = NULL){
      super$initialize(rootName = "dimensionInfo")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }
    },
   
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      enabled = xml2::xml_find_first(xml, "//enabled") %>% xml2::xml_text() %>% as.logical()
      self$setEnabled(enabled)
      presentation = xml2::xml_find_first(xml, "//presentation") %>% xml2::xml_text()
      self$setPresentation(presentation)
      units = xml2::xml_find_first(xml, "//units") %>% xml2::xml_text()
      self$setUnit(units)
      unitSymbol = xml2::xml_find_first(xml, "//unitSymbol") %>% xml2::xml_text()
      if(!is.na(unitSymbol)) self$setUnitSymbol(unitSymbol)
    },
    
    #'@description Set enabled
    #'@param enabled enabled
    setEnabled = function(enabled){
      self$enabled = enabled
    },
    
    #'@description Set presentation
    #'@param presentation presentation. Possible values: "LIST", "CONTINUOUS_INTERVAL", "DISCRETE_INTERVAL"
    #'@param interval interval
    setPresentation = function(presentation, interval = NULL){
      supportedPresentations <- c("LIST", "CONTINUOUS_INTERVAL", "DISCRETE_INTERVAL")
      if(!(presentation %in% supportedPresentations)){
        err <- sprintf("Unknown presentation '%s'. Supported values are: [%s]",
                       presentation, paste0(supportedPresentations, collapse=","))
        stop(err)
      }
      self$presentation = presentation
      if(self$presentation == "DISCRETE_INTERVAL"){
        if(missing(interval) | is.null(interval)){
          stop("Interval should be provided for DISCRETE_INTERVAL presentation")
        }
        self$resolution <- interval
      }
    },
    
    #'@description Set unit
    #'@param unit unit
    setUnit = function(unit){
      self$units = unit
    },
    
    #'@description Set unit symbol
    #'@param unitSymbol unit symbol
    setUnitSymbol = function(unitSymbol){
      self$unitSymbol = unitSymbol
    }
   
  )
                         
)

# test_GSDimension.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSDimension.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSDimension")

test_that("dimension encoding/decoding",{

  dim <- GSDimension$new()
  dim$setEnabled(TRUE)
  dim$setPresentation("CONTINUOUS_INTERVAL")
  dim$setUnit("ISO8601")
  expect_true(dim$enabled)
  expect_equal(dim$presentation, "CONTINUOUS_INTERVAL")
  expect_equal(dim$units, "ISO8601")

  #encoding to XML
  dimXML <- dim$encode()
  expect_is(dimXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  dim2 <- GSDimension$new(xml = dimXML)
  dim2XML <- dim2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(dimXML, dim2XML))==0)
  
})

# test_GSFeatureDimension.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSFeatureDimension.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSFeatureDimension")

test_that("feature dimension encoding/decoding",{
  
  dim <- GSFeatureDimension$new()
  dim$setEnabled(TRUE)
  dim$setAttribute("START_YEAR")
  dim$setEndAttribute("END_YEAR")
  dim$setPresentation("CONTINUOUS_INTERVAL")
  dim$setUnit("ISO8601")
  expect_true(dim$enabled)
  expect_equal(dim$attribute, "START_YEAR")
  expect_equal(dim$endAttribute, "END_YEAR")
  expect_equal(dim$presentation, "CONTINUOUS_INTERVAL")
  expect_equal(dim$units, "ISO8601")
  
  #encoding to XML
  dimXML <- dim$encode()
  expect_is(dimXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  dim2 <- GSFeatureDimension$new(xml = dimXML)
  dim2XML <- dim2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(dimXML, dim2XML))==0)
  
})

test_that("featureType metadata",{
  
  ft <- GSFeatureType$new()
  ft$setName("name")
  
  timeDimension <- GSFeatureDimension$new()
  timeDimension$setEnabled(TRUE)
  timeDimension$setAttribute("START_YEAR")
  timeDimension$setEndAttribute("END_YEAR")
  timeDimension$setPresentation("CONTINUOUS_INTERVAL")
  timeDimension$setUnit("ISO8601")
  ft$setMetadata("time", timeDimension)
  
  #encoding to XML
  ftXML <- ft$encode()
  expect_is(ftXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  ft2 <- GSFeatureType$new(xml = ftXML)
  ft2XML <- ft2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(ftXML, ft2XML))==0)
  
})

#' Geoserver REST API Resource
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSFeatureType
#' @title A GeoServer feature type
#' @description This class models a GeoServer feature type. This class is to be
#' used for manipulating representations of vector data with GeoServer.
#' @keywords geoserver rest api resource featureType
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer feature type
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   ft <- GSFeatureType$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSFeatureType <- R6Class("GSFeatureType",
  inherit = GSResource,

  public = list(
    #'@field cqlFilter CQL filter
    cqlFilter = NULL,
    
    #'@description Initializes an object of class \link{GSFeatureType}
    #'@param xml object of class \link[xml2]{xml_node-class}
    initialize = function(xml = NULL){
      super$initialize(rootName = "featureType")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }
    },
    
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      super$decode(xml)
    },
    
    #'@description Set CQL filter
    #'@param cqlFilter CQL filter
    setCqlFilter = function(cqlFilter){
      self$cqlFilter <- cqlFilter
    },
    
    #'@description Set virtual table
    #'@param vt object of class \link{GSVirtualTable}
    #'@return \code{TRUE} if set/added, \code{FALSE} otherwise
    setVirtualTable = function(vt){
      if(!is(vt, "GSVirtualTable")){
        stop("Argument 'vt' should be an object of class 'GSVirtualTable'")
      }
      added <- super$setMetadata("JDBC_VIRTUAL_TABLE", vt)
      return(added)
    },
    
    #'@description Deletes virtual table
    #'@param vt object of class \link{GSVirtualTable}
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    delVirtualTable = function(){
      deleted <- super$delMetadata("JDBC_VIRTUAL_TABLE")
      return(deleted)
    }
  
  )
                    
)

#' Geoserver REST API WorldImageCoverageStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api CoverageStore WorldImage
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer WorldImage CoverageStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSWorldImageCoverageStore <- R6Class("GSWorldImageCoverageStore",
  inherit = GSAbstractCoverageStore,
  private = list(
    TYPE = "WorldImage"
  ),
  public = list(
    #'@field url url
    url = NULL,
    
    #'@description Initializes an WorldImage coverage store
    #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
    #'@param name coverage store name
    #'@param description coverage store description
    #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
    #'@param url url
    initialize = function(xml = NULL, name = NULL, description = "", enabled = TRUE, url = NULL){
      super$initialize(xml = xml, type = private$TYPE, 
                       name = name, description = description, enabled = enabled, url = url)
    }
  )                     
)

#' Geoserver REST API GSCoverageView
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api coverageView
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer coverage view
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   GSCoverageView$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSCoverageView <- R6Class("GSCoverageView",
  inherit = GSRESTResource,                    
  public = list(
    #'@field name name
    name = NA,
    #'@field envelopeCompositionType envelope composition type
    envelopeCompositionType = NULL,
    #'@field selectedResolution selected resolution
    selectedResolution = NULL,
    #'@field selectedResolutionIndex selected resolution index
    selectedResolutionIndex = NULL,
    #'@field coverageBands coverage bands
    coverageBands = list(),
    
    #'@description Initializes an object of class \link{GSCoverageView}
    #'@param xml object of class \link[xml2]{xml_node-class}
    initialize = function(xml = NULL){
      super$initialize(rootName = "coverageView")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }
    },
    
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      names <- xml2::xml_find_first(xml, "//name") 
      self$name <- xml2::xml_text(names)
      
      ect <- xml2::xml_find_first(xml, "//envelopeCompositionType")
      self$envelopeCompositionType <- xml2::xml_text(ect)
      
      sr <- xml2::xml_find_first(xml, "//selectedResolution")
      self$selectedResolution <- xml2::xml_text(sr)
      
      sri <- xml2::xml_find_first(xml, "//selectedResolutionIndex")
      self$selectedResolutionIndex <- xml2::xml_text(sri)
      
      bands <- as(xml2::xml_find_all(xml, "//coverageBand"), "list")
      if(length(bands)>0){
        for(band in bands){
          band <- GSCoverageBand$new(xml = band)
          self$addBand(band)
        }
      }   
    },
    
    #'@description Set name
    #'@param name name
    setName = function(name){
      self$name = name
    },
    
    #'@description Sets the envelope composition type. Type of Envelope Composition, used to expose the bounding box 
    #' of the CoverageView, either 'UNION' or 'INTERSECTION'.
    #'@param envelopeCompositionType envelope composition type
    setEnvelopeCompositionType = function(envelopeCompositionType){
      ectValues <- c("UNION", "INTERSECTION")
      if(!envelopeCompositionType %in% ectValues){
        stop(sprintf("The 'envelopeCompositionType' should be among values [%s]",
                     paste0(ectValues, collapse = ",")))
      }
      self$envelopeCompositionType = envelopeCompositionType
    },
    
    #'@description Set selected resolution
    #'@param selectedResolution selected resolution
    setSelectedResolution = function(selectedResolution){
      srValues <- c("BEST", "WORST")
      if(!selectedResolution %in% srValues){
        stop(sprintf("The 'selectedResolution' should be among values [%s]",
                     paste0(srValues, collapse = ",")))
      }
      self$selectedResolution = selectedResolution
    },
    
    #'@description Set selected resolution index
    #'@param selectedResolutionIndex selected resolution index
    setSelectedResolutionIndex = function(selectedResolutionIndex){
      self$selectedResolutionIndex = selectedResolutionIndex
    },
    
    #'@description Adds band
    #'@param band object of class \link{GSCoverageBand}
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addBand = function(band){
      if(!is(band, "GSCoverageBand")){
        stop("The 'band' object object should be of class 'GSCoverageBand'")
      }
      startNb = length(self$coverageBands)
      availableBands <- sapply(self$coverageBands, function(x){x$index})
      if(length(which(availableBands == band$index)) == 0){
        self$coverageBands = c(self$coverageBands, band)
      }
      endNb = length(self$coverageBands)
      return(endNb == startNb+1)
    },
    
    #'@description Deletes band
    #'@param band object of class \link{GSCoverageBand}
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    delBand = function(band){
      if(!is(band, "GSCoverageBand")){
        stop("The 'band' object object should be of class 'GSCoverageBand'")
      }
      startNb = length(self$coverageBands)
      availableBands <- sapply(self$coverageBands, function(x){x$index})
      self$coverageBands = self$coverageBands[which(availableBands != band$index)]
      endNb = length(self$coverageBands)
      return(endNb == startNb-1)
    }
    
  )                     
)

# test_GSCoverageView.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSCoverageView.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSCoverageView")

test_that("GSCoverageView encoding/decoding",{

  coview <- GSCoverageView$new()
  coview$setName("sfdem_new")
  coview$setEnvelopeCompositionType("INTERSECTION")
  coview$setSelectedResolution("BEST")
  coview$setSelectedResolutionIndex(-1)
  coviewband <- GSCoverageBand$new()
  coviewband$setDefinition("sfdem_new@0")
  coviewband$setIndex(0)
  coviewband$addInputBand(GSInputCoverageBand$new( coverageName = "sfdem_new", band = 0))
  coview$addBand(coviewband)
  
  #encoding to XML
  coviewXML <- coview$encode()
  expect_is(coviewXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  coview2 <- GSCoverageView$new(xml = coviewXML)
  coview2XML <- coview2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(coviewXML, coview2XML))==0)
  
})

#' Geoserver REST API Workspace Setting
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api workspace settings
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer workspace settings
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   settings <- GSWorkspaceSettings$new()
#'   settings$setCharset("UTF-8")
#'   settings$setNumDecimals(5)
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSWorkspaceSettings <- R6Class("GSWorkspaceSettings",
   inherit = GSRESTResource,                    
   public = list(
     #'@field contact contact
     contact = NULL,
     #'@field charset charset
     charset = "UTF-8",
     #'@field numDecimals number of decimal
     numDecimals = 8,
     #'@field onlineResource online resource
     onlineResource = NULL,
     #'@field verbose verbose
     verbose = FALSE,
     #'@field verboseExceptions verbose exceptions
     verboseExceptions = FALSE,
     #'@field localWorkspaceIncludesPrefix local workspace includes prefix
     localWorkspaceIncludesPrefix = TRUE,
     
     #'@description This method is used to instantiate a \code{GSWorkspaceSettings}. This settings 
     #'    object is required to activate a workspace configuration, using the method
     #'    \code{GSManager$createWorkspaceSettings}. Supported from GeoServer 2.12
     #' @param xml object of class \link[xml2]{xml_node-class}
     initialize = function(xml = NULL){
       super$initialize(rootName = "settings")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
       charset <- xml2::xml_find_first(xml, "//charset")
       if(length(charset)>0) self$charset <- xml2::xml_text(charset)
       numDecimals <- xml2::xml_find_first(xml, "//numDecimals")
       if(length(numDecimals)>0) self$numDecimals <- as.integer(xml2::xml_text(numDecimals))
       onlineResource <- xml2::xml_find_first(xml, "//onlineResource")
       if(length(onlineResource)>0) self$onlineResource <- xml2::xml_text(onlineResource)
       verbose <- xml2::xml_find_first(xml, "//verbose")
       if(length(verbose)>0) self$verbose <- as.logical(toupper(xml2::xml_text(verbose)))
       verboseExceptions <- xml2::xml_find_first(xml, "//verboseExceptions")
       if(length(verboseExceptions)>0) self$verboseExceptions <- as.logical(toupper(xml2::xml_text(verboseExceptions)))
       includesPrefix <- xml2::xml_find_first(xml, "//localWorkspaceIncludesPrefix")
       if(length(includesPrefix)>0) self$localWorkspaceIncludesPrefix <- as.logical(toupper(xml2::xml_text(includesPrefix)))
     },
     
     #'@description Set charset
     #'@param charset charset
     setCharset = function(charset){
       self$charset <- charset
     },
     
     #'@description Set number of decimals
     #'@param numDecimals number of decimals
     setNumDecimals = function(numDecimals){
       self$numDecimals <- numDecimals
     },
     
     #'@description Set online resource
     #'@param onlineResource online resource
     setOnlineResource = function(onlineResource){
       self$onlineResource <- onlineResource
     },
     
     #'@description Set verbose
     #'@param verbose verbose
     setVerbose = function(verbose){
       self$verbose <- verbose
     },
     
     #'@description Set verbose exceptions
     #'@param verboseExceptions verbose exceptions
     setVerboseExceptions = function(verboseExceptions){
       self$verboseExceptions <- verboseExceptions
     },
     
     #'@description Set local workspace includes prefix
     #'@param includesPrefix includes prefix
     setLocalWorkspaceIncludesPrefix = function(includesPrefix){
       self$localWorkspaceIncludesPrefix <- includesPrefix
     }
     
   )                     
)

#' Geoserver REST API GeoTIFF CoverageStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api GeoTIFF CoverageStore
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer GeoTIFF CoverageStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSGeoTIFFCoverageStore <- R6Class("GSGeoTIFFCoverageStore",
   inherit = GSAbstractCoverageStore,
   private = list(
     TYPE = "GeoTIFF"
   ),
   public = list(
     #'@field url url
     url = NULL,
     
     #'@description Initializes an GeoTIFF coverage store
     #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
     #'@param name coverage store name
     #'@param description coverage store description
     #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
     #'@param url url
     initialize = function(xml = NULL, name = NULL, description = "", enabled = TRUE, url = NULL){
       super$initialize(xml = xml, type = private$TYPE, 
                        name = name, description = description, enabled = enabled, url = url)
     }
   )                     
)

#' Geoserver REST API Metadatalink
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSMetadataLink
#' @title A GeoServer resource metadataLink
#' @description This class models a GeoServer resource metadataLink made of a type
#' (free text e.g. text/xml, text/html), a metadataType (Possible values are 
#' ISO19115:2003, FGDC, TC211, 19139, other), and a content: an URL that gives 
#' the metadataLink
#' @keywords geoserver rest api resource metadataLink
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer resource metadataLink
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSMetadataLink <- R6Class("GSMetadataLink",
   inherit = GSRESTResource,
   
   public = list(
     #'@field type type
     type = NULL,
     #'@field metadataType metadata type
     metadataType = NULL,
     #'@field content content
     content = NULL,
     
     #'@description Initializes an object of class \link{GSMetadataLink}
     #'@param xml object of class \link[xml2]{xml_node-class}
     #'@param type type
     #'@param metadataType metadata type
     #'@param content content
     initialize = function(xml = NULL, type, metadataType, content){
       super$initialize(rootName = "metadataLink")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }else{
         self$setType(type)
         self$setMetadataType(metadataType)
         self$setContent(content)
       }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
        self$setType(xml2::xml_find_first(xml, "//type") %>% xml2::xml_text())
        self$setMetadataType(xml2::xml_find_first(xml, "//metadataType") %>% xml2::xml_text())
        self$setContent(xml2::xml_find_first(xml, "//content") %>% xml2::xml_text())
     },
     
     #'@description Set type type
     #'@param type type
     setType = function(type){
        self$type = type
     },
     
     #'@description Set metadata type
     #'@param metadataType metadata type. Supported values: "ISO19115:2003", "FGDC", "TC211", "19139", "other"
     setMetadataType = function(metadataType){
        supportedMetadataTypes <- c("ISO19115:2003", "FGDC", "TC211", "19139", "other")
        if(!(metadataType %in% supportedMetadataTypes)){
          err <- sprintf("Unknown metadataType '%s'. Supported values are: [%s]",
                         metadataType, paste0(supportedMetadataTypes, collapse=","))
          stop(err)
        }
        self$metadataType <- metadataType
     },
     
     #'@description Set content 
     #'@param content content
     setContent = function(content){
       self$content = content
     }
     
   )
                         
)

#' Geoserver REST API Manager Utils
#'
#' @docType class
#' @export
#' @keywords geoserver rest api
#' @return Object of \code{\link[R6]{R6Class}} with static util methods for communication
#' with the REST API of a GeoServer instance.
#' @format \code{\link[R6]{R6Class}} object.
#'
#' @section Static methods:
#' \describe{
#'  \item{\code{getUserAgent()}}{
#'    This method is used to get the user agent for performing GeoServer API requests.
#'    Here the user agent will be compound by geosapi package name and version.
#'  }
#'  \item{\code{getUserToken(user, pwd)}}{
#'    This method is used to get the user authentication token for performing GeoServer
#'    API requests. Token is given a Base64 encoded string.
#'  }
#'  \item{\code{GET(url, user, pwd, path, verbose)}}{
#'    This method performs a GET request for a given \code{path} to GeoServer REST API
#'  }
#'  \item{\code{PUT(url, user, pwd, path, filename, contentType, verbose)}}{
#'    This method performs a PUT request for a given \code{path} to GeoServer REST API,
#'    to upload a file of name \code{filename} with given \code{contentType}
#'  }
#'  \item{\code{POST(url, user, pwd, path, content, contentType, verbose)}}{
#'    This method performs a POST request for a given \code{path} to GeoServer REST API,
#'    to post content of given \code{contentType}
#'  }
#'  \item{\code{DELETE(url, user, pwd, path, verbose)}}{
#'    This method performs a DELETE request for a given GeoServer resource identified
#'    by a \code{path} in GeoServer REST API
#'  }
#'  \item{\code{parseResponseXML(req)}}{
#'    Convenience method to parse XML response from GeoServer REST API.
#'  }
#'  \item{\code{getPayloadXML(obj)}}{
#'    Convenience method to create payload XML to send to GeoServer.
#'  }
#'  \item{\code{setBbox(minx, miny, maxx, maxy, bbox, crs)}}{
#'    Creates an list object representing a bbox. Either from coordinates or from
#'    a \code{bbox} object (matrix).
#'  }
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSUtils <- R6Class("GSUtils")

GSUtils$getUserAgent <- function(){
  return(paste("geosapi", packageVersion("geosapi"), sep="-"))
}

GSUtils$getUserToken <- function(user, pwd){
  token <- openssl::base64_encode(charToRaw(paste(user, pwd, sep=":")))
  return(token)
}

GSUtils$GET <- function(url, user, pwd, path = "", contentType = "text/xml", verbose = FALSE, filename = NULL){
  req <- NULL
  if(verbose){
    if(is.null(filename)){
      req <- with_verbose(GSUtils$GET(url, user, pwd, path))
    }else{
      req <- with_verbose(GSUtils$GET(url, user, pwd, path, filename = filename))
    }
  }else{
    if(!grepl("^/", path) && path != "") path = paste0("/", path)
    url <- paste0(url, path)
    if(is.null(filename)){
      req <- httr::GET(
        url = url,
        add_headers(
          "User-Agent" = GSUtils$getUserAgent(),
          "Authorization" = paste("Basic", GSUtils$getUserToken(user, pwd)),
          "Content-Type" = contentType
        )
      )
    }else{
      req <- httr::GET(
        url = url,
        add_headers(
          "User-Agent" = GSUtils$getUserAgent(),
          "Authorization" = paste("Basic", GSUtils$getUserToken(user, pwd))
        ),
        httr::write_disk(path = filename, overwrite = TRUE)
      )
    }
  }
  return(req)
}

GSUtils$PUT <- function(url, user, pwd, path,
                        content = NULL, filename = NULL,
                        contentType, verbose = FALSE){
  if(verbose){
    req <- with_verbose(GSUtils$PUT(url, user, pwd, path, content, filename, contentType))
  }else{
    body <- NULL
    if(missing(content) | is.null(content)){
      if(missing(filename) | is.null(filename)){
        stop("The filename must be provided")
      }
      body <- httr::upload_file(filename)
    }else{
      body <- content
    }
    
    if(!grepl("^/", path)) path = paste0("/", path)
    url <- paste0(url, path)
    req <- httr::PUT(
      url = url,
      add_headers(
        "User-Agent" = GSUtils$getUserAgent(),
        "Authorization" = paste("Basic", GSUtils$getUserToken(user, pwd)),
        "Content-Type" = contentType
      ),    
      body = body
    )
  }
  return(req)
}

GSUtils$POST <- function(url, user, pwd, path, content, contentType, verbose = FALSE){
  if(verbose){
     req <- with_verbose(GSUtils$POST(url, user, pwd, path, content, contentType))
  }else{
    if(!grepl("^/", path)) path = paste0("/", path)
    url <- paste0(url, path)
    req <- httr::POST(
      url = url,
      add_headers(
        "User-Agent" = GSUtils$getUserAgent(),
        "Authorization" = paste("Basic", GSUtils$getUserToken(user, pwd)),
        "Content-Type" = contentType
      ),    
      body = content
    )
  }
  return(req)
}

GSUtils$DELETE <- function(url, user, pwd, path, verbose = FALSE){
  if(verbose){
    req <- with_verbose(GSUtils$DELETE(url, user, pwd, path))
  }else{
    if(!grepl("^/", path)) path = paste0("/", path)
    url <- paste0(url, path)
    req <- httr::DELETE(
      url = url,
      add_headers(
        "User-Agent" = GSUtils$getUserAgent(),
        "Authorization" = paste("Basic", GSUtils$getUserToken(user, pwd))
      )
    )
  }
  return(req)
}

GSUtils$parseResponseXML <- function(req){
  return(xml2::read_xml(httr::content(req, "text", encoding = "UTF-8")))
}

GSUtils$getPayloadXML <- function(obj){
  if(!("encode" %in% names(obj))){
    stop("R6 class with no XML encoder method!")
  }
  xml <- obj$encode()
  xmltext <- as(xml, "character")
  payload <- gsub("[\r\n ] ", "", xmltext)
  return(payload)
}

GSUtils$setBbox = function(minx, miny, maxx, maxy, bbox = NULL, crs){
  
  if(!missing(bbox) & !is.null(bbox)){
    if(!is(bbox,"matrix")) stop("Bbox is not a valid bounding box matrix")
    if(all(dim(bbox) != c(2,2))) stop("Bbox is not a valid bounding box matrix")
    minx = bbox[1L,1L]
    miny = bbox[2L,1L]
    maxx = bbox[1L,2L]
    maxy = bbox[2L,2L]
  }
  
  out <- list()
  out[["minx"]] = minx
  out[["miny"]] = miny
  out[["maxx"]] = maxx
  out[["maxy"]] = maxy
  out[["crs"]] = crs
  return(out)
}

GSUtils$isXMLString = function(str){
  return(length(grep("<([a-zA-Z]+:)?[a-zA-Z]+(/?>| [a-zA-Z]+=[\"'])", str)) > 0)
}

#' Geoserver REST API GSVirtualTable
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api virtualTable
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer virtual table
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSVirtualTable$new()
#'
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSVirtualTable <- R6Class("GSVirtualTable",
   inherit = GSRESTResource,                    
   public = list(
      #' @field name name
     name = NA,
     #' @field sql SQL statement
     sql = NA,
     #' @field escapeSql escape SQL?
     escapeSql = FALSE,
     #' @field keyColumn key column
     keyColumn = NULL,
     #' @field geometry geometry
     geometry = NULL,
     #' @field parameters list of virtual parameters
     parameters = list(),
     
     #'@description Initializes an object of class \link{GSVirtualTable}
     #'@param xml object of class \link[xml2]{xml_node-class}
     initialize = function(xml = NULL){
       super$initialize(rootName = "virtualTable")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
       self$name <- xml2::xml_find_first(xml, "//name") %>% xml2::xml_text()
       self$sql <- xml2::xml_find_first(xml, "//sql") %>% xml2::xml_text()
       escapeSql <- xml2::xml_find_first(xml, "//escapeSql")
       if(length(escapeSql)>0){
         self$escapeSql <- as.logical(xml2::xml_text(escapeSql))
       }
       keyColumns <- xml2::xml_find_first(xml, "//keyColumn")
       if(length(keyColumns)>0){
         self$keyColumn <- xml2::xml_text(keyColumns)
       }
       geoms <- xml2::xml_find_first(xml, "//geometry")
       if(length(geoms)>0){
         vtg <- GSVirtualTableGeometry$new(xml = geoms)
         self$setGeometry(vtg)
       }
       params <- as(xml2::xml_find_all(xml, "//parameter"), "list")
       if(length(params)>0){
        for(param in params){
          vtp <- GSVirtualTableParameter$new(xml = param)
          self$addParameter(vtp)
        }
       }   
     },
     
     #'@description Set name
     #'@param name name
     setName = function(name){
       self$name = name
     },
     
     #'@description Set SQL
     #'@param sql sql
     setSql = function(sql){
       self$sql = sql
     },
     
     #'@description Set escape SQL
     #'@param escapeSql escape SQL
     setEscapeSql = function(escapeSql){
       self$escapeSql = escapeSql
     },
     
     #'@description Set key column
     #'@param keyColumn key column
     setKeyColumn = function(keyColumn){
       self$keyColumn = keyColumn
     },
     
     #'@description Set geometry
     #'@param vtg object of class \link{GSVirtualTableGeometry}
     setGeometry = function(vtg){
       self$geometry = vtg
     },
     
     #'@description Adds parameter
     #'@param parameter object of class \link{GSVirtualTableParameter}
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addParameter = function(parameter){
       startNb = length(self$parameters)
       availableParams <- sapply(self$parameters, function(x){x$name})
       if(length(which(availableParams == parameter$name)) == 0){
         self$parameters = c(self$parameters, parameter)
       }
       endNb = length(self$parameters)
       return(endNb == startNb+1)
     },
    
     #'@description Deletes parameter
     #'@param parameter object of class \link{GSVirtualTableParameter}
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     delParameter = function(parameter){
       startNb = length(self$parameters)
       availableParams <- sapply(self$parameters, function(x){x$name})
       self$parameters = self$parameters[which(availableParams != parameter$name)]
       endNb = length(self$parameters)
       return(endNb == startNb-1)
     }
     
   )                     
)

# test_GSVirtualTable.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSVirtualTable.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSVirtualTable")

test_that("virtual table encoding/decoding",{
  
  vt <- GSVirtualTable$new()
  vt$setName("popstates")
  vt$setSql("select gid, state_name, the_geom from pgstates where persons between %low% and %high%")
  vtg <- GSVirtualTableGeometry$new(name = "the_geom", type = "MultiPolygon", srid = 4326)
  vt$setGeometry(vtg)
  vtp1 <- GSVirtualTableParameter$new(name = "low", defaultValue = "10000000",regexpValidator = "^[\\d]+$")
  vtp2 <- GSVirtualTableParameter$new(name = "high", defaultValue = "0",regexpValidator = "^[\\d]+$")
  vt$addParameter(vtp1)
  vt$addParameter(vtp2)
  
  vtXML <- vt$encode()
  expect_is(vtXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  vt2 <- GSVirtualTable$new(xml = vtXML)
  vt2XML <- vt2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(vtXML, vt2XML))==0)
  
})

#' Geoserver REST API Style
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSLayer
#' @title A GeoServer layer style
#' @description This class models a GeoServer style.
#' @keywords geoserver rest api resourcelayer style
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer style
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   lyr <- GSStyle$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSStyle <- R6Class("GSStyle",
  inherit = GSRESTResource,
  
  public = list(
    #'@field full full
    full = TRUE,
    #'@field name name
    name = NULL,
    #'@field filename filename
    filename = NULL,
    
    #'@description Initializes a \link{GSStyle}
    #'@param xml an object of class \link[xml2]{xml_node-class}
    #'@param name name
    #'@param filename filename
    initialize = function(xml = NULL, name = NULL, filename = NULL){
     super$initialize(rootName = "style")
     if(!missing(xml) & !is.null(xml)){
       self$decode(xml)
     }else{
       if(!missing(name) | !is.null(name)){
         self$setName(name)
       }
       if(!missing(filename) | !is.null(filename)){
         self$setFilename(filename)
       }
     }
    },
    
    #'@description Decodes from XML
    #'@param xml an object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      self$setName(xml2::xml_find_first(xml, "//name") %>% xml2::xml_text())
      filenames <- xml2::xml_find_first(xml, "//filename")
      if(length(filenames)==0) self$full <- FALSE
      if(self$full){
        self$setFilename(xml2::xml_text(filenames))
      }
    },
    
    #'@description set name
    #'@param name name
    setName = function(name){
     self$name = name
    },
    
    #'@description Set filename
    #'@param filename filename
    setFilename = function(filename){
     self$filename = filename
    }
  )                       
)

#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSFeatureDimension
#' @title A GeoServer dimension
#' @description This class models a GeoServer feature dimension.
#' @keywords geoserver rest api resource dimension
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer feature dimension
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   dim <- GSFeatureDimension$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSFeatureDimension <- R6Class("GSFeatureDimension",
  inherit = GSDimension,
  
  public = list(
   #'@field attribute attribute
   attribute = NULL,
   #'@field endAttribute end attribute
   endAttribute = NULL,
   
   #'@description Initializes an object of class \link{GSFeatureDimension}
   #'@param xml object of class \link[xml2]{xml_node-class}
   initialize = function(xml = NULL){
     super$initialize(xml)
     if(!missing(xml) & !is.null(xml)){
       self$decode(xml)
     }
   },
   
   #'@description Decodes from XML
   #'@param xml object of class \link[xml2]{xml_node-class}
   decode = function(xml){
     super$decode(xml)
     attribute = xml2::xml_find_first(xml, "//attribute") %>% xml2::xml_text()
     self$setAttribute(attribute)
     endAttribute = xml2::xml_find_first(xml, "//endAttribute") %>% xml2::xml_text()
     self$setEndAttribute(endAttribute)
   },
   
   #'@description Set attribute
   #'@param attribute attribute
   setAttribute = function(attribute){
     self$attribute = attribute
   },
   
   #'@description Set end attribute
   #'@param endAttribute end attribute
   setEndAttribute = function(endAttribute){
     self$endAttribute = endAttribute
   }
   
  )
                       
)

#' Geoserver REST API - Geoserver Version
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSVersion
#' @title A GeoServer version
#' @description This class allows to grab the GeoServer version. By default, a 
#' tentative is made to fetch version from web admin default page, since Geoserver 
#' REST API did not support GET operation for the Geoserver version in past releases
#' of Geoserver.
#' @keywords geoserver rest api version
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer version
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#' version <- GSVersion$new(
#'              url = "http://localhost:8080/geoserver",
#'              user = "admin", pwd = "geoserver"
#'            )
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSVersion <- R6Class("GSVersion",
  
  private = list(
    getVersionValue = function(version){
      version <- gsub("x", "0", version)
      version <- gsub("-SNAPSHOT", "", version)
      versions <- unlist(strsplit(version, "\\."))
      value <- list()
      value[["major"]] <- as.integer(versions[1])
      value[["minor"]] <- as.integer(versions[2])
      value[["revision"]] <- 0
      if(length(versions)==3){
        value[["revision"]] <- as.integer(versions[3])
      }
      return(value)
    }
  ),
                     
  public = list(
    #'@field version version
    version = NULL,
    #'@field value value
    value = NULL,
    
    #'@description Initializes an object of class \link{GSVersion}
    #'@param url url
    #'@param user user
    #'@param pwd pwd
    initialize = function(url, user, pwd){ 
      
      #try to grab version from web admin
      req <- httr::GET(paste(dirname(url), "web", sep = "/"))
      if(status_code(req) == 200){
        html <- xml2::read_html(content(req, "text", encoding = "ISO-8859-1"))
        trgSet <- xml2::xml_find_first(html, "//strong")
        if(length(trgSet) > 0){
          version <- xml2::xml_text(trgSet)
          value <- private$getVersionValue(version)
          if(is.list(value)){
            self$version <- version
            self$value <- value
          }
        }
      }
      
      #try to grab version from REST API
      if(is.null(self$version) & is.null(self$value)){
        req <- GSUtils$GET(url, user, pwd, "about/version.xml", verbose = FALSE)
        if(status_code(req) == 200){
          xml <- httr::content(req, encoding = "UTF-8")
          trgSet <- xml2::xml_find_first(xml, "//resource[@name='GeoServer']/Version")
          if(length(trgSet) > 0){
            version <- xml2::xml_text(trgSet)
            value <- private$getVersionValue(version)
            if(is.list(value)){
              self$version <- version
              self$value <- value
            }
          }
        }
      }
    },
    
    #'@description  Compares to a version and returns TRUE if it is lower, FALSE otherwise
    #'@param version version
    #'@return \code{TRUE} if lower, \code{FALSE} otherwise
    lowerThan = function(version){
      lower <- FALSE
      if(is.character(version)){
        value <- private$getVersionValue(version)
      }else if(is.list(version)){
        value <- version
      }
      lower <- (self$value$major < value$major)
      if(!lower & identical(self$value$major, value$major)){
        lower <- (self$value$minor < value$minor)
      }
      if(!lower & identical(self$value$minor, value$minor)){
        lower <- (self$value$revision < value$revision)
      }
      return(lower)
    },

    #'@description  Compares to a version and returns TRUE if it is greater, FALSE otherwise
    #'@param version version
    #'@return \code{TRUE} if greater, \code{FALSE} otherwise
    greaterThan = function(version){
      greater <- FALSE
      if(is.character(version)){
        value <- private$getVersionValue(version)
      }else if(is.list(version)){
        value <- version
      }
      greater <- (self$value$major > value$major)
      if(!greater & identical(self$value$major, value$major)){
        greater <- (self$value$minor > value$minor)
      } 
      if(!greater & identical(self$value$minor, value$minor)){
        greater <- (self$value$revision > value$revision)
      }
      return(greater)
    },
    
    #'@description  Compares to a version and returns TRUE if it is equal, FALSE otherwise
    #'@param version version
    #'@return \code{TRUE} if equal, \code{FALSE} otherwise
    equalTo = function(version){
      equal <- FALSE
      if(is.character(version)){
        value <- private$getVersionValue(version)
      }else if(is.list(version)){
        value <- version
      }
      equal <- !self$lowerThan(version) & !self$greaterThan(version)
      return(equal)
    }
  
  )                  
)

# test_GSVersion.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSVersion.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSVersion")

test_that("geoserver versions",{
  #gsVersion <- gsman$version
  #expect_is(gsVersion, "GSVersion")
  #GS_VERSION <- "2.12.1"
  #expect_equal(gsVersion$version, GS_VERSION)
  #expect_is(gsVersion$value, "list")
  #expect_equal(length(gsVersion$value), 3L)
  #expect_true(gsVersion$lowerThan("3.0.x"))
  #expect_true(gsVersion$lowerThan("2.11.3"))
  #expect_true(gsVersion$lowerThan("2.11.2"))
  #expect_true(gsVersion$greaterThan("1.5.1"))
  #expect_true(gsVersion$greaterThan("2.5.1"))
  #expect_true(gsVersion$greaterThan("2.10.x"))
  #expect_false(gsVersion$lowerThan("2.5.1"))
  #expect_false(gsVersion$greaterThan(GS_VERSION))
  #expect_false(gsVersion$lowerThan(GS_VERSION))
  #expect_true(gsVersion$equalTo(GS_VERSION))
  #expect_false(gsVersion$greaterThan("3.0.0"))
})

#' Geoserver REST API DataStore Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
#' DataStores (i.e. stores of vector data)
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSDataStoreManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#'  }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSDataStoreManager <- R6Class("GSDataStoreManager",
  inherit = GSManager,
  
  public = list(   
     
    #DataStore generic CRUD methods
    #===========================================================================
    
    #'@description Get the list of available dataStores. 
    #'@param ws workspace name
    #'@return an object of class \code{list} giving items of class \code{\link{GSAbstractDataStore}}
    getDataStores = function(ws){
      msg = sprintf("Fetching list of datastores in workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/datastores.xml", ws),
        verbose = self$verbose.debug)
      dsList <- NULL
      if(status_code(req) == 200){
        dsXML <- GSUtils$parseResponseXML(req)
        dsXMLList <- as(xml2::xml_find_all(dsXML, "//dataStore"), "list")
        dsList <- lapply(dsXMLList, function(xml){
          dsType <-  xml2::xml_find_first(xml, "//type") %>% xml2::xml_text()
          dataStore <- switch(dsType,
              "Shapefile" = GSShapefileDataStore$new(xml = xml),
              "Directory of spatial files (shapefiles)" = GSShapefileDirectoryDataStore$new(xml = xml),
              "GeoPackage" = GSGeoPackageDataStore$new(xml = xml),
              "PostGIS" = GSPostGISDataStore$new(xml = xml),
              "Oracle NG" = GSOracleNGDataStore$new(xml = xml),
              GSAbstractDataStore$new(xml = xml)
          )
          return(dataStore)
        })
        msg = sprintf("Successfully fetched %s datastores!", length(dsList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of datastores"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(dsList)
    },
    
    #'@description Get the list of available dataStore names.
    #'@param ws workspace name
    #'@return a vector of class \code{character}
    getDataStoreNames = function(ws){
      dsList <- sapply(self$getDataStores(ws), function(x){x$name})
      return(dsList)
    },
    
    #'@description Get an object of class \code{\link{GSAbstractDataStore}} given a workspace and datastore names.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@return the datastore
    getDataStore = function(ws, ds){
      msg = sprintf("Fetching datastore '%s' in workspace '%s'", ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/datastores/%s.xml", ws, ds),
        verbose = self$verbose.debug)
      dataStore <- NULL
      if(status_code(req) == 200){
        dsXML <- GSUtils$parseResponseXML(req)
        dsType <-  xml2::xml_find_first(dsXML, "//type") %>% xml2::xml_text()
        dataStore <- switch(dsType,
          "Shapefile" = GSShapefileDataStore$new(xml = dsXML),
          "Directory of spatial files (shapefiles)" = GSShapefileDirectoryDataStore$new(xml = dsXML),
          "GeoPackage" = GSGeoPackageDataStore$new(xml = dsXML),
          "PostGIS" = GSPostGISDataStore$new(xml = dsXML),
          "Oracle NG" = GSOracleNGDataStore$new(xml = dsXML),
          GSAbstractDataStore$new(xml = dsXML)
        )
        msg = "Successfully fetched datastore!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching datastore"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(dataStore)
    },
    
    #'@description Creates a datastore given a workspace and an object of class \code{\link{GSAbstractDataStore}}.
    #'@param ws workspace name
    #'@param dataStore datastore object of class \link{GSAbstractDataStore}
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createDataStore = function(ws, dataStore){
      msg = sprintf("Creating datastore '%s' in workspace '%s'", dataStore$name, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      req <- GSUtils$POST(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/datastores.xml", ws),
        content = GSUtils$getPayloadXML(dataStore),
        contentType = "text/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfully created datastore!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating datastore"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
    },
    
    
    #'@description Updates a datastore given a workspace and an object of class \code{\link{GSAbstractDataStore}}.
    #'@param ws workspace name
    #'@param dataStore datastore object of class \link{GSAbstractDataStore}
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateDataStore = function(ws, dataStore){
      updated <- FALSE
      msg = sprintf("Updating datastore '%s' in workspace '%s'", dataStore$name, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user, 
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/datastores/%s.xml", ws, dataStore$name),
        content = GSUtils$getPayloadXML(dataStore),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated datastore!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating datastore"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a datastore given workspace and datastore names.
    #'    By defaut, the option \code{recurse} is set to FALSE, ie datastore layers are not removed.
    #'    To remove all datastore layers, set this option to TRUE.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param recurse recurse 
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteDataStore = function(ws, ds, recurse = FALSE){
      msg = sprintf("Deleting datastore '%s' in workspace '%s'", ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/workspaces/%s/datastores/%s.xml", ws, ds)
      if(recurse) path <- paste0(path, "?recurse=true")
      req <- GSUtils$DELETE(self$getUrl(), private$user, 
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully deleted datastore!"
        cli::cli_alert_info(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting datastore"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)  
    },
  
    #DataStore featureType CRUD methods
    #===========================================================================
    
    #'@description Get the list of available feature types for given workspace and datastore.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param list list type value, among "configured", "available", "available_with_geom", "all"
    #'@return an object of class \code{list} giving items of class \code{\link{GSFeatureType}}
    getFeatureTypes = function(ws, ds, list = "configured"){
      msg = sprintf("Fetching featureTypes for datastore '%s' in workspace '%s'", ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      supportedListValues <- c("configured", "available", "available_with_geom", "all")
      if(!(list %in% supportedListValues)){
        err = sprintf("Unsupported 'list' parameter value '%s'. Possible values: [%s]",
                      list, paste0(supportedListValues, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/datastores/%s/featuretypes.xml?list=%s", ws, ds, list),
        verbose = self$verbose.debug)
      ftList <- NULL
      if(status_code(req) == 200){
        ftXML <- GSUtils$parseResponseXML(req)
        ftXMLList <- as(xml2::xml_find_all(ftXML, "//featureTypes/featureType"), "list")
        ftList <- lapply(ftXMLList, GSFeatureType$new)
        msg = sprintf("Successfully fetched %s featureTypes!", length(ftList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of featureTypes"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(ftList)
    },
    
    #'@description Get the list of available feature type names for given workspace and datastore.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@return a vector of class\code{character}
    getFeatureTypeNames = function(ws, ds){
      ftList <- sapply(self$getFeatureTypes(ws, ds), function(x){x$name})
      return(ftList)
    },
    
    #'@description Get an object of class \code{\link{GSFeatureType}} given a workspace, datastore and feature type names.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param ft feature type name
    #'@return an object of class \link{GSFeatureType}
    getFeatureType = function(ws, ds, ft){
      msg = sprintf("Fetching featureType '%s' in datastore '%s' (workspace '%s')", ft, ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, 
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/workspaces/%s/datastores/%s/featuretypes/%s.xml", ws, ds, ft),
        verbose = self$verbose.debug)
      featureType <- NULL
      if(status_code(req) == 200){
        ftXML <- GSUtils$parseResponseXML(req)
        featureType <- GSFeatureType$new(xml = ftXML)
        msg = "Successfully fetched featureType!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching featureType"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(featureType)
    },
    
    #'@description Creates a new featureType given a workspace, datastore names and an object of class \code{\link{GSFeatureType}}
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param featureType feature type
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createFeatureType = function(ws, ds, featureType){
      msg = sprintf("Creating featureType '%s' in datastore '%s' (workspace '%s')", featureType$name, ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      req <- GSUtils$POST(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/datastores/%s/featuretypes.xml", ws, ds),
        content = GSUtils$getPayloadXML(featureType),
        contentType = "text/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfully created featureType!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating featureType"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description Updates a featureType given a workspace, datastore names and an object of class \code{\link{GSFeatureType}}
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param featureType feature type
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateFeatureType = function(ws, ds, featureType){
      msg = sprintf("Updating featureType '%s' in datastore '%s' (workspace '%s')", featureType$name, ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      updated <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user, 
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/datastores/%s/featuretypes/%s.xml",
                       ws, ds, featureType$name),
        content = GSUtils$getPayloadXML(featureType),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated featureType!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating featureType"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a featureType given a workspace, datastore names, and an object of 
    #'    class \code{\link{GSFeatureType}}. By defaut, the option \code{recurse} is 
    #'    set to FALSE, ie datastore layers are not removed.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param ft feature type name
    #'@param recurse recurse
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteFeatureType = function(ws, ds, ft, recurse = FALSE){
      msg = sprintf("Deleting featureType '%s' in datastore '%s' (workspace '%s')", ft, ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/workspaces/%s/datastores/%s/featuretypes/%s.xml", ws, ds, ft)
      if(recurse) path <- paste0(path, "?recurse=true")
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfuly deleted featureType!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting featureType"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)  
    },
    
    #Featuretype Layer publication methods
    #===========================================================================    
    #'@description Publish a feature type/layer pair given a workspace and datastore. The name 'layer' here 
    #'encompasses both \link{GSFeatureType} and \link{GSLayer} resources.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param featureType object of class \link{GSFeatureType}
    #'@param layer object of class \link{GSLayer}
    #'@return \code{TRUE} if published, \code{FALSE} otherwise
    publishLayer = function(ws, ds, featureType, layer){
      msg = sprintf("Publishing layer '%s'", layer$name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      published <- FALSE
      if(featureType$name != layer$name){
        stop("FeatureType and Layer names differ!")
      }
      ftCreated <- self$createFeatureType(ws, ds, featureType)
      if(ftCreated){
        lyrCreated <- self$createLayer(layer)
        if(lyrCreated){
          published <- TRUE
          msg = "Successfully published layer!"
          cli::cli_alert_success(msg)
          self$INFO(msg)
        }else{
          #rolling back
          published <- FALSE
          msg = "Rolling back - deleting previously created FeatureType!"
          cli::cli_alert_warning(msg)
          self$WARN(msg)
          ftDeleted <- self$deleteFeatureType(ws, ds, featureType$name)
        }
      }
      if(!published){
        err = "Error while publishing layer"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(published)
    },
    
    #'@description Unpublish a feature type/layer pair given a workspace and datastore. The name 'layer' here 
    #'encompasses both \link{GSFeatureType} and \link{GSLayer} resources.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param lyr layer name
    #'@return \code{TRUE} if published, \code{FALSE} otherwise
    unpublishLayer = function(ws, ds, lyr){
      msg = sprintf("Unpublishing layer '%s'", lyr)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      unpublished <- FALSE
      layer <- self$getLayer(lyr)
      if(is(layer, "GSLayer")){
        lyrDeleted <- self$deleteLayer(lyr)
      }
      featureType <- self$getFeatureType(ws, ds, lyr)
      if(is(featureType, "GSFeatureType")){
        ftDeleted <- self$deleteFeatureType(ws, ds, lyr)
        if(ftDeleted){
            unpublished <- TRUE
            msg = "Successfully unpublished layer!"
            cli::cli_alert_success(msg)
            self$INFO(msg)
        }
      }
      return(unpublished)
    },
    
    #Upload methods
    #===========================================================================
    #'@description Uploads features data. The \code{extension} corresponds to the format/type of features to be uploaded among "shp", "spatialite", "h2", "gpkg". 
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param extension extension
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@param contentType content type
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadData = function(ws, ds, endpoint = "file", extension,
                          configure = "first", update = "append", filename,
                          charset, contentType){
      msg = sprintf("Uploading %s data in datastore '%s' (workspace '%s')",
                    toupper(extension), ds, ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      
      uploaded <- FALSE
      
      supportedEndpoints <- c("file","url","external")
      if(!(endpoint %in% supportedEndpoints)){
        err = sprintf("Unsupported endpoint '%s'. Possible values: [%s]",
                      endpoint, paste0(supportedEndpoints, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      supportedExtensions <- c("shp", "spatialite", "h2", "gpkg")
      if(!(extension %in% supportedExtensions)){
        err = sprintf("Unsupported extension '%s'. Possible values: [%s]",
                      extension, paste0(supportedExtensions, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      supportedConfigurations <- c("first", "none", "all")
      if(!(configure %in% supportedConfigurations)){
        err = sprintf("Unsupported configure parameter '%s'. Possible values: [%s]",
                      configure, paste0(supportedConfigurations, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      supportedUpdates <- c("append","overwrite")
      if(!(update %in% supportedUpdates)){
        err = sprintf("Unsupported update parameter '%s'. Possible values: [%s]",
                      update, paste0(supportedUpdates, collapse=","))
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/datastores/%s/%s.%s?configure=%s&update=%s",
                       ws, ds, endpoint, extension, configure, update),
        content = NULL,
        filename = filename,
        contentType = contentType,
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfull data upload!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        uploaded = TRUE
      }
      if(!uploaded){
        err = sprintf("Error while uploading data: %s", http_status(req)$message)
        cli::cli_alert_danger(err)
        self$ERROR(err)
        self$ERROR("Response headers -->")
        print(headers(req))
        self$ERROR("Response content -->")
        print(as(content(req), "character"))
      }
      return(uploaded)
    },
    
    #'@description Uploads zipped shapefile.
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadShapefile = function(ws, ds, endpoint = "file",
                                configure = "first", update = "append",
                               filename, charset = "UTF-8"){
      return(
        self$uploadData(ws, ds, endpoint, extension = "shp",
                        configure, update, filename, charset,
                        contentType = "application/zip")
      )
    },
    
    #'@description Uploads properties.
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadProperties = function(ws, ds, endpoint = "file",
                               configure = "first", update = "append",
                               filename, charset = "UTF-8"){
      return(
        self$uploadData(ws, ds, endpoint, extension = "properties",
                        configure, update, filename, charset,
                        contentType = "")
      )
    },
    
    #'@description Uploads H2 database.
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadH2 = function(ws, ds, endpoint = "file",
                        configure = "first", update = "append",
                        filename, charset = "UTF-8"){
      return(
        self$uploadData(ws, ds, endpoint, extension = "h2",
                        configure, update, filename, charset,
                        contentType = "")
      )
    },
    
    #'@description Uploads spatialite file.
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadSpatialite = function(ws, ds, endpoint = "file",
                        configure = "first", update = "append",
                        filename, charset = "UTF-8"){
      return(
        self$uploadData(ws, ds, endpoint, extension = "spatialite",
                        configure, update, filename, charset,
                        contentType = "application/x-sqlite3")
      )
    },
    
    #'@description Uploads App schema.
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadAppschema = function(ws, ds, endpoint = "file",
                                configure = "first", update = "append",
                                filename, charset = "UTF-8"){
      return(
        self$uploadData(ws, ds, endpoint, extension = "appschema",
                        configure, update, filename, charset,
                        contentType = "application/appschema")
      )
    },
    
    #'@description Uploads GeoPackage.
    #'    The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
    #'    of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
    #'    \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
    #'    both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
    #'    the first upload, \code{"overwrite"} in case the file should be overwriten.
    #'@param ws workspace name
    #'@param ds datastore name
    #'@param endpoint endpoint
    #'@param configure configure strategy among values: "first" or "none"
    #'@param update update strategy, among values: "append", "overwrite"
    #'@param filename file name of the resource to upload
    #'@param charset charset
    #'@return \code{TRUE} if uploaded, \code{FALSE} otherwise
    uploadGeoPackage = function(ws, ds, endpoint = "file",
                               configure = "first", update = "append",
                               filename, charset = "UTF-8"){
      contentType = ""
      if(endsWith(filename, ".zip")) contentType = "application/zip"
      return(
        self$uploadData(ws, ds, endpoint, extension = "gpkg",
                        configure, update, filename, charset,
                        contentType = contentType)
      )
    }
    
  )
                                                        
)

# test_GSDataStore.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSDataStore*.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSDataStore")

test_that("READ dataStore",{
  expect_is(gsman,"GSManager")
  ds <- gsman$getDataStore("topp", "taz_shapes")
  expect_is(ds, "GSAbstractDataStore")
  expect_true(all(c("name", "enabled", "type", "description", "connectionParameters", "full") %in% names(ds)))
  expect_equal(ds$name, "taz_shapes")
})

test_that("READ dataStores",{
  dslist <- gsman$getDataStores("topp")
  expect_true(all(sapply(dslist, function(x){class(x)[1] == "GSAbstractDataStore"})))
  dsnames <- gsman$getDataStoreNames("topp")
  expect_true(all(dsnames %in% c("states_shapefile","taz_shapes")))
})

#Shapefile
#---------------------------------------------------------------------------

test_that("CREATE dataStore - Shapefile",{
  ds = GSShapefileDataStore$new(name = "topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                url = "file:data/shapefiles/states.shp")
  created <- gsman$createDataStore("topp", ds)
  expect_true(created)
  ds <- gsman$getDataStore("topp", "topp_datastore")
  expect_is(ds, "GSShapefileDataStore")
  expect_equal(ds$description, "topp_datastore description")
  expect_true(ds$enabled)
})

test_that("UPDATE datastore - Shapefile",{
  dataStore <- gsman$getDataStore("topp", "topp_datastore")
  dataStore$setDescription("topp_datastore updated description")
  dataStore$setEnabled(FALSE)
  
  updated <- gsman$updateDataStore("topp", dataStore)
  expect_true(updated)
  ds <- gsman$getDataStore("topp", "topp_datastore")
  expect_is(ds, "GSShapefileDataStore")
  expect_equal(ds$description, "topp_datastore updated description")
  expect_false(ds$enabled)
})

test_that("DELETE dataStore - Shapefile",{
  deleted <- gsman$deleteDataStore("topp", "topp_datastore", TRUE)
  expect_true(deleted)
  ds <- gsman$getDataStore("topp", "topp_datastore")
  expect_is(ds, "NULL")
})

#Shapefile Directory
#---------------------------------------------------------------------------

test_that("CREATE dataStore - Directory of Shapefiles",{
  ds = GSShapefileDirectoryDataStore$new(name = "topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE, url = "file:data/shapefiles")
  created <- gsman$createDataStore("topp", ds)
  expect_true(created)
  ds <- gsman$getDataStore("topp", "topp_datastore")
  expect_is(ds, "GSShapefileDirectoryDataStore")
  expect_equal(ds$description, "topp_datastore description")
  expect_true(ds$enabled)
})

test_that("UPDATE datastore - Directory of Shapefiles",{
  dataStore <- gsman$getDataStore("topp", "topp_datastore")
  dataStore$setDescription("topp_datastore updated description")
  dataStore$setEnabled(FALSE)
  
  updated <- gsman$updateDataStore("topp", dataStore)
  expect_true(updated)
  ds <- gsman$getDataStore("topp", "topp_datastore")
  expect_is(ds, "GSShapefileDirectoryDataStore")
  expect_equal(ds$description, "topp_datastore updated description")
  expect_false(ds$enabled)
})

test_that("DELETE dataStore - Directory of Shapefiles",{
  deleted <- gsman$deleteDataStore("topp", "topp_datastore", TRUE)
  expect_true(deleted)
  ds <- gsman$getDataStore("topp", "topp_datastore")
  expect_is(ds, "NULL")
})

#GeoPackage
#------------------------------------------------------------------------
test_that("CREATE dataStore - GeoPackage",{
  ds = GSGeoPackageDataStore$new(name = "topp_datastore_gpkg",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                database = "file:data/somefile.gpkg")
  created <- gsman$createDataStore("topp", ds)
  expect_true(created)
  ds <- gsman$getDataStore("topp", "topp_datastore_gpkg")
  expect_is(ds, "GSGeoPackageDataStore")
  expect_equal(ds$description, "topp_datastore description")
  expect_true(ds$enabled)
})

test_that("UPDATE datastore - GeoPackage",{
  dataStore <- gsman$getDataStore("topp", "topp_datastore_gpkg")
  dataStore$setDescription("topp_datastore updated description")
  dataStore$setEnabled(FALSE)
  
  updated <- gsman$updateDataStore("topp", dataStore)
  expect_true(updated)
  ds <- gsman$getDataStore("topp", "topp_datastore_gpkg")
  expect_is(ds, "GSGeoPackageDataStore")
  expect_equal(ds$description, "topp_datastore updated description")
  expect_false(ds$enabled)
})

test_that("DELETE dataStore - GeoPackage",{
  deleted <- gsman$deleteDataStore("topp", "topp_datastore_gpkg", TRUE)
  expect_true(deleted)
  ds <- gsman$getDataStore("topp", "topp_datastore_gpkg")
  expect_is(ds, "NULL")
})

#' Geoserver REST API Access Control List Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api access control list ACL
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
#' Access Control List (ACL) operations.
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSAccessControlListManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#'  }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSAccessControlListManager <- R6Class("GSAccessControlListManager",
  inherit = GSManager,
  
  public = list(
    
    #'@description Set the catalog mode
    #'@param mode mode
    #'@return \code{TRUE} if set, \code{FALSE} otherwise
    setCatalogMode = function(mode = c("HIDE","MIXED","CHALLENGE")){
      mode = match.arg(mode)
      msg = sprintf("Setting catalog mode '%s'", mode)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      
      created <- FALSE
      
      catalogXml = xml2::xml_new_root("catalog")
      modeXml = xml2::xml_new_root("mode")
      xml2::xml_set_text(modeXml, mode)
      xml2::xml_add_child(catalogXml, modeXml)
      
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = "/security/acl/catalog.xml",
        content = as(catalogXml, "character"),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfuly set catalog mode!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while setting catalog mode"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description Get the catalog mode
    #'@return the mode either \code{HIDE}, \code{MIXED} or \code{CHALLENGE}
    getCatalogMode = function(){
      msg = "Fetching catalog mode"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, 
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        "/security/acl/catalog.xml", verbose = self$verbose.debug)
      catalogMode <- NULL
      if(status_code(req) == 200){
        ctXML <- GSUtils$parseResponseXML(req)
        catalogMode = xml2::xml_find_first(ctXML, "//catalog/mode") %>% xml2::xml_contents() %>% as.character()
        msg = "Successfuly fetched catalog mode!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching catalog mode"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(catalogMode)
    },
    
    #'@description Get rules
    #'@param domain the access control domain
    #'@return the list of rules for a given domain
    getRules = function(domain = c("layers", "services", "rest")){
      domain = match.arg(domain)
      msg = sprintf("Fetching rules for %s", domain)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, 
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/security/acl/%s.xml", domain), verbose = self$verbose.debug)
      rules <- NULL
      if(status_code(req) == 200){
        rXML <- GSUtils$parseResponseXML(req)
        rXMLList <- as(xml2::xml_find_all(rXML, "//rules/rule"), "list")
        
        GSRuleClass <- switch(domain,
          "layers" = GSLayerRule,
          "services" = GSServiceRule,
          "rest" = GSRestRule
        )
        
        rules <- lapply(rXMLList, GSRuleClass$new)
        msg = sprintf("Successfuly fetched rules for %s!", domain)
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = sprintf("Error while fetching %s rules", domain)
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(rules)
    },
    
    #'@description Method to get the list of 'layers' rules
    #'@return the list of 'layers' rules
    getLayerRules = function(){
      self$getRules(domain = 'layers')
    },
    
    #'@description Method to get the list of 'services' rules
    #'@return the list of 'services' rules
    getServiceRules = function(){
      self$getRules(domain = 'services')
    },
    
    #'@description Method to get the list of 'rest' rules
    #'@return the list of 'rest' rules
    getRestRules = function(){
      self$getRules(domain = 'rest')
    },
    
    #'@description Method to get a rule by resource
    #'@param domain the access control domain
    #'@param resource resource
    #'@return an object of class \link{GSRule} or \code{NULL} if no rule is found
    getRule = function(domain = c("layers", "services", "rest"), resource){
      out = NULL
      domain = match.arg(domain)
      rules = self$getRules(domain = domain)
      rules = rules[sapply(rules, function(x){x$attrs$resource == resource})]
      if(length(rules)>0) out = rules[[1]]
      return(out)
    },
    
    #'@description Method to get a 'layers' rule
    #'@param resource resource
    #'@return an object of class \link{GSLayerRule} or \code{NULL} if no rule is found
    getLayerRule = function(resource){
      self$getRule(domain = 'layers', resource = resource)
    },
    
    #'@description Method to get a 'services' rule
    #'@param resource resource
    #'@return an object of class \link{GSServiceRule} or \code{NULL} if no rule is found
    getServiceRule = function(resource){
      self$getRule(domain = 'services', resource = resource)
    },
    
    #'@description Method to get a 'rest' rule
    #'@param resource resource
    #'@return an object of class \link{GSRestRule} or \code{NULL} if no rule is found
    getRestRule = function(resource){
      self$getRule(domain = 'rest', resource = resource)
    },
    
    #'@description Generic method to add an access control rule
    #'@param rule object of class \link{GSRule}
    #'@param position position with the list of rules, either "first" or "last". The position
    #'is especially true to capture to the order of priority. For priority, put 'first'.
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addRule = function(rule, position = c("first", "last")){
      
      position = match.arg(position)

      if(!is(rule, "GSRule")){
        msg = "Argument 'rule' should be an object of class 'GSLayerRule', 'GSServiceRule', or 'GSRestRule'"
        cli::cli_alert_danger(msg)
        self$ERROR(msg)
        stop(msg)
      }
      
      domain <- switch(class(rule)[1],
        "GSLayerRule" = "layers",
        "GSServiceRule" = "services",
        "GSRestRule" = "rest"
      )
      
      msg = sprintf("Adding %s rule for resource '%s'", domain, rule$attrs$resource)
      cli::cli_alert_info(msg)
      self$INFO(msg)

      created <- FALSE
      req <- GSUtils$POST(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/security/acl/%s.xml", domain),
        content = {
          xml = xml2::xml_new_root("rules")
          xml2::xml_add_child(xml, rule$encode())
          gsub("[\r\n ] ", "", as(xml, "character"))
        },
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfuly added access control rule!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while adding access control rule"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      
      if(position == "first"){
        self$modifyRule(rule, position = position)
      }
      
      return(created)
    },
    
    #'@description Adds an access control layer rule
    #'@param ws the resource workspace. Default is \code{NULL}
    #'@param lyr the target layer to which the access control should be added
    #'@param permission the rule permission, either \code{r} (read), \code{w} (write) or \code{a} (administer)
    #'@param roles one or more roles to add for the rule
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addLayerRule = function(
      ws = NULL, lyr = "*",
      permission = c("r","w","a"), 
      roles){
      
      permission = match.arg(permission)
      
      #checks on workspace (if any) and layer
      if(!is.null(self$getWorkspace)){ #assumes parent GSManager is used
        if(!is.null(ws)) if(ws != "*"){
          ws_obj = self$getWorkspace(ws)
          if(is.null(ws_obj)){
            err = sprintf("No workspace '%s'", ws)
            cli::cli_alert_danger(err)
            self$ERROR(err)
            return(FALSE)
          }
        }
        
        if(lyr != "*"){
          lyr_obj = self$getLayer(lyr = lyr)
          if(is.null(lyr_obj)){
            err = sprintf("No layer '%s.'", lyr)
            cli::cli_alert_danger(err)
            self$ERROR(err)
            return(FALSE)
          }
        }
      }
      
      rule <- GSLayerRule$new(
        ws = ws, lyr = lyr,
        permission = permission,
        roles = roles
      )
      
      self$addRule(rule)
      
    },
    
    #'@description Adds an access control service rule
    #'@param service service subject to the access control rule, eg. 'wfs'
    #'@param method service method subject to the access control rule, eg. 'GetFeature'
    #'@param permission the rule permission, either \code{r} (read), \code{w} (write) or \code{a} (administer)
    #'@param roles one or more roles to add for the rule
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addServiceRule = function(
      service, method,
      permission = c("r","w","a"), 
      roles){
      
      permission = match.arg(permission)
      
      rule <- GSServiceRule$new(
        service = service, method = method,
        permission = permission,
        roles = roles
      )
      
      self$addRule(rule)
      
    },
    
    #'@description Adds an access control rest rule
    #'@param pattern a URL Ant pattern, only applicable for domain \code{rest}. Default is \code{/**}
    #'@param methods HTTP method(s)
    #'@param roles one or more roles to add for the rule
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addRestRule = function(
      pattern = "/**", methods,
      roles){
      
      rule <- GSRestRule$new(
        pattern = pattern, methods = methods,
        roles = roles
      )
      
      self$addRule(rule)
      
    },
    
    #'@description Generic method to modify an access control rule
    #'@param rule object of class \link{GSRule}
    #'@param position position with the list of rules, either "first" or "last". The position
    #'is especially true to capture to the order of priority. For priority, put 'first'.
    #'@return \code{TRUE} if modified, \code{FALSE} otherwise
    modifyRule = function(rule, position = c("first", "last")){
      
      if(!is(rule, "GSRule")){
        msg = "Argument 'rule' should be an object of class 'GSLayerRule', 'GSServiceRule', or 'GSRestRule'"
        cli::cli_alert_danger(msg)
        self$ERROR(msg)
        stop(msg)
      }
      
      domain <- switch(class(rule)[1],
                       "GSLayerRule" = "layers",
                       "GSServiceRule" = "services",
                       "GSRestRule" = "rest"
      )
      
      resource = rule$attrs$resource
      msg = sprintf("Modify %s rule for resource '%s'", domain, resource)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      
      modified <- FALSE
      if(domain == "rest"){
        resource = gsub("/", URLencode("/",reserved = T), gsub(":",";",resource))
      }
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/security/acl/%s.xml", domain),
        content = {
          xml = xml2::xml_new_root("rules")
          rules = self$getRules(domain = domain)
          rules = rules[sapply(rules, function(x){x$attrs$resource != rule$attrs$resource})]
          switch(position,
            "first" = {
              rules = c(rule, rules)
            },
            "last" = {
              rules = c(rules, rule)
            }
          )
          for(arule in rules){
            xml2::xml_add_child(xml, arule$encode())
          }
          gsub("[\r\n ] ", "", as(xml, "character"))
        },
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfuly modified access control rule!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        modified = TRUE
      }else{
        err = "Error while modifying access control rule"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(modified)
    },
    
    #'@description Generic method to delete an access control rule
    #'@param rule object of class \link{GSRule}
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteRule = function(rule){

      if(!is(rule, "GSRule")){
        msg = "Argument 'rule' should be an object of class 'GSLayerRule', 'GSServiceRule', or 'GSRestRule'"
        cli::cli_alert_danger(msg)
        self$ERROR(msg)
        stop(msg)
      }
      
      domain <- switch(class(rule)[1],
                       "GSLayerRule" = "layers",
                       "GSServiceRule" = "services",
                       "GSRestRule" = "rest"
      )
      
      resource = rule$attrs$resource
      
      msg = sprintf("Deleting access control rule for '%s'", resource)
      cli::cli_alert_info(msg)
      self$INFO(msg)
     
      deleted <- FALSE
      if(domain == "rest"){
        resource = gsub("/", URLencode("/",reserved = T), gsub(":",";",resource))
      }
      path <- sprintf("/security/acl/%s/%s", domain, resource)
      req <- GSUtils$DELETE(self$getUrl(), private$user, 
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfuly deleted access control rule!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting access control rule"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    }
  )
)

#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSPublishable
#' @title A GeoServer layer group publishable
#' @description This class models a GeoServer layer. This class is to be
#' used internally by \pkg{geosapi} for configuring layers or layer groups
#' within an object of class \code{GSLayerGroup}
#' @keywords geoserver rest api resource layer group publishable
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer layer group publishable
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   publishable <- GSPublishable$new(name = "name", type = "layer")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSPublishable <- R6Class("GSPublishable",
   inherit = GSRESTResource,
   
   public = list(
     #'@field full full
     full = TRUE,
     #'@field name name
     name = NULL,
     #'@field attr_type type of attribute
     attr_type = NULL,
     
     #'@description Initializes a \link{GSPublishable}
     #'@param xml an object of class \link[xml2]{xml_node-class}
     #'@param name name
     #'@param type type
     initialize = function(xml = NULL, name, type){
       super$initialize(rootName = "published")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }else{
         if(!missing(name) & !missing(type)){
           self$setName(name)
           self$setType(type)
         }
       }
     },
     
     #'@description Decodes from XML
     #'@param xml an object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
       self$name <- xml2::xml_find_first(xml, "//name") %>% xml2::xml_text()
       self$attr_type <- xml2::xml_attr(xml, "type")
     },
     
     #'@description set name
     #'@param name name
     setName = function(name){
       self$name = name
     },
     
     #'@description Set type
     #'@param type type
     setType = function(type){
       self$attr_type = type
     }
     
   )                       
)

#' Geoserver REST API Monitor Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api Layer
#' @return Object of \code{\link[R6]{R6Class}} with methods for the GeoServer Monitoring extension.
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSMonitorManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#'  }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSMonitorManager <- R6Class("GSMonitorManager",
  inherit = GSManager,
  
  public = list(
    
    #'@description Get the requests
    #'@param offset offset
    #'@return an object of class \code{data.frame}
    getRequests = function(offset = 0){
      msg = "Fetching requests"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      tmp = tempfile(fileext = ".csv")
      req <- GSUtils$GET(
        self$getUrl(), private$user, 
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/monitor/requests.csv?order=startTime;DESC&count=100&offset=%s", offset), verbose = self$verbose.debug, filename = tmp)
      out <- NULL
      if(status_code(req) == 200){
        out <- readr::read_csv(tmp)
        unlink(tmp)
        msg = sprintf("Successfully fetched %s requests", nrow(out))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching requests"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(out)
    }
  )
)

#' Geoserver REST API DataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver monitoring
#' @return Object of \code{\link[R6]{R6Class}} for setting a GS Shiny monitoring app
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @note Internal class used for \code{GSManager$monitor} method
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSShinyMonitor <- R6Class("GSShinyMonitor",
  private = list(
    manager = NULL,
    gs_url = NULL,
    monitor_file = NULL,
    sleep = 1,
    status_metrics = NULL,
    #shiny monitor ui
    monitor_ui = function(id){
      
      shiny::fluidPage(
        shinyWidgets::useShinydashboard(),
        h3(tags$a(tags$img(src = sprintf("%s/web/wicket/resource/org.geoserver.web.GeoServerBasePage/img/logo.png", private$gs_url), alt="logo"), href = private$gs_url, target = "_blank"),
           "GeoServer system status", tags$small(" powered by 'geosapi' R package")),
        hr(),
        shiny::fluidRow(
          shinydashboard::box(
            width = 6,
            title = "GeoServer identification",
            tags$ul(
              tags$li("URL: ", tags$b(tags$a(private$gs_url, href = private$gs_url, target = "_blank")))
            )                    
          ),
          shinydashboard::box(
            width = 6,
            title = "System identification",
            uiOutput("system_metrics")
          )
        ),
        shiny::fluidRow(
          shinydashboard::box(
            width = 12, title = "Monitoring",
            shinyWidgets::verticalTabsetPanel(
              id = "monitoring",
              shinyWidgets::verticalTabPanel(
                title = "CPU Usage", icon = icon("line-chart"),
                tags$div(
                  class = "row",
                  style = "padding:5px",
                  tags$div(class = "col-md-4", uiOutput("monitor_cpu_info")),
                  tags$div(class = "col-md-6", plotly::plotlyOutput("monitor_cpu_load"))
                )
              ),
              shinyWidgets::verticalTabPanel(
                title = "Data", icon = icon("table"),
                tags$div(DT::dataTableOutput("monitor_data"), style = "font-size:75%;overflow-x:scroll;")
              )
            )
          )
        )
      )
    },
    
    #shiny monitor server
    monitor_server = function(input, output, session) {

      monitoring <- reactivePoll(private$sleep*1000, session,
       checkFunc = function() {
         
         status <- private$manager$getSystemStatus()
         readr::write_csv(status$values, path = private$monitor_file, append = TRUE)
         
         output$system_metrics <- renderUI({
           system_metrics_df <- status$raw[status$raw$category == "SYSTEM",]
           tags$ul(
             shiny::tagList(
               lapply(1:nrow(system_metrics_df), function(i){
                 tags$li(paste0(system_metrics_df[i,]$description,": "), tags$b(system_metrics_df[i,]$value))
              })
             )
           )
         })
         
         if (file.exists(private$monitor_file)){
           file.info(private$monitor_file)$mtime[1]
         }else{
           ""
         }
       },
       valueFunc = function() {
         readr::read_csv(private$monitor_file)
       }
      )
      
      #monitor data
      require(DT)
      output$monitor_data <- renderDataTable(
        monitoring(),
        server = FALSE,
        escape = FALSE,
        rownames = FALSE,
        extensions = c("Buttons"),
        filter = list(position = 'top', clear = FALSE),
        
        options = list(
          autoWidth = FALSE,
          dom = 'Bfrtip',
          deferRender = TRUE,
          scroll = FALSE,
          buttons = list(
            list(extend = 'copy'),
            list(extend = 'csv', filename =  "geoserver_monitoring", title = NULL, header = TRUE),
            list(extend = 'excel', filename =  "geoserver_monitoring", title = NULL, header = TRUE),
            list(extend = "pdf", title = "Geoserver monitoring", header = TRUE, orientation = "landscape")
          ),
          exportOptions = list(
            modifiers = list(page = "all", selected = TRUE)
          ),
          
          pageLength = 10
        )
      )
      
      #cpu characteristics
      output$monitor_cpu_info <- renderUI({
        physical_cpus <- self$getMetric("PHYSICAL_CPUS")
        logical_cpus <- self$getMetric("LOGICAL_CPUS")
        running_process <- self$getMetric("RUNNING_PROCESS")
        running_threads <- self$getMetric("RUNNING_THREADS")
        tags$div(
          h4("CPU characteristics"), hr(),
          tags$ul(
            tags$li(paste0(physical_cpus$description,": "), tags$b(physical_cpus$value)),
            tags$li(paste0(logical_cpus$description,": "), tags$b(logical_cpus$value)),
            tags$li(paste0(running_process$description,": "), tags$b(running_process$value)),
            tags$li(paste0(running_threads$description,": "), tags$b(running_threads$value))
          )
        )
        
      })
      
      #monitor cpu load
      require(plotly)
      output$monitor_cpu_load <- renderPlotly({
        data = monitoring()
        p = plot_ly(data = data)
        p = p %>% add_lines(x=~ TIME, y = ~ CPU_LOAD, color = I("black"), name = self$getMetric("CPU_LOAD")$description, text = ~sprintf("%s %s",round(CPU_LOAD,2), self$getMetric("CPU_LOAD")$unit))
        logical_cpus <- data[,colnames(data)[startsWith(colnames(data), "CPU ")]]
        logical_cpu_names = names(logical_cpus)
        for(i in 1:length(logical_cpu_names)){
          dfk <- data.frame(y= data[[logical_cpu_names[i]]], TIME = data$TIME)
          p = p %>% add_trace(x= ~TIME, y = ~y, data = dfk, name = self$getMetric("PER_CPU_LOAD")[i,]$description, 
                              type = "scatter", mode = "lines",
                              text = ~sprintf("%s %s",round(y,2), self$getMetric("PER_CPU_LOAD")[i,]$unit))
        }
        p %>% plotly::layout(
          showlegend=T,
          hovermode ='closest',
          xaxis = list(
            titlefont = list(size = 10), 
            tickfont = list(size = 10),
            title = "TIME",
            zeroline = F
          ),
          yaxis = list(
            titlefont = list(size = 10), 
            tickfont = list(size = 10),
            title = "CPU LOAD (%s)",
            zeroline = F
          )
        )
      })
      
    }
    
  ),
  public = list(
    
    #'@description Initializes a Geoserver shiny monitoring tool
    #'@param manager object of class \link{GSManager}
    #'@param file file File where to store monitoring results
    #'@param append append. Whether results should be appended to existing file
    #'@param sleep sleep. Interval in seconds to trigger monitor calls
    initialize = function(manager, file = NULL, append = FALSE, sleep = 1){
      monitor_file <- file
      if(is.null(file)){
        monitor_file <- tempfile( fileext = ".csv")
        append <- FALSE
      }
      private$monitor_file <- monitor_file
      private$sleep <- sleep
      private$manager <- manager
      private$gs_url <- unlist(strsplit(private$manager$getUrl(), "/rest"))[1]
      
      status <- private$manager$getSystemStatus()
      readr::write_csv(status$values, path = monitor_file, append = append)
      private$status_metrics <- status$raw[,c("name", "description", "unit", "value")]
      
      requireNamespace("magrittr")
      requireNamespace("shiny")
      requireNamespace("shinydashboard")
      requireNamespace("shinyWidgets")
      requireNamespace("DT")
      requireNamespace("plotly")
    },
    
    #'@description Get metric
    #'@param name name
    #'@return the Geoserver monitored metric
    getMetric = function(name){
      return(private$status_metrics[private$status_metrics$name == name,])
    },
    
    #'@description Runs the application
    run = function(){
      shiny::shinyApp(ui = private$monitor_ui, server = private$monitor_server)
    }
 )                     
)

#' Geoserver REST API Access Control List REST Rule
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSRestRule
#' @title A GeoServer access control list service rule
#' @description This class models a GeoServer access control list service rule
#' @keywords geoserver rest api access control rule
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer access control list service rule
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSRestRule <- R6Class("GSRestRule",
   inherit = GSRule,
   
   public = list(
     
     #'@field roles one or more roles
     roles = list(),
     
     #'@description Initializes a \link{GSLayerRule}
     #'@param xml an object of class \link[xml2]{xml_node-class}
     #'@param pattern a URL Ant pattern, only applicable for domain \code{rest}. Default is \code{/**}
     #'@param methods HTTP method(s)
     #'@param roles one or more roles to add for the rule
     initialize = function(xml = NULL,
                           pattern = "/**", methods,
                           roles){
       super$initialize(xml = xml)
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }else{
         pat = pattern
         res_str = sprintf("%s:%s", pat, paste0(methods, collapse = ","))
         self$attrs$resource = res_str
         self$roles = roles
       }
     }
   )                       
)

#' Geoserver REST API Style Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api style
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing the styles
#' of a GeoServer instance.
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSStyleManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#' }
#'
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSStyleManager <- R6Class("GSStyleManager",
  inherit = GSManager,
  public = list(
    
    #'@description Get the list of available styles. 
    #'@param ws an optional workspace name
    #'@return an object of class \code{list} containing items of class \code{\link{GSStyle}}
    getStyles = function(ws = NULL){
      msg = "Fetching list of styles"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req_url <- "/styles.xml"
      if(!is.null(ws)) req_url <- sprintf("/workspaces/%s/styles.xml", ws)
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         req_url, self$verbose.debug)
      styleList <- NULL
      if(status_code(req) == 200){
        styleXML <- GSUtils$parseResponseXML(req)
        styleXMLList <- as(xml2::xml_find_all(styleXML, "//style"), "list")
        styleList <- lapply(styleXMLList, GSStyle$new)
        msg = sprintf("Successfully fetched %s styles", length(styleList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of styles"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(styleList)
    },
    
    #'@description Get the list of available style names
    #'@param ws an optional workspace name
    #'@return a vector of class \code{character}
    getStyleNames = function(ws = NULL){
      styleList <- sapply(self$getStyles(ws = ws), function(x){x$name})
      return(styleList)
    },
    
    #'@description Get a \code{\link{GSStyle}} object given a style name.
    #'@param style style name
    #'@param ws workspace name. Optional
    #'@return object of class \link{GSStyle}
    getStyle = function(style, ws = NULL){
      msg = sprintf("Fetching style '%s'", style)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      reqUrl <- ""
      if(!missing(ws) & !is.null(ws)){
        reqUrl <- sprintf("/workspaces/%s", ws)
      }
      reqUrl <- paste0(reqUrl, sprintf("/styles/%s.xml", style))
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         reqUrl, self$verbose.debug)
      style <- NULL
      if(status_code(req) == 200){
        styleXML <- GSUtils$parseResponseXML(req)
        style <- GSStyle$new(xml = styleXML)
        msg = "Successfully fetched style!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching style"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(style)
    },
    
    #'@description Creates a GeoServer style given a name.
    #'@param file file
    #'@param sldBody SLD body
    #'@param name name
    #'@param raw raw
    #'@param ws workspace name
    #'@return \code{TRUE} if the style has been successfully created, \code{FALSE} otherwise
    createStyle = function(file, sldBody = NULL, name, raw = FALSE, ws = NULL){
      msg = sprintf("Creating style '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      
      if(!missing(file)){
        content <- readChar(file, file.info(file)$size)
        if(!GSUtils$isXMLString(content)){
          err = "SLD style is not recognized XML"
          cli::cli_alert_danger(err)
          self$ERROR(err)
          stop(err)
        }
        sldBody <- xml2::read_xml(content)
      }
    
      if(!is(sldBody, "xml_document")){
        err = "SLD body is not an XML document object"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      contentType <- switch(self$getSLDVersion(sldBody),
                            "1.0.0" = "application/vnd.ogc.sld+xml",
                            "1.1.0" = "application/vnd.ogc.se+xml",
                            NULL
                     )
      if(is.null(contentType)){
        err = "No contentType specified for style creation"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      reqUrl <- ""
      if(!missing(ws) & !is.null(ws)){
        reqUrl <- sprintf("/workspaces/%s", ws)
      }
      reqUrl <- paste0(reqUrl, "/styles?name=", name)
      if(raw) reqUrl <- paste0(reqUrl, "&raw=", tolower(as.character(raw)))
      
      req <- GSUtils$POST(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = reqUrl,
        content = as(sldBody, "character"),
        contentType = contentType,
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfully created style!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating style"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description Updates a GeoServer style given a name.
    #'@param file file
    #'@param sldBody SLD body
    #'@param name name
    #'@param raw raw
    #'@param ws workspace name
    #'@return \code{TRUE} if the style has been successfully updated, \code{FALSE} otherwise
    updateStyle = function(file, sldBody = NULL, name, raw = FALSE, ws = NULL){
      msg = sprintf("Updating style '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      
      if(!missing(file)){
        content <- readChar(file, file.info(file)$size)
        if(!GSUtils$isXMLString(content)){
          err = "SLD style is not recognized XML"
          cli::cli_alert_danger(err)
          self$ERROR(err)
          stop(err)
        }
        sldBody <- xml2::read_xml(content)
      }
      
      if(!is(sldBody, "xml_document")){
        err = "SLD body is not an XML document object"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      contentType <- switch(self$getSLDVersion(sldBody),
                            "1.0.0" = "application/vnd.ogc.sld+xml",
                            "1.1.0" = "application/vnd.ogc.se+xml",
                            NULL
      )
      if(is.null(contentType)){
        err = "No contentType specified for style creation"
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      reqUrl <- ""
      if(!missing(ws) & !is.null(ws)){
        reqUrl <- sprintf("/workspaces/%s", ws)
      }
      reqUrl <- paste0(reqUrl, sprintf("/styles/%s.xml", name))
      if(raw) reqUrl <- paste0(reqUrl, "&raw=", tolower(as.character(raw)))
      
      updated <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = reqUrl,
        content = as(sldBody, "character"),
        contentType = contentType,
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated style!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating style"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a style given a name.
    #'    By defaut, the option \code{recurse} is set to FALSE, ie datastore layers are not removed.
    #'    To remove all coverage store layers, set this option to TRUE. The \code{purge} parameter is used 
    #'    to customize the delete of files on disk (in case the underlying reader implements a delete method).
    #'@param name name
    #'@param recurse recurse
    #'@param purge purge
    #'@param ws workspace name
    #'@return \code{TRUE} if the style has been successfully deleted, \code{FALSE} otherwise
    deleteStyle = function(name, recurse = FALSE, purge = FALSE, ws = NULL){
      msg = sprintf("Deleting style '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      
      path <- ""
      if(!missing(ws) & !is.null(ws)){
        path <- sprintf("/workspaces/%s", ws)
      }
      path <- paste0(path, sprintf("/styles/%s", name))
      path <- paste0(path, "?recurse=", tolower(as.character(recurse)))
      path <- paste0(path, "&purge=", tolower(as.character(recurse)))
      #TODO hack for style removing (not managed by REST API) - check version
      
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully deleted style!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting style"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    },
    
    #'@description Get SLD version
    #'@param sldBody SLD body
    getSLDVersion = function(sldBody){
      return(xml2::xml_attr(sldBody, "version"))
    },
    
    #'@description Get SLD body
    #'@param style style name
    #'@param ws workspace name
    #'@return an object of class \link[xml2]{xml_node-class}
    getSLDBody = function(style, ws = NULL){
      
      if(self$version$lowerThan("2.2")){
        err <- sprintf("Unsupported method for GeoServer %s", self$version$version)
        cli::cli_alert_danger(err)
        self$ERROR(err)
        stop(err)
      }
      
      msg = sprintf("Fetching SLD body for style '%s'", style)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      reqUrl <- ""
      if(!missing(ws) & !is.null(ws)){
        reqUrl <- sprintf("/workspaces/%s", ws)
      }
      reqUrl <- paste0(reqUrl, sprintf("/styles/%s.sld", style))
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         reqUrl, self$verbose.debug)
      style <- NULL
      if(status_code(req) == 200){
        style <- GSUtils$parseResponseXML(req)
        msg = "Successfully fetched SLD body!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching SLD body"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(style)
    }
    
  )
                              
)

# test_GSStyle.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSStyle.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSStyle")

test_that("style encoding/decoding",{
  style <- GSStyle$new()
  expect_is(style, "GSStyle")
  expect_is(style, "R6")
})

test_that("GET style",{
  style <- gsman$getStyle("generic")
  expect_is(style, "GSStyle")
})

test_that("READ style (SLD content)",{
  sldBody <- gsman$getSLDBody("generic")
  expect_is(sldBody, "xml_document")
})

test_that("GET styles",{ 
  styles <- gsman$getStyles()
  expect_equal(length(styles), 21L)
  expect_equal(unique(sapply(styles, function(x){class(x)[1]})), "GSStyle")
  expect_false(unique(sapply(styles, function(x){x$full})))
})

test_that("CREATE style",{
  sldFile <- system.file("extdata", "mystyle.sld", package = "geosapi")
  sldStyle <- xml2::read_xml(sldFile)
  expect_true(is(sldStyle, "xml_document"))
  
  created <- gsman$createStyle(file = sldFile, name = "mystyle")
  expect_true(created)
})

test_that("UPDATE style",{
  sldFile2 <- system.file("extdata", "mystyle2.sld", package = "geosapi")
  sldStyle2 <- xml2::read_xml(sldFile2)
  expect_true(is(sldStyle2, "xml_document"))
  updated <- gsman$updateStyle(sldBody = sldStyle2, name = "mystyle")
  expect_true(updated)
})

test_that("DELETE style",{
  deleted <- gsman$deleteStyle("mystyle", TRUE, TRUE)
  expect_true(deleted)
})

#' Geoserver REST API GSVirtualTableGeometry
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api virtualTable
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer virtual table geometry
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSVirtualTableGeometry$new(name = "work", type = "MultiPolygon", srid = 4326)
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSVirtualTableGeometry <- R6Class("GSVirtualTableGeometry",
   inherit = GSRESTResource,                    
   public = list(
     #' @field name geometry name
     name = NA,
     #' @field type geometry type
     type = NA,
     #' @field srid geometry SRID
     srid = NA,
     
     #'@description Initializes an object of class \link{GSVirtualTableGeometry}
     #'@param xml object of class \link[xml2]{xml_node-class}
     #'@param name name
     #'@param type type
     #'@param srid srid
     initialize = function(xml = NULL, name, type, srid){
       super$initialize(rootName = "geometry")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }else{
         self$name = name
         self$type = type
         self$srid = srid
       }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
       self$name <- xml2::xml_find_first(xml, "//name") %>% xml2::xml_text()
       self$type <- xml2::xml_find_first(xml, "//type") %>% xml2::xml_text()
       self$srid <- xml2::xml_find_first(xml, "//srid") %>% xml2::xml_text()
     }
     
   )                     
)

#' Geoserver REST API DataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer dataStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSAbstractDataStore <- R6Class("GSAbstractDataStore",
  inherit = GSAbstractStore,
  private = list(
    STORE_TYPE = "dataStore"
  ),
  public = list(
    #'@field connectionParameters the list of connection parameters
    connectionParameters = NULL,
    
    #'@description initializes an abstract data store
    #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
    #'@param type the type of coverage store
    #'@param name coverage store name
    #'@param description coverage store description
    #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
    #'@param connectionParameters the list of connection parameters
    initialize = function(xml = NULL, type = NULL,
                          name = NULL, description = "", enabled = TRUE, 
                          connectionParameters){
      super$initialize(xml = xml, storeType = private$STORE_TYPE, type = type, 
                       name = name, description = description, enabled = enabled)
      if(!missing(xml) & !is.null(xml)){
        if(!any(class(xml) %in% c("xml_document","xml_node"))){
          stop("The argument 'xml' is not a valid XML object")
        }
        self$decode(xml)
      }else{
        self$connectionParameters = GSRESTEntrySet$new(rootName = "connectionParameters")
        if(!missing(connectionParameters)){
          if(!is.list(connectionParameters)) stop("Connection parameters should be provided as named list")
          self$connectionParameters$setEntryset(connectionParameters)
        }
      }
    },
    
    #'@description Decodes a data store from XML
    #'@param xml an object of class \link[xml2]{xml_node-class}
    #'@return an object of class \link{GSAbstractDataStore}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      super$decode(xml)
      self$connectionParameters = GSRESTEntrySet$new(rootName = "connectionParameters", xml)
    },
    
    #'@description Set list connection parameters. The argument should be an object
    #'    of class \code{GSRESTEntrySet} giving a list of key/value parameter entries.
    #'@param parameters an object of class \link{GSRESTEntrySet}
    setConnectionParameters = function(parameters){
      self$connectionParameters = parameters
    },
    
    #'@description Adds a connection parameter
    #'@param key connection parameter key
    #'@param value connection parameter value
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addConnectionParameter = function(key, value){
      added <- self$connectionParameters$addEntry(key, value)
      return(added)
    },
    
    #'@description Sets a connection parameter
    #'@param key connection parameter key
    #'@param value connection parameter value
    setConnectionParameter = function(key, value){
      self$connectionParameters$setEntry(key, value)
    },
    
    #'@description Removes a connection parameter
    #'@param key connection parameter key
    #'@param value connection parameter value
    #'@return \code{TRUE} if removed, \code{FALSE} otherwise
    delConnectionParameter = function(key){
      deleted <- self$connectionParameters$delEntry(key)
      return(deleted)
    }
  )                     
)

#' Geoserver REST API Layer Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api Layer
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
#' Layers as results of published feature types or coverages
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSLayerManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#'  }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSLayerManager <- R6Class("GSLayerManager",
  inherit = GSManager,
  
  public = list(
    
    #Layer CRUD methods
    #===========================================================================
    
    #'@description Get the list of layers.
    #'@return an object of class \code{list} giving items of class \code{\link{GSLayer}}
    getLayers = function(){
      msg = "Fetching layers"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user, 
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        "/layers.xml", verbose = self$verbose.debug)
      lyrList <- NULL
      if(status_code(req) == 200){
        lyrXML <- GSUtils$parseResponseXML(req)
        lyrXMLList <- as(xml2::xml_find_all(lyrXML, "//layers/layer"), "list")
        lyrList <- lapply(lyrXMLList, GSLayer$new)
        msg = sprintf("Successfuly fetched %s layers!", length(lyrList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching layers"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(lyrList)
    },
    
    #'@description Get the list of layer names.
    #'@return a vector of class \code{character}
    getLayerNames = function(){
      lyrList <- sapply(self$getLayers(), function(x){x$name})
      return(lyrList)
    },
    
    #'@description Get layer by name
    #'@param lyr layer name
    #'@return an object of class \link{GSLayer}
    getLayer = function(lyr){
      msg = sprintf("Fetching layer '%s'", lyr)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        sprintf("/layers/%s.xml", lyr),
        verbose = self$verbose.debug)
      layer <- NULL
      if(status_code(req) == 200){
        lyrXML <- GSUtils$parseResponseXML(req)
        layer <- GSLayer$new(xml = lyrXML)
        msg = "Successfuly fetched layer!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching layer"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(layer)
    },
    
    #'@description Creates a new layer given an object of class \code{\link{GSLayer}}
    #'@param  layer object of class \link{GSLayer}
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createLayer = function(layer){
      msg = sprintf("Creating layer '%s'", layer$name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/layers/%s.xml", layer$name),
        content = GSUtils$getPayloadXML(layer),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfuly created layer!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating layer"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description Updates a layer given an object of class \code{\link{GSLayer}}
    #'@param  layer object of class \link{GSLayer}
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateLayer = function(layer){
      msg = sprintf("Updating layer '%s'", layer$name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      updated <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/layers/%s.xml", layer$name),
        content = GSUtils$getPayloadXML(layer),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfuly updated layer!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating layer"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes layer given an object of class \code{\link{GSLayer}}
    #'@param  lyr layer name
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteLayer = function(lyr){
      msg = sprintf("Deleting layer '%s'", lyr)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/layers/%s.xml", lyr)
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfuly deleted layer!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting layer"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    },
    
    #LayerGroup CRUD methods
    #===========================================================================
    
    #'@description Get layer groups
    #'@param ws workspace name. Optional
    #'@return a list of objects of class \link{GSLayerGroup}
    getLayerGroups = function(ws = NULL){
      if(missing(ws)){
        msg = "Fetching layer groups"
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }else{
        msg = sprintf("Fetching layer groups for workspace '%s'", ws)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        ifelse(missing(ws),"/layergroups.xml", sprintf("/workspaces/%s/layergroups.xml", ws)), 
        verbose = self$verbose.debug)
      lyrList <- NULL
      if(status_code(req) == 200){
        lyrXML <- GSUtils$parseResponseXML(req)
        lyrXMLList <- as(xml2::xml_find_all(lyrXML, "//layerGroups/layerGroup"), "list")
        lyrList <- lapply(lyrXMLList, GSLayerGroup$new)
        msg = sprintf("Successfuly fetched %s layer groups!", length(lyrList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching layer groups"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(lyrList)
    },
    
    #'@description Get layer group names
    #'@param ws workspace name
    #'@return a list of layer group names, as vector of class \code{character}
    getLayerGroupNames = function(ws = NULL){
      lyrList <- sapply(self$getLayerGroups(ws), function(x){x$name})
      return(lyrList)
    },
    
    #'@description Get layer group
    #'@param lyr lyr
    #'@param ws workspace name
    #'@return an object of class \link{GSLayerGroup}
    getLayerGroup = function(lyr, ws = NULL){
      if(is.null(ws)){
        msg = sprintf("Fetching layer group '%s'", lyr)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }else{
        msg = sprintf("Fetching layer group '%s' in workspace '%s'", lyr, ws)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }
      req <- GSUtils$GET(
        self$getUrl(), private$user,
        private$keyring_backend$get(service = private$keyring_service, username = private$user),
        ifelse(is.null(ws),
               sprintf("/layergroups/%s.xml", lyr),
               sprintf("/workspaces/%s/layergroups/%s.xml", ws, lyr)),
        verbose = self$verbose.debug)
      layer <- NULL
      if(status_code(req) == 200){
        lyrXML <- GSUtils$parseResponseXML(req)
        layer <- GSLayerGroup$new(xml = lyrXML)
        msg = "Successfuly fetched layer group!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching layer group"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(layer)
    },
    
    #'@description Creates a layer group
    #'@param layerGroup object of class \link{GSLayerGroup}
    #'@param ws workspace name. Optional
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createLayerGroup = function(layerGroup, ws = NULL){
      if(is.null(ws)){
        msg = sprintf("Creating layer group '%s'", layerGroup$name)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }else{
        msg = sprintf("Creating layer group '%s' in workspace '%s'", layerGroup$name, ws)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }
      created <- FALSE
      req <- GSUtils$POST(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = ifelse(is.null(ws),"/layergroups.xml",
                      sprintf("/workspaces/%s/layergroups.xml", ws)),
        content = GSUtils$getPayloadXML(layerGroup),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 201){
        msg = "Successfuly created layer group!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating layer group"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description Updates a layer group
    #'@param layerGroup object of class \link{GSLayerGroup}
    #'@param ws workspace name. Optional
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateLayerGroup = function(layerGroup, ws = NULL){
      if(is.null(ws)){
        msg = sprintf("Updating layer '%s'", layerGroup$name)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }else{
        msg = sprintf("Updating layer '%s' in workspace '%s'", layerGroup$name, ws)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }
      updated <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = ifelse(is.null(ws),
                      sprintf("/layergroups/%s.xml", layerGroup$name),
                      sprintf("/workspaces/%s/layergroups/%s.xml", ws, layerGroup$name)),
        content = GSUtils$getPayloadXML(layerGroup),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfuly updated layer group!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating layer group"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes a layer group
    #'@param lyr layer group name
    #'@param ws workspace name. Optional
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteLayerGroup = function(lyr, ws = NULL){
      if(is.null(ws)){
        msg = sprintf("Deleting layer group '%s'", lyr)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }else{
        msg = sprintf("Deleting layer group '%s' in workspace '%s'", lyr, ws)
        cli::cli_alert_info(msg)
        self$INFO(msg)
      }
      deleted <- FALSE
      path <- ifelse(is.null(ws),
                     sprintf("/layergroups/%s.xml", lyr),
                     sprintf("/workspaces/%s/layergroups/%s.xml", ws, lyr))
      req <- GSUtils$DELETE(self$getUrl(), private$user, 
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, verbose = self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfuly deleted layer group!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting layer group"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    }
  )
)

library(testthat)
library(geosapi)

#test environment
gsUrl <- "http://localhost:8080/geoserver"
gsUsr <- "admin"
gsPwd <- "geoserver"
gsLogger <- "INFO"
gsman <- try(GSManager$new(gsUrl, gsUsr, gsPwd, gsLogger))


if(is(gsman, "GSManager")){
  cat(sprintf("GeoServer test instance started at %s. Running integration tests...\n", gsUrl))
  test_check("geosapi")
}else{
  cat("GeoServer test instance is not started. Skipping integration tests...\n")
}

#' Geoserver REST API Service Setting
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSServiceSettings
#' @title A GeoServer service settings resource
#' @description This class models a GeoServer OWS service settings.
#' @keywords geoserver rest api service OGC OWS WMS WFS WCS
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer OWS service setting
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' settings <- GSServiceSettings$new(service = "WMS")
#' settings$setEnabled(TRUE)
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSServiceSettings <- R6Class("GSServiceSettings",
   inherit = GSRESTResource,
   public = list(
     #' @field enabled is service enabled or not?
     enabled = TRUE,
     #' @field citeCompliant is service cite compliant?
     citeCompliant = FALSE,
     #' @field name service name
     name = NULL,
     #' @field title service title
     title = NULL,
     #' @field maintainer service maintainer
     maintainer = NULL,
     #' @field abstrct service abastract
     abstrct = NULL,
     #' @field accessConstraints service access constraints
     accessConstraints = "NONE",
     #' @field fees service fees
     fees = "NONE",
     #' @field keywords services keywords
     keywords = list(),
     #' @field onlineResource service online resource
     onlineResource = NULL,
     #' @field schemaBaseURL service schema base URL
     schemaBaseURL = "http://schemas.opengis.net",
     #' @field verbose service verbose or not?
     verbose = FALSE,
     
     #'@description Initializes an object of class \link{GSServiceSettings}
     #'@param xml object of class \link[xml2]{xml_node-class}
     #'@param service service service acronym
     initialize = function(xml = NULL, service){
       super$initialize(rootName = tolower(service))
       self$setName(toupper(service))
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       enabled <- xml2::xml_find_first(xml, "//enabled")
       if(length(enabled)>0) self$enabled <- as.logical(xml2::xml_text(enabled))
       citeCompliant <- xml2::xml_find_first(xml, "//citeCompliant")
       if(length(citeCompliant)>0) self$citeCompliant <- as.logical(xml2::xml_text(citeCompliant))
       names <- xml2::xml_find_first(xml, paste0("//",self$rootName,"/name"))
       if(length(names)>0) self$name <- xml2::xml_text(names)
       titles <- xml2::xml_find_first(xml, "//title")
       if(length(titles)>0) self$title <- xml2::xml_text(titles)
       maintainers <- xml2::xml_find_first(xml, "//maintainer")
       if(length(maintainers)>0) self$maintainer <- xml2::xml_text(maintainers)
       abstracts <- xml2::xml_find_first(xml, "//abstrct")
       if(length(abstracts)>0) self$abstrct <- xml2::xml_text(abstracts)
       accessConstraints <- xml2::xml_find_first(xml, "//accessConstraints")
       if(length(accessConstraints)>0) self$accessConstraints <- xml2::xml_text(accessConstraints)
       fees <- xml2::xml_find_first(xml, "//fees")
       if(length(fees)>0) self$fees <- xml2::xml_text(fees)
       
       self$keywords <- lapply(as(xml2::xml_find_all(xml, "//keywords/string"), "list"), xml2::xml_text)
       onlineRes <- xml2::xml_find_first(xml, "//onlineResource")
       if(length(onlineRes)>0) self$onlineResource <- xml2::xml_text(onlineRes)
       schemaBaseURL <- xml2::xml_find_first(xml, "//schemaBaseURL")
       if(length(schemaBaseURL)>0) self$schemaBaseURL <- xml2::xml_text(schemaBaseURL)
       verbose <- xml2::xml_find_first(xml, "//verbose")
       if(length(verbose)>0) self$verbose <- as.logical(xml2::xml_text(verbose))
     },
     
     #'@description Set enabled
     #'@param enabled enabled
     setEnabled = function(enabled){
       self$enabled = enabled
     },
     
     #'@description Set cite compliant
     #'@param citeCompliant cite compliant
     setCiteCompliant = function(citeCompliant){
       self$citeCompliant <- citeCompliant
     },
     
     #'@description Set name
     #'@param name name
     setName = function(name){
       self$name = name
     },
     
     #'@description Set title
     #'@param title title
     setTitle = function(title){
       self$title = title
     },
     
     #'@description Set maintainer
     #'@param maintainer maintainer
     setMaintainer = function(maintainer){
       self$maintainer = maintainer
     },
     
     #'@description Set abstract
     #'@param abstract abstract
     setAbstract = function(abstract){
       self$abstrct = abstract
     },
     
     #'@description Set access constraints
     #'@param accessConstraints access constraints
     setAccessConstraints = function(accessConstraints){
       self$accessConstraints = accessConstraints
     },
     
     #'@description Set fees
     #'@param fees fees
     setFees = function(fees){
       self$fees = fees
     },
     
     #'@description Set keywords
     #'@param keywords keywords
     setKeywords = function(keywords){
       if(!is.list(keywords)) keywords = list(keywords)
       self$keywords = keywords
       return(TRUE)
     },
     
     #'@description Adds a keyword
     #'@param keyword keyword
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addKeyword = function(keyword){
       startNb = length(self$keywords)
       if(length(which(self$keywords == keyword)) == 0){
         self$keywords = c(self$keywords, keyword)
       }
       endNb = length(self$keywords)
       return(endNb == startNb+1)
     },
     
     #'@description Deletes a keyword
     #'@param keyword keyword
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     delKeyword = function(keyword){
       startNb = length(self$keywords)
       self$keywords = self$keywords[which(self$keywords != keyword)]
       endNb = length(self$keywords)
       return(endNb == startNb-1)
     }
   )                       
)

#' Geoserver REST API Workspace
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api workspace
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer workspace
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   GSWorkspace$new(name = "work")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSWorkspace <- R6Class("GSWorkspace",
  inherit = GSRESTResource,                    
  public = list(
    #'@field name name
    name = NA,
    
    #'@description initializes a \link{GSWorkspace}
    #'@param xml an object of class \link[xml2]{xml_node-class}
    #'@param name name
    initialize = function(xml = NULL, name){
      super$initialize(rootName = "workspace")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }else{
        self$name = name
      }
    },
    
    #'@description Decodes from XML
    #'@param xml an object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      self$name <- xml2::xml_child(xml) %>% xml2::xml_text()
    }
    
  )                     
)

#' Geoserver REST API CoverageStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api CoverageStore
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer CoverageStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSAbstractCoverageStore <- R6Class("GSAbstractCoverageStore",
 inherit = GSAbstractStore,
 private = list(
   STORE_TYPE = "coverageStore"
 ),
 public = list(
   #'@field url URL of the abstract coverage store
   url = NULL,
   
   #'@description initializes an abstract coverage store
   #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
   #'@param type the type of coverage store
   #'@param name coverage store name
   #'@param description coverage store description
   #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
   #'@param url URL of the store
   initialize = function(xml = NULL, type = NULL,
                         name = NULL, description = "", enabled = TRUE, url = NULL){
     super$initialize(xml = xml, storeType = private$STORE_TYPE, type = type, 
                      name = name, description = description, enabled = enabled)
     if(!missing(xml) & !is.null(xml)){
       if(!any(class(xml) %in% c("xml_document","xml_node"))){
         stop("The argument 'xml' is not a valid XML object")
       }
       self$decode(xml)
     }else{
       self$setUrl(url)
     }
   },
   
   #'@description Decodes a coverage store from XML
   #'@param xml an object of class \link[xml2]{xml_node-class}
   #'@return an object of class \link{GSAbstractCoverageStore}
   decode = function(xml){
     xml = xml2::as_xml_document(xml)
     super$decode(xml)
     urlXML <- xml2::xml_find_first(xml,"//url")
     if(length(urlXML) > 0) self$url <- xml2::xml_text(urlXML)
   },
   
   #'@description set coverage store URL
   #'@param url the store URL to set
   setUrl = function(url){
     self$url <- url
   }
 )                     
)

#' Geoserver REST API ShapeFileDataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore ESRI shapefile
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer Shapefile dataStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSShapefileDataStore$new(name = "ds", description = "des",
#'                          enabled = TRUE, url = "file://data/shape.shp")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSShapefileDataStore <- R6Class("GSShapefileDataStore",
 inherit = GSAbstractDataStore,
 private = list(
   TYPE = "Shapefile"
 ),
 public = list(
    
    #'@description initializes a shapefile data store
    #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
    #'@param name coverage store name
    #'@param description coverage store description
    #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
    #'@param url url
   initialize = function(xml = NULL, name = NULL, description = "",
                         enabled = TRUE, url){
     if(missing(xml)) xml <- NULL
     super$initialize(xml = xml, name = name,
                      description = description,
                      enabled = enabled)
     if(is.null(xml)){
      self$setUrl(url)
      self$setDefautConnectionParameters()
     }
     self$setType(private$TYPE)
   },
   
   #'@description Set the spatial files data URL
   #'@param url url
   setUrl = function(url){
     self$setConnectionParameter("url", url)
   },
   
   #'@description Set the charset used for DBF file. 
   #'@param charset charset. Default value is 'ISO-8859-1'
   setCharset = function(charset = "ISO-8859-1"){
     self$setConnectionParameter("charset", charset)
   },
   
   #'@description Set the 'Create Spatial Index' option
   #'@param create create. Default is \code{TRUE}
   setCreateSpatialIndex = function(create = TRUE){
     self$setConnectionParameter("create spatial index", create)
   },
   
   #'@description Set the 'Memory Mapped Buffer' option
   #'@param buffer buffer. Default is \code{FALSE}
   setMemoryMappedBuffer = function(buffer = FALSE){
     self$setConnectionParameter("memory mapped buffer", buffer)
   },
   
   #'@description Set the 'Cache & Reuse Memory Maps' option.
   #'@param maps maps. Default is \code{TRUE}
   setCacheReuseMemoryMaps = function(maps = TRUE){
     self$setConnectionParameter("cache and reuse memory maps", maps)
   },
   
   #'@description Set default connection parameters
   setDefautConnectionParameters = function(){
     self$setCharset()
     self$setCreateSpatialIndex()
     self$setMemoryMappedBuffer()
     self$setCacheReuseMemoryMaps()
   }
 )                     
)

#' Geoserver REST API OracleNGDataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore OracleNG
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer OracleNG dataStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   GSOracleNGDataStore$new(name = "ds", description = "des", enabled = TRUE)
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSOracleNGDataStore <- R6Class("GSOracleNGDataStore",
    inherit = GSAbstractDBDataStore,
    private = list(
      TYPE = "Oracle NG",
      DBTYPE = "oracle"
    ),
    public = list(
      
      #'@description initializes an Oracle NG data store
      #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
      #'@param name coverage store name
      #'@param description coverage store description
      #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
      initialize = function(xml = NULL, name = NULL, description = "", enabled = TRUE){
        super$initialize(xml = xml, type = private$TYPE, dbType = private$DBTYPE,
                         name = name, description = description,
                         enabled = enabled)
      }
    )                     
)

#' Geoserver REST API Access Control List Rule
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSRule
#' @title A GeoServer access control list rule
#' @description This class models a GeoServer access control list rule
#' @keywords geoserver rest api access control rule
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer access control list rule
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#' 
#' @note Abstract class
#'
GSRule <- R6Class("GSRule",
 inherit = GSRESTResource,
 
 public = list(
   
   #'@description Initializes a \link{GSRule}
   #'@param xml an object of class \link[xml2]{xml_node-class}
   initialize = function(xml = NULL){
     super$initialize(rootName = "rule")
   },
   
   #'@description Encodes as XML
   #'@return an object of class \link[xml2]{xml_node-class}
   encode = function(){
     ruleXML = xml2::xml_new_root("rule")
     xml2::xml_attrs(ruleXML) = unlist(self$attrs)
     xml2::xml_text(ruleXML) = paste0(self$roles, collapse=",")
     return(ruleXML)
   },
   
   #'@description Decodes from XML
   #'@param xml an object of class \link[xml2]{xml_node-class}
   decode = function(xml){
     xml = xml2::as_xml_document(xml)
     self$attrs$resource = xml2::xml_attr(xml, "resource")
     self$roles = unlist(strsplit(xml2::xml_contents(xml) %>% as.character(), ","))
   }
 )                       
)

#' Geoserver REST API Resource
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSResource
#' @title A GeoServer abstract resource
#' @description This class models an abstract GeoServer resource. This class is
#' used internally for modelling instances of class \code{GSFeatureType} or
#' \code{GSCoverage}
#' @keywords geoserver rest api resource
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer resource
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' res <- GSResource$new(rootName = "featureType")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSResource <- R6Class("GSResource",
   inherit = GSRESTResource,
   
   private = list(
     allowedProjectionPolicies = c("NONE", "FORCE_DECLARED", "REPROJECT_TO_DECLARED")
   ),
   
   public = list(
     #'@field full full
     full = TRUE,
     #' @field name resource name
     name = NULL,
     #' @field nativeName resource native name
     nativeName = NULL,
     #' @field title resource title
     title = NULL,
     #' @field description resource description
     description = NULL,
     #' @field abstract resource abstract
     abstract = NULL,
     #' @field keywords resource keywords
     keywords = list(),
     #' @field metadataLinks resource metadata links
     metadataLinks = list(),
     #' @field nativeCRS resource native CRS
     nativeCRS = NULL,
     #' @field srs resource srs
     srs = NULL,
     #' @field nativeBoundingBox resource lat/lon native bounding box
     nativeBoundingBox = list(minx = NA, miny = NA, maxx = NA, maxy = NA, crs = NA),
     #' @field latLonBoundingBox resource lat/lon bounding box
     latLonBoundingBox = list(minx = NA, miny = NA, maxx = NA, maxy = NA, crs = NA),
     #' @field projectionPolicy resource projection policy
     projectionPolicy = NULL,
     #'@field enabled enabled
     enabled = TRUE,
     #'@field metadata metadata
     metadata = NULL,
     
     #'@description Initializes a \link{GSResource}
     #'@param rootName root name
     #'@param xml object of class \link[xml2]{xml_node-class}
     initialize = function(rootName = NULL, xml = NULL){
        super$initialize(rootName = rootName)
        if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
        }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
       self$name <- xml2::xml_find_first(xml, "//name") %>% xml2::xml_text()
       enabled <- xml2::xml_find_first(xml, "//enabled")
       if(length(enabled)==0) self$full <- FALSE
       
       if(self$full){
         self$enabled <- as.logical(xml2::xml_text(enabled))
         nativeNames <- xml2::xml_find_first(xml, "//nativeName")
         if(length(nativeNames)>0) self$nativeName <- xml2::xml_text(nativeNames)
         titles <- xml2::xml_find_first(xml, "//title")
         if(length(titles)>0) self$title <- xml2::xml_text(titles)
         descriptions <- xml2::xml_find_first(xml, "//description")
         if(length(descriptions)>0) self$description <- xml2::xml_text(descriptions)
         abstracts <- xml2::xml_find_first(xml, "//abstract")
         if(length(abstracts)>0) self$abstract <- xml2::xml_text(abstracts)
         self$keywords <- lapply(as(xml2::xml_find_all(xml, "//keywords/string"), "list"), xml2::xml_text)
         
         metadataLinksXML <- as(xml2::xml_find_all(xml, "//metadataLinks/metadataLink"), "list")
         if(length(metadataLinksXML)>0){
           for(metadataLinkXML in metadataLinksXML){
             self$addMetadataLink(GSMetadataLink$new(xml = metadataLinkXML))
           }
         }
         
         nativeCRS <- xml2::xml_find_first(xml, "//nativeCRS")
         if(length(nativeCRS)>0){
           self$nativeCRS <- xml2::xml_text(nativeCRS)
         }
         
         
         srs <- xml2::xml_find_first(xml, "//srs")
         if(length(srs)>0){
           self$srs <- xml2::xml_text(srs)
         }

         latLonBboxXML <- as(xml2::xml_find_all(xml, "//latLonBoundingBox/*"), "list")
         self$latLonBoundingBox <- lapply(latLonBboxXML, xml2::xml_text)
         names(self$latLonBoundingBox) <-  lapply(latLonBboxXML, xml2::xml_name)
         
         nativeBboxXML <- as(xml2::xml_find_all(xml, "//nativeBoundingBox/*"), "list")
         self$nativeBoundingBox <- lapply(nativeBboxXML, xml2::xml_text)
         names(self$nativeBoundingBox) <-  lapply(nativeBboxXML, xml2::xml_name)
         
         projPolicies <- xml2::xml_find_first(xml, "//projectionPolicy")
         if(length(projPolicies)>0){
           self$projectionPolicy <- xml2::xml_text(projPolicies)
         }
         
         md_entries <- as(xml2::xml_find_all(xml, "//metadata/entry"), "list")
         if(length(md_entries)>0){
           for(md_entry in md_entries){
              key <- xml2::xml_attr(md_entry, "key")
              meta <- NULL
              child <- NULL
              if(length(xml2::xml_children(md_entry))>0){
                child = xml2::xml_child(md_entry)
              }else{
                meta = xml2::xml_text(md_entry)
              }
              if(!is.null(child)){
                if(is(self, "GSFeatureType")){
                  meta <- switch(xml2::xml_name(child),
                    "dimensionInfo" = GSFeatureDimension$new(xml = child),
                    "virtualTable" = GSVirtualTable$new(xml = child)
                  )
                }
                if(is(self, "GSCoverage")){
                  meta <- switch(xml2::xml_name(child),
                    "dimensionInfo" = GSDimension$new(xml = child),
                    "coverageView" = GSCoverageView$new(xml = child)
                  )
                }
              }
              if(!is.null(meta)) self$setMetadata(key, meta)
           }
         }
       }
     },
     
     #'@description Set enabled
     #'@param enabled enabled
     setEnabled = function(enabled){
       self$enabled = enabled
     },
     
     #'@description Set name
     #'@param name name
     setName = function(name){
       self$name = name
     },
     
     #'@description Set native name
     #'@param nativeName native name
     setNativeName = function(nativeName){
       self$nativeName = nativeName
     },
     
     #'@description Set title
     #'@param title title
     setTitle = function(title){
       self$title = title
     },
     
     #'@description Set description
     #'@param description description
     setDescription = function(description){
       self$description = description
     },
     
     #'@description Set abstract
     #'@param abstract abstract
     setAbstract = function(abstract){
       self$abstract = abstract
     },
     
     #'@description Set keyword(s)
     #'@param keywords keywords
     setKeywords = function(keywords){
       if(!is.list(keywords)) keywords = list(keywords)
       self$keywords = keywords
       return(TRUE)
     },
     
     #'@description Adds keyword
     #'@param keyword keyword
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addKeyword = function(keyword){
       startNb = length(self$keywords)
       if(length(which(self$keywords == keyword)) == 0){
         self$keywords = c(self$keywords, keyword)
       }
       endNb = length(self$keywords)
       return(endNb == startNb+1)
     },
     
     #'@description Deletes keyword
     #'@param keyword keyword
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     delKeyword = function(keyword){
       startNb = length(self$keywords)
       self$keywords = self$keywords[which(self$keywords != keyword)]
       endNb = length(self$keywords)
       return(endNb == startNb-1)
     },
     
     #'@description Set metadata links
     #'@param metadataLinks metadata links
     setMetadataLinks = function(metadataLinks){
       self$metadataLinks <- metadataLinks
     },
     
     #'@description Adds metadata link
     #'@param metadataLink object of class \link{GSMetadataLink}
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addMetadataLink = function(metadataLink){
       startNb = length(self$metadataLinks)
       links <- sapply(self$metadataLinks, function(x){x$content})
       if(!metadataLink$content %in% links){
         self$metadataLinks = c(self$metadataLinks, metadataLink)
       }
       endNb = length(self$metadataLinks)
       return(endNb == startNb+1)
     },
     
     #'@description Deletes metadata link
     #'@param metadataLink object of class \link{GSMetadataLink}
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     deleteMetadataLink = function(metadataLink){
       startNb = length(self$metadataLinks)
       links <- lapply(self$metadataLinks, function(x){x$content})
       self$metadataLinks = self$metadataLinks[which(links != metadataLink$content)]
       endNb = length(self$metadataLinks)
       return(endNb == startNb-1)
     },
     
     #'@description Set projection policy
     #'@param projectionPolicy projection policy
     setProjectionPolicy = function(projectionPolicy){
       if(!(projectionPolicy %in% private$allowedProjectionPolicies)){
        stop(sprintf("'%s' is not a valid projection policy", projectionPolicy))
       }
       self$projectionPolicy = projectionPolicy
     },
     
     #'@description Set SRS
     #'@param srs srs
     setSrs = function(srs){
       self$srs = srs
     },
     
     #'@description Set native CRS
     #'@param nativeCRS native crs
     setNativeCRS = function(nativeCRS){
       self$nativeCRS = nativeCRS
     },

     #'@description Set LatLon bounding box
     #'@param minx minx
     #'@param miny miny
     #'@param maxx maxx
     #'@param maxy maxy
     #'@param bbox bbox
     #'@param crs crs
     setLatLonBoundingBox = function(minx, miny, maxx, maxy, bbox = NULL, crs){
       self$latLonBoundingBox <- GSUtils$setBbox(minx, miny, maxx, maxy, bbox, crs)
     },
     
     #'@description Set native bounding box
     #'@param minx minx
     #'@param miny miny
     #'@param maxx maxx
     #'@param maxy maxy
     #'@param bbox bbox
     #'@param crs crs
     setNativeBoundingBox = function(minx, miny, maxx, maxy, bbox = NULL, crs){
       self$nativeBoundingBox <- GSUtils$setBbox(minx, miny, maxx, maxy, bbox, crs)
     },
     
     #'@description Set metadata
     #'@param key key
     #'@param metadata metadata
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     setMetadata = function(key, metadata){
       if(is.null(self$metadata)){
         self$metadata = GSRESTEntrySet$new(rootName = "metadata")
       }
       added <- self$metadata$addEntry(key, metadata)
       return(added)
     },
     
     #'@description Deletes metadata
     #'@param key key
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     delMetadata = function(key){
       deleted <- FALSE
       if(!is.null(self$metadata)){
        deleted <- self$metadata$delEntry(key)
        if(length(self$metadataentryset)==0) self$metadata = NULL
       }
       return(deleted)
     },
     
     #'@description Set metadata dimension
     #'@param key key
     #'@param dimension dimension
     #'@param custom custom
     setMetadataDimension = function(key, dimension, custom = FALSE){
      if(custom) key <- paste0("custom_dimension_", toupper(key))
      added <- self$setMetadata(key, dimension)
      return(added)
     }
   )                     
)

# test_GSFeatureType.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSFeatureType.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSFeatureType")

test_that("featureType encoding/decoding",{
  
  ft <- GSFeatureType$new()
  expect_is(ft, "GSFeatureType")
  expect_is(ft, "R6")
  
  ft$setName("name")
  expect_equal(ft$name, "name")
  
  ft$setNativeName("native_name")
  expect_equal(ft$nativeName, "native_name")
  
  ft$setTitle("this is a title")
  expect_equal(ft$title, "this is a title")
  
  ft$setDescription("this is a description")
  expect_equal(ft$description, "this is a description")
  
  ft$setAbstract("this is an abstract")
  expect_equal(ft$abstract, "this is an abstract")
  
  expect_true(ft$addKeyword("keyword1"))
  expect_true(ft$addKeyword("keyword2"))
  expect_true(ft$addKeyword("keyword3"))
  expect_false(ft$addKeyword("keyword1")) #because already added
  expect_true(ft$delKeyword("keyword3"))
  expect_false(ft$delKeyword("keyword3")) #because already deleted
  expect_equal(length(ft$keywords), 2L)
  expect_true(all(unlist(ft$keywords) == c("keyword1","keyword2")))
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/xml"
  )
  expect_true(ft$addMetadataLink(md1))
  expect_false(ft$addMetadataLink(md1))
  expect_equal(length(ft$metadataLinks), 1L)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/html"
  )
  expect_true(ft$addMetadataLink(md2))
  expect_false(ft$addMetadataLink(md2))
  expect_equal(length(ft$metadataLinks), 2L)
  
  ft$setProjectionPolicy("NONE")
  expect_equal(ft$projectionPolicy, "NONE")
  
  ft$setSrs("EPSG:4326")
  expect_equal("EPSG:4326", ft$srs)
  ft$setNativeCRS("EPSG:4326")
  expect_equal("EPSG:4326", ft$nativeCRS)
  
  ft$setLatLonBoundingBox(-180, -90, 180, 90, crs = "EPSG:4326")
  ft$setNativeBoundingBox(-180, -90, 180, 90, crs = "EPSG:4326")
  
  #encoding to XML
  ftXML <- ft$encode()
  expect_is(ftXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  ft2 <- GSFeatureType$new(xml = ftXML)
  ft2XML <- ft2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(ftXML, ft2XML))==0)
  
})

test_that("READ featuretype",{ 
  ft <- gsman$getFeatureType("topp","taz_shapes", "tasmania_cities")
  expect_true(any(class(ft) == "GSFeatureType"))
  expect_true(ft$full)
})

test_that("READ featuretypes",{
  fts <- gsman$getFeatureTypes("topp","taz_shapes")
  expect_equal(length(fts), 4L)
  expect_equal(unique(sapply(fts, function(x){class(x)[1]})), "GSFeatureType")
  expect_false(unique(sapply(fts, function(x){x$full})))
  expect_true(all(sapply(fts,function(x){x$name}) %in% paste0("tasmania_", c("cities", "roads", "state_boundaries", "water_bodies"))))
})

test_that("CREATE featureType",{
  ft <- GSFeatureType$new()
  ft$setName("tasmania_cities2")
  ft$setNativeName("tasmania_cities")
  ft$setAbstract("abstract")
  ft$setTitle("title")
  ft$setSrs("EPSG:4326")
  ft$setNativeCRS("EPSG:4326")
  ft$setEnabled(TRUE)
  ft$setProjectionPolicy("REPROJECT_TO_DECLARED")
  ft$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  ft$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326")
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/xml"
  )
  expect_true(ft$addMetadataLink(md1))
  expect_false(ft$addMetadataLink(md1))
  expect_equal(length(ft$metadataLinks), 1L)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/html"
  )
  expect_true(ft$addMetadataLink(md2))
  expect_false(ft$addMetadataLink(md2))
  expect_equal(length(ft$metadataLinks), 2L)
  
  created <- gsman$createFeatureType("topp", "taz_shapes", ft)
  expect_true(created)
  
  featureType <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities2")
  expect_is(featureType, "GSFeatureType")
  expect_true(featureType$enabled)
  expect_equal(featureType$abstract, "abstract")
})

test_that("UPDATE featuretype",{
  featureType <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities2")
  featureType$setAbstract("abstract updated")
  featureType$setEnabled(FALSE)
  
  updated <- gsman$updateFeatureType("topp", "taz_shapes", featureType)
  expect_true(updated)
  ft <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities2")
  expect_is(ft, "GSFeatureType")
  expect_equal(ft$abstract, "abstract updated")
  expect_false(ft$enabled)
})

test_that("DELETE featureType",{
  deleted <- gsman$deleteLayer("tasmania_cities2")
  expect_true(deleted)
  if(deleted){
    deleted <- gsman$deleteFeatureType("topp", "taz_shapes", "tasmania_cities2")
    expect_true(deleted)
    ft <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities2")
    expect_is(ft, "NULL")
  }
  
})

# test_GSResource.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSResource.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSResource")

test_that("resource encoding/decoding",{
  
  res <- GSResource$new(rootName = "featureType")
  expect_is(res, "GSResource")
  expect_is(res, "R6")
  
  res$setName("name")
  expect_equal("name", res$name)
  
  res$setNativeName("native_name")
  expect_equal("native_name", res$nativeName)
  
  res$setTitle("this is a title")
  expect_equal("this is a title", res$title)
  
  res$setDescription("this is a description")
  expect_equal("this is a description", res$description)
  
  res$setAbstract("this is an abstract")
  expect_equal("this is an abstract", res$abstract)
  
  expect_true(res$addKeyword("keyword1"))
  expect_true(res$addKeyword("keyword2"))
  expect_true(res$addKeyword("keyword3"))
  expect_false(res$addKeyword("keyword1")) #because already added
  expect_true(res$delKeyword("keyword3"))
  expect_false(res$delKeyword("keyword3")) #because already deleted
  expect_equal(length(res$keywords), 2L)
  expect_true(all(unlist(res$keywords) == c("keyword1","keyword2")))
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/xml"
  )
  expect_true(res$addMetadataLink(md1))
  expect_false(res$addMetadataLink(md1))
  expect_equal(length(res$metadataLinks), 1L)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/html"
  )
  expect_true(res$addMetadataLink(md2))
  expect_false(res$addMetadataLink(md2))
  expect_equal(length(res$metadataLinks), 2L)
  
  res$setProjectionPolicy("NONE")
  expect_equal("NONE", res$projectionPolicy)
  
  res$setSrs("EPSG:4326")
  expect_equal("EPSG:4326", res$srs)
  res$setNativeCRS("EPSG:4326")
  expect_equal("EPSG:4326", res$nativeCRS)
  
  res$setLatLonBoundingBox(-180, -90, 180, 90, crs = "EPSG:4326")
  res$setNativeBoundingBox(-180, -90, 180, 90, crs = "EPSG:4326")
  
  #encoding to XML
  resXML <- res$encode()
  expect_is(resXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  res2 <- GSResource$new(rootName = "featureType", xml = resXML)
  res2XML <- res2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(resXML, res2XML))==0)
  
})

# test_GSCoverage.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSCoverage.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSCoverage")

test_that("GSCoverage encoding/decoding",{
  
  cov <- GSCoverage$new()
  cov$setName("sfdem_new")
  cov$setNativeName("sfdem_new")
  cov$setTitle("Title for sfdem")
  cov$setDescription("Description for sfdem")
  cov$addKeyword("sfdem keyword1")
  cov$addKeyword("sfdem keyword2")
  cov$addKeyword("sfdem keyword3")
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/xml"
  )
  cov$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/html"
  )
  cov$addMetadataLink(md2)
  
  cov$setSrs("EPSG:4326")
  cov$setNativeCRS("EPSG:26713")
  cov$setLatLonBoundingBox(-103.87108701853181, 44.370187074132616, -103.62940739432703, 44.5016011535299, crs = "EPSG:4326")
  cov$setNativeBoundingBox(589980, 4913700, 609000, 4928010, crs = "EPSG:26713")
  
  coview <- GSCoverageView$new()
  coview$setName("sfdem_new")
  coview$setEnvelopeCompositionType("INTERSECTION")
  coview$setSelectedResolution("BEST")
  coview$setSelectedResolutionIndex(-1)
  coviewband <- GSCoverageBand$new()
  coviewband$setDefinition("sfdem_new@0")
  coviewband$setIndex(0)
  coviewband$addInputBand(GSInputCoverageBand$new( coverageName = "sfdem_new", band = 0))
  coview$addBand(coviewband)
  cov$setView(coview)
  
  #encoding to XML
  covXML <- cov$encode()
  expect_is(covXML, "xml_node")
  
  #decoding from XML
  cov2 <- GSCoverage$new(xml = covXML)
  cov2XML <- cov2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(covXML, cov2XML))==0)
  
})

# test_GSLayer.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSLayer.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSLayer")

test_that("layer encoding/decoding",{
  
  lyr <- GSLayer$new()
  expect_is(lyr, "GSLayer")
  expect_is(lyr, "R6")
  
  lyr$setName("name")
  expect_equal(lyr$name, "name")
  
  lyr$setDefaultStyle("mystyle")
  expect_equal(length(lyr$defaultStyle), 1L)
  expect_equal(lyr$defaultStyle[[1]], "mystyle")
  
  expect_true(lyr$addStyle("mystyle2"))
  expect_false(lyr$addStyle("mystyle2"))
  expect_true(lyr$addStyle("mystyle3"))
  expect_false(lyr$addStyle("mystyle3"))
  expect_true(lyr$addStyle("mystyle4"))
  expect_false(lyr$addStyle("mystyle4"))
  expect_equal(length(lyr$styles), 3L)
  expect_true(lyr$delStyle("mystyle4"))
  expect_false(lyr$delStyle("mystyle4"))
  expect_equal(length(lyr$styles), 2L)
  expect_equal(sapply(lyr$styles, function(x){x$name}), c("mystyle2","mystyle3"))
  
  #encoding to XML
  lyrXML <- lyr$encode()
  expect_is(lyrXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  lyr2 <- GSLayer$new(xml = lyrXML)
  lyr2XML <- lyr2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(lyrXML, lyr2XML))==0)
  
})

test_that("READ layer",{
  lyr <- gsman$getLayer("tasmania_cities")
  expect_is(lyr,"GSLayer")
  expect_true(lyr$enabled)
  expect_true(lyr$queryable)
  expect_true(lyr$advertised)
  expect_equal(lyr$name, "tasmania_cities")
  expect_equal(length(lyr$defaultStyle), 1L)
  expect_equal(lyr$defaultStyle[[1]], "capitals")
})

test_that("READ layers",{ 
  lyrs <- gsman$getLayers()
  expect_equal(length(lyrs), 24L)
  expect_equal(unique(sapply(lyrs, function(x){class(x)[1]})), "GSLayer")
  expect_false(unique(sapply(lyrs, function(x){x$full})))
})

test_that("CREATE layer",{
  #create featuretype before
  ft <- GSFeatureType$new()
  ft$setName("tasmania_cities2")
  ft$setNativeName("tasmania_cities")
  ft$setAbstract("abstract")
  ft$setTitle("title")
  ft$setSrs("EPSG:4326")
  ft$setNativeCRS("EPSG:4326")
  ft$setEnabled(TRUE)
  ft$setProjectionPolicy("REPROJECT_TO_DECLARED")
  ft$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  ft$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326") 
  created <- gsman$createFeatureType("topp", "taz_shapes", ft)
  
  #test layer creation
  layer <- GSLayer$new()
  layer$setName("tasmania_cities2")
  layer$setDefaultStyle("capitals")
  layer$addStyle("generic")
  created <- gsman$createLayer(layer)
  expect_true(created)
})

test_that("UPDATE layer",{
  lyr <- gsman$getLayer("tasmania_cities")
  expect_equal(lyr$defaultStyle$name, "capitals")
  
  lyr$setQueryable(FALSE)
  updated <- gsman$updateLayer(lyr)
  lyr <- gsman$getLayer("tasmania_cities")
  expect_false(lyr$queryable)
  lyr$setQueryable(TRUE)
  updated <- gsman$updateLayer(lyr)
  lyr <- gsman$getLayer("tasmania_cities")
  expect_true(lyr$queryable)
})

test_that("DELETE layer",{
  deleted <- gsman$deleteLayer("tasmania_cities2")
  expect_true(deleted)
  if(deleted){
    #delete feature after
    gsman$deleteFeatureType("topp", "taz_shapes", "tasmania_cities2")
  }
})

test_that("PUBLISH layer",{
  #create featuretype
  featureType <- GSFeatureType$new()
  featureType$setName("tasmania_cities2")
  featureType$setNativeName("tasmania_cities")
  featureType$setAbstract("abstract")
  featureType$setTitle("title")
  featureType$setSrs("EPSG:4326")
  featureType$setNativeCRS("EPSG:4326")
  featureType$setEnabled(TRUE)
  featureType$setProjectionPolicy("REPROJECT_TO_DECLARED")
  featureType$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  featureType$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326") 
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/xml"
  )
  expect_true(featureType$addMetadataLink(md1))
  expect_false(featureType$addMetadataLink(md1))
  expect_equal(length(featureType$metadataLinks), 1L)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/html"
  )
  expect_true(featureType$addMetadataLink(md2))
  expect_false(featureType$addMetadataLink(md2))
  expect_equal(length(featureType$metadataLinks), 2L)
  
  #create layer
  layer <- GSLayer$new()
  layer$setName("tasmania_cities2")
  layer$setDefaultStyle("capitals")
  layer$addStyle("generic")
  
  #try to publish the complete layer (featuretype + layer)
  published <- gsman$publishLayer("topp", "taz_shapes", featureType, layer)
  expect_true(published)
  
})

test_that("UNPUBLISH layer",{
  #try to unpublish the complete layer (featuretype + layer)
  unpublished <- gsman$unpublishLayer("topp", "taz_shapes", "tasmania_cities2")
  expect_true(unpublished)
})

# test_GSCoverageStore.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSCoverageStore*.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSCoverageStore")

test_that("READ coverageStore",{
  expect_is(gsman,"GSManager")
  cs <- gsman$getCoverageStore("nurc", "mosaic")
  expect_is(cs, "GSAbstractCoverageStore")
  expect_true(all(c("name", "enabled", "type", "description", "full") %in% names(cs)))
  expect_equal(cs$name, "mosaic")
})

test_that("READ coverageStores",{
  cslist <- gsman$getCoverageStores("nurc")
  expect_true(all(sapply(cslist, function(x){class(x)[1] == "GSAbstractCoverageStore"})))
  csnames <- gsman$getCoverageStoreNames("nurc")
  expect_true(all(csnames %in% c("arcGridSample","img_sample2", "mosaic","worldImageSample")))
})


#GeoTIFF
#==========================================================================
#GeoTIFF CoverageStore CRUD 
#---------------------------------------------------------------------------

test_that("CRUD operations coverageStore - GeoTIFF",{
  cs = GSGeoTIFFCoverageStore$new(name="sfdem_new",
                                description = "sfdem_new description", enabled = TRUE)
  #CREATE
  created <- gsman$createCoverageStore("sf", cs)
  expect_true(created)
  cs <- gsman$getCoverageStore("sf", "sfdem_new")
  expect_is(cs, "GSGeoTIFFCoverageStore")
  expect_equal(cs$description, "sfdem_new description")
  expect_true(cs$enabled)

  #UPDATE
  coverageStore <- gsman$getCoverageStore("sf", "sfdem_new")
  coverageStore$setDescription("sfdem_new updated description")
  coverageStore$setEnabled(FALSE)
  
  updated <- gsman$updateCoverageStore("sf", coverageStore)
  expect_true(updated)
  cs <- gsman$getCoverageStore("sf", "sfdem_new")
  expect_is(cs, "GSGeoTIFFCoverageStore")
  expect_equal(cs$description, "sfdem_new updated description")
  expect_false(cs$enabled)

  #DELETE
  deleted <- gsman$deleteCoverageStore("sf", "sfdem_new", TRUE)
  expect_true(deleted)
  cs <- gsman$getCoverageStore("sf", "sfdem_new")
  expect_is(cs, "NULL")
})

#GeoTIFF Upload/Coverage CRUD 
#---------------------------------------------------------------------------
test_that("Upload coverage file and Create coverage - GeoTIFF",{
  #prerequisite - create store
  cs = GSGeoTIFFCoverageStore$new(name="sfdem_new",
                                  description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
  expect_true(created)
  
  #UPLOAD coverage file
  uploaded <- gsman$uploadGeoTIFF(ws = "sf", cs = "sfdem_new",
                                  endpoint = "file", configure = "none", update = "overwrite",
                                  filename = system.file("extdata/sfdem_new.tif", package = "geosapi"))
  expect_true(uploaded)
  
  #CREATE coverage
  cov <- GSCoverage$new()
  cov$setName("sfdem_new")
  cov$setNativeName("sfdem_new")
  cov$setTitle("Title for sfdem")
  cov$setDescription("Description for sfdem")
  cov$addKeyword("sfdem keyword1")
  cov$addKeyword("sfdem keyword2")
  cov$addKeyword("sfdem keyword3")
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/xml"
  )
  cov$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/html"
  )
  cov$addMetadataLink(md2)
  
  cov$setSrs("EPSG:4326")
  cov$setNativeCRS("EPSG:26713")
  cov$setLatLonBoundingBox(-103.87108701853181, 44.370187074132616, -103.62940739432703, 44.5016011535299, crs = "EPSG:4326")
  cov$setNativeBoundingBox(589980, 4913700, 609000, 4928010, crs = "EPSG:26713")
  
  created <- gsman$createCoverage(ws = "sf", cs = "sfdem_new", coverage = cov)
  expect_true(created)

  #READ coverage
  cov <- gsman$getCoverage(ws = "sf", cs = "sfdem_new", cv = "sfdem_new")
  expect_is(cov, "GSCoverage")
  expect_equal(cov$name, "sfdem_new")

  #UPDATE coverage
  cov$setEnabled(FALSE)
  cov$setAbstract("this is a modified abstract")
  updated <- gsman$updateCoverage(ws = "sf", cs = "sfdem_new", coverage = cov)
  expect_true(updated)
  cov_updated <- gsman$getCoverage(ws = "sf", cs = "sfdem_new", cv = "sfdem_new")
  expect_false(cov_updated$enabled)
  expect_equal(cov_updated$abstract, "this is a modified abstract")

  #DELETE coverage
  deleted <- gsman$deleteCoverage(ws = "sf", cs = "sfdem_new", cv = "sfdem_new", recurse = TRUE)
  expect_true(deleted)
  cov <- gsman$getCoverage(ws = "sf", cs = "sfdem_new", cv = "sfdem_new")
  expect_null(cov)
})

#' Geoserver REST API XML entry set
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSRESTEntrySet
#' 
#' @keywords geoserver rest api entryset
#' @return Object of \code{\link[R6]{R6Class}} for modelling a entry set
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSRESTEntrySet <- R6Class("GSRESTEntrySet",
  inherit = GSRESTResource,
  public = list(
   #'@field entryset entryset
   entryset = list(),
   
   #'@description Initializes an object of class \link{GSRESTEntrySet}
   #'@param rootName root name
   #'@param xml object of class \link[xml2]{xml_node-class}
   #'@param entryset entry set
   initialize = function(rootName, xml = NULL, entryset){
     super$initialize(rootName = rootName)
     if(!missing(xml) & !is.null(xml)){
       if(!any(class(xml) %in% c("xml_document","xml_node"))){
         stop("The argument 'xml' is not a valid XML object")
       }
       self$decode(xml)
     }else{
       if(!missing(entryset)){
         self$entryset = entryset
       }
     }
   },
   
   #'@description Decodes from XML
   #'@param xml object of class \link[xml2]{xml_node-class} 
   decode = function(xml){
     xml = xml2::as_xml_document(xml)
    entriesXML <- as(xml2::xml_find_all(xml, sprintf("//%s/entry", self$rootName)), "list")
     self$entryset = lapply(entriesXML, function(x){
       entry <- xml2::xml_text(x)
       if(entry %in% c("true","false")){
         entry <- as.logical(entry)
       }
       return(entry)
     })
     names(self$entryset) = lapply(entriesXML, xml2::xml_attr, "key")
   },
   
   #'@description Set entry set
   #'@param entryset entry set
   setEntryset = function(entryset){
     self$entryset = entryset
   },
   
   #'@description Adds entry set
   #'@param key key
   #'@param value value
   #'@return \code{TRUE} if added, \code{FALSE} otherwise
   addEntry = function(key, value){
     startNb <- length(self$entryset)
     if(length(which(names(self$entryset) == key)) == 0){
       self$entryset[[key]] <- value
     }
     endNb <- length(self$entryset)
     return(endNb == startNb+1)
   },
   
   #'@description Sets entry set
   #'@param key key
   #'@param value value
   setEntry = function(key, value){
     self$entryset[[key]] <- value
   },
   
   #'@description Deletes entry set
   #'@param key key
   #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
   delEntry = function(key){
     startNb <- length(self$entryset)
     self$entryset = self$entryset[which(names(self$entryset) != key)]
     endNb <- length(self$entryset)
     return(endNb == startNb-1)
   }
   
  )                     
)

#' Geoserver REST API AbstractDBDataStore
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api DataStore DB database
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer abstract DB dataStore
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @note Internal abstract class used for setting DB stores
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSAbstractDBDataStore <- R6Class("GSAbstractDBDataStore",
    inherit = GSAbstractDataStore,
    public = list(
      
      #'@description initializes an abstract DB data store
      #'@param xml an object of class \link[xml2]{xml_node-class} to create object from XML
      #'@param type the type of DB data store
      #'@param dbType DB type
      #'@param name coverage store name
      #'@param description coverage store description
      #'@param enabled whether the store should be enabled or not. Default is \code{TRUE}
      initialize = function(xml = NULL, type = NULL, dbType = NULL,
                            name = NULL, description = "", enabled = TRUE){
        if(missing(xml)) xml <- NULL
        super$initialize(xml = xml, name = name,
                         description = description,
                         enabled = enabled)
        if(is.null(xml)){
          self$setDefautConnectionParameters()
        }
        self$setType(type)
        self$setDatabaseType(dbType)
      },
      
      #'@description Set database type
      #'@param dbtype DB type
      setDatabaseType = function(dbtype) {
        self$setConnectionParameter("dbtype", dbtype);
      },
      
      #'@description Set namespace
      #'@param namespace namespace
      setNamespace = function(namespace) {
        self$setConnectionParameter("namespace", namespace);
      },
      
      #'@description Set host
      #'@param host host
      setHost = function(host) {
        self$setConnectionParameter("host", host);
      },
      
      #'@description Set port
      #'@param port port
      setPort = function(port) {
        self$setConnectionParameter("port", port);
      },
      
      #'@description Set database
      #'@param database database
      setDatabase = function(database) {
        self$setConnectionParameter("database", database);
      },
      
      #'@description Set schema
      #'@param schema schema
      setSchema = function(schema) {
        self$setConnectionParameter("schema", schema);
      },
      
      #'@description Set user
      #'@param user user
      setUser = function(user) {
        self$setConnectionParameter("user", user);
      },
      
      #'@description Set password
      #'@param password password
      setPassword = function(password) {
        self$setConnectionParameter("passwd", password);
      },
      
      #'@description Set JNDI reference name
      #'@param jndiReferenceName JNDI reference name
      setJndiReferenceName = function(jndiReferenceName) {
        self$setConnectionParameter("jndiReferenceName", jndiReferenceName);
      },
      
      #'@description Set expose primary keyws
      #'@param exposePrimaryKeys expose primary keys
      setExposePrimaryKeys = function(exposePrimaryKeys) {
        self$setConnectionParameter("Expose primary keys", exposePrimaryKeys);
      },
      
      #'@description Set min connections
      #'@param minConnections min connections. Default is 11
      setMinConnections = function(minConnections = 1) {
        self$setConnectionParameter("min connections", minConnections);
      },
      
      #'@description Set max connections
      #'@param maxConnections max connections. Default is 10
      setMaxConnections = function(maxConnections = 10) {
        self$setConnectionParameter("max connections", maxConnections);
      },
      
      #'@description Set fetch size
      #'@param fetchSize fetch size. Default is 1000
      setFetchSize = function(fetchSize = 1000) {
        self$setConnectionParameter("fetch size", fetchSize);
      },
      
      #'@description Set connection timeout
      #'@param seconds timeout (in seconds). Default is 20
      setConnectionTimeout = function(seconds = 20) {
        self$setConnectionParameter("Connection timeout", seconds);
      },
      
      #'@description Set validate connection
      #'@param validateConnections Validate connections
      setValidateConnections = function(validateConnections) {
        self$setConnectionParameter("validate connections", validateConnections);
      },
      
      #'@description Set primary key metadata table
      #'@param primaryKeyMetadataTable primary key metadata table
      setPrimaryKeyMetadataTable = function(primaryKeyMetadataTable) {
        self$setConnectionParameter("Primary key metadata table", primaryKeyMetadataTable);
      },
      
      #'@description Set loose bbox
      #'@param looseBBox loose bbox. Default is \code{TRUE}
      setLooseBBox = function(looseBBox = TRUE) {
        self$setConnectionParameter("Loose bbox", looseBBox);
      },
      
      #'@description Set prepared statemnts
      #'@param preparedStatements prepared Statements. Default is \code{FALSE}
      setPreparedStatements = function(preparedStatements = FALSE) {
        self$setConnectionParameter("preparedStatements", preparedStatements);
      },
      
      #'@description Set max open prepared statements
      #'@param maxOpenPreparedStatements max open preepared statements. Default is 50
      setMaxOpenPreparedStatements = function(maxOpenPreparedStatements = 50) {
        self$setConnectionParameter("Max open prepared statements", maxOpenPreparedStatements);
      },
      
      #'@description Set estimatedExtends
      #'@param estimatedExtends estimated extends. Default is \code{FALSE}
      setEstimatedExtends = function(estimatedExtends = FALSE){
        self$setConnectionParameter("Estimated extends", estimatedExtends);
      },
      
      #'@description Set default connection parameters
      setDefautConnectionParameters = function(){
        self$setMinConnections()
        self$setMaxConnections()
        self$setFetchSize()
        self$setConnectionTimeout()
        self$setLooseBBox()
        self$setPreparedStatements()
        self$setMaxOpenPreparedStatements()
        self$setEstimatedExtends()
      }
    )                     
)

#' Geoserver REST API GSInputCoverageBand
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api inputCoverageBand
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer input coverage band
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   GSInputCoverageBand$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSInputCoverageBand <- R6Class("GSInputCoverageBand",
  inherit = GSRESTResource,                    
  public = list(
    #'@field coverageName coverage name
    coverageName = NULL,
    #'@field band band
    band = NULL,
    
    #'@description Initializes an object of class \link{GSInputCoverageBand}
    #'@param xml object of class \link[xml2]{xml_node-class}
    #'@param coverageName coverage name
    #'@param band band name
    initialize = function(xml = NULL, coverageName = NULL, band = NULL){
      super$initialize(rootName = "inputCoverageBand")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }else{
        self$setCoverageName(coverageName)
        self$setBand(band)
      }
    },
    
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      coverageNames <- xml2::xml_find_first(xml, "//coverageName")
      self$coverageName <- xml2::xml_text(coverageNames)
      bands <- xml2::xml_find_first(xml, "//band")
      self$band <- xml2::xml_text(bands)
    },
    
    #'@description Set coverage name
    #'@param coverageName coverage name
    setCoverageName = function(coverageName){
      self$coverageName = coverageName
    },
  
    #'@description Set band
    #'@param band band
    setBand = function(band){
      self$band = band
    }
    
  )                     
)

#' Geoserver REST API Access Control List Layer Rule
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSLayerRule
#' @title A GeoServer access control list layer rule
#' @description This class models a GeoServer access control list layer rule
#' @keywords geoserver rest api access control rule
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer access control list layer rule
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSLayerRule <- R6Class("GSLayerRule",
  inherit = GSRule,
  
  public = list(
    
    #'@field roles one or more roles
    roles = list(),
    
    #'@description Initializes a \link{GSLayerRule}
    #'@param xml an object of class \link[xml2]{xml_node-class}
    #'@param ws the resource workspace. Default is \code{NULL}
    #'@param lyr the target layer to which the access control should be added
    #'@param permission the rule permission, either \code{r} (read), \code{w} (write) or \code{a} (administer)
    #'@param roles one or more roles to add for the rule
    initialize = function(xml = NULL,
                          ws = NULL, lyr,
                          permission = c("r","w","a"),
                          roles){
      super$initialize(xml = xml)
      permission = match.arg(permission)
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }else{
        res_str = sprintf("%s.%s", lyr, permission)
        if(!is.null(ws)) res_str = sprintf("%s.%s", ws, res_str)
        self$attrs$resource = res_str
        self$roles = roles
      }
    }
  )                       
)

#' Geoserver REST API Namespace
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api namespace
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer namespace
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' GSNamespace$new(prefix = "my_ns", uri = "http://my_ns")
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSNamespace <- R6Class("GSNamespace",
  inherit = GSRESTResource,  
  public = list(
    #' @field name namespace name
    name = NA,
    #' @field prefix namespace prefix
    prefix = NA,
    #' @field uri namespace URI
    uri = NA,
    #' @field full completeness of the namespace description
    full = FALSE,
   
    #'@description Initializes an object of class \link{GSNamespace}
    #'@param xml object of class \link[xml2]{xml_node-class}
    #'@param prefix prefix
    #'@param uri uri
    initialize = function(xml = NULL, prefix, uri){
      super$initialize(rootName = "namespace")
      if(!missing(xml) & !is.null(xml)){
        self$decode(xml)
      }else{
        self$prefix <- prefix
        self$name <- prefix
        self$uri <- uri
        self$full <- TRUE
      }
    },
    
    #'@description Decodes from XML
    #'@param xml object of class \link[xml2]{xml_node-class}
    decode = function(xml){
      xml = xml2::as_xml_document(xml)
      names <- xml2::xml_find_first(xml, "//name")
      if(length(names)>0){
        self$full <- FALSE
        self$name <- xml2::xml_text(names)
      }else{
        self$full <- TRUE
        self$prefix <- xml2::xml_find_first(xml, "//prefix") %>% xml2::xml_text()
        self$name <- self$prefix
        self$uri <- xml2::xml_find_first(xml, "//uri") %>% xml2::xml_text()
      }
    }
  )                     
)

#' Geoserver REST API Workspace Manager
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' @keywords geoserver rest api workspace
#' @return Object of \code{\link[R6]{R6Class}} with methods for managing the workspaces
#'  of a GeoServer instance.
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' \dontrun{
#'    GSWorkspaceManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSWorkspaceManager <- R6Class("GSWorkspaceManager",
  inherit = GSManager,

  public = list(
    
    #'@description Get the list of available workspace. Returns an object of class \code{list}
    #'    containing items of class \code{\link{GSWorkspace}}
    #'@param a list of \link{GSWorkspace}
    getWorkspaces = function(){
      msg = "Fetching list of workspaces"
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         "/workspaces.xml", self$verbose.debug)
      wsList <- NULL
      if(status_code(req) == 200){
        wsXML <- GSUtils$parseResponseXML(req)
        wsXMLList <- as(xml2::xml_find_all(wsXML, "//workspace"), "list")
        wsList <- lapply(wsXMLList, GSWorkspace$new)
        msg = sprintf("Successfully fetched %s workspaces", length(wsList))
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching list of workspaces"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(wsList)
    },
    
    #'@description Get the list of available workspace names. Returns an vector of class \code{character}
    #'@return a list of workspace names
    getWorkspaceNames = function(){
      wsList <- sapply(self$getWorkspaces(), function(x){x$name})
      return(wsList)
    },
    
    #'@description Get a \code{\link{GSWorkspace}} object given a workspace name.
    #'@param ws workspace name
    #'@return an object of class \link{GSWorkspace}
    getWorkspace = function(ws){
      msg = sprintf("Fetching workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                      sprintf("/workspaces/%s.xml", ws), self$verbose.debug)
      workspace <- NULL
      if(status_code(req) == 200){
        wsXML <- GSUtils$parseResponseXML(req)
        workspace <- GSWorkspace$new(xml = wsXML)
        msg = "Successfully fetched workspace!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching workspace"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(workspace)
    },
    
    #'@description Creates a GeoServer workspace given a name, and an optional URI. If the URI
    #'    is not specified, GeoServer will automatically create an associated Namespace 
    #'    with the URI built from the workspace name. If the URI is specified, the method
    #'    invokes the method \code{createNamespace(ns, uri)} of the \code{\link{GSNamespaceManager}}.
    #'    Returns \code{TRUE} if the workspace has been successfully created, \code{FALSE} otherwise
    #'@param name name
    #'@param uri uri
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createWorkspace = function(name, uri){
      msg = sprintf("Creating workspace '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      if(missing(uri)){
        
        ws <- GSWorkspace$new(name = name)
        
        req <- GSUtils$POST(
          url = self$getUrl(),
          user = private$user,
          pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
          path = "/workspaces",
          content = GSUtils$getPayloadXML(ws),
          contentType = "text/xml",
          verbose = self$verbose.debug
        )
        if(status_code(req) == 201){
          msg = "Successfully created workspace!"
          cli::cli_alert_info(msg)
          self$INFO(msg)
          created = TRUE
        }else{
          err = "Error while creating workspace"
          cli::cli_alert_danger(err)
          self$ERROR(err)
        }
      }else{
        msg = "Delegating workspace creation to namespace manager"
        cli::cli_alert_info(msg)
        self$INFO(msg)
        nsman <- GSNamespaceManager$new(self$getUrl(), private$user,
                                        private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                        self$loggerType)
        created <- nsman$createNamespace(name, uri)
      }
      return(created)
    },
    
    #'@description Updates a GeoServer workspace given a name, and an optional URI. If the URI
    #'    is not specified, GeoServer will automatically update the associated Namespace 
    #'    with the URI built from the workspace name. If the URI is specified, the method
    #'    invokes the method \code{updateNamespace(ns, uri)} of the \code{\link{GSNamespaceManager}}.
    #'    Returns \code{TRUE} if the workspace has been successfully updated, \code{FALSE} otherwise
    #'@param name name
    #'@param uri uri
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    updateWorkspace = function(name, uri){
      msg = sprintf("Updating workspace '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      updated <- FALSE
      if(missing(uri)){
        workspace <- GSWorkspace$new(name = name)
        req <- GSUtils$PUT(
          url = self$getUrl(), user = private$user,
          pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
          path = sprintf("/workspaces/%s.xml", name),
          content = GSUtils$getPayloadXML(workspace),
          contentType = "application/xml",
          verbose = self$verbose.debug
        )
        if(status_code(req) == 200){
          msg = "Successfully updated workspace!"
          cli::cli_alert_success(msg)
          self$INFO(msg)
          updated = TRUE
        }else{
          err = "Error while updating workspace"
          cli::cli_alert_danger(err)
          self$ERROR(err)
        }
      }else{
        msg = "Delegating workspace update to namespace manager"
        cli::cli_alert_info(msg)
        self$INFO(msg)
        nsman <- GSNamespaceManager$new(self$getUrl(), private$user, 
                                        private$keyring_backend$get(service = private$keyring_service, username = private$user),
                                        self$loggerType)
        updated <- nsman$updateNamespace(name, uri)
      }
      return(updated)
    },
    
    #'@description Deletes a GeoServer workspace given a name. 
    #'@param name name
    #'@param recurse recurse
    #'@return \code{TRUE} if the workspace has been successfully deleted, \code{FALSE} otherwise
    deleteWorkspace = function(name, recurse = FALSE){
      msg = sprintf("Deleting workspace '%s'", name)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/workspaces/%s", name)
      if(recurse) path <- paste0(path, "?recurse=true")
      #TODO hack for style removing (not managed by REST API)
      
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         path = path, self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully deleted workspace!"
        cli::cli_alert_info(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting workspace"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    },
    
    #'@description Updates workspace settings
    #'@param ws workspace name
    #'@return an object of class \link{GSWorkspaceSettings}
    getWorkspaceSettings = function(ws){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        stop(err)
      }
      msg = sprintf("Fetching settings for workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      req <- GSUtils$GET(self$getUrl(), private$user,
                         private$keyring_backend$get(service = private$keyring_service, username = private$user),
                         sprintf("/workspaces/%s/settings.xml", ws), self$verbose.debug)
      workspaceSettings <- NULL
      if(status_code(req) == 200){
        wsSettingXML <- GSUtils$parseResponseXML(req)
        workspaceSettings <- GSWorkspaceSettings$new(xml = wsSettingXML)
        msg = "Successfully fetched workspace settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
      }else{
        err = "Error while fetching workspace settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(workspaceSettings)
    },
    
    #'@description Creates workspace settings
    #'@param ws workspace name
    #'@param workspaceSettings object of class \link{GSWorkspaceSettings}
    #'@return \code{TRUE} if created, \code{FALSE} otherwise
    createWorkspaceSettings = function(ws, workspaceSettings){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        stop(err)
      }
      msg = sprintf("Creating settings for workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      created <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(),
        user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/settings.xml", ws),
        content = GSUtils$getPayloadXML(workspaceSettings),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully created workspace settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        created = TRUE
      }else{
        err = "Error while creating workspace settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(created)
    },
    
    #'@description Updates workspace settings
    #'@param ws workspace name
    #'@param workspaceSettings object of class \link{GSWorkspaceSettings}
    #'@return \code{TRUE} if updated, \code{FALSE} otherwise
    updateWorkspaceSettings = function(ws, workspaceSettings){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        stop(err)
      }
      msg = sprintf("Updating settings for workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      updated <- FALSE
      req <- GSUtils$PUT(
        url = self$getUrl(), user = private$user,
        pwd = private$keyring_backend$get(service = private$keyring_service, username = private$user),
        path = sprintf("/workspaces/%s/settings.xml", ws),
        content = GSUtils$getPayloadXML(workspaceSettings),
        contentType = "application/xml",
        verbose = self$verbose.debug
      )
      if(status_code(req) == 200){
        msg = "Successfully updated workspace settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        updated = TRUE
      }else{
        err = "Error while updating workspace settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(updated)
    },
    
    #'@description Deletes workspace settings
    #'@param ws workspace name
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteWorkspaceSettings = function(ws){
      if(self$version$lowerThan("2.12")){
        err = "This feature is available starting from GeoServer 2.12"
        cli::cli_alert_danger(err)
        stop(err)
      }
      msg = sprintf("Deleting settings for workspace '%s'", ws)
      cli::cli_alert_info(msg)
      self$INFO(msg)
      deleted <- FALSE
      path <- sprintf("/workspaces/%s/settings", ws)
      req <- GSUtils$DELETE(self$getUrl(), private$user,
                            private$keyring_backend$get(service = private$keyring_service, username = private$user),
                            path = path, self$verbose.debug)
      if(status_code(req) == 200){
        msg = "Successfully deleted workspace settings!"
        cli::cli_alert_success(msg)
        self$INFO(msg)
        deleted = TRUE
      }else{
        err = "Error while deleting workspace settings"
        cli::cli_alert_danger(err)
        self$ERROR(err)
      }
      return(deleted)
    }
    
  )
  
)

# test_GSWorkspace.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSWorkspace*.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSWorkspace")

test_that("new workspace",{
  ws <- GSWorkspace$new(name = "test")
  expect_is(ws, "GSWorkspace")
  expect_equal(ws$name, "test")
})

test_that("READ workspace",{
  ws <- gsman$getWorkspace("topp")
  expect_is(ws, "GSWorkspace")
  expect_true("name" %in% names(ws))
  expect_equal(ws$name, "topp")
})

test_that("READ workspaces",{
  wslist <- gsman$getWorkspaces()
  expect_true(all(sapply(wslist, function(x){class(x)[1] == "GSWorkspace"})))
})

test_that("CREATE workspace",{
  created <- gsman$createWorkspace("geosapi")
  expect_true(created)
  ws <- gsman$getWorkspace("geosapi")
  expect_equal(ws$name, "geosapi")
  nsman <- GSNamespaceManager$new(gsUrl, gsUsr, gsPwd)
  ns <- nsman$getNamespace("geosapi")
  expect_equal(ns$name, "geosapi")
  expect_equal(ns$prefix, "geosapi")
  expect_equal(ns$uri, "http://geosapi")
})

test_that("UPDATE namespace",{
  updated <- gsman$updateWorkspace("geosapi", "http://www.my.org/geosapi2")
  expect_true(updated)
  nsman <- GSNamespaceManager$new(gsUrl, gsUsr, gsPwd)
  ns <- nsman$getNamespace("geosapi")
  expect_equal(ns$name, "geosapi")
  expect_equal(ns$prefix, "geosapi")
  expect_equal(ns$uri, "http://www.my.org/geosapi2")
})

test_that("DELETE workspace",{
  deleted <- gsman$deleteWorkspace("geosapi")
  expect_true(deleted)
  ws <- gsman$getWorkspace("geosapi")
  expect_is(ws, "NULL")
})

# test_GSWorkspaceSettings.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSWorkspaceSettings.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSWorkspaceSettings")

gsman$createWorkspace("geosapi")
minVersion <- "2.12"

test_that("CREATE workspace settings",{
  if(gsman$version$equalTo(minVersion)|gsman$version$greaterThan(minVersion)){
    settings <- GSWorkspaceSettings$new()
    created <- gsman$createWorkspaceSettings("geosapi", settings)
    expect_true(created)
  }
})

test_that("READ workspace settings",{
  if(gsman$version$equalTo(minVersion)|gsman$version$greaterThan(minVersion)){
    settings <- gsman$getWorkspaceSettings("geosapi")
    expect_true(!is.null(settings))
  }
})

test_that("UPDATE workspace settings",{
  if(gsman$version$equalTo(minVersion)|gsman$version$greaterThan(minVersion)){
    settings <- GSWorkspaceSettings$new()
    settings$setNumDecimals(4)
    settings$setVerbose(TRUE)
    updated <- gsman$updateWorkspaceSettings("geosapi", settings)
    settings2 <- gsman$getWorkspaceSettings("geosapi")
    expect_equal(settings2$numDecimals, 4)
    expect_true(settings2$verbose)
    expect_true(updated)
  }
})

test_that("DELETE workspace settings",{
  if(gsman$version$equalTo(minVersion)|gsman$version$greaterThan(minVersion)){
    deleted <- gsman$deleteWorkspaceSettings("geosapi")
    expect_true(deleted)
  }
})

gsman$deleteWorkspace("geosapi")

#' Geoserver REST API LayerGroup
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSLayerGroup
#' @title A GeoServer layergroup resource
#' @description This class models a GeoServer layer group. This class is to be
#' used for clustering layers into a group.
#' @keywords geoserver rest api resource featureType coverage layer layer group
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer layergroup
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#'   lyr <- GSLayerGroup$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSLayerGroup <- R6Class("GSLayerGroup",
   inherit = GSRESTResource,
   
   public = list(
     #'@field full full
     full = TRUE,
     #'@field name name
     name = NULL,
     #'@field mode mode
     mode = NULL,
     #'@field title title
     title = NULL,
     #'@field abstractTxt abstract
     abstractTxt = NULL,
     #'@field workspace workspace
     workspace = NULL,
     #'@field publishables publishables
     publishables = list(),
     #'@field styles styles
     styles = list(),
     #'@field metadataLinks metadata links
     metadataLinks = list(),
     #'@field bounds bounds
     bounds = list(minx = NA, miny = NA, maxx = NA, maxy = NA, crs = NA),
     
     #'@description Initializes an object of class \link{GSLayerGroup}
     #'@param xml object of class \link[xml2]{xml_node-class}
     initialize = function(xml = NULL){
       super$initialize(rootName = "layerGroup")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }
     },
     
     #'@description Decodes from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       xml = xml2::as_xml_document(xml)
       self$name <- xml2::xml_find_first(xml, "//name") %>% xml2::xml_text()
       
       if(self$full){
         
         titles <- xml2::xml_find_first(xml, "//title")
         if(length(titles)>0) self$title <- xml2::xml_text(titles)
         abstracts <- xml2::xml_find_first(xml, "//abstractTxt")
         if(length(abstracts)>0) self$abstractTxt <- xml2::xml_text(abstracts)
         modes <- xml2::xml_find_first(xml, "//mode")
         if(length(modes)>0) self$mode <- xml2::xml_text(modes)
         workspaces <- xml2::xml_find_first(xml, "//workspace")
         if(length(workspaces)>0){
           self$workspace <- GSWorkspace$new(name = xml2::xml_text(workspaces))
         }
         
         #publishables
         publishables <- as(xml2::xml_find_all(xml, "//publishables/published"), "list")
         if(length(publishables)>0){
           self$publishables <- lapply(publishables, GSPublishable$new)
         }
         
         #styles
         styles <- as(xml2::xml_find_all(xml, "//styles/style"), "list")
         if(length(styles)>0){
           styles <- lapply(styles, GSStyle$new)
           self$setStyles(styles)
         }
         
         #metadataLinks
         metadataLinksXML <- as(xml2::xml_find_all(xml, "//metadataLinks/metadataLink"), "list")
         if(length(metadataLinksXML)>0){
           for(metadataLinkXML in metadataLinksXML){
             md <- GSMetadataLink$new(xml = metadataLinkXML)
             self$addMetadataLink(md)
           }
         }
         
         #bounds
         boundsXML <- as(xml2::xml_find_all(xml, "//bounds/*"), "list")
         self$bounds <- lapply(boundsXML, xml2::xml_text)
         names(self$bounds) <-  lapply(boundsXML, xml2::xml_name)
       }
     },
     
     #'@description Set name
     #'@param name name
     setName = function(name){
       self$name = name
     },
     
     #'@description Set mode
     #'@param mode a mode value among "SINGLE", "NAMED", "CONTAINER", "EO"
     setMode = function(mode){
       allowedModes <- c("SINGLE", "NAMED", "CONTAINER", "EO")
       if(!(mode %in% allowedModes)){
         stop(sprintf("Mode should one among values [%s]", paste(allowedModes, collapse=",")))
       }
       self$mode = mode
     },
     
     #'@description Set title
     #'@param title title
     setTitle = function(title){
       self$title = title
     },
     
     #'@description Set abstract
     #'@param abstract abstract
     setAbstract = function(abstract){
       self$abstractTxt = abstract
     },
     
     #'@description Set workspace
     #'@param workspace workspace name, object of class \link{GSWorkspace} or \code{character}
     setWorkspace = function(workspace){
       if(!is(workspace, "GSWorkspace")){
         workspace <- GSWorkspace$new(name = workspace)
       }
       self$workspace = workspace
     },
     
     #'@description Adds layer
     #'@param layer layer name
     #'@param style style name
     addLayer = function(layer, style){
       ps <- GSPublishable$new(name = layer, type = "layer")
       self$addPublishable(ps)
       self$addStyle(style)
     },
     
     #'@description Adds layer group
     #'@param layerGroup layer group
     addLayerGroup = function(layerGroup){
       ps <- GSPublishable$new(name = layer, type = "layerGroup")
       self$addPublishable(ps)
       self$addStyle(NA)
     },
     
     #'@description Adds publishable
     #'@param publishable publishable
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addPublishable = function(publishable){
       startNb = length(self$publishables)
       add <- TRUE
       invisible(sapply(self$publishables, function(x){
         if(publishable$name == x$name & publishable$attr_type == x$attr_type){
           add <<- FALSE
         }
       }))
       if(add){
         self$publishables = c(self$publishables, publishable)
       }
       endNb = length(self$publishables)
       return(endNb == startNb+1)
     },
     
     #'@description Set styles
     #'@param styles styles
     setStyles = function(styles){
       if(!is.list(styles)) styles = list(styles)
       self$styles = styles
       return(TRUE)
     },
     
     #'@description Adds a style
     #'@param style style
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addStyle = function(style){
        if(is(style, "character")) style <- GSStyle$new(xml=NULL, name = style)
        startNb = length(self$styles)
        self$styles = c(self$styles, style)
        endNb = length(self$styles)
        return(endNb == startNb+1)
     },
     
     #'@description Set metadata links
     #'@param metadataLinks metadata links
     setMetadataLinks = function(metadataLinks){
       self$metadataLinks <- metadataLinks
     },
     
     #'@description Adds metadata link
     #'@param metadataLink object of class \link{GSMetadataLink}
     #'@return \code{TRUE} if added, \code{FALSE} otherwise
     addMetadataLink = function(metadataLink){
       startNb = length(self$metadataLinks)
       links <- lapply(self$metadataLinks, function(x){x$content})
       if(length(which(links == metadataLink$content)) == 0){
         self$metadataLinks = c(self$metadataLinks, metadataLink)
       }
       endNb = length(self$metadataLinks)
       return(endNb == startNb+1)
     },
     
     #'@description Deletes metadata link
     #'@param metadataLink object of class \link{GSMetadataLink}
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     deleteMetadataLink = function(metadataLink){
       startNb = length(self$metadataLinks)
       links <- lapply(self$metadataLinks, function(x){x$content})
       self$metadataLinks = self$metadataLinks[which(links != metadataLink$content)]
       endNb = length(self$metadataLinks)
       return(endNb == startNb-1)
     },
     
     #'@description Set bounds
     #'@param minx minx
     #'@param miny miny
     #'@param maxx maxx
     #'@param maxy maxy
     #'@param bbox bbox
     #'@param crs crs
     setBounds = function(minx, miny, maxx, maxy, bbox = NULL, crs){
       self$bounds <- GSUtils$setBbox(minx, miny, maxx, maxy, bbox, crs)
     }
     
   )                       
)

# test_GSLayerGroup.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for GSLayerGroup.R
#=======================
require(geosapi, quietly = TRUE)
require(testthat)

context("GSLayerGroup")

test_that("layer group encoding/decoding",{
  
  lyr <- GSLayerGroup$new()
  expect_is(lyr, "GSLayerGroup")
  expect_is(lyr, "R6")
  
  lyr$setName("name")
  lyr$setTitle("title")
  lyr$setAbstract("abstract")
  lyr$setMode("SINGLE")
  lyr$setWorkspace("staging")
  lyr$addLayer(layer = "grc_farms", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  
  #encoding to XML
  lyrXML <- lyr$encode()
  expect_is(lyrXML, c("xml_document", "xml_node"))
  
  #decoding from XML
  lyr2 <- GSLayerGroup$new(xml = lyrXML)
  lyr2XML <- lyr2$encode()
  
  #check encoded XML is equal to decoded XML
  testthat::expect_true(length(waldo::compare(lyrXML, lyr2XML))==0)
  
})

test_that("CREATE layer group",{
  lyr <- GSLayerGroup$new()
  lyr$setName("test_layergroup")
  lyr$setTitle("title")
  lyr$setAbstract("abstract")
  lyr$setMode("SINGLE")
  lyr$setWorkspace("topp")
  lyr$addLayer(layer = "tasmania_cities", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  expect_true(gsman$createLayerGroup(layerGroup = lyr, ws = "topp"))
})

test_that("UPDATE layer group",{
  lyr <- GSLayerGroup$new()
  lyr$setName("test_layergroup")
  lyr$setTitle("title")
  lyr$setAbstract("abstract 2")
  lyr$setMode("SINGLE")
  lyr$setWorkspace("topp")
  lyr$addLayer(layer = "tasmania_cities", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  expect_true(gsman$updateLayerGroup(layerGroup = lyr, ws = "topp"))
})

test_that("READ layer group",{
  lyr <- gsman$getLayerGroup(lyr = "test_layergroup", ws = "topp")
  expect_equal(lyr$abstractTxt, "abstract 2")
  expect_is(lyr, "GSLayerGroup")
})

test_that("READ layer groups",{ 
  lyrs <- gsman$getLayerGroups(ws = "topp")
  expect_is(lyrs, "list")
  expect_equal(length(lyrs), 1L)
  expect_is(lyrs[[1]], "GSLayerGroup")
})

test_that("DELETE layer group",{
  expect_true(gsman$deleteLayerGroup(lyr = "test_layergroup", ws = "topp"))
})

#' Geoserver REST API Resource
#'
#' @docType class
#' @importFrom R6 R6Class
#' @export
#' 
#' @name GSCoverage
#' @title A GeoServer coverage
#' @description This class models a GeoServer coverage. This class is to be
#' used for manipulating representations of vector data with GeoServer.
#' @keywords geoserver rest api resource coverageType
#' @return Object of \code{\link[R6]{R6Class}} for modelling a GeoServer coverage
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @examples
#' gt <- GSCoverage$new()
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
GSCoverage <- R6Class("GSCoverage",
   inherit = GSResource,
   
   public = list(
     #'@field cqlFilter CQL filter
     cqlFilter = NULL,
     
     #'@description Initializes a \link{GSCoverage} from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     initialize = function(xml = NULL){
       super$initialize(rootName = "coverage")
       if(!missing(xml) & !is.null(xml)){
         self$decode(xml)
       }
     },
     
     #'@description Decodes coverage from XML
     #'@param xml object of class \link[xml2]{xml_node-class}
     decode = function(xml){
       super$decode(xml)
     },
     
     #'@description Set view
     #'@param cv cv, object of class \link{GSCoverageView}
     #'@return \code{TRUE} if set, \code{FALSE} otherwise
     setView = function(cv){
       if(!is(cv, "GSCoverageView")){
         stop("Argument 'cv' should be an object of class 'GSCoverageView'")
       }
       added <- super$setMetadata("COVERAGE_VIEW", cv)
       return(added)
     },
     
     #'@description Deletes view
     #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
     delView = function(){
       deleted <- super$delMetadata("COVERAGE_VIEW")
       return(deleted)
     }
     
   )
                         
)

---
title: geosapi User Manual
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{geosapi User Manual}
  %\VignetteEngine{knitr::knitr}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```


# geosapi - R Interface to GeoServer REST API


R Interface to GeoServer REST API – Allows to perform programmatically from R

* CRUD operations (Create, Read, Update, Delete)  on GeoServer resources including ``workspace``, ``namespaces``, ``dataStores`` ``featureTypes``, ``layers``, ``styles``, ``settings`` (workspaces, OGC services) etc. 
* Data upload to Geoserver data directory (e.g. upload shapefile)

***

If you wish to sponsor geosapi, do not hesitate to [contact me](mailto:eblondel.pro@gmail.com)

Many thanks to the following organizations that have provided fundings for strenghtening the ``geosapi`` package:

<a href="https://unepgrid.ch/en"><img height=100 width=200 src="https://www.weadapt.org/sites/weadapt.org/files/styles/large/public/screenshot_2021-05-25_at_16.19.32_0.png?itok=iiEMf_S3"></a><a href="https://www.fao.org/home/en/"><img height=100 width=200 src="https://www.fao.org/fileadmin/templates/family-farming-decade/images/FAO-IFAD-Logos/FAO-Logo-EN.svg"></a>


***

**Table of contents**

[**1. Overview**](#package_overview)<br/>
[**2. Package status**](#package_status)<br/>
[**3. Credits**](#package_credits)<br/>
[**4. User guide**](#user_guide)<br/>
&nbsp;&nbsp;&nbsp;[4.1 Installation](#install_guide)<br/>
&nbsp;&nbsp;&nbsp;[4.2 Connect to GeoServer REST API](#GSManager)<br/>
&nbsp;&nbsp;&nbsp;[**4.3 GeoServer ``workspaces``**](#GSWorkspace)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.1 Get a (list of) ``workspace(s)``](#GSWorkspace-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.2 Create a new ``workspace``](#GSWorkspace-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.3 Update a new ``workspace``](#GSWorkspace-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.4 Delete a new ``workspace``](#GSWorkspace-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.4 GeoServer ``namespaces``**](#GSNamespace)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.1 Get a (list of) ``namespace(s)``](#GSNamespace-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.2 Create a new ``namespace``](#GSNamespace-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.3 Update a new ``namespace``](#GSNamespace-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.4 Delete a new ``namespace``](#GSNamespace-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.5 Manage vector data in GeoServer**](#vectordata)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.1 GeoServer ``dataStores``**](#GSDataStore)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.1 Get a (list of) ``dataStore(s)``](#GSDataStore-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2 Create a new ``dataStore``](#GSDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2.1 Create a **Shapefile** ``dataStore``](#GSShapefileDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2.2 Create a **Shapefiles Directory** ``dataStore``](#GSShapefileDirectoryDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2.3 Create a **GeoPackage** ``dataStore``](#GSGeoPackageDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.3 Update a new ``dataStore``](#GSDataStore-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.4 Delete a new ``dataStore``](#GSDataStore-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.2 Upload features/vector data**](#GSFeatureType-upload)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.3 GeoServer ``featureTypes``**](#GSFeatureType)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.1 Get a (list of) ``featureType(s)``](#GSFeatureType-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.2 Create a new ``featureType``](#GSFeatureType-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.3 Update a new ``featureType``](#GSFeatureType-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.4 Delete a new ``featureType``](#GSFeatureType-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.3 GeoServer ``layers``**](#GSLayer)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.1 Get a (list of) ``layer(s)``](#GSLayer-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.2 Create a new ``layer``](#GSLayer-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.3 Update a new ``layer``](#GSLayer-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.4 Delete a new ``layer``](#GSLayer-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.5 Publication of GeoServer feature layer**](#publishFeatureLayer)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.5.1 Publish Geoserver feature layer](#publishFeatureLayer-publish)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.5.2 Unpublish Geoserver feature layer](#publishFeatureLayer-unpublish)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.6 GeoServer ``layer groups``**](#GSLayerGroup)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.1 Get a (list of) ``layer group(s)``](#GSLayerGroup-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.2 Create a new ``layer group``](#GSLayerGroup-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.3 Update a new ``layer group``](#GSLayerGroup-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.4 Delete a new ``layer group``](#GSLayerGroup-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.6 Manage raster data in GeoServer**](#rasterdata)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.6.1 GeoServer ``coveragestores``**](#GSCoverageStore)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.1 Get a (list of) ``coveragestore(s)``](#GSCoverageStore-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2 Create a new ``coveragestore``](#GSCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.1 Create a **GeoTIFF** ``coveragestore``](#GSGeoTIFFCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.2 Create a **WorldImage** ``coveragestore``](#GSWorldImageCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.3 Create a **ArcGrid** ``coveragestore``](#GSArcGridCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.4 Create a **ImageMosaic** ``coveragestore``](#GSImageMosaicCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.3 Update a new ``coveragestore``](#GSCoverageStore-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.4 Delete a new ``coveragetore``](#GSCoverageStore-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.6.2 Upload coverage/raster data**](#GSCoverage-upload)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.6.3 GeoServer ``coverages``**](#GSCoverage)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.1 Get a (list of) ``coverage(s)``](#GSCoverage-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.2 Create a new ``coverage``](#GSCoverage-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.3 Update a new ``coverage``](#GSCoverage-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.4 Delete a new ``coverage``](#GSCoverage-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.7 Manage styles in GeoServer**](#styles)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.1 Get a (list of) ``style(s)``](#GSStyle-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.2 Create a new ``style``](#GSStyle-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.3 Update a new ``style``](#GSStyle-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.4 Delete a new ``style``](#GSStyle-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.8 Manage workspace settings in GeoServer**](#settings_workspaces)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.1 Create a workspace settings](#GSWorkspaceSettings-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.2 Get a workspace settings](#GSWorkspaceSettings-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.3 Update a workspace settings](#GSWorkspaceSettings-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.4 Delete a workspace settings](#GSWorkspaceSettings-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.9 Manage OGC service (WMS/WFS/WCS) settings in GeoServer**](#settings_services)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.9.1 Get an OGC service (WMS/WFS/WCS) settings](#GSServiceSettings-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.9.2 Update an OGC service (WMS/WFS/WCS) settings](#GSServiceSettings-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.9.3 Enable/Disable an OGC service (WMS/WFS/WCS)](#GSServiceSettings-enable)<br/>
[**5. Issue reporting**](#package_issues)<br/>

<a name="package_overview"/>

### 1. Overview and vision
***

Until now, equivalent tools were existing for other programming languages (e.g. Java, Python) but not in R. [geosapi](https://github.com/eblondel/geosapi) intends to provide R native interface to the GeoServer REST API, in order to facilitate publication of geographic data resources from R to [GeoServer](https://geoserver.org/).

<a name="package_status"/>

### 2. Development status
***

Published on CRAN

<a name="package_credits"/>

### 3. Credits
***

(c) 2016, Emmanuel Blondel

Package distributed under MIT license.

If you use ``geosapi``, i would be very grateful if you can add a citation in your published work. By citing ``geosapi``, beyond acknowledging the work, you contribute to make it more visible and guarantee its growing and sustainability. You can get the preferred citation by running ```citation("geosapi)``` in R.

You can reference ``geosapi`` through its DOI: [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.1184895.svg)](https://doi.org/10.5281/zenodo.1184895)

<a name="user_guide"/>

### 4. User guide
***

<a name="install_guide"/>

#### 4.1 How to install geosapi in R

For now, the package can be installed from Github
```R
install.packages("remotes")
```
Once the devtools package loaded, you can use the install_github to install ``geosapi. By default, package will be installed from ``master`` which is the current version in development (likely to be unstable). 

```R
remotes::install_github("eblondel/geosapi")
```

<a name="GSManager"/>

#### 4.2 Connect to GeoServer REST API 

The main entry point of ``geosapi`` is the ``GSManager``. To configure it, enter the following line, specifying the base ``URL`` of your GeoServer, and your credentials:
```R
gsman <- GSManager$new(
    url = "http://localhost:8080/geoserver", #baseUrl of the Geoserver
    user = "admin", pwd = "geoserver", #credentials
    logger = NULL #logger, for info or debugging purpose
)
```

By default, the ``geosapi`` **logger** is deactivated. To enable the logger, specify the level of log you wish as parameter of the above R code. Two logging levels are available:

* ``INFO``: will print the ``geosapi`` logs. Three types of messages can be distinguished: ``INFO``, ``WARN``, ``ERROR``. The latter is generally associated with a ``stop`` and indicate an blocking error for the R method executed.
* ``DEBUG`` will print the above ``geosapi`` logs, and report all logs from HTTP requests performed with ``cURL``

The ``GSManager`` inherits all methods of resource dependent managers, to provide the users with a single R interface to GeoServer REST API operations. In addition, the ``GSManager`` allows accessing the different resource managers through specific methods. The following ``managers`` are available:
* ``GSNamespaceManager``: Manage ``namespaces``
* ``GSWorkspaceManager``: Manage ``workspaces``
* ``GSDataStoreManager``: Manage ``dataStores``

<a name="GSWorkspace"/>

#### 4.3 Manipulate GeoServer ``workspaces``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/workspaces.html

<a name="GSWorkspace-get"/>

##### 4.3.1 Get a (list of) ``workspace(s)``

```R
  #get workspace objects
  wslist <- gsman$getWorkspaces()

  #get workspace names only
  wsnames <- gsman$getWorkspaceNames()

  #get workspace by name
  ws <- gsman$getWorkspace("topp")
```

<a name="GSWorkspace-create"/>

##### 4.3.2 Create a new ``workspace``

```R
  created <- gsman$createWorkspace("geosapi", "http://geoapi")
```

<a name="GSWorkspace-update"/>

##### 4.3.3 Update an existing ``workspace``

```R
  updated <- gsman$updateWorkspace("geosapi", "http://newgeoapi")
```

<a name="GSWorkspace-delete"/>

##### 4.3.4 Delete an existing ``workspace``

```R
  deleted <- gsman$deleteWorkspace("geosapi", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all layers published under the deleted workspace.

<a name="GSNamespace"/>

#### 4.4 Manipulate GeoServer ``namespaces``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/namespaces.html

<a name="GSNamespace-get"/>

##### 4.4.1 Get a (list of) ``namespace(s)``

```R
  #get namespace objects
  nslist <- gsman$getNamespaces()

  #get namespace names only
  nsnames <- gsman$getNamespaceNames()

  #get namespace by name
  ns <- gsman$getNamespace("topp")
```

<a name="GSNamespace-create"/>

##### 4.4.2 Create a new ``namespace``

```R
  created <- gsman$createNamespace("geosapi", "http://geoapi")
```

<a name="GSNamespace-update"/>

##### 4.4.3 Update an existing ``namespace``

```R
  updated <- gsman$updateNamespace("geosapi", "http://newgeoapi")
```

<a name="GSNamespace-delete"/>

##### 4.4.4 Delete an existing ``namespace``

```R
  deleted <- gsman$deleteNamespace("geosapi", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all layers published under the deleted namespace.

<a name="vectordata"/>

#### 4.5 Manage **vector data** with GeoServer

<a name="GSDataStore"/>

##### 4.5.1 Manipulate GeoServer ``dataStores``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/datastores.html

<a name="GSDataStore-get"/>

###### 4.5.1.1 Get a (list of) ``dataStore(s)``

```R
  #get datastore objects
  dslist <- gsman$getDataStores("topp")

  #get datastore names only
  dsnames <- gsman$getDataStoreNames("topp")

  #get datastore by name
  ns <- gsman$getDataStore("topp", "states_shapefile")
```

<a name="GSDataStore-create"/>

###### 4.5.1.2 Create a new ``dataStore``

<a name="GSShapefileDataStore-create"/>

* 4.5.1.2.1 Create a **Shapefile** ``dataStore``

```R
  ds = GSShapefileDataStore$new(dataStore="topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                url = "file:data/shapefiles/states.shp")
  created <- gsman$createDataStore("topp", ds)
```

<a name="GSShapefileDirectoryDataStore-create"/>

* 4.5.1.2.2 Create a **Shapefiles Directory** ``dataStore``

```R
  ds = GSShapefileDirectoryDataStore$new(dataStore="topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                url = "file:data/shapefiles")
  created <- gsman$createDataStore("topp", ds)
```

<a name="GSShapefileDirectoryDataStore-create"/>

* 4.5.1.2.3 Create a **GeoPackage** ``dataStore``

```R
  ds = GSGeoPackageDataStore$new(dataStore="topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                database = "file:data/somefile.gpkg")
  created <- gsman$createDataStore("topp", ds)
```

<a name="GSDataStore-update"/>

###### 4.5.1.3 Update an existing ``dataStore``

```R
  dataStore <- gsman$getDataStore("topp", "topp_datastore")
  dataStore$setDescription("topp_datastore updated description")
  dataStore$setEnabled(FALSE)
  
  updated <- gsman$updateDataStore("topp", dataStore)
```

<a name="GSDataStore-delete"/>

###### 4.5.1.4 Delete an existing ``dataStore``

```R
  deleted <- gsman$deleteDataStore("topp", "topp_datastore", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all layers published under the deleted datastore.

<a name="GSFeatureType-upload"/>

##### 4.5.2 Upload features/vector data

The `geosapi` offers methods to upload vector data to configured `datastores`. A generic method `gsman$uploadData` allows to upload any kind of data type supported by GeoServer, by specifying the file extension. To upload data, several parameters have to be specified including:
* `ws`: the workspace in which the datastore is configured
* `ds`: the datastore in which data has to be uploaded
* `endpoint`: choice among `file` (use a local file content as body), `url` (use a file URL), or `external` (path to a file). Important note: So far only `file` was tested and is supported.
* `extension`: the data file extension
* `configure`: either `none` (upload only, with no featuretype/layer configured by default), `first` (upload and configure featuretype/layer over the uploaded vector data)
* `update`: update strategy, either `append` (upload that will fails in case data was already uploaded), or `overwrite` (overwrite the already uploaded data if existing)
* `filename`: filename of the data resource to upload 
* `charset`: character set of the data resource
* `contentType: data content type

To simplify the upload, several data-specific methods were designed targeting specific data file extensions. These methods share the same parameters as above except the `extension` and `contentType` that will be pre-set according to the data upload format:
* `uploadShapefile`: upload a shapefile
* `uploadProperties`: upload a properties file
* `uploadH2`: upload a H2 database
* `uploadSpatialite`: upload a spatialite file
* `uploadAppSchema`: upload an app schema
* `uploadGeoPackage`: upload an OGC GeoPackage

Example with an OGC Geopackage upload

```r
  uploaded <- gsman$uploadGeoPackage(
	ws = "my_workspace", ds = "my_gpkg_datastore",
	endpoint = "file", configure = "first", update = "overwrite",
	charset = "UTF-8", filename = "myfile.gpkg"
  )
```

<a name="GSFeatureType"/>

##### 4.5.3 Manipulate GeoServer ``featureType``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/featuretypes.html

<a name="GSFeatureType-get"/>

###### 4.5.3.1 Get a (list of) ``featureType(s)``

```R
  #get featureType objects
  ftlist <- gsman$getFeatureTypes("topp", "taz_shapes")

  #get featureType names only
  ftnames <- gsman$getFeatureTypeNames()

  #get featureType by name
  ft <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities")
```

<a name="GSFeatureType-create"/>

###### 4.5.3.2 Create a new ``featureType``

```R
  #create featuretype object
  featureType <- GSFeatureType$new()
  featureType$setName("tasmania_cities2")
  featureType$setNativeName("tasmania_cities")
  featureType$setAbstract("abstract")
  featureType$setTitle("title")
  featureType$setSrs("EPSG:4326")
  featureType$setNativeCRS("EPSG:4326")
  featureType$setEnabled(TRUE)
  featureType$setProjectionPolicy("REPROJECT_TO_DECLARED")
  featureType$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  featureType$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326") 
  
  md1 <- GSMetadataLink$new(type = "text/xml", metadataType = "ISO19115:2003", content = "http://somelink.org/xml")
  featureType$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(type = "text/html", metadataType = "ISO19115:2003", content = "http://somelink.org/html")
  featureType$addMetadataLink(md2)

  #create the feature type in GeoServer
  created <- gsman$createFeatureType("topp", "taz_shapes", featureType)
```

**How to set a Geoserver SQL View**

With ``geosapi``, it is also possible to configure a Geoserver SQL View, possibly with dynamic parameters. For this, GeoServer uses the concept of _VirtualTable_. The code below explains how to configure such virtual table with ``geosapi``:

```R

##virtual table
vt <- GSVirtualTable$new()
vt$setName("view name")
vt$setSql("select * from mydataset where flag = '%flag%' and year between %start_year% and %end_year%")

## incase of spatial virtual table, specify the geometry column name, type and SRID
vtg <- GSVirtualTableGeometry$new(name = "the_geom", type = "MultiPolygon", srid = 4326)
vt$setGeometry(vtg)

##in case of dynamic parameters handled in your sql (bound with %)
vtp1 <- GSVirtualTableParameter$new(name = "flag", defaultValue = "FRA", regexpValidator = "^[\\w\\d\\s]+$")
vtp2 <- GSVirtualTableParameter$new(name = "start_year", defaultValue = "2000",regexpValidator = "^[\\d]+$")
vtp3 <- GSVirtualTableParameter$new(name = "end_year", defaultValue = "2010",regexpValidator = "^[\\d]+$")
vt$addParameter(vtp1)
vt$addParameter(vtp2)
vt$addParameter(vtp3)

##add virtual table to featureType
featureType$setVirtualTable(vt)
```

<a name="GSFeatureType-update"/>

###### 4.5.3.3 Update an existing ``featureType``

```R
  #fetch featureType and update some properties
  featureType <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities2")
  featureType$setAbstract("abstract updated")
  featureType$setEnabled(FALSE)
  
  #update featureType in GeoServer
  updated <- gsman$updateFeatureType("topp", "taz_shapes", featureType)
```

<a name="GSFeatureType-delete"/>

###### 4.5.3.4 Delete an existing ``featureType``

```R
  deleted <- gsman$deleteFeatureType("topp", "taz_shapes", "tasmania_cities2", recurse = TRUE)
```

The ``recurse`` parameter allows to delete the layer published for the deleted featureType.

<a name="GSLayer"/>

##### 4.5.4 Manipulate GeoServer ``layers``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/layers.html

<a name="GSLayer-get"/>

###### 4.5.4.1 Get a (list of) ``layer(s)``

```R
  #get layer objects
  layers <- gsman$getLayers()

  #get layer names only
  layerNames <- gsman$getLayerNames()

  #get layer by name
  layer <- gsman$getLayer("tasmania_cities")
```

<a name="GSLayer-create"/>

###### 4.5.4.2 Create a new ``layer``

This method shows how to _publish_ a created ``featureType`` (as created in [4.5.3 Create a new ``FeatureType``](https://github.com/eblondel/geosapi/wiki#GSFeatureType-create))

```R
  layer <- GSLayer$new()
  layer$setName("tasmania_cities2")
  layer$setDefaultStyle("capitals")
  layer$addStyle("generic")
  created <- gsman$createLayer(layer)
```

<a name="GSLayer-update"/>

###### 4.5.4.3 Update an existing ``layer``

```R
  lyr <- gsman$getLayer("tasmania_cities")
  lyr$setDefaultStyle("generic")
  updated <- gsman$updateLayer(lyr)
```

<a name="GSLayer-delete"/>

###### 4.5.4.4 Delete an existing ``layer``

```R
  deleted <- gsman$deleteLayer("tasmania_cities2")
```

<a name="publishFeatureLayer"/>

##### 4.5.5 Publication of GeoServer feature layer

GeoServer manages feature data layers by means of two different types of resources: the ``featureType`` and the ``layer``. When created, the latter will _publish_ the former. Hence, in order to **publish** a complete feature data layer, two GeoServer resources should be created. The below methods allow the user to do both creation through a **single** method. To distinguish them from the above API methods and avoid confusion, we don't use the terminology **create** / **delete** but **publish** / **unpublish**.


<a name="publishFeatureLayer-publish"/>

###### 4.5.5.1 Publish Geoserver feature layer

The method ``publishLayer`` will sequentially do two requests to GeoServer:

* Create ``featureType``
* Create ``layer``

```R
  #create featuretype
  featureType <- GSFeatureType$new()
  featureType$setName("tasmania_cities2")
  featureType$setNativeName("tasmania_cities")
  featureType$setAbstract("abstract")
  featureType$setTitle("title")
  featureType$setSrs("EPSG:4326")
  featureType$setNativeCRS("EPSG:4326")
  featureType$setEnabled(TRUE)
  featureType$setProjectionPolicy("REPROJECT_TO_DECLARED")
  featureType$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  featureType$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326") 
  md1 <- GSMetadataLink$new(type = "text/xml", metadataType = "ISO19115:2003", content = "http://somelink.org/xml")
  featureType$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(type = "text/html", metadataType = "ISO19115:2003", content = "http://somelink.org/html")
  featureType$addMetadataLink(md2)
  
  #create layer
  layer <- GSLayer$new()
  layer$setName("tasmania_cities2")
  layer$setDefaultStyle("capitals")
  layer$addStyle("generic")
  
  #try to publish the complete layer (featuretype + layer)
  published <- gsman$publishLayer("topp", "taz_shapes", featureType, layer)
```

<a name="publishFeatureLayer-unpublish"/>

###### 4.5.5.2 Unpublish Geoserver feature layer

The method ``publishLayer`` will sequentially do two requests to GeoServer:

* Delete ``layer``
* Delete ``featureType`` 

```R
  #try to unpublish the complete layer (featuretype + layer)
  published <- gsman$unpublishLayer("topp", "taz_shapes", "tasmania_cities2")
```

<a name="GSLayerGroup"/>

##### 4.5.6 Manipulate GeoServer ``layer groups``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/layergroups.html

<a name="GSLayerGroup-get"/>

###### 4.5.6.1 Get a (list of) ``layer group(s)``

```R
  #get layer groups objects
  layerGroups <- gsman$getLayerGroups()

  #get layer group names only
  layerGroupNames <- gsman$getLayerGroupNames()

  #get layer group by name
  layerGroup <- gsman$getLayerGroup("test_layergroup")
```

<a name="GSLayerGroup-create"/>

###### 4.5.6.2 Create a new ``layer group``

```R
  lyr <- GSLayerGroup$new()
  lyr$setName("test_layergroup")
  lyr$setTitle("title")
  lyr$setAbstract("abstract")
  lyr$setMode("SINGLE")
  lyr$addLayer(layer = "tasmania_cities", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  gsman$createLayerGroup(layerGroup = lyr)
```

<a name="GSLayerGroup-update"/>

###### 4.5.6.3 Update an existing ``layer group``

```R
  lyr <- GSLayerGroup$new()
  lyr$setName("test_layergroup")
  lyr$setTitle("title")
  lyr$setAbstract("abstract 2")
  lyr$setMode("SINGLE")
  lyr$addLayer(layer = "tasmania_cities", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  gsman$updateLayerGroup(layerGroup = lyr)
```

<a name="GSLayerGroup-delete"/>

###### 4.5.6.4 Delete an existing ``layer group``

```R
  deleted <- gsman$deleteLayerGroup("test_layergroup")
```

<a name="rasterdata"/>

#### 4.6. Manage **raster data** with GeoServer

<a name="GSCoverageStore"/>

##### 4.6.1 Manipulate GeoServer ``coveragestores``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/coveragestores.html

<a name="GSCoverageStore-get"/>

###### 4.6.1.1 Get a (list of) ``coveragestore(s)``

```R
  #get coveragestore objects
  cslist <- gsman$getCoverageStores("nurc")

  #get coveragestore names only
  csnames <- gsman$getCoverageStoreNames("nurc")

  #get coveragestore by name
  cs <- gsman$getCoverageStore("nurc", "mosaic")
```

<a name="GSCoverageStore-create"/>

###### 4.6.1.2 Create a new ``coveragetore``

<a name="GSGeoTIFFCoverageStore-create"/>

* 4.6.1.2.1 Create a **GeoTIFF** ``coveragestore``

```R
  cs = GSGeoTIFFCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```

<a name="GSWorldImageCoverageStore-create"/>

* 4.6.1.2.2 Create a **WorldImage** ``coveragestore``

```R
  cs = GSWorldImageCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```

<a name="GSArcGridCoverageStore-create"/>

* 4.6.1.2.3 Create a **ArcGrid** ``coveragestore``

```R
  cs = GSArcGridCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```

<a name="GSImageMosaicCoverageStore-create"/>

* 4.6.1.2.4 Create a **ImageMosaic** ``coveragestore``

```R
  cs = GSImageMosaicCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```


<a name="GSCoverageStore-update"/>

###### 4.6.1.3 Update an existing ``coveragestore``

```R
  cs <- gsman$getCoverageStore("sf", "sfdem")
  cs$setDescription("sfdem updated description")
  cs$setEnabled(FALSE)
  
  updated <- gsman$updateCoverageStore("sf", cs)
```

<a name="GSCoverageStore-delete"/>

###### 4.6.1.4 Delete an existing ``coveragestore``

```R
  deleted <- gsman$deleteCoverageStore("sf", "sfdem", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all coverages published under the deleted coveragestore.

<a name="GSCoverage-upload"/>

##### 4.6.2 Upload coverage/raster data

The `geosapi` offers methods to upload coverage data to configured `coveragestores`. A generic method `gsman$uploadCoverage` allows to upload any kind of coverage data type supported by GeoServer, by specifying the file extension. To upload data, several parameters have to be specified including:
* `ws`: the workspace in which the datastore is configured
* `ds`: the datastore in which data has to be uploaded
* `endpoint`: choice among `file` (use a local file content as body), `url` (use a file URL), or `external` (path to a file). Important note: So far only `file` was tested and is supported.
* `extension`: the data file extension
* `configure`: either `none` (upload only, with no coverage/layer configured by default), `first` (upload and configure coverae/layer over the uploaded data)
* `update`: update strategy, either `append` (upload that will fails in case data was already uploaded), or `overwrite` (overwrite the already uploaded data if existing)
* `filename`: filename of the data resource to upload 
* `charset`: character set of the data resource
* `contentType: data content type

To simplify the upload, several coverage-specific methods were designed targeting specific data file extensions. These methods share the same parameters as above except the `extension` and `contentType` that will be pre-set according to the data upload format:
* `uploadGeoTIFF`: upload a GeoTIFF file
* `uploadWorldImage`: upload a WorldImage file
* `uploadArcGrid`: upload an ArcGrid file
* `uploadImageMosaic`: upload an ImageMosaic

Example with a GeoTIFF upload

```r
  uploaded <- gsman$uploadGeoTIFF(
	ws = "sf", cs = "sfdem_new",
    endpoint = "file", configure = "none", update = "overwrite",
    filename = system.file("extdata/sfdem_new.tif", package = "geosapi")
  )
```

<a name="GSCoverage"/>

##### 4.6.3 Manipulate GeoServer ``coverages``

>**GeoServer API doc**: https://docs.geoserver.org/stable/en/user/rest/api/coverages.html

<a name="GSCoverage-get"/>

###### 4.6.3.1 Get a (list of) ``coverage(s)``

```R
  #get coverage objects
  covlist <- gsman$getCoverages("sf", "sfdem")

  #get coverage names only
  covnames <- gsman$getCoverageNames("sf", "sfdem")

  #get coverage by name
  cov <- gsman$getCoverage("sf", "sfdem", "sfdem")
```

<a name="GSCoverage-create"/>

###### 4.6.3.2 Create a new ``coverage``

```R
  #create coverage object
  cov <- GSCoverage$new()
  cov$setName("sfdem_new")
  cov$setNativeName("sfdem_new")
  cov$setTitle("Title for sfdem")
  cov$setDescription("Description for sfdem")
  cov$addKeyword("sfdem keyword1")
  cov$addKeyword("sfdem keyword2")
  cov$addKeyword("sfdem keyword3")
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/xml"
  )
  cov$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/html"
  )
  cov$addMetadataLink(md2)
  
  cov$setSrs("EPSG:4326")
  cov$setNativeCRS("EPSG:26713")
  cov$setLatLonBoundingBox(-103.87108701853181, 44.370187074132616, -103.62940739432703, 44.5016011535299, crs = "EPSG:4326")
  cov$setNativeBoundingBox(589980, 4913700, 609000, 4928010, crs = "EPSG:26713")
  
  created <- gsman$createCoverage(ws = "sf", cs = "sfdem_new", coverage = cov)
```

**How to set a Geoserver Coverage View**

With ``geosapi``, it is also possible to configure a Geoserver Coverage View. For this, GeoServer uses the concept of _CoverageView_. The code below explains how to configure such coverage view with ``geosapi``:

```R

  ##coverage view
  coview <- GSCoverageView$new()
  coview$setName("sfdem_new")
  coview$setEnvelopeCompositionType("INTERSECTION")
  coview$setSelectedResolution("BEST")
  coview$setSelectedResolutionIndex(-1)
  coviewband <- GSCoverageBand$new()
  coviewband$setDefinition("sfdem_new@0")
  coviewband$setIndex(0)
  coviewband$addInputBand(GSInputCoverageBand$new( coverageName = "sfdem_new", band = 0))
  coview$addBand(coviewband)

  ##add coverage view to coverage
  cov$setView(coview)
```

<a name="GSCoverage-update"/>

###### 4.6.3.3 Update an existing ``coverage``

```R
  #fetch coverage and update some properties
  cov <- gsman$getCoverage("sf", "sfdem", "sfdem")
  cov$setAbstract("abstract updated")
  cov$setEnabled(FALSE)
  
  #update coverage in GeoServer
  updated <- gsman$updateCoverage("sf", "sfdem", cov)
```

<a name="GSCoverage-delete"/>

###### 4.6.3.4 Delete an existing ``coverage``

```R
  deleted <- gsman$deleteCoverage("sf", "sfdem", "sfdem", recurse = TRUE)
```

The ``recurse`` parameter allows to delete the layer published for the deleted featureType.

<a name="styles"/>

#### 4.7. Manage **styles** with GeoServer

<a name="GSStyle-get"/>

##### 4.7.1 Get a (list of) ``style``(s)

```R
  styles <- gsman$getStyles()
  stylenames <- gsman$getStyleNames()
  style <- gsman$getStyle("capitals")
```

From GeoServer 2.2, it is also possible to get the content of a style, ie the SLD body:

```R
  sld <- gsman$getSLDBody("capitals")
```

<a name="GSStyle-create"/>

##### 4.7.2 Create a new ``style``

```R
  sldFile <- system.file("extdata", "mystyle.sld", package = "geosapi") #to use 'file' argument of gsman$createStyle
  sldStyle <- xmlParse(sldFile) #to use 'sldBody' argument of gsman$createStyle
  created <- gsman$createStyle(file = sldFile, name = "mystyle")
```

<a name="GSStyle-update"/>

##### 4.7.3 Update a new ``style``

```R
  sldFile2 <- system.file("extdata", "mystyle2.sld", package = "geosapi") #to use 'file' argument of gsman$updateStyle
  sldStyle2 <- xmlParse(sldFile2) #to use 'sldBody' argument of gsman$updateStyle
  updated <- gsman$updateStyle(sldBody = sldStyle2, name = "mystyle")
```

<a name="GSStyle-delete"/>

##### 4.7.4 Delete an existing ``style``

```R
  deleted <- gsman$deleteStyle("mystyle", recurse = TRUE, purge = TRUE)
```

All methods describe here above also support a ``ws`` argument where a workspace can be specified.

<a name="settings_workspaces"/>

#### 4.8. Manage **workspace settings** with GeoServer

Geoserver allows to activate settings by workspace. In such case, this will overwrite the global Geoserver settings. Activating these settings is required to set-up specific OGC service capabilities par workspace (see next section).

<a name="GSWorkspaceSettings-create"/>

##### 4.8.1 Create a workspace settings

Creating a workspace settings is equivalent to check the "Enabled" checkbox in the _Settings_ section of a workspace edition page in the GeoServer UI. To create it (assuming a workspace named ``geosapi``), run the following code:

```R
settings <- GSWorkspaceSettings$new()
settings$setNumDecimals(5) #custom number of decimals for data published in this workspace
created <- gsman$createWorkspaceSettings("geosapi", settings)
```

<a name="GSWorkspaceSettings-get"/>

##### 4.8.2 Get a workspace settings

To get a workspace settings, use the following code:

```R
geosapi_settings <- gsman$getWorkspaceSettings("geosapi")
```

This method can be used in view of updating a workspace settings.

<a name="GSWorkspaceSettings-update"/>

##### 4.8.3 Update a workspace settings

Similar method can be used for updating the workspace configuration:

```R
settings <- gsman$getWorkspaceSettings("geosapi")
settings$setNumDecimals(9) #change number of decimals
created <- gsman$updateWorkspaceSettings("geosapi", settings)
```

<a name="GSWorkspaceSettings-delete"/>

##### 4.8.4 Delete a workspace settings

The settings can be deleted (equivalent of unchecking the "Enabled" checkbox in a workspace settings section):

```R
deleted <- gsman$deleteWorkspaceSettings("geosapi")
```

<a name="settings_services"/>

#### 4.9 Manage OGC Service (WMS/WFS/WCS) settings with GeoServer

<a name="GSServiceSettings-get"/>

##### 4.9.1 Get an OGC service (WMS/WFS/WCS) settings

To get an OGC service settings (global, or workspace specific), geosapi offers a generic method that can be used as follows:

```R
#get WMS global settings
wms_global_settings <- gsman$getServiceSettings(service = "WMS")

#get WMS geosapi settings
wms_geosapi_settings <- gsman$getServiceSettings(service = "WMS", ws = "geosapi")
```
Shortcut methods can also be used as follow:

```R
#get WMS global settings
wms_global_settings <- gsman$getWmsSettings()

#get WMS geosapi settings
wms_geosapi_settings <- gsman$getWmsSettings(ws = "geosapi")
```

Analog methods are available for WFS and WCS.


<a name="GSServiceSettings-update"/>

##### 4.9.2 Update an OGC service (WMS/WFS/WCS) settings

OGC service settings can be updated, for example to activate/deactivate the service and fill minimum metadata (name, title, abstract). The following examples shows how to do update WMS global configuration and by workspace:

* Update the WMS global configuration:

```R
wmsSettings <- gsman$getWmsSettings()
wmsSettings$setTitle("Tile for my global WMS")
updated <- gsman$updateServiceSettings(wmsSettings, service = "WMS")
```

Shortcut update methods are also available. The last code line above can be replaced by:

```R
updated <- gsman$updateWmsSettings(wmsSettings)
```
* Update  WMS workspace configuration

```R
wmsSettings <- GSServiceSettings$new(service = "WMS")
wmsSettings$setTitle("Tile for my workspace WMS")
wmsSettings$setAbstract("This WMS is set-up specifically for the workspace geosapi")
updated <- gsman$updateServiceSettings(wmsSettings, service = "WMS", ws = "geosapi")
```

The last code line above could be replaced by the shortcut method:

``
updated <- gsman$updateWmsSettings(wmsSettings, ws = "geosapi")
``

<a name="GSServiceSettings-enable"/>

##### 4.9.3 Enable/Disable an OGC service (WMS/WFS/WCS)

To make OGC service enabling/disabling easier for users, additional shortcut methods were set-up in geosapi. The following code examples show you how to enable/disable WMS/WFS/WCS

* Enable/Disable Global OGC services

```
#disable global WMS
gsman$disableWMS()
#enable global WFS and WCS
gsman$enableWFS()
gsman$enableWCS()
```

* Enable/Disable OGC services at workspace level

```R
#activates WMS for workspace 'geosapi'
enabled <- gsman$enableWMS("geosapi")

#deactivates WMS for workspace 'geosapi'
disabled <- gsman$disableWMS("geosapi")
```

Analog functions are available for WFS and WCS:

```
#enable WFS and WCS for geosapi workspace
gsman$enableWFS("geosapi")
gsman$enableWCS("geosapi")
```

```
#disable WFS and WCS for geosapi workspace
gsman$disableWFS("geosapi")
gsman$disableWCS("geosapi")
```

<a name="package_issues"/>

### 5. Issue reporting
***

Issues can be reported at https://github.com/eblondel/geosapi/issues

---
title: geosapi User Manual
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{geosapi User Manual}
  %\VignetteEngine{knitr::knitr}
  \usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```


# geosapi - R Interface to GeoServer REST API


R Interface to GeoServer REST API – Allows to perform programmatically from R

* CRUD operations (Create, Read, Update, Delete)  on GeoServer resources including ``workspace``, ``namespaces``, ``dataStores`` ``featureTypes``, ``layers``, ``styles``, ``settings`` (workspaces, OGC services) etc. 
* Data upload to Geoserver data directory (e.g. upload shapefile)

***

If you wish to sponsor geosapi, do not hesitate to [contact me](mailto:eblondel.pro@gmail.com)

Many thanks to the following organizations that have provided fundings for strenghtening the ``geosapi`` package:

<a href="https://unepgrid.ch/en"><img height=100 width=200 src="https://www.weadapt.org/sites/weadapt.org/files/styles/large/public/screenshot_2021-05-25_at_16.19.32_0.png?itok=iiEMf_S3"></a><a href="https://www.fao.org/home/en/"><img height=100 width=200 src="https://www.fao.org/fileadmin/templates/family-farming-decade/images/FAO-IFAD-Logos/FAO-Logo-EN.svg"></a>


***

**Table of contents**

[**1. Overview**](#package_overview)<br/>
[**2. Package status**](#package_status)<br/>
[**3. Credits**](#package_credits)<br/>
[**4. User guide**](#user_guide)<br/>
&nbsp;&nbsp;&nbsp;[4.1 Installation](#install_guide)<br/>
&nbsp;&nbsp;&nbsp;[4.2 Connect to GeoServer REST API](#GSManager)<br/>
&nbsp;&nbsp;&nbsp;[**4.3 GeoServer ``workspaces``**](#GSWorkspace)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.1 Get a (list of) ``workspace(s)``](#GSWorkspace-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.2 Create a new ``workspace``](#GSWorkspace-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.3 Update a new ``workspace``](#GSWorkspace-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.3.4 Delete a new ``workspace``](#GSWorkspace-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.4 GeoServer ``namespaces``**](#GSNamespace)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.1 Get a (list of) ``namespace(s)``](#GSNamespace-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.2 Create a new ``namespace``](#GSNamespace-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.3 Update a new ``namespace``](#GSNamespace-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.4.4 Delete a new ``namespace``](#GSNamespace-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.5 Manage vector data in GeoServer**](#vectordata)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.1 GeoServer ``dataStores``**](#GSDataStore)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.1 Get a (list of) ``dataStore(s)``](#GSDataStore-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2 Create a new ``dataStore``](#GSDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2.1 Create a **Shapefile** ``dataStore``](#GSShapefileDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2.2 Create a **Shapefiles Directory** ``dataStore``](#GSShapefileDirectoryDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.2.3 Create a **GeoPackage** ``dataStore``](#GSGeoPackageDataStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.3 Update a new ``dataStore``](#GSDataStore-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.1.4 Delete a new ``dataStore``](#GSDataStore-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.2 Upload features/vector data**](#GSFeatureType-upload)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.3 GeoServer ``featureTypes``**](#GSFeatureType)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.1 Get a (list of) ``featureType(s)``](#GSFeatureType-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.2 Create a new ``featureType``](#GSFeatureType-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.3 Update a new ``featureType``](#GSFeatureType-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.3.4 Delete a new ``featureType``](#GSFeatureType-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.3 GeoServer ``layers``**](#GSLayer)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.1 Get a (list of) ``layer(s)``](#GSLayer-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.2 Create a new ``layer``](#GSLayer-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.3 Update a new ``layer``](#GSLayer-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.4.4 Delete a new ``layer``](#GSLayer-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.5 Publication of GeoServer feature layer**](#publishFeatureLayer)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.5.1 Publish Geoserver feature layer](#publishFeatureLayer-publish)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.5.2 Unpublish Geoserver feature layer](#publishFeatureLayer-unpublish)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.5.6 GeoServer ``layer groups``**](#GSLayerGroup)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.1 Get a (list of) ``layer group(s)``](#GSLayerGroup-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.2 Create a new ``layer group``](#GSLayerGroup-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.3 Update a new ``layer group``](#GSLayerGroup-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.5.6.4 Delete a new ``layer group``](#GSLayerGroup-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.6 Manage raster data in GeoServer**](#rasterdata)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.6.1 GeoServer ``coveragestores``**](#GSCoverageStore)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.1 Get a (list of) ``coveragestore(s)``](#GSCoverageStore-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2 Create a new ``coveragestore``](#GSCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.1 Create a **GeoTIFF** ``coveragestore``](#GSGeoTIFFCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.2 Create a **WorldImage** ``coveragestore``](#GSWorldImageCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.3 Create a **ArcGrid** ``coveragestore``](#GSArcGridCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.2.4 Create a **ImageMosaic** ``coveragestore``](#GSImageMosaicCoverageStore-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.3 Update a new ``coveragestore``](#GSCoverageStore-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.1.4 Delete a new ``coveragetore``](#GSCoverageStore-delete)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.6.2 Upload coverage/raster data**](#GSCoverage-upload)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[**4.6.3 GeoServer ``coverages``**](#GSCoverage)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.1 Get a (list of) ``coverage(s)``](#GSCoverage-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.2 Create a new ``coverage``](#GSCoverage-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.3 Update a new ``coverage``](#GSCoverage-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.6.3.4 Delete a new ``coverage``](#GSCoverage-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.7 Manage styles in GeoServer**](#styles)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.1 Get a (list of) ``style(s)``](#GSStyle-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.2 Create a new ``style``](#GSStyle-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.3 Update a new ``style``](#GSStyle-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.7.4 Delete a new ``style``](#GSStyle-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.8 Manage workspace settings in GeoServer**](#settings_workspaces)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.1 Create a workspace settings](#GSWorkspaceSettings-create)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.2 Get a workspace settings](#GSWorkspaceSettings-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.3 Update a workspace settings](#GSWorkspaceSettings-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.8.4 Delete a workspace settings](#GSWorkspaceSettings-delete)<br/>
&nbsp;&nbsp;&nbsp;[**4.9 Manage OGC service (WMS/WFS/WCS) settings in GeoServer**](#settings_services)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.9.1 Get an OGC service (WMS/WFS/WCS) settings](#GSServiceSettings-get)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.9.2 Update an OGC service (WMS/WFS/WCS) settings](#GSServiceSettings-update)<br/>
&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;[4.9.3 Enable/Disable an OGC service (WMS/WFS/WCS)](#GSServiceSettings-enable)<br/>
[**5. Issue reporting**](#package_issues)<br/>

<a name="package_overview"/>

### 1. Overview and vision
***

Until now, equivalent tools were existing for other programming languages (e.g. Java, Python) but not in R. [geosapi](https://github.com/eblondel/geosapi) intends to provide R native interface to the GeoServer REST API, in order to facilitate publication of geographic data resources from R to [GeoServer](https://geoserver.org/).

<a name="package_status"/>

### 2. Development status
***

Published on CRAN

<a name="package_credits"/>

### 3. Credits
***

(c) 2016, Emmanuel Blondel

Package distributed under MIT license.

If you use ``geosapi``, i would be very grateful if you can add a citation in your published work. By citing ``geosapi``, beyond acknowledging the work, you contribute to make it more visible and guarantee its growing and sustainability. You can get the preferred citation by running ```citation("geosapi)``` in R.

You can reference ``geosapi`` through its DOI: [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.1184895.svg)](https://doi.org/10.5281/zenodo.1184895)

<a name="user_guide"/>

### 4. User guide
***

<a name="install_guide"/>

#### 4.1 How to install geosapi in R

For now, the package can be installed from Github
```R
install.packages("remotes")
```
Once the devtools package loaded, you can use the install_github to install ``geosapi. By default, package will be installed from ``master`` which is the current version in development (likely to be unstable). 

```R
remotes::install_github("eblondel/geosapi")
```

<a name="GSManager"/>

#### 4.2 Connect to GeoServer REST API 

The main entry point of ``geosapi`` is the ``GSManager``. To configure it, enter the following line, specifying the base ``URL`` of your GeoServer, and your credentials:
```R
gsman <- GSManager$new(
    url = "http://localhost:8080/geoserver", #baseUrl of the Geoserver
    user = "admin", pwd = "geoserver", #credentials
    logger = NULL #logger, for info or debugging purpose
)
```

By default, the ``geosapi`` **logger** is deactivated. To enable the logger, specify the level of log you wish as parameter of the above R code. Two logging levels are available:

* ``INFO``: will print the ``geosapi`` logs. Three types of messages can be distinguished: ``INFO``, ``WARN``, ``ERROR``. The latter is generally associated with a ``stop`` and indicate an blocking error for the R method executed.
* ``DEBUG`` will print the above ``geosapi`` logs, and report all logs from HTTP requests performed with ``cURL``

The ``GSManager`` inherits all methods of resource dependent managers, to provide the users with a single R interface to GeoServer REST API operations. In addition, the ``GSManager`` allows accessing the different resource managers through specific methods. The following ``managers`` are available:
* ``GSNamespaceManager``: Manage ``namespaces``
* ``GSWorkspaceManager``: Manage ``workspaces``
* ``GSDataStoreManager``: Manage ``dataStores``

<a name="GSWorkspace"/>

#### 4.3 Manipulate GeoServer ``workspaces``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/workspaces/

<a name="GSWorkspace-get"/>

##### 4.3.1 Get a (list of) ``workspace(s)``

```R
  #get workspace objects
  wslist <- gsman$getWorkspaces()

  #get workspace names only
  wsnames <- gsman$getWorkspaceNames()

  #get workspace by name
  ws <- gsman$getWorkspace("topp")
```

<a name="GSWorkspace-create"/>

##### 4.3.2 Create a new ``workspace``

```R
  created <- gsman$createWorkspace("geosapi", "http://geoapi")
```

<a name="GSWorkspace-update"/>

##### 4.3.3 Update an existing ``workspace``

```R
  updated <- gsman$updateWorkspace("geosapi", "http://newgeoapi")
```

<a name="GSWorkspace-delete"/>

##### 4.3.4 Delete an existing ``workspace``

```R
  deleted <- gsman$deleteWorkspace("geosapi", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all layers published under the deleted workspace.

<a name="GSNamespace"/>

#### 4.4 Manipulate GeoServer ``namespaces``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/namespaces/

<a name="GSNamespace-get"/>

##### 4.4.1 Get a (list of) ``namespace(s)``

```R
  #get namespace objects
  nslist <- gsman$getNamespaces()

  #get namespace names only
  nsnames <- gsman$getNamespaceNames()

  #get namespace by name
  ns <- gsman$getNamespace("topp")
```

<a name="GSNamespace-create"/>

##### 4.4.2 Create a new ``namespace``

```R
  created <- gsman$createNamespace("geosapi", "http://geoapi")
```

<a name="GSNamespace-update"/>

##### 4.4.3 Update an existing ``namespace``

```R
  updated <- gsman$updateNamespace("geosapi", "http://newgeoapi")
```

<a name="GSNamespace-delete"/>

##### 4.4.4 Delete an existing ``namespace``

```R
  deleted <- gsman$deleteNamespace("geosapi", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all layers published under the deleted namespace.

<a name="vectordata"/>

#### 4.5 Manage **vector data** with GeoServer

<a name="GSDataStore"/>

##### 4.5.1 Manipulate GeoServer ``dataStores``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/datastores/

<a name="GSDataStore-get"/>

###### 4.5.1.1 Get a (list of) ``dataStore(s)``

```R
  #get datastore objects
  dslist <- gsman$getDataStores("topp")

  #get datastore names only
  dsnames <- gsman$getDataStoreNames("topp")

  #get datastore by name
  ns <- gsman$getDataStore("topp", "states_shapefile")
```

<a name="GSDataStore-create"/>

###### 4.5.1.2 Create a new ``dataStore``

<a name="GSShapefileDataStore-create"/>

* 4.5.1.2.1 Create a **Shapefile** ``dataStore``

```R
  ds = GSShapefileDataStore$new(dataStore="topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                url = "file:data/shapefiles/states.shp")
  created <- gsman$createDataStore("topp", ds)
```

<a name="GSShapefileDirectoryDataStore-create"/>

* 4.5.1.2.2 Create a **Shapefiles Directory** ``dataStore``

```R
  ds = GSShapefileDirectoryDataStore$new(dataStore="topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                url = "file:data/shapefiles")
  created <- gsman$createDataStore("topp", ds)
```

<a name="GSShapefileDirectoryDataStore-create"/>

* 4.5.1.2.3 Create a **GeoPackage** ``dataStore``

```R
  ds = GSGeoPackageDataStore$new(dataStore="topp_datastore",
                                description = "topp_datastore description",
                                enabled = TRUE,
                                database = "file:data/somefile.gpkg")
  created <- gsman$createDataStore("topp", ds)
```

<a name="GSDataStore-update"/>

###### 4.5.1.3 Update an existing ``dataStore``

```R
  dataStore <- gsman$getDataStore("topp", "topp_datastore")
  dataStore$setDescription("topp_datastore updated description")
  dataStore$setEnabled(FALSE)
  
  updated <- gsman$updateDataStore("topp", dataStore)
```

<a name="GSDataStore-delete"/>

###### 4.5.1.4 Delete an existing ``dataStore``

```R
  deleted <- gsman$deleteDataStore("topp", "topp_datastore", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all layers published under the deleted datastore.

<a name="GSFeatureType-upload"/>

##### 4.5.2 Upload features/vector data

The `geosapi` offers methods to upload vector data to configured `datastores`. A generic method `gsman$uploadData` allows to upload any kind of data type supported by GeoServer, by specifying the file extension. To upload data, several parameters have to be specified including:
* `ws`: the workspace in which the datastore is configured
* `ds`: the datastore in which data has to be uploaded
* `endpoint`: choice among `file` (use a local file content as body), `url` (use a file URL), or `external` (path to a file). Important note: So far only `file` was tested and is supported.
* `extension`: the data file extension
* `configure`: either `none` (upload only, with no featuretype/layer configured by default), `first` (upload and configure featuretype/layer over the uploaded vector data)
* `update`: update strategy, either `append` (upload that will fails in case data was already uploaded), or `overwrite` (overwrite the already uploaded data if existing)
* `filename`: filename of the data resource to upload 
* `charset`: character set of the data resource
* `contentType: data content type

To simplify the upload, several data-specific methods were designed targeting specific data file extensions. These methods share the same parameters as above except the `extension` and `contentType` that will be pre-set according to the data upload format:
* `uploadShapefile`: upload a shapefile
* `uploadProperties`: upload a properties file
* `uploadH2`: upload a H2 database
* `uploadSpatialite`: upload a spatialite file
* `uploadAppSchema`: upload an app schema
* `uploadGeoPackage`: upload an OGC GeoPackage

Example with an OGC Geopackage upload

```r
  uploaded <- gsman$uploadGeoPackage(
	ws = "my_workspace", ds = "my_gpkg_datastore",
	endpoint = "file", configure = "first", update = "overwrite",
	charset = "UTF-8", filename = "myfile.gpkg"
  )
```

<a name="GSFeatureType"/>

##### 4.5.3 Manipulate GeoServer ``featureType``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/featuretypes/

<a name="GSFeatureType-get"/>

###### 4.5.3.1 Get a (list of) ``featureType(s)``

```R
  #get featureType objects
  ftlist <- gsman$getFeatureTypes("topp", "taz_shapes")

  #get featureType names only
  ftnames <- gsman$getFeatureTypeNames()

  #get featureType by name
  ft <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities")
```

<a name="GSFeatureType-create"/>

###### 4.5.3.2 Create a new ``featureType``

```R
  #create featuretype object
  featureType <- GSFeatureType$new()
  featureType$setName("tasmania_cities2")
  featureType$setNativeName("tasmania_cities")
  featureType$setAbstract("abstract")
  featureType$setTitle("title")
  featureType$setSrs("EPSG:4326")
  featureType$setNativeCRS("EPSG:4326")
  featureType$setEnabled(TRUE)
  featureType$setProjectionPolicy("REPROJECT_TO_DECLARED")
  featureType$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  featureType$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326") 
  
  md1 <- GSMetadataLink$new(type = "text/xml", metadataType = "ISO19115:2003", content = "http://somelink.org/xml")
  featureType$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(type = "text/html", metadataType = "ISO19115:2003", content = "http://somelink.org/html")
  featureType$addMetadataLink(md2)

  #create the feature type in GeoServer
  created <- gsman$createFeatureType("topp", "taz_shapes", featureType)
```

**How to set a Geoserver SQL View**

With ``geosapi``, it is also possible to configure a Geoserver SQL View, possibly with dynamic parameters. For this, GeoServer uses the concept of _VirtualTable_. The code below explains how to configure such virtual table with ``geosapi``:

```R

##virtual table
vt <- GSVirtualTable$new()
vt$setName("view name")
vt$setSql("select * from mydataset where flag = '%flag%' and year between %start_year% and %end_year%")

## incase of spatial virtual table, specify the geometry column name, type and SRID
vtg <- GSVirtualTableGeometry$new(name = "the_geom", type = "MultiPolygon", srid = 4326)
vt$setGeometry(vtg)

##in case of dynamic parameters handled in your sql (bound with %)
vtp1 <- GSVirtualTableParameter$new(name = "flag", defaultValue = "FRA", regexpValidator = "^[\\w\\d\\s]+$")
vtp2 <- GSVirtualTableParameter$new(name = "start_year", defaultValue = "2000",regexpValidator = "^[\\d]+$")
vtp3 <- GSVirtualTableParameter$new(name = "end_year", defaultValue = "2010",regexpValidator = "^[\\d]+$")
vt$addParameter(vtp1)
vt$addParameter(vtp2)
vt$addParameter(vtp3)

##add virtual table to featureType
featureType$setVirtualTable(vt)
```

<a name="GSFeatureType-update"/>

###### 4.5.3.3 Update an existing ``featureType``

```R
  #fetch featureType and update some properties
  featureType <- gsman$getFeatureType("topp", "taz_shapes", "tasmania_cities2")
  featureType$setAbstract("abstract updated")
  featureType$setEnabled(FALSE)
  
  #update featureType in GeoServer
  updated <- gsman$updateFeatureType("topp", "taz_shapes", featureType)
```

<a name="GSFeatureType-delete"/>

###### 4.5.3.4 Delete an existing ``featureType``

```R
  deleted <- gsman$deleteFeatureType("topp", "taz_shapes", "tasmania_cities2", recurse = TRUE)
```

The ``recurse`` parameter allows to delete the layer published for the deleted featureType.

<a name="GSLayer"/>

##### 4.5.4 Manipulate GeoServer ``layers``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/layers/

<a name="GSLayer-get"/>

###### 4.5.4.1 Get a (list of) ``layer(s)``

```R
  #get layer objects
  layers <- gsman$getLayers()

  #get layer names only
  layerNames <- gsman$getLayerNames()

  #get layer by name
  layer <- gsman$getLayer("tasmania_cities")
```

<a name="GSLayer-create"/>

###### 4.5.4.2 Create a new ``layer``

This method shows how to _publish_ a created ``featureType`` (as created in [4.5.3 Create a new ``FeatureType``](https://github.com/eblondel/geosapi/wiki#GSFeatureType-create))

```R
  layer <- GSLayer$new()
  layer$setName("tasmania_cities2")
  layer$setDefaultStyle("capitals")
  layer$addStyle("generic")
  created <- gsman$createLayer(layer)
```

<a name="GSLayer-update"/>

###### 4.5.4.3 Update an existing ``layer``

```R
  lyr <- gsman$getLayer("tasmania_cities")
  lyr$setDefaultStyle("generic")
  updated <- gsman$updateLayer(lyr)
```

<a name="GSLayer-delete"/>

###### 4.5.4.4 Delete an existing ``layer``

```R
  deleted <- gsman$deleteLayer("tasmania_cities2")
```

<a name="publishFeatureLayer"/>

##### 4.5.5 Publication of GeoServer feature layer

GeoServer manages feature data layers by means of two different types of resources: the ``featureType`` and the ``layer``. When created, the latter will _publish_ the former. Hence, in order to **publish** a complete feature data layer, two GeoServer resources should be created. The below methods allow the user to do both creation through a **single** method. To distinguish them from the above API methods and avoid confusion, we don't use the terminology **create** / **delete** but **publish** / **unpublish**.


<a name="publishFeatureLayer-publish"/>

###### 4.5.5.1 Publish Geoserver feature layer

The method ``publishLayer`` will sequentially do two requests to GeoServer:

* Create ``featureType``
* Create ``layer``

```R
  #create featuretype
  featureType <- GSFeatureType$new()
  featureType$setName("tasmania_cities2")
  featureType$setNativeName("tasmania_cities")
  featureType$setAbstract("abstract")
  featureType$setTitle("title")
  featureType$setSrs("EPSG:4326")
  featureType$setNativeCRS("EPSG:4326")
  featureType$setEnabled(TRUE)
  featureType$setProjectionPolicy("REPROJECT_TO_DECLARED")
  featureType$setLatLonBoundingBox(-180,-90,180,90, crs = "EPSG:4326")
  featureType$setNativeBoundingBox(-180,-90,180,90, crs ="EPSG:4326") 
  md1 <- GSMetadataLink$new(type = "text/xml", metadataType = "ISO19115:2003", content = "http://somelink.org/xml")
  featureType$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(type = "text/html", metadataType = "ISO19115:2003", content = "http://somelink.org/html")
  featureType$addMetadataLink(md2)
  
  #create layer
  layer <- GSLayer$new()
  layer$setName("tasmania_cities2")
  layer$setDefaultStyle("capitals")
  layer$addStyle("generic")
  
  #try to publish the complete layer (featuretype + layer)
  published <- gsman$publishLayer("topp", "taz_shapes", featureType, layer)
```

<a name="publishFeatureLayer-unpublish"/>

###### 4.5.5.2 Unpublish Geoserver feature layer

The method ``publishLayer`` will sequentially do two requests to GeoServer:

* Delete ``layer``
* Delete ``featureType`` 

```R
  #try to unpublish the complete layer (featuretype + layer)
  published <- gsman$unpublishLayer("topp", "taz_shapes", "tasmania_cities2")
```

<a name="GSLayerGroup"/>

##### 4.5.6 Manipulate GeoServer ``layer groups``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/layergroups/

<a name="GSLayerGroup-get"/>

###### 4.5.6.1 Get a (list of) ``layer group(s)``

```R
  #get layer groups objects
  layerGroups <- gsman$getLayerGroups()

  #get layer group names only
  layerGroupNames <- gsman$getLayerGroupNames()

  #get layer group by name
  layerGroup <- gsman$getLayerGroup("test_layergroup")
```

<a name="GSLayerGroup-create"/>

###### 4.5.6.2 Create a new ``layer group``

```R
  lyr <- GSLayerGroup$new()
  lyr$setName("test_layergroup")
  lyr$setTitle("title")
  lyr$setAbstract("abstract")
  lyr$setMode("SINGLE")
  lyr$addLayer(layer = "tasmania_cities", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  gsman$createLayerGroup(layerGroup = lyr)
```

<a name="GSLayerGroup-update"/>

###### 4.5.6.3 Update an existing ``layer group``

```R
  lyr <- GSLayerGroup$new()
  lyr$setName("test_layergroup")
  lyr$setTitle("title")
  lyr$setAbstract("abstract 2")
  lyr$setMode("SINGLE")
  lyr$addLayer(layer = "tasmania_cities", style = "generic")
  lyr$setBounds(-180,-90,180,90,crs = "EPSG:4326")
  gsman$updateLayerGroup(layerGroup = lyr)
```

<a name="GSLayerGroup-delete"/>

###### 4.5.6.4 Delete an existing ``layer group``

```R
  deleted <- gsman$deleteLayerGroup("test_layergroup")
```

<a name="rasterdata"/>

#### 4.6. Manage **raster data** with GeoServer

<a name="GSCoverageStore"/>

##### 4.6.1 Manipulate GeoServer ``coveragestores``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/coveragestores/

<a name="GSCoverageStore-get"/>

###### 4.6.1.1 Get a (list of) ``coveragestore(s)``

```R
  #get coveragestore objects
  cslist <- gsman$getCoverageStores("nurc")

  #get coveragestore names only
  csnames <- gsman$getCoverageStoreNames("nurc")

  #get coveragestore by name
  cs <- gsman$getCoverageStore("nurc", "mosaic")
```

<a name="GSCoverageStore-create"/>

###### 4.6.1.2 Create a new ``coveragetore``

<a name="GSGeoTIFFCoverageStore-create"/>

* 4.6.1.2.1 Create a **GeoTIFF** ``coveragestore``

```R
  cs = GSGeoTIFFCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```

<a name="GSWorldImageCoverageStore-create"/>

* 4.6.1.2.2 Create a **WorldImage** ``coveragestore``

```R
  cs = GSWorldImageCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```

<a name="GSArcGridCoverageStore-create"/>

* 4.6.1.2.3 Create a **ArcGrid** ``coveragestore``

```R
  cs = GSArcGridCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```

<a name="GSImageMosaicCoverageStore-create"/>

* 4.6.1.2.4 Create a **ImageMosaic** ``coveragestore``

```R
  cs = GSImageMosaicCoverageStore$new(name="sfdem_new", description = "sfdem_new description", enabled = TRUE)
  created <- gsman$createCoverageStore("sf", cs)
```


<a name="GSCoverageStore-update"/>

###### 4.6.1.3 Update an existing ``coveragestore``

```R
  cs <- gsman$getCoverageStore("sf", "sfdem")
  cs$setDescription("sfdem updated description")
  cs$setEnabled(FALSE)
  
  updated <- gsman$updateCoverageStore("sf", cs)
```

<a name="GSCoverageStore-delete"/>

###### 4.6.1.4 Delete an existing ``coveragestore``

```R
  deleted <- gsman$deleteCoverageStore("sf", "sfdem", recurse = TRUE)
```

The ``recurse`` parameter allows to delete all coverages published under the deleted coveragestore.

<a name="GSCoverage-upload"/>

##### 4.6.2 Upload coverage/raster data

The `geosapi` offers methods to upload coverage data to configured `coveragestores`. A generic method `gsman$uploadCoverage` allows to upload any kind of coverage data type supported by GeoServer, by specifying the file extension. To upload data, several parameters have to be specified including:
* `ws`: the workspace in which the datastore is configured
* `ds`: the datastore in which data has to be uploaded
* `endpoint`: choice among `file` (use a local file content as body), `url` (use a file URL), or `external` (path to a file). Important note: So far only `file` was tested and is supported.
* `extension`: the data file extension
* `configure`: either `none` (upload only, with no coverage/layer configured by default), `first` (upload and configure coverae/layer over the uploaded data)
* `update`: update strategy, either `append` (upload that will fails in case data was already uploaded), or `overwrite` (overwrite the already uploaded data if existing)
* `filename`: filename of the data resource to upload 
* `charset`: character set of the data resource
* `contentType: data content type

To simplify the upload, several coverage-specific methods were designed targeting specific data file extensions. These methods share the same parameters as above except the `extension` and `contentType` that will be pre-set according to the data upload format:
* `uploadGeoTIFF`: upload a GeoTIFF file
* `uploadWorldImage`: upload a WorldImage file
* `uploadArcGrid`: upload an ArcGrid file
* `uploadImageMosaic`: upload an ImageMosaic

Example with a GeoTIFF upload

```r
  uploaded <- gsman$uploadGeoTIFF(
	ws = "sf", cs = "sfdem_new",
    endpoint = "file", configure = "none", update = "overwrite",
    filename = system.file("extdata/sfdem_new.tif", package = "geosapi")
  )
```

<a name="GSCoverage"/>

##### 4.6.3 Manipulate GeoServer ``coverages``

>**GeoServer API doc**: https://docs.geoserver.org/main/en/user/rest/api/coverages/

<a name="GSCoverage-get"/>

###### 4.6.3.1 Get a (list of) ``coverage(s)``

```R
  #get coverage objects
  covlist <- gsman$getCoverages("sf", "sfdem")

  #get coverage names only
  covnames <- gsman$getCoverageNames("sf", "sfdem")

  #get coverage by name
  cov <- gsman$getCoverage("sf", "sfdem", "sfdem")
```

<a name="GSCoverage-create"/>

###### 4.6.3.2 Create a new ``coverage``

```R
  #create coverage object
  cov <- GSCoverage$new()
  cov$setName("sfdem_new")
  cov$setNativeName("sfdem_new")
  cov$setTitle("Title for sfdem")
  cov$setDescription("Description for sfdem")
  cov$addKeyword("sfdem keyword1")
  cov$addKeyword("sfdem keyword2")
  cov$addKeyword("sfdem keyword3")
  
  md1 <- GSMetadataLink$new(
    type = "text/xml",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/xml"
  )
  cov$addMetadataLink(md1)
  md2 <- GSMetadataLink$new(
    type = "text/html",
    metadataType = "ISO19115:2003",
    content = "http://somelink.org/sfdem_new/html"
  )
  cov$addMetadataLink(md2)
  
  cov$setSrs("EPSG:4326")
  cov$setNativeCRS("EPSG:26713")
  cov$setLatLonBoundingBox(-103.87108701853181, 44.370187074132616, -103.62940739432703, 44.5016011535299, crs = "EPSG:4326")
  cov$setNativeBoundingBox(589980, 4913700, 609000, 4928010, crs = "EPSG:26713")
  
  created <- gsman$createCoverage(ws = "sf", cs = "sfdem_new", coverage = cov)
```

**How to set a Geoserver Coverage View**

With ``geosapi``, it is also possible to configure a Geoserver Coverage View. For this, GeoServer uses the concept of _CoverageView_. The code below explains how to configure such coverage view with ``geosapi``:

```R

  ##coverage view
  coview <- GSCoverageView$new()
  coview$setName("sfdem_new")
  coview$setEnvelopeCompositionType("INTERSECTION")
  coview$setSelectedResolution("BEST")
  coview$setSelectedResolutionIndex(-1)
  coviewband <- GSCoverageBand$new()
  coviewband$setDefinition("sfdem_new@0")
  coviewband$setIndex(0)
  coviewband$addInputBand(GSInputCoverageBand$new( coverageName = "sfdem_new", band = 0))
  coview$addBand(coviewband)

  ##add coverage view to coverage
  cov$setView(coview)
```

<a name="GSCoverage-update"/>

###### 4.6.3.3 Update an existing ``coverage``

```R
  #fetch coverage and update some properties
  cov <- gsman$getCoverage("sf", "sfdem", "sfdem")
  cov$setAbstract("abstract updated")
  cov$setEnabled(FALSE)
  
  #update coverage in GeoServer
  updated <- gsman$updateCoverage("sf", "sfdem", cov)
```

<a name="GSCoverage-delete"/>

###### 4.6.3.4 Delete an existing ``coverage``

```R
  deleted <- gsman$deleteCoverage("sf", "sfdem", "sfdem", recurse = TRUE)
```

The ``recurse`` parameter allows to delete the layer published for the deleted featureType.

<a name="styles"/>

#### 4.7. Manage **styles** with GeoServer

<a name="GSStyle-get"/>

##### 4.7.1 Get a (list of) ``style``(s)

```R
  styles <- gsman$getStyles()
  stylenames <- gsman$getStyleNames()
  style <- gsman$getStyle("capitals")
```

From GeoServer 2.2, it is also possible to get the content of a style, ie the SLD body:

```R
  sld <- gsman$getSLDBody("capitals")
```

<a name="GSStyle-create"/>

##### 4.7.2 Create a new ``style``

```R
  sldFile <- system.file("extdata", "mystyle.sld", package = "geosapi") #to use 'file' argument of gsman$createStyle
  sldStyle <- xmlParse(sldFile) #to use 'sldBody' argument of gsman$createStyle
  created <- gsman$createStyle(file = sldFile, name = "mystyle")
```

<a name="GSStyle-update"/>

##### 4.7.3 Update a new ``style``

```R
  sldFile2 <- system.file("extdata", "mystyle2.sld", package = "geosapi") #to use 'file' argument of gsman$updateStyle
  sldStyle2 <- xmlParse(sldFile2) #to use 'sldBody' argument of gsman$updateStyle
  updated <- gsman$updateStyle(sldBody = sldStyle2, name = "mystyle")
```

<a name="GSStyle-delete"/>

##### 4.7.4 Delete an existing ``style``

```R
  deleted <- gsman$deleteStyle("mystyle", recurse = TRUE, purge = TRUE)
```

All methods describe here above also support a ``ws`` argument where a workspace can be specified.

<a name="settings_workspaces"/>

#### 4.8. Manage **workspace settings** with GeoServer

Geoserver allows to activate settings by workspace. In such case, this will overwrite the global Geoserver settings. Activating these settings is required to set-up specific OGC service capabilities par workspace (see next section).

<a name="GSWorkspaceSettings-create"/>

##### 4.8.1 Create a workspace settings

Creating a workspace settings is equivalent to check the "Enabled" checkbox in the _Settings_ section of a workspace edition page in the GeoServer UI. To create it (assuming a workspace named ``geosapi``), run the following code:

```R
settings <- GSWorkspaceSettings$new()
settings$setNumDecimals(5) #custom number of decimals for data published in this workspace
created <- gsman$createWorkspaceSettings("geosapi", settings)
```

<a name="GSWorkspaceSettings-get"/>

##### 4.8.2 Get a workspace settings

To get a workspace settings, use the following code:

```R
geosapi_settings <- gsman$getWorkspaceSettings("geosapi")
```

This method can be used in view of updating a workspace settings.

<a name="GSWorkspaceSettings-update"/>

##### 4.8.3 Update a workspace settings

Similar method can be used for updating the workspace configuration:

```R
settings <- gsman$getWorkspaceSettings("geosapi")
settings$setNumDecimals(9) #change number of decimals
created <- gsman$updateWorkspaceSettings("geosapi", settings)
```

<a name="GSWorkspaceSettings-delete"/>

##### 4.8.4 Delete a workspace settings

The settings can be deleted (equivalent of unchecking the "Enabled" checkbox in a workspace settings section):

```R
deleted <- gsman$deleteWorkspaceSettings("geosapi")
```

<a name="settings_services"/>

#### 4.9 Manage OGC Service (WMS/WFS/WCS) settings with GeoServer

<a name="GSServiceSettings-get"/>

##### 4.9.1 Get an OGC service (WMS/WFS/WCS) settings

To get an OGC service settings (global, or workspace specific), geosapi offers a generic method that can be used as follows:

```R
#get WMS global settings
wms_global_settings <- gsman$getServiceSettings(service = "WMS")

#get WMS geosapi settings
wms_geosapi_settings <- gsman$getServiceSettings(service = "WMS", ws = "geosapi")
```
Shortcut methods can also be used as follow:

```R
#get WMS global settings
wms_global_settings <- gsman$getWmsSettings()

#get WMS geosapi settings
wms_geosapi_settings <- gsman$getWmsSettings(ws = "geosapi")
```

Analog methods are available for WFS and WCS.


<a name="GSServiceSettings-update"/>

##### 4.9.2 Update an OGC service (WMS/WFS/WCS) settings

OGC service settings can be updated, for example to activate/deactivate the service and fill minimum metadata (name, title, abstract). The following examples shows how to do update WMS global configuration and by workspace:

* Update the WMS global configuration:

```R
wmsSettings <- gsman$getWmsSettings()
wmsSettings$setTitle("Tile for my global WMS")
updated <- gsman$updateServiceSettings(wmsSettings, service = "WMS")
```

Shortcut update methods are also available. The last code line above can be replaced by:

```R
updated <- gsman$updateWmsSettings(wmsSettings)
```
* Update  WMS workspace configuration

```R
wmsSettings <- GSServiceSettings$new(service = "WMS")
wmsSettings$setTitle("Tile for my workspace WMS")
wmsSettings$setAbstract("This WMS is set-up specifically for the workspace geosapi")
updated <- gsman$updateServiceSettings(wmsSettings, service = "WMS", ws = "geosapi")
```

The last code line above could be replaced by the shortcut method:

``
updated <- gsman$updateWmsSettings(wmsSettings, ws = "geosapi")
``

<a name="GSServiceSettings-enable"/>

##### 4.9.3 Enable/Disable an OGC service (WMS/WFS/WCS)

To make OGC service enabling/disabling easier for users, additional shortcut methods were set-up in geosapi. The following code examples show you how to enable/disable WMS/WFS/WCS

* Enable/Disable Global OGC services

```
#disable global WMS
gsman$disableWMS()
#enable global WFS and WCS
gsman$enableWFS()
gsman$enableWCS()
```

* Enable/Disable OGC services at workspace level

```R
#activates WMS for workspace 'geosapi'
enabled <- gsman$enableWMS("geosapi")

#deactivates WMS for workspace 'geosapi'
disabled <- gsman$disableWMS("geosapi")
```

Analog functions are available for WFS and WCS:

```
#enable WFS and WCS for geosapi workspace
gsman$enableWFS("geosapi")
gsman$enableWCS("geosapi")
```

```
#disable WFS and WCS for geosapi workspace
gsman$disableWFS("geosapi")
gsman$disableWCS("geosapi")
```

<a name="package_issues"/>

### 5. Issue reporting
***

Issues can be reported at https://github.com/eblondel/geosapi/issues

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSGeoPackageDataStore.R
\docType{class}
\name{GSGeoPackageDataStore}
\alias{GSGeoPackageDataStore}
\title{Geoserver REST API GeoPackageDataStore}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer GeoPackage dataStore
}
\description{
Geoserver REST API GeoPackageDataStore

Geoserver REST API GeoPackageDataStore
}
\examples{
ds <- GSGeoPackageDataStore$new(
 name = "ds", description = "des", 
 enabled = TRUE, database = NULL
)

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{DataStore}
\keyword{GeoPackage}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSAbstractStore]{geosapi::GSAbstractStore}} -> \code{\link[geosapi:GSAbstractDataStore]{geosapi::GSAbstractDataStore}} -> \code{\link[geosapi:GSAbstractDBDataStore]{geosapi::GSAbstractDBDataStore}} -> \code{GSGeoPackageDataStore}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSGeoPackageDataStore-new}{\code{GSGeoPackageDataStore$new()}}
\item \href{#method-GSGeoPackageDataStore-clone}{\code{GSGeoPackageDataStore$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setDescription"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setDescription'><code>geosapi::GSAbstractStore$setDescription()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setEnabled"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setEnabled'><code>geosapi::GSAbstractStore$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setType"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setType'><code>geosapi::GSAbstractStore$setType()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="addConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-addConnectionParameter'><code>geosapi::GSAbstractDataStore$addConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="decode"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-decode'><code>geosapi::GSAbstractDataStore$decode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="delConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-delConnectionParameter'><code>geosapi::GSAbstractDataStore$delConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="setConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-setConnectionParameter'><code>geosapi::GSAbstractDataStore$setConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="setConnectionParameters"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-setConnectionParameters'><code>geosapi::GSAbstractDataStore$setConnectionParameters()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setConnectionTimeout"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setConnectionTimeout'><code>geosapi::GSAbstractDBDataStore$setConnectionTimeout()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setDatabase"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setDatabase'><code>geosapi::GSAbstractDBDataStore$setDatabase()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setDatabaseType"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setDatabaseType'><code>geosapi::GSAbstractDBDataStore$setDatabaseType()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setDefautConnectionParameters"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setDefautConnectionParameters'><code>geosapi::GSAbstractDBDataStore$setDefautConnectionParameters()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setEstimatedExtends"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setEstimatedExtends'><code>geosapi::GSAbstractDBDataStore$setEstimatedExtends()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setExposePrimaryKeys"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setExposePrimaryKeys'><code>geosapi::GSAbstractDBDataStore$setExposePrimaryKeys()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setFetchSize"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setFetchSize'><code>geosapi::GSAbstractDBDataStore$setFetchSize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setHost"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setHost'><code>geosapi::GSAbstractDBDataStore$setHost()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setJndiReferenceName"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setJndiReferenceName'><code>geosapi::GSAbstractDBDataStore$setJndiReferenceName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setLooseBBox"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setLooseBBox'><code>geosapi::GSAbstractDBDataStore$setLooseBBox()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setMaxConnections"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setMaxConnections'><code>geosapi::GSAbstractDBDataStore$setMaxConnections()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setMaxOpenPreparedStatements"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setMaxOpenPreparedStatements'><code>geosapi::GSAbstractDBDataStore$setMaxOpenPreparedStatements()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setMinConnections"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setMinConnections'><code>geosapi::GSAbstractDBDataStore$setMinConnections()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setNamespace"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setNamespace'><code>geosapi::GSAbstractDBDataStore$setNamespace()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setPassword"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setPassword'><code>geosapi::GSAbstractDBDataStore$setPassword()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setPort"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setPort'><code>geosapi::GSAbstractDBDataStore$setPort()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setPreparedStatements"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setPreparedStatements'><code>geosapi::GSAbstractDBDataStore$setPreparedStatements()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setPrimaryKeyMetadataTable"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setPrimaryKeyMetadataTable'><code>geosapi::GSAbstractDBDataStore$setPrimaryKeyMetadataTable()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setSchema"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setSchema'><code>geosapi::GSAbstractDBDataStore$setSchema()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setUser"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setUser'><code>geosapi::GSAbstractDBDataStore$setUser()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDBDataStore" data-id="setValidateConnections"><a href='../../geosapi/html/GSAbstractDBDataStore.html#method-GSAbstractDBDataStore-setValidateConnections'><code>geosapi::GSAbstractDBDataStore$setValidateConnections()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSGeoPackageDataStore-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSGeoPackageDataStore-new}{}}}
\subsection{Method \code{new()}}{
initializes an GeoPackage data store
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSGeoPackageDataStore$new(
  xml = NULL,
  name = NULL,
  description = "",
  enabled = TRUE,
  database = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class} to create object from XML}

\item{\code{name}}{coverage store name}

\item{\code{description}}{coverage store description}

\item{\code{enabled}}{whether the store should be enabled or not. Default is \code{TRUE}}

\item{\code{database}}{database}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSGeoPackageDataStore-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSGeoPackageDataStore-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSGeoPackageDataStore$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geosapi.R
\docType{package}
\name{geosapi}
\alias{geosapi}
\alias{geosapi-package}
\title{GeoServer REST API R Interface}
\description{
Provides an R interface to the GeoServer REST API, allowing to 
upload and publish data in a GeoServer web-application and expose data to OGC 
Web-Services. The package currently supports all CRUD (Create,Read,Update,Delete) 
operations on GeoServer workspaces, namespaces, datastores (stores of vector data), 
featuretypes, layers, styles, as well as vector data upload operations. For 
more information about the GeoServer REST API, see <https://docs.geoserver.org/main/en/user/rest/> .
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/eblondel/geosapi}
  \item \url{https://eblondel.github.io/geosapi/}
  \item \url{https://geoserver.org/}
  \item Report bugs at \url{https://github.com/eblondel/geosapi/issues}
}

}
\author{
Emmanuel Blondel \email{emmanuel.blondel1@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSRESTEntrySet.R
\docType{class}
\name{GSRESTEntrySet}
\alias{GSRESTEntrySet}
\title{Geoserver REST API XML entry set}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a entry set
}
\description{
Geoserver REST API XML entry set

Geoserver REST API XML entry set
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{entryset}
\keyword{geoserver}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSRESTEntrySet}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{entryset}}{entryset}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSRESTEntrySet-new}{\code{GSRESTEntrySet$new()}}
\item \href{#method-GSRESTEntrySet-decode}{\code{GSRESTEntrySet$decode()}}
\item \href{#method-GSRESTEntrySet-setEntryset}{\code{GSRESTEntrySet$setEntryset()}}
\item \href{#method-GSRESTEntrySet-addEntry}{\code{GSRESTEntrySet$addEntry()}}
\item \href{#method-GSRESTEntrySet-setEntry}{\code{GSRESTEntrySet$setEntry()}}
\item \href{#method-GSRESTEntrySet-delEntry}{\code{GSRESTEntrySet$delEntry()}}
\item \href{#method-GSRESTEntrySet-clone}{\code{GSRESTEntrySet$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSRESTEntrySet}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$new(rootName, xml = NULL, entryset)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rootName}}{root name}

\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}

\item{\code{entryset}}{entry set}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-setEntryset"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-setEntryset}{}}}
\subsection{Method \code{setEntryset()}}{
Set entry set
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$setEntryset(entryset)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{entryset}}{entry set}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-addEntry"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-addEntry}{}}}
\subsection{Method \code{addEntry()}}{
Adds entry set
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$addEntry(key, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{key}

\item{\code{value}}{value}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-setEntry"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-setEntry}{}}}
\subsection{Method \code{setEntry()}}{
Sets entry set
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$setEntry(key, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{key}

\item{\code{value}}{value}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-delEntry"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-delEntry}{}}}
\subsection{Method \code{delEntry()}}{
Deletes entry set
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$delEntry(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{key}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTEntrySet-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTEntrySet-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTEntrySet$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSFeatureDimension.R
\docType{class}
\name{GSFeatureDimension}
\alias{GSFeatureDimension}
\title{A GeoServer dimension}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer feature dimension
}
\description{
This class models a GeoServer feature dimension.
}
\examples{
  dim <- GSFeatureDimension$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{dimension}
\keyword{geoserver}
\keyword{resource}
\keyword{rest}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSDimension]{geosapi::GSDimension}} -> \code{GSFeatureDimension}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{attribute}}{attribute}

\item{\code{endAttribute}}{end attribute}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSFeatureDimension-new}{\code{GSFeatureDimension$new()}}
\item \href{#method-GSFeatureDimension-decode}{\code{GSFeatureDimension$decode()}}
\item \href{#method-GSFeatureDimension-setAttribute}{\code{GSFeatureDimension$setAttribute()}}
\item \href{#method-GSFeatureDimension-setEndAttribute}{\code{GSFeatureDimension$setEndAttribute()}}
\item \href{#method-GSFeatureDimension-clone}{\code{GSFeatureDimension$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSDimension" data-id="setEnabled"><a href='../../geosapi/html/GSDimension.html#method-GSDimension-setEnabled'><code>geosapi::GSDimension$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSDimension" data-id="setPresentation"><a href='../../geosapi/html/GSDimension.html#method-GSDimension-setPresentation'><code>geosapi::GSDimension$setPresentation()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSDimension" data-id="setUnit"><a href='../../geosapi/html/GSDimension.html#method-GSDimension-setUnit'><code>geosapi::GSDimension$setUnit()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSDimension" data-id="setUnitSymbol"><a href='../../geosapi/html/GSDimension.html#method-GSDimension-setUnitSymbol'><code>geosapi::GSDimension$setUnitSymbol()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSFeatureDimension-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSFeatureDimension-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSFeatureDimension}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSFeatureDimension$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSFeatureDimension-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSFeatureDimension-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSFeatureDimension$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSFeatureDimension-setAttribute"></a>}}
\if{latex}{\out{\hypertarget{method-GSFeatureDimension-setAttribute}{}}}
\subsection{Method \code{setAttribute()}}{
Set attribute
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSFeatureDimension$setAttribute(attribute)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{attribute}}{attribute}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSFeatureDimension-setEndAttribute"></a>}}
\if{latex}{\out{\hypertarget{method-GSFeatureDimension-setEndAttribute}{}}}
\subsection{Method \code{setEndAttribute()}}{
Set end attribute
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSFeatureDimension$setEndAttribute(endAttribute)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{endAttribute}}{end attribute}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSFeatureDimension-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSFeatureDimension-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSFeatureDimension$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSStyleManager.R
\docType{class}
\name{GSStyleManager}
\alias{GSStyleManager}
\title{Geoserver REST API Style Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing the styles
of a GeoServer instance.
}
\description{
Geoserver REST API Style Manager

Geoserver REST API Style Manager
}
\examples{
\dontrun{
   GSStyleManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
}

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{style}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSStyleManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSStyleManager-getStyles}{\code{GSStyleManager$getStyles()}}
\item \href{#method-GSStyleManager-getStyleNames}{\code{GSStyleManager$getStyleNames()}}
\item \href{#method-GSStyleManager-getStyle}{\code{GSStyleManager$getStyle()}}
\item \href{#method-GSStyleManager-createStyle}{\code{GSStyleManager$createStyle()}}
\item \href{#method-GSStyleManager-updateStyle}{\code{GSStyleManager$updateStyle()}}
\item \href{#method-GSStyleManager-deleteStyle}{\code{GSStyleManager$deleteStyle()}}
\item \href{#method-GSStyleManager-getSLDVersion}{\code{GSStyleManager$getSLDVersion()}}
\item \href{#method-GSStyleManager-getSLDBody}{\code{GSStyleManager$getSLDBody()}}
\item \href{#method-GSStyleManager-clone}{\code{GSStyleManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-getStyles"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-getStyles}{}}}
\subsection{Method \code{getStyles()}}{
Get the list of available styles.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$getStyles(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{an optional workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \code{list} containing items of class \code{\link{GSStyle}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-getStyleNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-getStyleNames}{}}}
\subsection{Method \code{getStyleNames()}}{
Get the list of available style names
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$getStyleNames(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{an optional workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a vector of class \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-getStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-getStyle}{}}}
\subsection{Method \code{getStyle()}}{
Get a \code{\link{GSStyle}} object given a style name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$getStyle(style, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{style}}{style name}

\item{\code{ws}}{workspace name. Optional}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
object of class \link{GSStyle}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-createStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-createStyle}{}}}
\subsection{Method \code{createStyle()}}{
Creates a GeoServer style given a name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$createStyle(file, sldBody = NULL, name, raw = FALSE, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{file}}{file}

\item{\code{sldBody}}{SLD body}

\item{\code{name}}{name}

\item{\code{raw}}{raw}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the style has been successfully created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-updateStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-updateStyle}{}}}
\subsection{Method \code{updateStyle()}}{
Updates a GeoServer style given a name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$updateStyle(file, sldBody = NULL, name, raw = FALSE, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{file}}{file}

\item{\code{sldBody}}{SLD body}

\item{\code{name}}{name}

\item{\code{raw}}{raw}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the style has been successfully updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-deleteStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-deleteStyle}{}}}
\subsection{Method \code{deleteStyle()}}{
Deletes a style given a name.
   By defaut, the option \code{recurse} is set to FALSE, ie datastore layers are not removed.
   To remove all coverage store layers, set this option to TRUE. The \code{purge} parameter is used 
   to customize the delete of files on disk (in case the underlying reader implements a delete method).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$deleteStyle(name, recurse = FALSE, purge = FALSE, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}

\item{\code{recurse}}{recurse}

\item{\code{purge}}{purge}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the style has been successfully deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-getSLDVersion"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-getSLDVersion}{}}}
\subsection{Method \code{getSLDVersion()}}{
Get SLD version
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$getSLDVersion(sldBody)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{sldBody}}{SLD body}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-getSLDBody"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-getSLDBody}{}}}
\subsection{Method \code{getSLDBody()}}{
Get SLD body
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$getSLDBody(style, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{style}}{style name}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link[xml2]{xml_node-class}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyleManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyleManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyleManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSServiceRule.R
\docType{class}
\name{GSServiceRule}
\alias{GSServiceRule}
\title{A GeoServer access control list service rule}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer access control list service rule
}
\description{
This class models a GeoServer access control list service rule
}
\details{
Geoserver REST API Access Control List Service Rule
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{access}
\keyword{api}
\keyword{control}
\keyword{geoserver}
\keyword{rest}
\keyword{rule}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSRule]{geosapi::GSRule}} -> \code{GSServiceRule}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{roles}}{one or more roles}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSServiceRule-new}{\code{GSServiceRule$new()}}
\item \href{#method-GSServiceRule-clone}{\code{GSServiceRule$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRule" data-id="decode"><a href='../../geosapi/html/GSRule.html#method-GSRule-decode'><code>geosapi::GSRule$decode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRule" data-id="encode"><a href='../../geosapi/html/GSRule.html#method-GSRule-encode'><code>geosapi::GSRule$encode()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceRule-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceRule-new}{}}}
\subsection{Method \code{new()}}{
Initializes a \link{GSLayerRule}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceRule$new(
  xml = NULL,
  service,
  method,
  permission = c("r", "w", "a"),
  roles
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class}}

\item{\code{service}}{service subject to the access control rule, eg. 'wfs'}

\item{\code{method}}{service method subject to the access control rule, eg. 'GetFeature'}

\item{\code{permission}}{the rule permission, either \code{r} (read), \code{w} (write) or \code{a} (administer)}

\item{\code{roles}}{one or more roles to add for the rule}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceRule-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceRule-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceRule$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSServiceManager.R
\docType{class}
\name{GSServiceManager}
\alias{GSServiceManager}
\title{Geoserver REST API Service Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer services
}
\description{
Geoserver REST API Service Manager

Geoserver REST API Service Manager
}
\examples{
\dontrun{
   GSServiceManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
 }

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{service}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSServiceManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSServiceManager-getServiceSettings}{\code{GSServiceManager$getServiceSettings()}}
\item \href{#method-GSServiceManager-getWmsSettings}{\code{GSServiceManager$getWmsSettings()}}
\item \href{#method-GSServiceManager-getWfsSettings}{\code{GSServiceManager$getWfsSettings()}}
\item \href{#method-GSServiceManager-getWcsSettings}{\code{GSServiceManager$getWcsSettings()}}
\item \href{#method-GSServiceManager-updateServiceSettings}{\code{GSServiceManager$updateServiceSettings()}}
\item \href{#method-GSServiceManager-deleteServiceSettings}{\code{GSServiceManager$deleteServiceSettings()}}
\item \href{#method-GSServiceManager-updateWmsSettings}{\code{GSServiceManager$updateWmsSettings()}}
\item \href{#method-GSServiceManager-updateWfsSettings}{\code{GSServiceManager$updateWfsSettings()}}
\item \href{#method-GSServiceManager-updateWcsSettings}{\code{GSServiceManager$updateWcsSettings()}}
\item \href{#method-GSServiceManager-enableWMS}{\code{GSServiceManager$enableWMS()}}
\item \href{#method-GSServiceManager-enableWFS}{\code{GSServiceManager$enableWFS()}}
\item \href{#method-GSServiceManager-enableWCS}{\code{GSServiceManager$enableWCS()}}
\item \href{#method-GSServiceManager-disableServiceSettings}{\code{GSServiceManager$disableServiceSettings()}}
\item \href{#method-GSServiceManager-disableWMS}{\code{GSServiceManager$disableWMS()}}
\item \href{#method-GSServiceManager-disableWFS}{\code{GSServiceManager$disableWFS()}}
\item \href{#method-GSServiceManager-disableWCS}{\code{GSServiceManager$disableWCS()}}
\item \href{#method-GSServiceManager-clone}{\code{GSServiceManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-getServiceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-getServiceSettings}{}}}
\subsection{Method \code{getServiceSettings()}}{
Get the service settings. To get the service settings for a specific workspace,
   specify the workspace name as \code{ws} parameter, otherwise global settings are
   retrieved.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$getServiceSettings(service, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{service}}{service}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSServiceSettings}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-getWmsSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-getWmsSettings}{}}}
\subsection{Method \code{getWmsSettings()}}{
Get WMS settings. To get the WMS settings for a specific workspace,
   specify the workspace name as \code{ws} parameter, otherwise global settings are
   retrieved.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$getWmsSettings(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSServiceSettings}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-getWfsSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-getWfsSettings}{}}}
\subsection{Method \code{getWfsSettings()}}{
Get WFS settings. To get the WFS settings for a specific workspace,
   specify the workspace name as \code{ws} parameter, otherwise global settings are
   retrieved.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$getWfsSettings(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSServiceSettings}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-getWcsSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-getWcsSettings}{}}}
\subsection{Method \code{getWcsSettings()}}{
Get WCS settings. To get the WCS settings for a specific workspace,
   specify the workspace name as \code{ws} parameter, otherwise global settings are
   retrieved.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$getWcsSettings(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSServiceSettings}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-updateServiceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-updateServiceSettings}{}}}
\subsection{Method \code{updateServiceSettings()}}{
Updates the service settings with an object of class \code{GSServiceSettings}.
   An optional workspace name \code{ws} can be specified to update service settings
   applying to a workspace.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$updateServiceSettings(serviceSettings, service, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{serviceSettings}}{serviceSettings object of class \link{GSServiceSettings}}

\item{\code{service}}{service}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-deleteServiceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-deleteServiceSettings}{}}}
\subsection{Method \code{deleteServiceSettings()}}{
Deletes the service settings. This method is used internally by \pkg{geosapi} 
   for disabling a service setting at workspace level.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$deleteServiceSettings(service, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{service}}{service}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-updateWmsSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-updateWmsSettings}{}}}
\subsection{Method \code{updateWmsSettings()}}{
Updates the WMS settings with an object of class \code{GSServiceSettings}.
   An optional workspace name \code{ws} can be specified to update WMS settings
   applying to a workspace.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$updateWmsSettings(serviceSettings, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{serviceSettings}}{service settings object of class \link{GSServiceSettings}}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-updateWfsSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-updateWfsSettings}{}}}
\subsection{Method \code{updateWfsSettings()}}{
Updates the WFS settings with an object of class \code{GSServiceSettings}.
   An optional workspace name \code{ws} can be specified to update WFS settings
   applying to a workspace.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$updateWfsSettings(serviceSettings, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{serviceSettings}}{service settings object of class \link{GSServiceSettings}}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-updateWcsSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-updateWcsSettings}{}}}
\subsection{Method \code{updateWcsSettings()}}{
Updates the WCS settings with an object of class \code{GSServiceSettings}.
   An optional workspace name \code{ws} can be specified to update WCS settings
   applying to a workspace.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$updateWcsSettings(serviceSettings, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{serviceSettings}}{service settings object of class \link{GSServiceSettings}}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-enableWMS"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-enableWMS}{}}}
\subsection{Method \code{enableWMS()}}{
Enables WMS service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$enableWMS(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if enabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-enableWFS"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-enableWFS}{}}}
\subsection{Method \code{enableWFS()}}{
Enables WFS service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$enableWFS(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if enabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-enableWCS"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-enableWCS}{}}}
\subsection{Method \code{enableWCS()}}{
Enables WCS service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$enableWCS(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if enabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-disableServiceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-disableServiceSettings}{}}}
\subsection{Method \code{disableServiceSettings()}}{
Disables service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$disableServiceSettings(service, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{service}}{service}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if disabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-disableWMS"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-disableWMS}{}}}
\subsection{Method \code{disableWMS()}}{
Disables WMS service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$disableWMS(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if disabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-disableWFS"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-disableWFS}{}}}
\subsection{Method \code{disableWFS()}}{
Disables WFS service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$disableWFS(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if disabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-disableWCS"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-disableWCS}{}}}
\subsection{Method \code{disableWCS()}}{
Disables WCS service settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$disableWCS(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if disabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSWorkspace.R
\docType{class}
\name{GSWorkspace}
\alias{GSWorkspace}
\title{Geoserver REST API Workspace}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer workspace
}
\description{
Geoserver REST API Workspace

Geoserver REST API Workspace
}
\examples{
  GSWorkspace$new(name = "work")

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{workspace}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSWorkspace}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSWorkspace-new}{\code{GSWorkspace$new()}}
\item \href{#method-GSWorkspace-decode}{\code{GSWorkspace$decode()}}
\item \href{#method-GSWorkspace-clone}{\code{GSWorkspace$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspace-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspace-new}{}}}
\subsection{Method \code{new()}}{
initializes a \link{GSWorkspace}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspace$new(xml = NULL, name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class}}

\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspace-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspace-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspace$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspace-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspace-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspace$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSAbstractStore.R
\docType{class}
\name{GSAbstractStore}
\alias{GSAbstractStore}
\title{Geoserver REST API Store}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer store
}
\description{
Geoserver REST API Store

Geoserver REST API Store
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{store}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSAbstractStore}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{full}}{whether store object is fully described}

\item{\code{name}}{store name}

\item{\code{enabled}}{if the store is enabled or not}

\item{\code{description}}{store description}

\item{\code{type}}{store type}

\item{\code{workspace}}{workspace name}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSAbstractStore-new}{\code{GSAbstractStore$new()}}
\item \href{#method-GSAbstractStore-decode}{\code{GSAbstractStore$decode()}}
\item \href{#method-GSAbstractStore-setType}{\code{GSAbstractStore$setType()}}
\item \href{#method-GSAbstractStore-setEnabled}{\code{GSAbstractStore$setEnabled()}}
\item \href{#method-GSAbstractStore-setDescription}{\code{GSAbstractStore$setDescription()}}
\item \href{#method-GSAbstractStore-clone}{\code{GSAbstractStore$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractStore-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractStore-new}{}}}
\subsection{Method \code{new()}}{
initializes an abstract store
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractStore$new(
  xml = NULL,
  storeType,
  type = NULL,
  name = NULL,
  description = "",
  enabled = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class} to create object from XML}

\item{\code{storeType}}{store type}

\item{\code{type}}{the type of coverage store}

\item{\code{name}}{coverage store name}

\item{\code{description}}{coverage store description}

\item{\code{enabled}}{whether the store should be enabled or not. Default is \code{TRUE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractStore-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractStore-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes store from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractStore$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractStore-setType"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractStore-setType}{}}}
\subsection{Method \code{setType()}}{
Set type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractStore$setType(type)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{type}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractStore-setEnabled"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractStore-setEnabled}{}}}
\subsection{Method \code{setEnabled()}}{
Set enabled
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractStore$setEnabled(enabled)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{enabled}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractStore-setDescription"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractStore-setDescription}{}}}
\subsection{Method \code{setDescription()}}{
Set description
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractStore$setDescription(description)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{description}}{description}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractStore-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractStore-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractStore$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSLayerManager.R
\docType{class}
\name{GSLayerManager}
\alias{GSLayerManager}
\title{Geoserver REST API Layer Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
Layers as results of published feature types or coverages
}
\description{
Geoserver REST API Layer Manager

Geoserver REST API Layer Manager
}
\examples{
\dontrun{
   GSLayerManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
 }

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{Layer}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSLayerManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSLayerManager-getLayers}{\code{GSLayerManager$getLayers()}}
\item \href{#method-GSLayerManager-getLayerNames}{\code{GSLayerManager$getLayerNames()}}
\item \href{#method-GSLayerManager-getLayer}{\code{GSLayerManager$getLayer()}}
\item \href{#method-GSLayerManager-createLayer}{\code{GSLayerManager$createLayer()}}
\item \href{#method-GSLayerManager-updateLayer}{\code{GSLayerManager$updateLayer()}}
\item \href{#method-GSLayerManager-deleteLayer}{\code{GSLayerManager$deleteLayer()}}
\item \href{#method-GSLayerManager-getLayerGroups}{\code{GSLayerManager$getLayerGroups()}}
\item \href{#method-GSLayerManager-getLayerGroupNames}{\code{GSLayerManager$getLayerGroupNames()}}
\item \href{#method-GSLayerManager-getLayerGroup}{\code{GSLayerManager$getLayerGroup()}}
\item \href{#method-GSLayerManager-createLayerGroup}{\code{GSLayerManager$createLayerGroup()}}
\item \href{#method-GSLayerManager-updateLayerGroup}{\code{GSLayerManager$updateLayerGroup()}}
\item \href{#method-GSLayerManager-deleteLayerGroup}{\code{GSLayerManager$deleteLayerGroup()}}
\item \href{#method-GSLayerManager-clone}{\code{GSLayerManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-getLayers"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-getLayers}{}}}
\subsection{Method \code{getLayers()}}{
Get the list of layers.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$getLayers()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \code{list} giving items of class \code{\link{GSLayer}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-getLayerNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-getLayerNames}{}}}
\subsection{Method \code{getLayerNames()}}{
Get the list of layer names.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$getLayerNames()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
a vector of class \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-getLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-getLayer}{}}}
\subsection{Method \code{getLayer()}}{
Get layer by name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$getLayer(lyr)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{lyr}}{layer name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSLayer}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-createLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-createLayer}{}}}
\subsection{Method \code{createLayer()}}{
Creates a new layer given an object of class \code{\link{GSLayer}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$createLayer(layer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{layer}}{object of class \link{GSLayer}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-updateLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-updateLayer}{}}}
\subsection{Method \code{updateLayer()}}{
Updates a layer given an object of class \code{\link{GSLayer}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$updateLayer(layer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{layer}}{object of class \link{GSLayer}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-deleteLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-deleteLayer}{}}}
\subsection{Method \code{deleteLayer()}}{
Deletes layer given an object of class \code{\link{GSLayer}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$deleteLayer(lyr)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{lyr}}{layer name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-getLayerGroups"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-getLayerGroups}{}}}
\subsection{Method \code{getLayerGroups()}}{
Get layer groups
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$getLayerGroups(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name. Optional}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a list of objects of class \link{GSLayerGroup}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-getLayerGroupNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-getLayerGroupNames}{}}}
\subsection{Method \code{getLayerGroupNames()}}{
Get layer group names
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$getLayerGroupNames(ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a list of layer group names, as vector of class \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-getLayerGroup"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-getLayerGroup}{}}}
\subsection{Method \code{getLayerGroup()}}{
Get layer group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$getLayerGroup(lyr, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{lyr}}{lyr}

\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSLayerGroup}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-createLayerGroup"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-createLayerGroup}{}}}
\subsection{Method \code{createLayerGroup()}}{
Creates a layer group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$createLayerGroup(layerGroup, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{layerGroup}}{object of class \link{GSLayerGroup}}

\item{\code{ws}}{workspace name. Optional}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-updateLayerGroup"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-updateLayerGroup}{}}}
\subsection{Method \code{updateLayerGroup()}}{
Updates a layer group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$updateLayerGroup(layerGroup, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{layerGroup}}{object of class \link{GSLayerGroup}}

\item{\code{ws}}{workspace name. Optional}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-deleteLayerGroup"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-deleteLayerGroup}{}}}
\subsection{Method \code{deleteLayerGroup()}}{
Deletes a layer group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$deleteLayerGroup(lyr, ws = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{lyr}}{layer group name}

\item{\code{ws}}{workspace name. Optional}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSLayer.R, R/GSStyle.R
\docType{class}
\name{GSLayer}
\alias{GSLayer}
\alias{GSStyle}
\title{A GeoServer layer resource}
\format{
\code{\link[R6]{R6Class}} object.

\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer layer

Object of \code{\link[R6]{R6Class}} for modelling a GeoServer style
}
\description{
This class models a GeoServer layer. This class is to be
used for published resource (feature type or coverage).

This class models a GeoServer style.
}
\details{
Geoserver REST API Resource

Geoserver REST API Style
}
\examples{
  lyr <- GSLayer$new()

  lyr <- GSStyle$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{coverage}
\keyword{featureType}
\keyword{geoserver}
\keyword{layer}
\keyword{resource}
\keyword{resourcelayer}
\keyword{rest}
\keyword{style}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSLayer}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{full}}{full}

\item{\code{name}}{name}

\item{\code{path}}{path}

\item{\code{defaultStyle}}{default style}

\item{\code{styles}}{styles}

\item{\code{enabled}}{enabled}

\item{\code{queryable}}{queryable}

\item{\code{advertised}}{advertised}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSLayer-new}{\code{GSLayer$new()}}
\item \href{#method-GSLayer-decode}{\code{GSLayer$decode()}}
\item \href{#method-GSLayer-setName}{\code{GSLayer$setName()}}
\item \href{#method-GSLayer-setPath}{\code{GSLayer$setPath()}}
\item \href{#method-GSLayer-setEnabled}{\code{GSLayer$setEnabled()}}
\item \href{#method-GSLayer-setQueryable}{\code{GSLayer$setQueryable()}}
\item \href{#method-GSLayer-setAdvertised}{\code{GSLayer$setAdvertised()}}
\item \href{#method-GSLayer-setDefaultStyle}{\code{GSLayer$setDefaultStyle()}}
\item \href{#method-GSLayer-setStyles}{\code{GSLayer$setStyles()}}
\item \href{#method-GSLayer-addStyle}{\code{GSLayer$addStyle()}}
\item \href{#method-GSLayer-delStyle}{\code{GSLayer$delStyle()}}
\item \href{#method-GSLayer-clone}{\code{GSLayer$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSLayer}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setPath"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setPath}{}}}
\subsection{Method \code{setPath()}}{
Set path
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setPath(path)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{path}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setEnabled"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setEnabled}{}}}
\subsection{Method \code{setEnabled()}}{
Set enabled
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setEnabled(enabled)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{enabled}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setQueryable"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setQueryable}{}}}
\subsection{Method \code{setQueryable()}}{
Set queryable
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setQueryable(queryable)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{queryable}}{queryable}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setAdvertised"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setAdvertised}{}}}
\subsection{Method \code{setAdvertised()}}{
Set advertised
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setAdvertised(advertised)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{advertised}}{advertised}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setDefaultStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setDefaultStyle}{}}}
\subsection{Method \code{setDefaultStyle()}}{
Set default style
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setDefaultStyle(style)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{style}}{object o class \link{GSStyle} or \code{character}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-setStyles"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-setStyles}{}}}
\subsection{Method \code{setStyles()}}{
Set styles
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$setStyles(styles)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{styles}}{styles}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-addStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-addStyle}{}}}
\subsection{Method \code{addStyle()}}{
Adds style
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$addStyle(style)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{style}}{style, object o class \link{GSStyle} or \code{character}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-delStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-delStyle}{}}}
\subsection{Method \code{delStyle()}}{
Deletes style
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$delStyle(style)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{style}}{style, object o class \link{GSStyle} or \code{character}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayer-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayer-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayer$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSStyle}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{full}}{full}

\item{\code{name}}{name}

\item{\code{filename}}{filename}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSStyle-new}{\code{GSStyle$new()}}
\item \href{#method-GSStyle-decode}{\code{GSStyle$decode()}}
\item \href{#method-GSStyle-setName}{\code{GSStyle$setName()}}
\item \href{#method-GSStyle-setFilename}{\code{GSStyle$setFilename()}}
\item \href{#method-GSStyle-clone}{\code{GSStyle$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyle-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyle-new}{}}}
\subsection{Method \code{new()}}{
Initializes a \link{GSStyle}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyle$new(xml = NULL, name = NULL, filename = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class}}

\item{\code{name}}{name}

\item{\code{filename}}{filename}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyle-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyle-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyle$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyle-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyle-setName}{}}}
\subsection{Method \code{setName()}}{
set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyle$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyle-setFilename"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyle-setFilename}{}}}
\subsection{Method \code{setFilename()}}{
Set filename
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyle$setFilename(filename)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{filename}}{filename}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSStyle-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSStyle-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSStyle$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSVirtualTableParameter.R
\docType{class}
\name{GSVirtualTableParameter}
\alias{GSVirtualTableParameter}
\title{Geoserver REST API GSVirtualTableParameter}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer virtual table parameter
}
\description{
Geoserver REST API GSVirtualTableParameter

Geoserver REST API GSVirtualTableParameter
}
\examples{
GSVirtualTableParameter$new(name = "fieldname", defaultValue = "default_value",
                            regexpValidator = "someregexp")

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{virtualTable}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSVirtualTableParameter}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{name}}{parameter name}

\item{\code{defaultValue}}{parameter default value}

\item{\code{regexpValidator}}{parameter regexp validator}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSVirtualTableParameter-new}{\code{GSVirtualTableParameter$new()}}
\item \href{#method-GSVirtualTableParameter-decode}{\code{GSVirtualTableParameter$decode()}}
\item \href{#method-GSVirtualTableParameter-clone}{\code{GSVirtualTableParameter$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTableParameter-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTableParameter-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSVirtualTableParameter}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTableParameter$new(xml = NULL, name, defaultValue, regexpValidator)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}

\item{\code{name}}{name}

\item{\code{defaultValue}}{default value}

\item{\code{regexpValidator}}{regexp validator}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTableParameter-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTableParameter-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTableParameter$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTableParameter-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTableParameter-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTableParameter$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSCoverageBand.R
\docType{class}
\name{GSCoverageBand}
\alias{GSCoverageBand}
\title{Geoserver REST API GSCoverageBand}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer coverage band
}
\description{
Geoserver REST API GSCoverageBand

Geoserver REST API GSCoverageBand
}
\section{Methods}{

\describe{
 \item{\code{new(xml)}}{
   This method is used to instantiate a \code{GSCoverageBand}
 }
 \item{\code{decode(xml)}}{
   This method is used to decode a \code{GSCoverageBand} from XML
 }
 \item{\code{encode()}}{
   This method is used to encode a \code{GSCoverageBand} to XML
 }
 \item{\code{setDefinition(definition)}}{
   Sets the coverage band definition
 }
 \item{\code{setIndex(index)}}{
   Sets the coverage band index
 }
 \item{\code{setCompositionType}}{
   Sets the composition type. Only 'BAND_SELECT' is supported by GeoServer for now.
 }
 \item{\code{addInputBand(band)}}{
   Adds a input coverage band, object of class \code{GSInputCoverageBand}
 }
 \item{\code{delInputBand(band)}}{
   Removes a input coverage band, object of class \code{GSInputCoverageBand}
 }
}
}

\examples{
GSCoverageBand$new()


}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{coverageBand}
\keyword{geoserver}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSCoverageBand}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{inputCoverageBands}}{list of input coverage bands}

\item{\code{definition}}{coverage band definition}

\item{\code{index}}{coverage band index}

\item{\code{compositionType}}{coverage band composition type}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSCoverageBand-new}{\code{GSCoverageBand$new()}}
\item \href{#method-GSCoverageBand-decode}{\code{GSCoverageBand$decode()}}
\item \href{#method-GSCoverageBand-setName}{\code{GSCoverageBand$setName()}}
\item \href{#method-GSCoverageBand-setDefinition}{\code{GSCoverageBand$setDefinition()}}
\item \href{#method-GSCoverageBand-setIndex}{\code{GSCoverageBand$setIndex()}}
\item \href{#method-GSCoverageBand-setCompositionType}{\code{GSCoverageBand$setCompositionType()}}
\item \href{#method-GSCoverageBand-addInputBand}{\code{GSCoverageBand$addInputBand()}}
\item \href{#method-GSCoverageBand-delInputBand}{\code{GSCoverageBand$delInputBand()}}
\item \href{#method-GSCoverageBand-clone}{\code{GSCoverageBand$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-new}{}}}
\subsection{Method \code{new()}}{
Initalizes a \link{GSCoverageBand}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-setDefinition"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-setDefinition}{}}}
\subsection{Method \code{setDefinition()}}{
Set definition
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$setDefinition(definition)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{definition}}{definition}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-setIndex"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-setIndex}{}}}
\subsection{Method \code{setIndex()}}{
Set index
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$setIndex(index)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{index}}{index}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-setCompositionType"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-setCompositionType}{}}}
\subsection{Method \code{setCompositionType()}}{
Set composition type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$setCompositionType(compositionType)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{compositionType}}{composition type}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-addInputBand"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-addInputBand}{}}}
\subsection{Method \code{addInputBand()}}{
Adds an input band
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$addInputBand(band)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{band}}{object of class \link{GSInputCoverageBand}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-delInputBand"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-delInputBand}{}}}
\subsection{Method \code{delInputBand()}}{
Deletes an input band
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$delInputBand(band)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{band}}{object of class \link{GSInputCoverageBand}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageBand-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageBand-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageBand$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSShapefileDirectoryDataStore.R
\docType{class}
\name{GSShapefileDirectoryDataStore}
\alias{GSShapefileDirectoryDataStore}
\title{Geoserver REST API ShapeFileDirectoryDataStore}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer Shapefile directory dataStore
}
\description{
Geoserver REST API ShapeFileDirectoryDataStore

Geoserver REST API ShapeFileDirectoryDataStore
}
\examples{
GSShapefileDirectoryDataStore$new(name = "ds", description = "des",
                         enabled = TRUE, url = "file://data")

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{DataStore}
\keyword{ESRI}
\keyword{api}
\keyword{directory}
\keyword{geoserver}
\keyword{rest}
\keyword{shapefile}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSAbstractStore]{geosapi::GSAbstractStore}} -> \code{\link[geosapi:GSAbstractDataStore]{geosapi::GSAbstractDataStore}} -> \code{\link[geosapi:GSShapefileDataStore]{geosapi::GSShapefileDataStore}} -> \code{GSShapefileDirectoryDataStore}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSShapefileDirectoryDataStore-new}{\code{GSShapefileDirectoryDataStore$new()}}
\item \href{#method-GSShapefileDirectoryDataStore-setUrl}{\code{GSShapefileDirectoryDataStore$setUrl()}}
\item \href{#method-GSShapefileDirectoryDataStore-setCharset}{\code{GSShapefileDirectoryDataStore$setCharset()}}
\item \href{#method-GSShapefileDirectoryDataStore-setCreateSpatialIndex}{\code{GSShapefileDirectoryDataStore$setCreateSpatialIndex()}}
\item \href{#method-GSShapefileDirectoryDataStore-setMemoryMappedBuffer}{\code{GSShapefileDirectoryDataStore$setMemoryMappedBuffer()}}
\item \href{#method-GSShapefileDirectoryDataStore-setCacheReuseMemoryMaps}{\code{GSShapefileDirectoryDataStore$setCacheReuseMemoryMaps()}}
\item \href{#method-GSShapefileDirectoryDataStore-setDefautConnectionParameters}{\code{GSShapefileDirectoryDataStore$setDefautConnectionParameters()}}
\item \href{#method-GSShapefileDirectoryDataStore-clone}{\code{GSShapefileDirectoryDataStore$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setDescription"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setDescription'><code>geosapi::GSAbstractStore$setDescription()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setEnabled"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setEnabled'><code>geosapi::GSAbstractStore$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setType"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setType'><code>geosapi::GSAbstractStore$setType()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="addConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-addConnectionParameter'><code>geosapi::GSAbstractDataStore$addConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="decode"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-decode'><code>geosapi::GSAbstractDataStore$decode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="delConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-delConnectionParameter'><code>geosapi::GSAbstractDataStore$delConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="setConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-setConnectionParameter'><code>geosapi::GSAbstractDataStore$setConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="setConnectionParameters"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-setConnectionParameters'><code>geosapi::GSAbstractDataStore$setConnectionParameters()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-new}{}}}
\subsection{Method \code{new()}}{
initializes a shapefile directory data store
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$new(
  xml = NULL,
  name = NULL,
  description = "",
  enabled = TRUE,
  url
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class} to create object from XML}

\item{\code{name}}{coverage store name}

\item{\code{description}}{coverage store description}

\item{\code{enabled}}{whether the store should be enabled or not. Default is \code{TRUE}}

\item{\code{url}}{url}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-setUrl"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-setUrl}{}}}
\subsection{Method \code{setUrl()}}{
Set the spatial files data URL
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$setUrl(url)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{url}}{url}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-setCharset"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-setCharset}{}}}
\subsection{Method \code{setCharset()}}{
Set the charset used for DBF file.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$setCharset(charset = "ISO-8859-1")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{charset}}{charset. Default value is 'ISO-8859-1'}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-setCreateSpatialIndex"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-setCreateSpatialIndex}{}}}
\subsection{Method \code{setCreateSpatialIndex()}}{
Set the 'Create Spatial Index' option
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$setCreateSpatialIndex(create = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{create}}{create. Default is \code{TRUE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-setMemoryMappedBuffer"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-setMemoryMappedBuffer}{}}}
\subsection{Method \code{setMemoryMappedBuffer()}}{
Set the 'Memory Mapped Buffer' option
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$setMemoryMappedBuffer(buffer = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{buffer}}{buffer. Default is \code{FALSE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-setCacheReuseMemoryMaps"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-setCacheReuseMemoryMaps}{}}}
\subsection{Method \code{setCacheReuseMemoryMaps()}}{
Set the 'Cache & Reuse Memory Maps' option.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$setCacheReuseMemoryMaps(maps = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{maps}}{maps. Default is \code{TRUE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-setDefautConnectionParameters"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-setDefautConnectionParameters}{}}}
\subsection{Method \code{setDefautConnectionParameters()}}{
Set default connection parameters
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$setDefautConnectionParameters()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShapefileDirectoryDataStore-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSShapefileDirectoryDataStore-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShapefileDirectoryDataStore$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSCoverageView.R
\docType{class}
\name{GSCoverageView}
\alias{GSCoverageView}
\title{Geoserver REST API GSCoverageView}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer coverage view
}
\description{
Geoserver REST API GSCoverageView

Geoserver REST API GSCoverageView
}
\examples{
  GSCoverageView$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{coverageView}
\keyword{geoserver}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSCoverageView}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{name}}{name}

\item{\code{envelopeCompositionType}}{envelope composition type}

\item{\code{selectedResolution}}{selected resolution}

\item{\code{selectedResolutionIndex}}{selected resolution index}

\item{\code{coverageBands}}{coverage bands}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSCoverageView-new}{\code{GSCoverageView$new()}}
\item \href{#method-GSCoverageView-decode}{\code{GSCoverageView$decode()}}
\item \href{#method-GSCoverageView-setName}{\code{GSCoverageView$setName()}}
\item \href{#method-GSCoverageView-setEnvelopeCompositionType}{\code{GSCoverageView$setEnvelopeCompositionType()}}
\item \href{#method-GSCoverageView-setSelectedResolution}{\code{GSCoverageView$setSelectedResolution()}}
\item \href{#method-GSCoverageView-setSelectedResolutionIndex}{\code{GSCoverageView$setSelectedResolutionIndex()}}
\item \href{#method-GSCoverageView-addBand}{\code{GSCoverageView$addBand()}}
\item \href{#method-GSCoverageView-delBand}{\code{GSCoverageView$delBand()}}
\item \href{#method-GSCoverageView-clone}{\code{GSCoverageView$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSCoverageView}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-setEnvelopeCompositionType"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-setEnvelopeCompositionType}{}}}
\subsection{Method \code{setEnvelopeCompositionType()}}{
Sets the envelope composition type. Type of Envelope Composition, used to expose the bounding box 
of the CoverageView, either 'UNION' or 'INTERSECTION'.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$setEnvelopeCompositionType(envelopeCompositionType)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{envelopeCompositionType}}{envelope composition type}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-setSelectedResolution"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-setSelectedResolution}{}}}
\subsection{Method \code{setSelectedResolution()}}{
Set selected resolution
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$setSelectedResolution(selectedResolution)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{selectedResolution}}{selected resolution}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-setSelectedResolutionIndex"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-setSelectedResolutionIndex}{}}}
\subsection{Method \code{setSelectedResolutionIndex()}}{
Set selected resolution index
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$setSelectedResolutionIndex(selectedResolutionIndex)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{selectedResolutionIndex}}{selected resolution index}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-addBand"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-addBand}{}}}
\subsection{Method \code{addBand()}}{
Adds band
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$addBand(band)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{band}}{object of class \link{GSCoverageBand}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-delBand"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-delBand}{}}}
\subsection{Method \code{delBand()}}{
Deletes band
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$delBand(band)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{band}}{object of class \link{GSCoverageBand}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageView-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageView-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageView$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSVersion.R
\docType{class}
\name{GSVersion}
\alias{GSVersion}
\title{A GeoServer version}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer version
}
\description{
This class allows to grab the GeoServer version. By default, a 
tentative is made to fetch version from web admin default page, since Geoserver 
REST API did not support GET operation for the Geoserver version in past releases
of Geoserver.
}
\details{
Geoserver REST API - Geoserver Version
}
\examples{
\dontrun{
version <- GSVersion$new(
             url = "http://localhost:8080/geoserver",
             user = "admin", pwd = "geoserver"
           )
}

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{version}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{version}}{version}

\item{\code{value}}{value}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSVersion-new}{\code{GSVersion$new()}}
\item \href{#method-GSVersion-lowerThan}{\code{GSVersion$lowerThan()}}
\item \href{#method-GSVersion-greaterThan}{\code{GSVersion$greaterThan()}}
\item \href{#method-GSVersion-equalTo}{\code{GSVersion$equalTo()}}
\item \href{#method-GSVersion-clone}{\code{GSVersion$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVersion-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSVersion-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSVersion}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVersion$new(url, user, pwd)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{url}}{url}

\item{\code{user}}{user}

\item{\code{pwd}}{pwd}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVersion-lowerThan"></a>}}
\if{latex}{\out{\hypertarget{method-GSVersion-lowerThan}{}}}
\subsection{Method \code{lowerThan()}}{
Compares to a version and returns TRUE if it is lower, FALSE otherwise
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVersion$lowerThan(version)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{version}}{version}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if lower, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVersion-greaterThan"></a>}}
\if{latex}{\out{\hypertarget{method-GSVersion-greaterThan}{}}}
\subsection{Method \code{greaterThan()}}{
Compares to a version and returns TRUE if it is greater, FALSE otherwise
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVersion$greaterThan(version)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{version}}{version}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if greater, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVersion-equalTo"></a>}}
\if{latex}{\out{\hypertarget{method-GSVersion-equalTo}{}}}
\subsection{Method \code{equalTo()}}{
Compares to a version and returns TRUE if it is equal, FALSE otherwise
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVersion$equalTo(version)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{version}}{version}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if equal, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVersion-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSVersion-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVersion$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSImageMosaicCoverageStore.R
\docType{class}
\name{GSImageMosaicCoverageStore}
\alias{GSImageMosaicCoverageStore}
\title{Geoserver REST API ImageMosaicCoverageStore}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer ImageMosaic CoverageStore
}
\description{
Geoserver REST API ImageMosaicCoverageStore

Geoserver REST API ImageMosaicCoverageStore
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{CoverageStore}
\keyword{ImageMosaic}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSAbstractStore]{geosapi::GSAbstractStore}} -> \code{\link[geosapi:GSAbstractCoverageStore]{geosapi::GSAbstractCoverageStore}} -> \code{GSImageMosaicCoverageStore}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{url}}{url}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSImageMosaicCoverageStore-new}{\code{GSImageMosaicCoverageStore$new()}}
\item \href{#method-GSImageMosaicCoverageStore-clone}{\code{GSImageMosaicCoverageStore$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setDescription"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setDescription'><code>geosapi::GSAbstractStore$setDescription()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setEnabled"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setEnabled'><code>geosapi::GSAbstractStore$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setType"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setType'><code>geosapi::GSAbstractStore$setType()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractCoverageStore" data-id="decode"><a href='../../geosapi/html/GSAbstractCoverageStore.html#method-GSAbstractCoverageStore-decode'><code>geosapi::GSAbstractCoverageStore$decode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractCoverageStore" data-id="setUrl"><a href='../../geosapi/html/GSAbstractCoverageStore.html#method-GSAbstractCoverageStore-setUrl'><code>geosapi::GSAbstractCoverageStore$setUrl()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSImageMosaicCoverageStore-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSImageMosaicCoverageStore-new}{}}}
\subsection{Method \code{new()}}{
Initializes an Image Mosaic coverage store
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSImageMosaicCoverageStore$new(
  xml = NULL,
  name = NULL,
  description = "",
  enabled = TRUE,
  url = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class} to create object from XML}

\item{\code{name}}{coverage store name}

\item{\code{description}}{coverage store description}

\item{\code{enabled}}{whether the store should be enabled or not. Default is \code{TRUE}}

\item{\code{url}}{url}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSImageMosaicCoverageStore-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSImageMosaicCoverageStore-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSImageMosaicCoverageStore$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSLayerGroup.R
\docType{class}
\name{GSLayerGroup}
\alias{GSLayerGroup}
\title{A GeoServer layergroup resource}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer layergroup
}
\description{
This class models a GeoServer layer group. This class is to be
used for clustering layers into a group.
}
\details{
Geoserver REST API LayerGroup
}
\examples{
  lyr <- GSLayerGroup$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{coverage}
\keyword{featureType}
\keyword{geoserver}
\keyword{group}
\keyword{layer}
\keyword{resource}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSLayerGroup}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{full}}{full}

\item{\code{name}}{name}

\item{\code{mode}}{mode}

\item{\code{title}}{title}

\item{\code{abstractTxt}}{abstract}

\item{\code{workspace}}{workspace}

\item{\code{publishables}}{publishables}

\item{\code{styles}}{styles}

\item{\code{metadataLinks}}{metadata links}

\item{\code{bounds}}{bounds}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSLayerGroup-new}{\code{GSLayerGroup$new()}}
\item \href{#method-GSLayerGroup-decode}{\code{GSLayerGroup$decode()}}
\item \href{#method-GSLayerGroup-setName}{\code{GSLayerGroup$setName()}}
\item \href{#method-GSLayerGroup-setMode}{\code{GSLayerGroup$setMode()}}
\item \href{#method-GSLayerGroup-setTitle}{\code{GSLayerGroup$setTitle()}}
\item \href{#method-GSLayerGroup-setAbstract}{\code{GSLayerGroup$setAbstract()}}
\item \href{#method-GSLayerGroup-setWorkspace}{\code{GSLayerGroup$setWorkspace()}}
\item \href{#method-GSLayerGroup-addLayer}{\code{GSLayerGroup$addLayer()}}
\item \href{#method-GSLayerGroup-addLayerGroup}{\code{GSLayerGroup$addLayerGroup()}}
\item \href{#method-GSLayerGroup-addPublishable}{\code{GSLayerGroup$addPublishable()}}
\item \href{#method-GSLayerGroup-setStyles}{\code{GSLayerGroup$setStyles()}}
\item \href{#method-GSLayerGroup-addStyle}{\code{GSLayerGroup$addStyle()}}
\item \href{#method-GSLayerGroup-setMetadataLinks}{\code{GSLayerGroup$setMetadataLinks()}}
\item \href{#method-GSLayerGroup-addMetadataLink}{\code{GSLayerGroup$addMetadataLink()}}
\item \href{#method-GSLayerGroup-deleteMetadataLink}{\code{GSLayerGroup$deleteMetadataLink()}}
\item \href{#method-GSLayerGroup-setBounds}{\code{GSLayerGroup$setBounds()}}
\item \href{#method-GSLayerGroup-clone}{\code{GSLayerGroup$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSLayerGroup}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setMode"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setMode}{}}}
\subsection{Method \code{setMode()}}{
Set mode
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setMode(mode)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{mode}}{a mode value among "SINGLE", "NAMED", "CONTAINER", "EO"}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setTitle"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setTitle}{}}}
\subsection{Method \code{setTitle()}}{
Set title
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setTitle(title)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{title}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setAbstract"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setAbstract}{}}}
\subsection{Method \code{setAbstract()}}{
Set abstract
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setAbstract(abstract)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{abstract}}{abstract}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setWorkspace"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setWorkspace}{}}}
\subsection{Method \code{setWorkspace()}}{
Set workspace
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setWorkspace(workspace)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{workspace}}{workspace name, object of class \link{GSWorkspace} or \code{character}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-addLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-addLayer}{}}}
\subsection{Method \code{addLayer()}}{
Adds layer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$addLayer(layer, style)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{layer}}{layer name}

\item{\code{style}}{style name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-addLayerGroup"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-addLayerGroup}{}}}
\subsection{Method \code{addLayerGroup()}}{
Adds layer group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$addLayerGroup(layerGroup)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{layerGroup}}{layer group}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-addPublishable"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-addPublishable}{}}}
\subsection{Method \code{addPublishable()}}{
Adds publishable
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$addPublishable(publishable)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{publishable}}{publishable}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setStyles"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setStyles}{}}}
\subsection{Method \code{setStyles()}}{
Set styles
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setStyles(styles)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{styles}}{styles}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-addStyle"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-addStyle}{}}}
\subsection{Method \code{addStyle()}}{
Adds a style
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$addStyle(style)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{style}}{style}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setMetadataLinks"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setMetadataLinks}{}}}
\subsection{Method \code{setMetadataLinks()}}{
Set metadata links
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setMetadataLinks(metadataLinks)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataLinks}}{metadata links}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-addMetadataLink"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-addMetadataLink}{}}}
\subsection{Method \code{addMetadataLink()}}{
Adds metadata link
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$addMetadataLink(metadataLink)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataLink}}{object of class \link{GSMetadataLink}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-deleteMetadataLink"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-deleteMetadataLink}{}}}
\subsection{Method \code{deleteMetadataLink()}}{
Deletes metadata link
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$deleteMetadataLink(metadataLink)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataLink}}{object of class \link{GSMetadataLink}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-setBounds"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-setBounds}{}}}
\subsection{Method \code{setBounds()}}{
Set bounds
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$setBounds(minx, miny, maxx, maxy, bbox = NULL, crs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{minx}}{minx}

\item{\code{miny}}{miny}

\item{\code{maxx}}{maxx}

\item{\code{maxy}}{maxy}

\item{\code{bbox}}{bbox}

\item{\code{crs}}{crs}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSLayerGroup-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSLayerGroup-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSLayerGroup$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSDimension.R
\docType{class}
\name{GSDimension}
\alias{GSDimension}
\title{A GeoServer dimension}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer dimension
}
\description{
This class models a GeoServer resource dimension.
}
\details{
Geoserver REST API Dimension
}
\examples{
  dim <- GSDimension$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{dimension}
\keyword{geoserver}
\keyword{resource}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSDimension}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{enabled}}{true/false}

\item{\code{presentation}}{dimension presentation}

\item{\code{resolution}}{dimension resolution}

\item{\code{units}}{dimension units}

\item{\code{unitSymbol}}{dimension unitsSymbol}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSDimension-new}{\code{GSDimension$new()}}
\item \href{#method-GSDimension-decode}{\code{GSDimension$decode()}}
\item \href{#method-GSDimension-setEnabled}{\code{GSDimension$setEnabled()}}
\item \href{#method-GSDimension-setPresentation}{\code{GSDimension$setPresentation()}}
\item \href{#method-GSDimension-setUnit}{\code{GSDimension$setUnit()}}
\item \href{#method-GSDimension-setUnitSymbol}{\code{GSDimension$setUnitSymbol()}}
\item \href{#method-GSDimension-clone}{\code{GSDimension$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSDimension}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-setEnabled"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-setEnabled}{}}}
\subsection{Method \code{setEnabled()}}{
Set enabled
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$setEnabled(enabled)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{enabled}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-setPresentation"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-setPresentation}{}}}
\subsection{Method \code{setPresentation()}}{
Set presentation
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$setPresentation(presentation, interval = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{presentation}}{presentation. Possible values: "LIST", "CONTINUOUS_INTERVAL", "DISCRETE_INTERVAL"}

\item{\code{interval}}{interval}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-setUnit"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-setUnit}{}}}
\subsection{Method \code{setUnit()}}{
Set unit
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$setUnit(unit)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{unit}}{unit}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-setUnitSymbol"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-setUnitSymbol}{}}}
\subsection{Method \code{setUnitSymbol()}}{
Set unit symbol
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$setUnitSymbol(unitSymbol)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{unitSymbol}}{unit symbol}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDimension-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSDimension-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDimension$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSShinyMonitor.R
\docType{class}
\name{GSShinyMonitor}
\alias{GSShinyMonitor}
\title{Geoserver REST API DataStore}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for setting a GS Shiny monitoring app
}
\description{
Geoserver REST API DataStore

Geoserver REST API DataStore
}
\note{
Internal class used for \code{GSManager$monitor} method
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{geoserver}
\keyword{monitoring}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSShinyMonitor-new}{\code{GSShinyMonitor$new()}}
\item \href{#method-GSShinyMonitor-getMetric}{\code{GSShinyMonitor$getMetric()}}
\item \href{#method-GSShinyMonitor-run}{\code{GSShinyMonitor$run()}}
\item \href{#method-GSShinyMonitor-clone}{\code{GSShinyMonitor$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShinyMonitor-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSShinyMonitor-new}{}}}
\subsection{Method \code{new()}}{
Initializes a Geoserver shiny monitoring tool
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShinyMonitor$new(manager, file = NULL, append = FALSE, sleep = 1)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{manager}}{object of class \link{GSManager}}

\item{\code{file}}{file File where to store monitoring results}

\item{\code{append}}{append. Whether results should be appended to existing file}

\item{\code{sleep}}{sleep. Interval in seconds to trigger monitor calls}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShinyMonitor-getMetric"></a>}}
\if{latex}{\out{\hypertarget{method-GSShinyMonitor-getMetric}{}}}
\subsection{Method \code{getMetric()}}{
Get metric
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShinyMonitor$getMetric(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the Geoserver monitored metric
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShinyMonitor-run"></a>}}
\if{latex}{\out{\hypertarget{method-GSShinyMonitor-run}{}}}
\subsection{Method \code{run()}}{
Runs the application
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShinyMonitor$run()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSShinyMonitor-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSShinyMonitor-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSShinyMonitor$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSCoverage.R
\docType{class}
\name{GSCoverage}
\alias{GSCoverage}
\title{A GeoServer coverage}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer coverage
}
\description{
This class models a GeoServer coverage. This class is to be
used for manipulating representations of vector data with GeoServer.
}
\details{
Geoserver REST API Resource
}
\examples{
gt <- GSCoverage$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{coverageType}
\keyword{geoserver}
\keyword{resource}
\keyword{rest}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSResource]{geosapi::GSResource}} -> \code{GSCoverage}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{cqlFilter}}{CQL filter}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSCoverage-new}{\code{GSCoverage$new()}}
\item \href{#method-GSCoverage-decode}{\code{GSCoverage$decode()}}
\item \href{#method-GSCoverage-setView}{\code{GSCoverage$setView()}}
\item \href{#method-GSCoverage-delView}{\code{GSCoverage$delView()}}
\item \href{#method-GSCoverage-clone}{\code{GSCoverage$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="addKeyword"><a href='../../geosapi/html/GSResource.html#method-GSResource-addKeyword'><code>geosapi::GSResource$addKeyword()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="addMetadataLink"><a href='../../geosapi/html/GSResource.html#method-GSResource-addMetadataLink'><code>geosapi::GSResource$addMetadataLink()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="delKeyword"><a href='../../geosapi/html/GSResource.html#method-GSResource-delKeyword'><code>geosapi::GSResource$delKeyword()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="delMetadata"><a href='../../geosapi/html/GSResource.html#method-GSResource-delMetadata'><code>geosapi::GSResource$delMetadata()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="deleteMetadataLink"><a href='../../geosapi/html/GSResource.html#method-GSResource-deleteMetadataLink'><code>geosapi::GSResource$deleteMetadataLink()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setAbstract"><a href='../../geosapi/html/GSResource.html#method-GSResource-setAbstract'><code>geosapi::GSResource$setAbstract()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setDescription"><a href='../../geosapi/html/GSResource.html#method-GSResource-setDescription'><code>geosapi::GSResource$setDescription()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setEnabled"><a href='../../geosapi/html/GSResource.html#method-GSResource-setEnabled'><code>geosapi::GSResource$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setKeywords"><a href='../../geosapi/html/GSResource.html#method-GSResource-setKeywords'><code>geosapi::GSResource$setKeywords()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setLatLonBoundingBox"><a href='../../geosapi/html/GSResource.html#method-GSResource-setLatLonBoundingBox'><code>geosapi::GSResource$setLatLonBoundingBox()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setMetadata"><a href='../../geosapi/html/GSResource.html#method-GSResource-setMetadata'><code>geosapi::GSResource$setMetadata()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setMetadataDimension"><a href='../../geosapi/html/GSResource.html#method-GSResource-setMetadataDimension'><code>geosapi::GSResource$setMetadataDimension()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setMetadataLinks"><a href='../../geosapi/html/GSResource.html#method-GSResource-setMetadataLinks'><code>geosapi::GSResource$setMetadataLinks()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setName"><a href='../../geosapi/html/GSResource.html#method-GSResource-setName'><code>geosapi::GSResource$setName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setNativeBoundingBox"><a href='../../geosapi/html/GSResource.html#method-GSResource-setNativeBoundingBox'><code>geosapi::GSResource$setNativeBoundingBox()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setNativeCRS"><a href='../../geosapi/html/GSResource.html#method-GSResource-setNativeCRS'><code>geosapi::GSResource$setNativeCRS()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setNativeName"><a href='../../geosapi/html/GSResource.html#method-GSResource-setNativeName'><code>geosapi::GSResource$setNativeName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setProjectionPolicy"><a href='../../geosapi/html/GSResource.html#method-GSResource-setProjectionPolicy'><code>geosapi::GSResource$setProjectionPolicy()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setSrs"><a href='../../geosapi/html/GSResource.html#method-GSResource-setSrs'><code>geosapi::GSResource$setSrs()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSResource" data-id="setTitle"><a href='../../geosapi/html/GSResource.html#method-GSResource-setTitle'><code>geosapi::GSResource$setTitle()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverage-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverage-new}{}}}
\subsection{Method \code{new()}}{
Initializes a \link{GSCoverage} from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverage$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverage-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverage-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes coverage from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverage$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverage-setView"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverage-setView}{}}}
\subsection{Method \code{setView()}}{
Set view
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverage$setView(cv)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{cv}}{cv, object of class \link{GSCoverageView}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if set, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverage-delView"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverage-delView}{}}}
\subsection{Method \code{delView()}}{
Deletes view
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverage$delView()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverage-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverage-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverage$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSAbstractDataStore.R
\docType{class}
\name{GSAbstractDataStore}
\alias{GSAbstractDataStore}
\title{Geoserver REST API DataStore}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer dataStore
}
\description{
Geoserver REST API DataStore

Geoserver REST API DataStore
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{DataStore}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSAbstractStore]{geosapi::GSAbstractStore}} -> \code{GSAbstractDataStore}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{connectionParameters}}{the list of connection parameters}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSAbstractDataStore-new}{\code{GSAbstractDataStore$new()}}
\item \href{#method-GSAbstractDataStore-decode}{\code{GSAbstractDataStore$decode()}}
\item \href{#method-GSAbstractDataStore-setConnectionParameters}{\code{GSAbstractDataStore$setConnectionParameters()}}
\item \href{#method-GSAbstractDataStore-addConnectionParameter}{\code{GSAbstractDataStore$addConnectionParameter()}}
\item \href{#method-GSAbstractDataStore-setConnectionParameter}{\code{GSAbstractDataStore$setConnectionParameter()}}
\item \href{#method-GSAbstractDataStore-delConnectionParameter}{\code{GSAbstractDataStore$delConnectionParameter()}}
\item \href{#method-GSAbstractDataStore-clone}{\code{GSAbstractDataStore$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setDescription"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setDescription'><code>geosapi::GSAbstractStore$setDescription()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setEnabled"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setEnabled'><code>geosapi::GSAbstractStore$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setType"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setType'><code>geosapi::GSAbstractStore$setType()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-new}{}}}
\subsection{Method \code{new()}}{
initializes an abstract data store
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$new(
  xml = NULL,
  type = NULL,
  name = NULL,
  description = "",
  enabled = TRUE,
  connectionParameters
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class} to create object from XML}

\item{\code{type}}{the type of coverage store}

\item{\code{name}}{coverage store name}

\item{\code{description}}{coverage store description}

\item{\code{enabled}}{whether the store should be enabled or not. Default is \code{TRUE}}

\item{\code{connectionParameters}}{the list of connection parameters}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes a data store from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSAbstractDataStore}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-setConnectionParameters"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-setConnectionParameters}{}}}
\subsection{Method \code{setConnectionParameters()}}{
Set list connection parameters. The argument should be an object
   of class \code{GSRESTEntrySet} giving a list of key/value parameter entries.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$setConnectionParameters(parameters)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{parameters}}{an object of class \link{GSRESTEntrySet}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-addConnectionParameter"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-addConnectionParameter}{}}}
\subsection{Method \code{addConnectionParameter()}}{
Adds a connection parameter
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$addConnectionParameter(key, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{connection parameter key}

\item{\code{value}}{connection parameter value}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-setConnectionParameter"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-setConnectionParameter}{}}}
\subsection{Method \code{setConnectionParameter()}}{
Sets a connection parameter
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$setConnectionParameter(key, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{connection parameter key}

\item{\code{value}}{connection parameter value}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-delConnectionParameter"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-delConnectionParameter}{}}}
\subsection{Method \code{delConnectionParameter()}}{
Removes a connection parameter
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$delConnectionParameter(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{connection parameter key}

\item{\code{value}}{connection parameter value}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if removed, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDataStore-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDataStore-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDataStore$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSInputCoverageBand.R
\docType{class}
\name{GSInputCoverageBand}
\alias{GSInputCoverageBand}
\title{Geoserver REST API GSInputCoverageBand}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer input coverage band
}
\description{
Geoserver REST API GSInputCoverageBand

Geoserver REST API GSInputCoverageBand
}
\examples{
  GSInputCoverageBand$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{inputCoverageBand}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSInputCoverageBand}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{coverageName}}{coverage name}

\item{\code{band}}{band}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSInputCoverageBand-new}{\code{GSInputCoverageBand$new()}}
\item \href{#method-GSInputCoverageBand-decode}{\code{GSInputCoverageBand$decode()}}
\item \href{#method-GSInputCoverageBand-setCoverageName}{\code{GSInputCoverageBand$setCoverageName()}}
\item \href{#method-GSInputCoverageBand-setBand}{\code{GSInputCoverageBand$setBand()}}
\item \href{#method-GSInputCoverageBand-clone}{\code{GSInputCoverageBand$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSInputCoverageBand-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSInputCoverageBand-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSInputCoverageBand}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSInputCoverageBand$new(xml = NULL, coverageName = NULL, band = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}

\item{\code{coverageName}}{coverage name}

\item{\code{band}}{band name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSInputCoverageBand-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSInputCoverageBand-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSInputCoverageBand$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSInputCoverageBand-setCoverageName"></a>}}
\if{latex}{\out{\hypertarget{method-GSInputCoverageBand-setCoverageName}{}}}
\subsection{Method \code{setCoverageName()}}{
Set coverage name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSInputCoverageBand$setCoverageName(coverageName)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{coverageName}}{coverage name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSInputCoverageBand-setBand"></a>}}
\if{latex}{\out{\hypertarget{method-GSInputCoverageBand-setBand}{}}}
\subsection{Method \code{setBand()}}{
Set band
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSInputCoverageBand$setBand(band)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{band}}{band}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSInputCoverageBand-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSInputCoverageBand-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSInputCoverageBand$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSUtils.R
\docType{class}
\name{GSUtils}
\alias{GSUtils}
\title{Geoserver REST API Manager Utils}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with static util methods for communication
with the REST API of a GeoServer instance.
}
\description{
Geoserver REST API Manager Utils

Geoserver REST API Manager Utils
}
\section{Static methods}{

\describe{
 \item{\code{getUserAgent()}}{
   This method is used to get the user agent for performing GeoServer API requests.
   Here the user agent will be compound by geosapi package name and version.
 }
 \item{\code{getUserToken(user, pwd)}}{
   This method is used to get the user authentication token for performing GeoServer
   API requests. Token is given a Base64 encoded string.
 }
 \item{\code{GET(url, user, pwd, path, verbose)}}{
   This method performs a GET request for a given \code{path} to GeoServer REST API
 }
 \item{\code{PUT(url, user, pwd, path, filename, contentType, verbose)}}{
   This method performs a PUT request for a given \code{path} to GeoServer REST API,
   to upload a file of name \code{filename} with given \code{contentType}
 }
 \item{\code{POST(url, user, pwd, path, content, contentType, verbose)}}{
   This method performs a POST request for a given \code{path} to GeoServer REST API,
   to post content of given \code{contentType}
 }
 \item{\code{DELETE(url, user, pwd, path, verbose)}}{
   This method performs a DELETE request for a given GeoServer resource identified
   by a \code{path} in GeoServer REST API
 }
 \item{\code{parseResponseXML(req)}}{
   Convenience method to parse XML response from GeoServer REST API.
 }
 \item{\code{getPayloadXML(obj)}}{
   Convenience method to create payload XML to send to GeoServer.
 }
 \item{\code{setBbox(minx, miny, maxx, maxy, bbox, crs)}}{
   Creates an list object representing a bbox. Either from coordinates or from
   a \code{bbox} object (matrix).
 }
}
}

\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSUtils-clone}{\code{GSUtils$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSUtils-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSUtils-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSUtils$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSVirtualTable.R
\docType{class}
\name{GSVirtualTable}
\alias{GSVirtualTable}
\title{Geoserver REST API GSVirtualTable}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer virtual table
}
\description{
Geoserver REST API GSVirtualTable

Geoserver REST API GSVirtualTable
}
\examples{
GSVirtualTable$new()

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{virtualTable}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSVirtualTable}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{name}}{name}

\item{\code{sql}}{SQL statement}

\item{\code{escapeSql}}{escape SQL?}

\item{\code{keyColumn}}{key column}

\item{\code{geometry}}{geometry}

\item{\code{parameters}}{list of virtual parameters}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSVirtualTable-new}{\code{GSVirtualTable$new()}}
\item \href{#method-GSVirtualTable-decode}{\code{GSVirtualTable$decode()}}
\item \href{#method-GSVirtualTable-setName}{\code{GSVirtualTable$setName()}}
\item \href{#method-GSVirtualTable-setSql}{\code{GSVirtualTable$setSql()}}
\item \href{#method-GSVirtualTable-setEscapeSql}{\code{GSVirtualTable$setEscapeSql()}}
\item \href{#method-GSVirtualTable-setKeyColumn}{\code{GSVirtualTable$setKeyColumn()}}
\item \href{#method-GSVirtualTable-setGeometry}{\code{GSVirtualTable$setGeometry()}}
\item \href{#method-GSVirtualTable-addParameter}{\code{GSVirtualTable$addParameter()}}
\item \href{#method-GSVirtualTable-delParameter}{\code{GSVirtualTable$delParameter()}}
\item \href{#method-GSVirtualTable-clone}{\code{GSVirtualTable$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSVirtualTable}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-setSql"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-setSql}{}}}
\subsection{Method \code{setSql()}}{
Set SQL
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$setSql(sql)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{sql}}{sql}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-setEscapeSql"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-setEscapeSql}{}}}
\subsection{Method \code{setEscapeSql()}}{
Set escape SQL
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$setEscapeSql(escapeSql)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{escapeSql}}{escape SQL}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-setKeyColumn"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-setKeyColumn}{}}}
\subsection{Method \code{setKeyColumn()}}{
Set key column
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$setKeyColumn(keyColumn)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keyColumn}}{key column}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-setGeometry"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-setGeometry}{}}}
\subsection{Method \code{setGeometry()}}{
Set geometry
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$setGeometry(vtg)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{vtg}}{object of class \link{GSVirtualTableGeometry}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-addParameter"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-addParameter}{}}}
\subsection{Method \code{addParameter()}}{
Adds parameter
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$addParameter(parameter)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{parameter}}{object of class \link{GSVirtualTableParameter}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-delParameter"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-delParameter}{}}}
\subsection{Method \code{delParameter()}}{
Deletes parameter
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$delParameter(parameter)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{parameter}}{object of class \link{GSVirtualTableParameter}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSVirtualTable-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSVirtualTable-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSVirtualTable$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSResource.R
\docType{class}
\name{GSResource}
\alias{GSResource}
\title{A GeoServer abstract resource}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer resource
}
\description{
This class models an abstract GeoServer resource. This class is
used internally for modelling instances of class \code{GSFeatureType} or
\code{GSCoverage}
}
\details{
Geoserver REST API Resource
}
\examples{
res <- GSResource$new(rootName = "featureType")

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{resource}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSResource}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{full}}{full}

\item{\code{name}}{resource name}

\item{\code{nativeName}}{resource native name}

\item{\code{title}}{resource title}

\item{\code{description}}{resource description}

\item{\code{abstract}}{resource abstract}

\item{\code{keywords}}{resource keywords}

\item{\code{metadataLinks}}{resource metadata links}

\item{\code{nativeCRS}}{resource native CRS}

\item{\code{srs}}{resource srs}

\item{\code{nativeBoundingBox}}{resource lat/lon native bounding box}

\item{\code{latLonBoundingBox}}{resource lat/lon bounding box}

\item{\code{projectionPolicy}}{resource projection policy}

\item{\code{enabled}}{enabled}

\item{\code{metadata}}{metadata}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSResource-new}{\code{GSResource$new()}}
\item \href{#method-GSResource-decode}{\code{GSResource$decode()}}
\item \href{#method-GSResource-setEnabled}{\code{GSResource$setEnabled()}}
\item \href{#method-GSResource-setName}{\code{GSResource$setName()}}
\item \href{#method-GSResource-setNativeName}{\code{GSResource$setNativeName()}}
\item \href{#method-GSResource-setTitle}{\code{GSResource$setTitle()}}
\item \href{#method-GSResource-setDescription}{\code{GSResource$setDescription()}}
\item \href{#method-GSResource-setAbstract}{\code{GSResource$setAbstract()}}
\item \href{#method-GSResource-setKeywords}{\code{GSResource$setKeywords()}}
\item \href{#method-GSResource-addKeyword}{\code{GSResource$addKeyword()}}
\item \href{#method-GSResource-delKeyword}{\code{GSResource$delKeyword()}}
\item \href{#method-GSResource-setMetadataLinks}{\code{GSResource$setMetadataLinks()}}
\item \href{#method-GSResource-addMetadataLink}{\code{GSResource$addMetadataLink()}}
\item \href{#method-GSResource-deleteMetadataLink}{\code{GSResource$deleteMetadataLink()}}
\item \href{#method-GSResource-setProjectionPolicy}{\code{GSResource$setProjectionPolicy()}}
\item \href{#method-GSResource-setSrs}{\code{GSResource$setSrs()}}
\item \href{#method-GSResource-setNativeCRS}{\code{GSResource$setNativeCRS()}}
\item \href{#method-GSResource-setLatLonBoundingBox}{\code{GSResource$setLatLonBoundingBox()}}
\item \href{#method-GSResource-setNativeBoundingBox}{\code{GSResource$setNativeBoundingBox()}}
\item \href{#method-GSResource-setMetadata}{\code{GSResource$setMetadata()}}
\item \href{#method-GSResource-delMetadata}{\code{GSResource$delMetadata()}}
\item \href{#method-GSResource-setMetadataDimension}{\code{GSResource$setMetadataDimension()}}
\item \href{#method-GSResource-clone}{\code{GSResource$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-new}{}}}
\subsection{Method \code{new()}}{
Initializes a \link{GSResource}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$new(rootName = NULL, xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rootName}}{root name}

\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setEnabled"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setEnabled}{}}}
\subsection{Method \code{setEnabled()}}{
Set enabled
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setEnabled(enabled)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{enabled}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setNativeName"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setNativeName}{}}}
\subsection{Method \code{setNativeName()}}{
Set native name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setNativeName(nativeName)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{nativeName}}{native name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setTitle"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setTitle}{}}}
\subsection{Method \code{setTitle()}}{
Set title
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setTitle(title)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{title}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setDescription"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setDescription}{}}}
\subsection{Method \code{setDescription()}}{
Set description
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setDescription(description)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{description}}{description}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setAbstract"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setAbstract}{}}}
\subsection{Method \code{setAbstract()}}{
Set abstract
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setAbstract(abstract)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{abstract}}{abstract}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setKeywords"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setKeywords}{}}}
\subsection{Method \code{setKeywords()}}{
Set keyword(s)
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setKeywords(keywords)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keywords}}{keywords}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-addKeyword"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-addKeyword}{}}}
\subsection{Method \code{addKeyword()}}{
Adds keyword
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$addKeyword(keyword)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keyword}}{keyword}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-delKeyword"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-delKeyword}{}}}
\subsection{Method \code{delKeyword()}}{
Deletes keyword
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$delKeyword(keyword)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keyword}}{keyword}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setMetadataLinks"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setMetadataLinks}{}}}
\subsection{Method \code{setMetadataLinks()}}{
Set metadata links
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setMetadataLinks(metadataLinks)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataLinks}}{metadata links}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-addMetadataLink"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-addMetadataLink}{}}}
\subsection{Method \code{addMetadataLink()}}{
Adds metadata link
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$addMetadataLink(metadataLink)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataLink}}{object of class \link{GSMetadataLink}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-deleteMetadataLink"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-deleteMetadataLink}{}}}
\subsection{Method \code{deleteMetadataLink()}}{
Deletes metadata link
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$deleteMetadataLink(metadataLink)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataLink}}{object of class \link{GSMetadataLink}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setProjectionPolicy"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setProjectionPolicy}{}}}
\subsection{Method \code{setProjectionPolicy()}}{
Set projection policy
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setProjectionPolicy(projectionPolicy)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{projectionPolicy}}{projection policy}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setSrs"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setSrs}{}}}
\subsection{Method \code{setSrs()}}{
Set SRS
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setSrs(srs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{srs}}{srs}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setNativeCRS"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setNativeCRS}{}}}
\subsection{Method \code{setNativeCRS()}}{
Set native CRS
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setNativeCRS(nativeCRS)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{nativeCRS}}{native crs}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setLatLonBoundingBox"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setLatLonBoundingBox}{}}}
\subsection{Method \code{setLatLonBoundingBox()}}{
Set LatLon bounding box
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setLatLonBoundingBox(minx, miny, maxx, maxy, bbox = NULL, crs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{minx}}{minx}

\item{\code{miny}}{miny}

\item{\code{maxx}}{maxx}

\item{\code{maxy}}{maxy}

\item{\code{bbox}}{bbox}

\item{\code{crs}}{crs}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setNativeBoundingBox"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setNativeBoundingBox}{}}}
\subsection{Method \code{setNativeBoundingBox()}}{
Set native bounding box
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setNativeBoundingBox(minx, miny, maxx, maxy, bbox = NULL, crs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{minx}}{minx}

\item{\code{miny}}{miny}

\item{\code{maxx}}{maxx}

\item{\code{maxy}}{maxy}

\item{\code{bbox}}{bbox}

\item{\code{crs}}{crs}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setMetadata"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setMetadata}{}}}
\subsection{Method \code{setMetadata()}}{
Set metadata
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setMetadata(key, metadata)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{key}

\item{\code{metadata}}{metadata}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-delMetadata"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-delMetadata}{}}}
\subsection{Method \code{delMetadata()}}{
Deletes metadata
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$delMetadata(key)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{key}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-setMetadataDimension"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-setMetadataDimension}{}}}
\subsection{Method \code{setMetadataDimension()}}{
Set metadata dimension
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$setMetadataDimension(key, dimension, custom = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{key}}{key}

\item{\code{dimension}}{dimension}

\item{\code{custom}}{custom}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSAbstractDBDataStore.R
\docType{class}
\name{GSAbstractDBDataStore}
\alias{GSAbstractDBDataStore}
\title{Geoserver REST API AbstractDBDataStore}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer abstract DB dataStore
}
\description{
Geoserver REST API AbstractDBDataStore

Geoserver REST API AbstractDBDataStore
}
\note{
Internal abstract class used for setting DB stores
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{DB}
\keyword{DataStore}
\keyword{api}
\keyword{database}
\keyword{geoserver}
\keyword{rest}
\section{Super classes}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{\link[geosapi:GSAbstractStore]{geosapi::GSAbstractStore}} -> \code{\link[geosapi:GSAbstractDataStore]{geosapi::GSAbstractDataStore}} -> \code{GSAbstractDBDataStore}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSAbstractDBDataStore-new}{\code{GSAbstractDBDataStore$new()}}
\item \href{#method-GSAbstractDBDataStore-setDatabaseType}{\code{GSAbstractDBDataStore$setDatabaseType()}}
\item \href{#method-GSAbstractDBDataStore-setNamespace}{\code{GSAbstractDBDataStore$setNamespace()}}
\item \href{#method-GSAbstractDBDataStore-setHost}{\code{GSAbstractDBDataStore$setHost()}}
\item \href{#method-GSAbstractDBDataStore-setPort}{\code{GSAbstractDBDataStore$setPort()}}
\item \href{#method-GSAbstractDBDataStore-setDatabase}{\code{GSAbstractDBDataStore$setDatabase()}}
\item \href{#method-GSAbstractDBDataStore-setSchema}{\code{GSAbstractDBDataStore$setSchema()}}
\item \href{#method-GSAbstractDBDataStore-setUser}{\code{GSAbstractDBDataStore$setUser()}}
\item \href{#method-GSAbstractDBDataStore-setPassword}{\code{GSAbstractDBDataStore$setPassword()}}
\item \href{#method-GSAbstractDBDataStore-setJndiReferenceName}{\code{GSAbstractDBDataStore$setJndiReferenceName()}}
\item \href{#method-GSAbstractDBDataStore-setExposePrimaryKeys}{\code{GSAbstractDBDataStore$setExposePrimaryKeys()}}
\item \href{#method-GSAbstractDBDataStore-setMinConnections}{\code{GSAbstractDBDataStore$setMinConnections()}}
\item \href{#method-GSAbstractDBDataStore-setMaxConnections}{\code{GSAbstractDBDataStore$setMaxConnections()}}
\item \href{#method-GSAbstractDBDataStore-setFetchSize}{\code{GSAbstractDBDataStore$setFetchSize()}}
\item \href{#method-GSAbstractDBDataStore-setConnectionTimeout}{\code{GSAbstractDBDataStore$setConnectionTimeout()}}
\item \href{#method-GSAbstractDBDataStore-setValidateConnections}{\code{GSAbstractDBDataStore$setValidateConnections()}}
\item \href{#method-GSAbstractDBDataStore-setPrimaryKeyMetadataTable}{\code{GSAbstractDBDataStore$setPrimaryKeyMetadataTable()}}
\item \href{#method-GSAbstractDBDataStore-setLooseBBox}{\code{GSAbstractDBDataStore$setLooseBBox()}}
\item \href{#method-GSAbstractDBDataStore-setPreparedStatements}{\code{GSAbstractDBDataStore$setPreparedStatements()}}
\item \href{#method-GSAbstractDBDataStore-setMaxOpenPreparedStatements}{\code{GSAbstractDBDataStore$setMaxOpenPreparedStatements()}}
\item \href{#method-GSAbstractDBDataStore-setEstimatedExtends}{\code{GSAbstractDBDataStore$setEstimatedExtends()}}
\item \href{#method-GSAbstractDBDataStore-setDefautConnectionParameters}{\code{GSAbstractDBDataStore$setDefautConnectionParameters()}}
\item \href{#method-GSAbstractDBDataStore-clone}{\code{GSAbstractDBDataStore$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setDescription"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setDescription'><code>geosapi::GSAbstractStore$setDescription()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setEnabled"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setEnabled'><code>geosapi::GSAbstractStore$setEnabled()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractStore" data-id="setType"><a href='../../geosapi/html/GSAbstractStore.html#method-GSAbstractStore-setType'><code>geosapi::GSAbstractStore$setType()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="addConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-addConnectionParameter'><code>geosapi::GSAbstractDataStore$addConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="decode"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-decode'><code>geosapi::GSAbstractDataStore$decode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="delConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-delConnectionParameter'><code>geosapi::GSAbstractDataStore$delConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="setConnectionParameter"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-setConnectionParameter'><code>geosapi::GSAbstractDataStore$setConnectionParameter()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSAbstractDataStore" data-id="setConnectionParameters"><a href='../../geosapi/html/GSAbstractDataStore.html#method-GSAbstractDataStore-setConnectionParameters'><code>geosapi::GSAbstractDataStore$setConnectionParameters()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-new}{}}}
\subsection{Method \code{new()}}{
initializes an abstract DB data store
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$new(
  xml = NULL,
  type = NULL,
  dbType = NULL,
  name = NULL,
  description = "",
  enabled = TRUE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{an object of class \link[xml2]{xml_node-class} to create object from XML}

\item{\code{type}}{the type of DB data store}

\item{\code{dbType}}{DB type}

\item{\code{name}}{coverage store name}

\item{\code{description}}{coverage store description}

\item{\code{enabled}}{whether the store should be enabled or not. Default is \code{TRUE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setDatabaseType"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setDatabaseType}{}}}
\subsection{Method \code{setDatabaseType()}}{
Set database type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setDatabaseType(dbtype)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{dbtype}}{DB type}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setNamespace"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setNamespace}{}}}
\subsection{Method \code{setNamespace()}}{
Set namespace
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setNamespace(namespace)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{namespace}}{namespace}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setHost"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setHost}{}}}
\subsection{Method \code{setHost()}}{
Set host
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setHost(host)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{host}}{host}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setPort"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setPort}{}}}
\subsection{Method \code{setPort()}}{
Set port
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setPort(port)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{port}}{port}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setDatabase"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setDatabase}{}}}
\subsection{Method \code{setDatabase()}}{
Set database
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setDatabase(database)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{database}}{database}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setSchema"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setSchema}{}}}
\subsection{Method \code{setSchema()}}{
Set schema
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setSchema(schema)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{schema}}{schema}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setUser"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setUser}{}}}
\subsection{Method \code{setUser()}}{
Set user
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setUser(user)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{user}}{user}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setPassword"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setPassword}{}}}
\subsection{Method \code{setPassword()}}{
Set password
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setPassword(password)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{password}}{password}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setJndiReferenceName"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setJndiReferenceName}{}}}
\subsection{Method \code{setJndiReferenceName()}}{
Set JNDI reference name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setJndiReferenceName(jndiReferenceName)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{jndiReferenceName}}{JNDI reference name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setExposePrimaryKeys"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setExposePrimaryKeys}{}}}
\subsection{Method \code{setExposePrimaryKeys()}}{
Set expose primary keyws
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setExposePrimaryKeys(exposePrimaryKeys)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{exposePrimaryKeys}}{expose primary keys}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setMinConnections"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setMinConnections}{}}}
\subsection{Method \code{setMinConnections()}}{
Set min connections
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setMinConnections(minConnections = 1)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{minConnections}}{min connections. Default is 11}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setMaxConnections"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setMaxConnections}{}}}
\subsection{Method \code{setMaxConnections()}}{
Set max connections
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setMaxConnections(maxConnections = 10)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{maxConnections}}{max connections. Default is 10}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setFetchSize"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setFetchSize}{}}}
\subsection{Method \code{setFetchSize()}}{
Set fetch size
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setFetchSize(fetchSize = 1000)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fetchSize}}{fetch size. Default is 1000}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setConnectionTimeout"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setConnectionTimeout}{}}}
\subsection{Method \code{setConnectionTimeout()}}{
Set connection timeout
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setConnectionTimeout(seconds = 20)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{seconds}}{timeout (in seconds). Default is 20}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setValidateConnections"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setValidateConnections}{}}}
\subsection{Method \code{setValidateConnections()}}{
Set validate connection
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setValidateConnections(validateConnections)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{validateConnections}}{Validate connections}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setPrimaryKeyMetadataTable"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setPrimaryKeyMetadataTable}{}}}
\subsection{Method \code{setPrimaryKeyMetadataTable()}}{
Set primary key metadata table
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setPrimaryKeyMetadataTable(primaryKeyMetadataTable)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{primaryKeyMetadataTable}}{primary key metadata table}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setLooseBBox"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setLooseBBox}{}}}
\subsection{Method \code{setLooseBBox()}}{
Set loose bbox
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setLooseBBox(looseBBox = TRUE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{looseBBox}}{loose bbox. Default is \code{TRUE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setPreparedStatements"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setPreparedStatements}{}}}
\subsection{Method \code{setPreparedStatements()}}{
Set prepared statemnts
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setPreparedStatements(preparedStatements = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{preparedStatements}}{prepared Statements. Default is \code{FALSE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setMaxOpenPreparedStatements"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setMaxOpenPreparedStatements}{}}}
\subsection{Method \code{setMaxOpenPreparedStatements()}}{
Set max open prepared statements
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setMaxOpenPreparedStatements(
  maxOpenPreparedStatements = 50
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{maxOpenPreparedStatements}}{max open preepared statements. Default is 50}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setEstimatedExtends"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setEstimatedExtends}{}}}
\subsection{Method \code{setEstimatedExtends()}}{
Set estimatedExtends
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setEstimatedExtends(estimatedExtends = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{estimatedExtends}}{estimated extends. Default is \code{FALSE}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-setDefautConnectionParameters"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-setDefautConnectionParameters}{}}}
\subsection{Method \code{setDefautConnectionParameters()}}{
Set default connection parameters
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$setDefautConnectionParameters()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAbstractDBDataStore-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSAbstractDBDataStore-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAbstractDBDataStore$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSManager.R
\docType{class}
\name{GSManager}
\alias{GSManager}
\title{Geoserver REST API Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for communication with
the REST API of a GeoServer instance.
}
\description{
Geoserver REST API Manager

Geoserver REST API Manager
}
\examples{
\dontrun{
   GSManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
}

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{verbose.info}}{if geosapi logs have to be printed}

\item{\code{verbose.debug}}{if curl logs have to be printed}

\item{\code{loggerType}}{the type of logger}

\item{\code{url}}{the Base url of GeoServer}

\item{\code{version}}{the version of Geoserver. Handled as \code{GSVersion} object}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSManager-logger}{\code{GSManager$logger()}}
\item \href{#method-GSManager-INFO}{\code{GSManager$INFO()}}
\item \href{#method-GSManager-WARN}{\code{GSManager$WARN()}}
\item \href{#method-GSManager-ERROR}{\code{GSManager$ERROR()}}
\item \href{#method-GSManager-new}{\code{GSManager$new()}}
\item \href{#method-GSManager-getUrl}{\code{GSManager$getUrl()}}
\item \href{#method-GSManager-connect}{\code{GSManager$connect()}}
\item \href{#method-GSManager-reload}{\code{GSManager$reload()}}
\item \href{#method-GSManager-getSystemStatus}{\code{GSManager$getSystemStatus()}}
\item \href{#method-GSManager-monitor}{\code{GSManager$monitor()}}
\item \href{#method-GSManager-getClassName}{\code{GSManager$getClassName()}}
\item \href{#method-GSManager-getWorkspaceManager}{\code{GSManager$getWorkspaceManager()}}
\item \href{#method-GSManager-getNamespaceManager}{\code{GSManager$getNamespaceManager()}}
\item \href{#method-GSManager-getDataStoreManager}{\code{GSManager$getDataStoreManager()}}
\item \href{#method-GSManager-getCoverageStoreManager}{\code{GSManager$getCoverageStoreManager()}}
\item \href{#method-GSManager-getServiceManager}{\code{GSManager$getServiceManager()}}
\item \href{#method-GSManager-getStyleManager}{\code{GSManager$getStyleManager()}}
\item \href{#method-GSManager-clone}{\code{GSManager$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-logger"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-logger}{}}}
\subsection{Method \code{logger()}}{
Prints a log message
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$logger(type, text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{type of log, "INFO", "WARN", "ERROR"}

\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-INFO"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-INFO}{}}}
\subsection{Method \code{INFO()}}{
Prints an INFO log message
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$INFO(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-WARN"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-WARN}{}}}
\subsection{Method \code{WARN()}}{
Prints an WARN log message
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$WARN(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-ERROR"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-ERROR}{}}}
\subsection{Method \code{ERROR()}}{
Prints an ERROR log message
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$ERROR(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-new}{}}}
\subsection{Method \code{new()}}{
This method is used to instantiate a GSManager with the \code{url} of the
   GeoServer and credentials to authenticate (\code{user}/\code{pwd}). 
   
   By default, the \code{logger} argument will be set to \code{NULL} (no logger). 
   This argument accepts two possible values: \code{INFO}: to print only geosapi logs,
   \code{DEBUG}: to print geosapi and CURL logs.
   
   The \code{keyring_backend} can be set to use a different backend for storing 
   the Geoserver user password with \pkg{keyring} (Default value is 'env').
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$new(url, user, pwd, logger = NULL, keyring_backend = "env")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{url}}{url}

\item{\code{user}}{user}

\item{\code{pwd}}{pwd}

\item{\code{logger}}{logger}

\item{\code{keyring_backend}}{keyring backend. Default is 'env'}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getUrl"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getUrl}{}}}
\subsection{Method \code{getUrl()}}{
Get URL
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getUrl()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the Geoserver URL
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-connect"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-connect}{}}}
\subsection{Method \code{connect()}}{
Connects to geoServer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$connect()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
\code{TRUE} if connected, raises an error otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-reload"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-reload}{}}}
\subsection{Method \code{reload()}}{
Reloads the GeoServer catalog
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$reload()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
\code{TRUE} if reloaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getSystemStatus"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getSystemStatus}{}}}
\subsection{Method \code{getSystemStatus()}}{
Get system status
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getSystemStatus()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \code{data.frame} given the date time and metrics value
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-monitor"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-monitor}{}}}
\subsection{Method \code{monitor()}}{
Monitors the Geoserver by launching a small shiny monitoring application
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$monitor(file = NULL, append = FALSE, sleep = 1)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{file}}{file where to store monitoring results}

\item{\code{append}}{whether to append results to existing files}

\item{\code{sleep}}{sleeping interval to trigger a system status call}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getClassName"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getClassName}{}}}
\subsection{Method \code{getClassName()}}{
Get class name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getClassName()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the self class name, as \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getWorkspaceManager"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getWorkspaceManager}{}}}
\subsection{Method \code{getWorkspaceManager()}}{
Get Workspace manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getWorkspaceManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link{GSWorkspaceManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getNamespaceManager"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getNamespaceManager}{}}}
\subsection{Method \code{getNamespaceManager()}}{
Get Namespace manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getNamespaceManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link{GSNamespaceManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getDataStoreManager"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getDataStoreManager}{}}}
\subsection{Method \code{getDataStoreManager()}}{
Get Datastore manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getDataStoreManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link{GSDataStoreManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getCoverageStoreManager"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getCoverageStoreManager}{}}}
\subsection{Method \code{getCoverageStoreManager()}}{
Get Coverage store manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getCoverageStoreManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link{GSCoverageStoreManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getServiceManager"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getServiceManager}{}}}
\subsection{Method \code{getServiceManager()}}{
Get service manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getServiceManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link{GSServiceManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-getStyleManager"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-getStyleManager}{}}}
\subsection{Method \code{getStyleManager()}}{
Get style manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$getStyleManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link{GSStyleManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSNamespaceManager.R
\docType{class}
\name{GSNamespaceManager}
\alias{GSNamespaceManager}
\title{Geoserver REST API Namespace Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing the namespaces
 of a GeoServer instance.
}
\description{
Geoserver REST API Namespace Manager

Geoserver REST API Namespace Manager
}
\examples{
\dontrun{
   GSNamespaceManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
}

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{namespace}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSNamespaceManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSNamespaceManager-getNamespaces}{\code{GSNamespaceManager$getNamespaces()}}
\item \href{#method-GSNamespaceManager-getNamespaceNames}{\code{GSNamespaceManager$getNamespaceNames()}}
\item \href{#method-GSNamespaceManager-getNamespace}{\code{GSNamespaceManager$getNamespace()}}
\item \href{#method-GSNamespaceManager-createNamespace}{\code{GSNamespaceManager$createNamespace()}}
\item \href{#method-GSNamespaceManager-updateNamespace}{\code{GSNamespaceManager$updateNamespace()}}
\item \href{#method-GSNamespaceManager-deleteNamespace}{\code{GSNamespaceManager$deleteNamespace()}}
\item \href{#method-GSNamespaceManager-clone}{\code{GSNamespaceManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-getNamespaces"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-getNamespaces}{}}}
\subsection{Method \code{getNamespaces()}}{
Get the list of available namespace. Re
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$getNamespaces()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \code{list} containing items of class \code{\link{GSNamespace}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-getNamespaceNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-getNamespaceNames}{}}}
\subsection{Method \code{getNamespaceNames()}}{
Get the list of available namespace names.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$getNamespaceNames()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
a vector of class \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-getNamespace"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-getNamespace}{}}}
\subsection{Method \code{getNamespace()}}{
Get a \code{\link{GSNamespace}} object given a namespace name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$getNamespace(ns)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ns}}{namespace}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSNamespace}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-createNamespace"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-createNamespace}{}}}
\subsection{Method \code{createNamespace()}}{
Creates a GeoServer namespace given a prefix, and an optional URI.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$createNamespace(prefix, uri)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{prefix}}{prefix}

\item{\code{uri}}{uri}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the namespace has been successfully created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-updateNamespace"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-updateNamespace}{}}}
\subsection{Method \code{updateNamespace()}}{
Updates a GeoServer namespace given a prefix, and an optional URI.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$updateNamespace(prefix, uri)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{prefix}}{prefix}

\item{\code{uri}}{uri}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the namespace has been successfully updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-deleteNamespace"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-deleteNamespace}{}}}
\subsection{Method \code{deleteNamespace()}}{
Deletes a GeoServer namespace given a name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$deleteNamespace(name, recurse = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}

\item{\code{recurse}}{recurse}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the namespace has been successfully deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSNamespaceManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSNamespaceManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSNamespaceManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSWorkspaceManager.R
\docType{class}
\name{GSWorkspaceManager}
\alias{GSWorkspaceManager}
\title{Geoserver REST API Workspace Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing the workspaces
 of a GeoServer instance.
}
\description{
Geoserver REST API Workspace Manager

Geoserver REST API Workspace Manager
}
\examples{
\dontrun{
   GSWorkspaceManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
}

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{workspace}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSWorkspaceManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSWorkspaceManager-getWorkspaces}{\code{GSWorkspaceManager$getWorkspaces()}}
\item \href{#method-GSWorkspaceManager-getWorkspaceNames}{\code{GSWorkspaceManager$getWorkspaceNames()}}
\item \href{#method-GSWorkspaceManager-getWorkspace}{\code{GSWorkspaceManager$getWorkspace()}}
\item \href{#method-GSWorkspaceManager-createWorkspace}{\code{GSWorkspaceManager$createWorkspace()}}
\item \href{#method-GSWorkspaceManager-updateWorkspace}{\code{GSWorkspaceManager$updateWorkspace()}}
\item \href{#method-GSWorkspaceManager-deleteWorkspace}{\code{GSWorkspaceManager$deleteWorkspace()}}
\item \href{#method-GSWorkspaceManager-getWorkspaceSettings}{\code{GSWorkspaceManager$getWorkspaceSettings()}}
\item \href{#method-GSWorkspaceManager-createWorkspaceSettings}{\code{GSWorkspaceManager$createWorkspaceSettings()}}
\item \href{#method-GSWorkspaceManager-updateWorkspaceSettings}{\code{GSWorkspaceManager$updateWorkspaceSettings()}}
\item \href{#method-GSWorkspaceManager-deleteWorkspaceSettings}{\code{GSWorkspaceManager$deleteWorkspaceSettings()}}
\item \href{#method-GSWorkspaceManager-clone}{\code{GSWorkspaceManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-getWorkspaces"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-getWorkspaces}{}}}
\subsection{Method \code{getWorkspaces()}}{
Get the list of available workspace. Returns an object of class \code{list}
   containing items of class \code{\link{GSWorkspace}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$getWorkspaces()}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{a}}{list of \link{GSWorkspace}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-getWorkspaceNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-getWorkspaceNames}{}}}
\subsection{Method \code{getWorkspaceNames()}}{
Get the list of available workspace names. Returns an vector of class \code{character}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$getWorkspaceNames()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
a list of workspace names
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-getWorkspace"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-getWorkspace}{}}}
\subsection{Method \code{getWorkspace()}}{
Get a \code{\link{GSWorkspace}} object given a workspace name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$getWorkspace(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSWorkspace}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-createWorkspace"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-createWorkspace}{}}}
\subsection{Method \code{createWorkspace()}}{
Creates a GeoServer workspace given a name, and an optional URI. If the URI
   is not specified, GeoServer will automatically create an associated Namespace 
   with the URI built from the workspace name. If the URI is specified, the method
   invokes the method \code{createNamespace(ns, uri)} of the \code{\link{GSNamespaceManager}}.
   Returns \code{TRUE} if the workspace has been successfully created, \code{FALSE} otherwise
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$createWorkspace(name, uri)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}

\item{\code{uri}}{uri}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-updateWorkspace"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-updateWorkspace}{}}}
\subsection{Method \code{updateWorkspace()}}{
Updates a GeoServer workspace given a name, and an optional URI. If the URI
   is not specified, GeoServer will automatically update the associated Namespace 
   with the URI built from the workspace name. If the URI is specified, the method
   invokes the method \code{updateNamespace(ns, uri)} of the \code{\link{GSNamespaceManager}}.
   Returns \code{TRUE} if the workspace has been successfully updated, \code{FALSE} otherwise
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$updateWorkspace(name, uri)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}

\item{\code{uri}}{uri}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-deleteWorkspace"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-deleteWorkspace}{}}}
\subsection{Method \code{deleteWorkspace()}}{
Deletes a GeoServer workspace given a name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$deleteWorkspace(name, recurse = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}

\item{\code{recurse}}{recurse}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if the workspace has been successfully deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-getWorkspaceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-getWorkspaceSettings}{}}}
\subsection{Method \code{getWorkspaceSettings()}}{
Updates workspace settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$getWorkspaceSettings(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSWorkspaceSettings}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-createWorkspaceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-createWorkspaceSettings}{}}}
\subsection{Method \code{createWorkspaceSettings()}}{
Creates workspace settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$createWorkspaceSettings(ws, workspaceSettings)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{workspaceSettings}}{object of class \link{GSWorkspaceSettings}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-updateWorkspaceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-updateWorkspaceSettings}{}}}
\subsection{Method \code{updateWorkspaceSettings()}}{
Updates workspace settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$updateWorkspaceSettings(ws, workspaceSettings)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{workspaceSettings}}{object of class \link{GSWorkspaceSettings}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-deleteWorkspaceSettings"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-deleteWorkspaceSettings}{}}}
\subsection{Method \code{deleteWorkspaceSettings()}}{
Deletes workspace settings
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$deleteWorkspaceSettings(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSAccessControlListManager.R
\docType{class}
\name{GSAccessControlListManager}
\alias{GSAccessControlListManager}
\title{Geoserver REST API Access Control List Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
Access Control List (ACL) operations.
}
\description{
Geoserver REST API Access Control List Manager

Geoserver REST API Access Control List Manager
}
\examples{
\dontrun{
   GSAccessControlListManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
 }

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{ACL}
\keyword{access}
\keyword{api}
\keyword{control}
\keyword{geoserver}
\keyword{list}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSAccessControlListManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSAccessControlListManager-setCatalogMode}{\code{GSAccessControlListManager$setCatalogMode()}}
\item \href{#method-GSAccessControlListManager-getCatalogMode}{\code{GSAccessControlListManager$getCatalogMode()}}
\item \href{#method-GSAccessControlListManager-getRules}{\code{GSAccessControlListManager$getRules()}}
\item \href{#method-GSAccessControlListManager-getLayerRules}{\code{GSAccessControlListManager$getLayerRules()}}
\item \href{#method-GSAccessControlListManager-getServiceRules}{\code{GSAccessControlListManager$getServiceRules()}}
\item \href{#method-GSAccessControlListManager-getRestRules}{\code{GSAccessControlListManager$getRestRules()}}
\item \href{#method-GSAccessControlListManager-getRule}{\code{GSAccessControlListManager$getRule()}}
\item \href{#method-GSAccessControlListManager-getLayerRule}{\code{GSAccessControlListManager$getLayerRule()}}
\item \href{#method-GSAccessControlListManager-getServiceRule}{\code{GSAccessControlListManager$getServiceRule()}}
\item \href{#method-GSAccessControlListManager-getRestRule}{\code{GSAccessControlListManager$getRestRule()}}
\item \href{#method-GSAccessControlListManager-addRule}{\code{GSAccessControlListManager$addRule()}}
\item \href{#method-GSAccessControlListManager-addLayerRule}{\code{GSAccessControlListManager$addLayerRule()}}
\item \href{#method-GSAccessControlListManager-addServiceRule}{\code{GSAccessControlListManager$addServiceRule()}}
\item \href{#method-GSAccessControlListManager-addRestRule}{\code{GSAccessControlListManager$addRestRule()}}
\item \href{#method-GSAccessControlListManager-modifyRule}{\code{GSAccessControlListManager$modifyRule()}}
\item \href{#method-GSAccessControlListManager-deleteRule}{\code{GSAccessControlListManager$deleteRule()}}
\item \href{#method-GSAccessControlListManager-clone}{\code{GSAccessControlListManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-setCatalogMode"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-setCatalogMode}{}}}
\subsection{Method \code{setCatalogMode()}}{
Set the catalog mode
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$setCatalogMode(
  mode = c("HIDE", "MIXED", "CHALLENGE")
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{mode}}{mode}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if set, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getCatalogMode"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getCatalogMode}{}}}
\subsection{Method \code{getCatalogMode()}}{
Get the catalog mode
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getCatalogMode()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the mode either \code{HIDE}, \code{MIXED} or \code{CHALLENGE}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getRules"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getRules}{}}}
\subsection{Method \code{getRules()}}{
Get rules
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getRules(domain = c("layers", "services", "rest"))}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{domain}}{the access control domain}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the list of rules for a given domain
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getLayerRules"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getLayerRules}{}}}
\subsection{Method \code{getLayerRules()}}{
Method to get the list of 'layers' rules
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getLayerRules()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the list of 'layers' rules
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getServiceRules"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getServiceRules}{}}}
\subsection{Method \code{getServiceRules()}}{
Method to get the list of 'services' rules
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getServiceRules()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the list of 'services' rules
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getRestRules"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getRestRules}{}}}
\subsection{Method \code{getRestRules()}}{
Method to get the list of 'rest' rules
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getRestRules()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the list of 'rest' rules
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getRule}{}}}
\subsection{Method \code{getRule()}}{
Method to get a rule by resource
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getRule(
  domain = c("layers", "services", "rest"),
  resource
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{domain}}{the access control domain}

\item{\code{resource}}{resource}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSRule} or \code{NULL} if no rule is found
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getLayerRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getLayerRule}{}}}
\subsection{Method \code{getLayerRule()}}{
Method to get a 'layers' rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getLayerRule(resource)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{resource}}{resource}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSLayerRule} or \code{NULL} if no rule is found
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getServiceRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getServiceRule}{}}}
\subsection{Method \code{getServiceRule()}}{
Method to get a 'services' rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getServiceRule(resource)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{resource}}{resource}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSServiceRule} or \code{NULL} if no rule is found
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-getRestRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-getRestRule}{}}}
\subsection{Method \code{getRestRule()}}{
Method to get a 'rest' rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$getRestRule(resource)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{resource}}{resource}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSRestRule} or \code{NULL} if no rule is found
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-addRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-addRule}{}}}
\subsection{Method \code{addRule()}}{
Generic method to add an access control rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$addRule(rule, position = c("first", "last"))}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rule}}{object of class \link{GSRule}}

\item{\code{position}}{position with the list of rules, either "first" or "last". The position
is especially true to capture to the order of priority. For priority, put 'first'.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-addLayerRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-addLayerRule}{}}}
\subsection{Method \code{addLayerRule()}}{
Adds an access control layer rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$addLayerRule(
  ws = NULL,
  lyr = "*",
  permission = c("r", "w", "a"),
  roles
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{the resource workspace. Default is \code{NULL}}

\item{\code{lyr}}{the target layer to which the access control should be added}

\item{\code{permission}}{the rule permission, either \code{r} (read), \code{w} (write) or \code{a} (administer)}

\item{\code{roles}}{one or more roles to add for the rule}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-addServiceRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-addServiceRule}{}}}
\subsection{Method \code{addServiceRule()}}{
Adds an access control service rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$addServiceRule(
  service,
  method,
  permission = c("r", "w", "a"),
  roles
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{service}}{service subject to the access control rule, eg. 'wfs'}

\item{\code{method}}{service method subject to the access control rule, eg. 'GetFeature'}

\item{\code{permission}}{the rule permission, either \code{r} (read), \code{w} (write) or \code{a} (administer)}

\item{\code{roles}}{one or more roles to add for the rule}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-addRestRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-addRestRule}{}}}
\subsection{Method \code{addRestRule()}}{
Adds an access control rest rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$addRestRule(pattern = "/**", methods, roles)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{pattern}}{a URL Ant pattern, only applicable for domain \code{rest}. Default is \code{/**}}

\item{\code{methods}}{HTTP method(s)}

\item{\code{roles}}{one or more roles to add for the rule}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-modifyRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-modifyRule}{}}}
\subsection{Method \code{modifyRule()}}{
Generic method to modify an access control rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$modifyRule(rule, position = c("first", "last"))}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rule}}{object of class \link{GSRule}}

\item{\code{position}}{position with the list of rules, either "first" or "last". The position
is especially true to capture to the order of priority. For priority, put 'first'.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if modified, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-deleteRule"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-deleteRule}{}}}
\subsection{Method \code{deleteRule()}}{
Generic method to delete an access control rule
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$deleteRule(rule)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{rule}}{object of class \link{GSRule}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSAccessControlListManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSAccessControlListManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSAccessControlListManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSRESTResource.R
\docType{class}
\name{GSRESTResource}
\alias{GSRESTResource}
\title{Geoserver REST API REST Resource interface}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer REST resource interface
}
\description{
Geoserver REST API REST Resource interface

Geoserver REST API REST Resource interface
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{rootName}}{root name}

\item{\code{attrs}}{attrs}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSRESTResource-new}{\code{GSRESTResource$new()}}
\item \href{#method-GSRESTResource-decode}{\code{GSRESTResource$decode()}}
\item \href{#method-GSRESTResource-encode}{\code{GSRESTResource$encode()}}
\item \href{#method-GSRESTResource-print}{\code{GSRESTResource$print()}}
\item \href{#method-GSRESTResource-getClassName}{\code{GSRESTResource$getClassName()}}
\item \href{#method-GSRESTResource-clone}{\code{GSRESTResource$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTResource-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTResource-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSRESTResource}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTResource$new(xml, rootName)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}

\item{\code{rootName}}{root name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTResource-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTResource-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML. Abstract method to be implemented by sub-classes
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTResource$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTResource-encode"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTResource-encode}{}}}
\subsection{Method \code{encode()}}{
Encodes as XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTResource$encode()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \link[xml2]{xml_node-class}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTResource-print"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTResource-print}{}}}
\subsection{Method \code{print()}}{
Provides a custom print output (as tree) of the current class
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTResource$print(..., depth = 1)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{args}

\item{\code{depth}}{class nesting depth}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTResource-getClassName"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTResource-getClassName}{}}}
\subsection{Method \code{getClassName()}}{
Get class name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTResource$getClassName()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an object of class \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSRESTResource-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSRESTResource-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSRESTResource$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSWorkspaceSettings.R
\docType{class}
\name{GSWorkspaceSettings}
\alias{GSWorkspaceSettings}
\title{Geoserver REST API Workspace Setting}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer workspace settings
}
\description{
Geoserver REST API Workspace Setting

Geoserver REST API Workspace Setting
}
\examples{
  settings <- GSWorkspaceSettings$new()
  settings$setCharset("UTF-8")
  settings$setNumDecimals(5)

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{settings}
\keyword{workspace}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSWorkspaceSettings}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{contact}}{contact}

\item{\code{charset}}{charset}

\item{\code{numDecimals}}{number of decimal}

\item{\code{onlineResource}}{online resource}

\item{\code{verbose}}{verbose}

\item{\code{verboseExceptions}}{verbose exceptions}

\item{\code{localWorkspaceIncludesPrefix}}{local workspace includes prefix}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSWorkspaceSettings-new}{\code{GSWorkspaceSettings$new()}}
\item \href{#method-GSWorkspaceSettings-decode}{\code{GSWorkspaceSettings$decode()}}
\item \href{#method-GSWorkspaceSettings-setCharset}{\code{GSWorkspaceSettings$setCharset()}}
\item \href{#method-GSWorkspaceSettings-setNumDecimals}{\code{GSWorkspaceSettings$setNumDecimals()}}
\item \href{#method-GSWorkspaceSettings-setOnlineResource}{\code{GSWorkspaceSettings$setOnlineResource()}}
\item \href{#method-GSWorkspaceSettings-setVerbose}{\code{GSWorkspaceSettings$setVerbose()}}
\item \href{#method-GSWorkspaceSettings-setVerboseExceptions}{\code{GSWorkspaceSettings$setVerboseExceptions()}}
\item \href{#method-GSWorkspaceSettings-setLocalWorkspaceIncludesPrefix}{\code{GSWorkspaceSettings$setLocalWorkspaceIncludesPrefix()}}
\item \href{#method-GSWorkspaceSettings-clone}{\code{GSWorkspaceSettings$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-new}{}}}
\subsection{Method \code{new()}}{
This method is used to instantiate a \code{GSWorkspaceSettings}. This settings 
   object is required to activate a workspace configuration, using the method
   \code{GSManager$createWorkspaceSettings}. Supported from GeoServer 2.12
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$new(xml = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-setCharset"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-setCharset}{}}}
\subsection{Method \code{setCharset()}}{
Set charset
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$setCharset(charset)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-setNumDecimals"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-setNumDecimals}{}}}
\subsection{Method \code{setNumDecimals()}}{
Set number of decimals
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$setNumDecimals(numDecimals)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{numDecimals}}{number of decimals}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-setOnlineResource"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-setOnlineResource}{}}}
\subsection{Method \code{setOnlineResource()}}{
Set online resource
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$setOnlineResource(onlineResource)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{onlineResource}}{online resource}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-setVerbose"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-setVerbose}{}}}
\subsection{Method \code{setVerbose()}}{
Set verbose
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$setVerbose(verbose)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{verbose}}{verbose}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-setVerboseExceptions"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-setVerboseExceptions}{}}}
\subsection{Method \code{setVerboseExceptions()}}{
Set verbose exceptions
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$setVerboseExceptions(verboseExceptions)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{verboseExceptions}}{verbose exceptions}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-setLocalWorkspaceIncludesPrefix"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-setLocalWorkspaceIncludesPrefix}{}}}
\subsection{Method \code{setLocalWorkspaceIncludesPrefix()}}{
Set local workspace includes prefix
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$setLocalWorkspaceIncludesPrefix(includesPrefix)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{includesPrefix}}{includes prefix}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSWorkspaceSettings-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSWorkspaceSettings-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSWorkspaceSettings$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSMetadataLink.R
\docType{class}
\name{GSMetadataLink}
\alias{GSMetadataLink}
\title{A GeoServer resource metadataLink}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer resource metadataLink
}
\description{
This class models a GeoServer resource metadataLink made of a type
(free text e.g. text/xml, text/html), a metadataType (Possible values are 
ISO19115:2003, FGDC, TC211, 19139, other), and a content: an URL that gives 
the metadataLink
}
\details{
Geoserver REST API Metadatalink
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{geoserver}
\keyword{metadataLink}
\keyword{resource}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSMetadataLink}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{type}}{type}

\item{\code{metadataType}}{metadata type}

\item{\code{content}}{content}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSMetadataLink-new}{\code{GSMetadataLink$new()}}
\item \href{#method-GSMetadataLink-decode}{\code{GSMetadataLink$decode()}}
\item \href{#method-GSMetadataLink-setType}{\code{GSMetadataLink$setType()}}
\item \href{#method-GSMetadataLink-setMetadataType}{\code{GSMetadataLink$setMetadataType()}}
\item \href{#method-GSMetadataLink-setContent}{\code{GSMetadataLink$setContent()}}
\item \href{#method-GSMetadataLink-clone}{\code{GSMetadataLink$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSMetadataLink-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSMetadataLink-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSMetadataLink}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSMetadataLink$new(xml = NULL, type, metadataType, content)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}

\item{\code{type}}{type}

\item{\code{metadataType}}{metadata type}

\item{\code{content}}{content}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSMetadataLink-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSMetadataLink-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSMetadataLink$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSMetadataLink-setType"></a>}}
\if{latex}{\out{\hypertarget{method-GSMetadataLink-setType}{}}}
\subsection{Method \code{setType()}}{
Set type type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSMetadataLink$setType(type)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{type}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSMetadataLink-setMetadataType"></a>}}
\if{latex}{\out{\hypertarget{method-GSMetadataLink-setMetadataType}{}}}
\subsection{Method \code{setMetadataType()}}{
Set metadata type
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSMetadataLink$setMetadataType(metadataType)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{metadataType}}{metadata type. Supported values: "ISO19115:2003", "FGDC", "TC211", "19139", "other"}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSMetadataLink-setContent"></a>}}
\if{latex}{\out{\hypertarget{method-GSMetadataLink-setContent}{}}}
\subsection{Method \code{setContent()}}{
Set content
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSMetadataLink$setContent(content)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{content}}{content}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSMetadataLink-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSMetadataLink-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSMetadataLink$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSServiceSettings.R
\docType{class}
\name{GSServiceSettings}
\alias{GSServiceSettings}
\title{A GeoServer service settings resource}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a GeoServer OWS service setting
}
\description{
This class models a GeoServer OWS service settings.
}
\details{
Geoserver REST API Service Setting
}
\examples{
settings <- GSServiceSettings$new(service = "WMS")
settings$setEnabled(TRUE)

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{OGC}
\keyword{OWS}
\keyword{WCS}
\keyword{WFS}
\keyword{WMS}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\keyword{service}
\section{Super class}{
\code{\link[geosapi:GSRESTResource]{geosapi::GSRESTResource}} -> \code{GSServiceSettings}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{enabled}}{is service enabled or not?}

\item{\code{citeCompliant}}{is service cite compliant?}

\item{\code{name}}{service name}

\item{\code{title}}{service title}

\item{\code{maintainer}}{service maintainer}

\item{\code{abstrct}}{service abastract}

\item{\code{accessConstraints}}{service access constraints}

\item{\code{fees}}{service fees}

\item{\code{keywords}}{services keywords}

\item{\code{onlineResource}}{service online resource}

\item{\code{schemaBaseURL}}{service schema base URL}

\item{\code{verbose}}{service verbose or not?}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSServiceSettings-new}{\code{GSServiceSettings$new()}}
\item \href{#method-GSServiceSettings-decode}{\code{GSServiceSettings$decode()}}
\item \href{#method-GSServiceSettings-setEnabled}{\code{GSServiceSettings$setEnabled()}}
\item \href{#method-GSServiceSettings-setCiteCompliant}{\code{GSServiceSettings$setCiteCompliant()}}
\item \href{#method-GSServiceSettings-setName}{\code{GSServiceSettings$setName()}}
\item \href{#method-GSServiceSettings-setTitle}{\code{GSServiceSettings$setTitle()}}
\item \href{#method-GSServiceSettings-setMaintainer}{\code{GSServiceSettings$setMaintainer()}}
\item \href{#method-GSServiceSettings-setAbstract}{\code{GSServiceSettings$setAbstract()}}
\item \href{#method-GSServiceSettings-setAccessConstraints}{\code{GSServiceSettings$setAccessConstraints()}}
\item \href{#method-GSServiceSettings-setFees}{\code{GSServiceSettings$setFees()}}
\item \href{#method-GSServiceSettings-setKeywords}{\code{GSServiceSettings$setKeywords()}}
\item \href{#method-GSServiceSettings-addKeyword}{\code{GSServiceSettings$addKeyword()}}
\item \href{#method-GSServiceSettings-delKeyword}{\code{GSServiceSettings$delKeyword()}}
\item \href{#method-GSServiceSettings-clone}{\code{GSServiceSettings$clone()}}
}
}
\if{html}{\out{
<details open><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="encode"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-encode'><code>geosapi::GSRESTResource$encode()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="getClassName"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-getClassName'><code>geosapi::GSRESTResource$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSRESTResource" data-id="print"><a href='../../geosapi/html/GSRESTResource.html#method-GSRESTResource-print'><code>geosapi::GSRESTResource$print()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-new"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-new}{}}}
\subsection{Method \code{new()}}{
Initializes an object of class \link{GSServiceSettings}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$new(xml = NULL, service)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}

\item{\code{service}}{service service acronym}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-decode"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-decode}{}}}
\subsection{Method \code{decode()}}{
Decodes from XML
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$decode(xml)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{xml}}{object of class \link[xml2]{xml_node-class}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setEnabled"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setEnabled}{}}}
\subsection{Method \code{setEnabled()}}{
Set enabled
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setEnabled(enabled)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{enabled}}{enabled}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setCiteCompliant"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setCiteCompliant}{}}}
\subsection{Method \code{setCiteCompliant()}}{
Set cite compliant
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setCiteCompliant(citeCompliant)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{citeCompliant}}{cite compliant}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setName"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setName}{}}}
\subsection{Method \code{setName()}}{
Set name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setName(name)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setTitle"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setTitle}{}}}
\subsection{Method \code{setTitle()}}{
Set title
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setTitle(title)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{title}}{title}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setMaintainer"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setMaintainer}{}}}
\subsection{Method \code{setMaintainer()}}{
Set maintainer
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setMaintainer(maintainer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{maintainer}}{maintainer}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setAbstract"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setAbstract}{}}}
\subsection{Method \code{setAbstract()}}{
Set abstract
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setAbstract(abstract)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{abstract}}{abstract}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setAccessConstraints"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setAccessConstraints}{}}}
\subsection{Method \code{setAccessConstraints()}}{
Set access constraints
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setAccessConstraints(accessConstraints)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{accessConstraints}}{access constraints}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setFees"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setFees}{}}}
\subsection{Method \code{setFees()}}{
Set fees
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setFees(fees)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{fees}}{fees}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-setKeywords"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-setKeywords}{}}}
\subsection{Method \code{setKeywords()}}{
Set keywords
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$setKeywords(keywords)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keywords}}{keywords}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-addKeyword"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-addKeyword}{}}}
\subsection{Method \code{addKeyword()}}{
Adds a keyword
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$addKeyword(keyword)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keyword}}{keyword}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-delKeyword"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-delKeyword}{}}}
\subsection{Method \code{delKeyword()}}{
Deletes a keyword
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$delKeyword(keyword)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{keyword}}{keyword}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSServiceSettings-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSServiceSettings-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSServiceSettings$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSDatastoreManager.R
\docType{class}
\name{GSDataStoreManager}
\alias{GSDataStoreManager}
\title{Geoserver REST API DataStore Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
DataStores (i.e. stores of vector data)
}
\description{
Geoserver REST API DataStore Manager

Geoserver REST API DataStore Manager
}
\examples{
\dontrun{
   GSDataStoreManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
 }

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{DataStore}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSDataStoreManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSDataStoreManager-getDataStores}{\code{GSDataStoreManager$getDataStores()}}
\item \href{#method-GSDataStoreManager-getDataStoreNames}{\code{GSDataStoreManager$getDataStoreNames()}}
\item \href{#method-GSDataStoreManager-getDataStore}{\code{GSDataStoreManager$getDataStore()}}
\item \href{#method-GSDataStoreManager-createDataStore}{\code{GSDataStoreManager$createDataStore()}}
\item \href{#method-GSDataStoreManager-updateDataStore}{\code{GSDataStoreManager$updateDataStore()}}
\item \href{#method-GSDataStoreManager-deleteDataStore}{\code{GSDataStoreManager$deleteDataStore()}}
\item \href{#method-GSDataStoreManager-getFeatureTypes}{\code{GSDataStoreManager$getFeatureTypes()}}
\item \href{#method-GSDataStoreManager-getFeatureTypeNames}{\code{GSDataStoreManager$getFeatureTypeNames()}}
\item \href{#method-GSDataStoreManager-getFeatureType}{\code{GSDataStoreManager$getFeatureType()}}
\item \href{#method-GSDataStoreManager-createFeatureType}{\code{GSDataStoreManager$createFeatureType()}}
\item \href{#method-GSDataStoreManager-updateFeatureType}{\code{GSDataStoreManager$updateFeatureType()}}
\item \href{#method-GSDataStoreManager-deleteFeatureType}{\code{GSDataStoreManager$deleteFeatureType()}}
\item \href{#method-GSDataStoreManager-publishLayer}{\code{GSDataStoreManager$publishLayer()}}
\item \href{#method-GSDataStoreManager-unpublishLayer}{\code{GSDataStoreManager$unpublishLayer()}}
\item \href{#method-GSDataStoreManager-uploadData}{\code{GSDataStoreManager$uploadData()}}
\item \href{#method-GSDataStoreManager-uploadShapefile}{\code{GSDataStoreManager$uploadShapefile()}}
\item \href{#method-GSDataStoreManager-uploadProperties}{\code{GSDataStoreManager$uploadProperties()}}
\item \href{#method-GSDataStoreManager-uploadH2}{\code{GSDataStoreManager$uploadH2()}}
\item \href{#method-GSDataStoreManager-uploadSpatialite}{\code{GSDataStoreManager$uploadSpatialite()}}
\item \href{#method-GSDataStoreManager-uploadAppschema}{\code{GSDataStoreManager$uploadAppschema()}}
\item \href{#method-GSDataStoreManager-uploadGeoPackage}{\code{GSDataStoreManager$uploadGeoPackage()}}
\item \href{#method-GSDataStoreManager-clone}{\code{GSDataStoreManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-getDataStores"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-getDataStores}{}}}
\subsection{Method \code{getDataStores()}}{
Get the list of available dataStores.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$getDataStores(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \code{list} giving items of class \code{\link{GSAbstractDataStore}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-getDataStoreNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-getDataStoreNames}{}}}
\subsection{Method \code{getDataStoreNames()}}{
Get the list of available dataStore names.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$getDataStoreNames(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a vector of class \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-getDataStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-getDataStore}{}}}
\subsection{Method \code{getDataStore()}}{
Get an object of class \code{\link{GSAbstractDataStore}} given a workspace and datastore names.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$getDataStore(ws, ds)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the datastore
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-createDataStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-createDataStore}{}}}
\subsection{Method \code{createDataStore()}}{
Creates a datastore given a workspace and an object of class \code{\link{GSAbstractDataStore}}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$createDataStore(ws, dataStore)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{dataStore}}{datastore object of class \link{GSAbstractDataStore}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-updateDataStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-updateDataStore}{}}}
\subsection{Method \code{updateDataStore()}}{
Updates a datastore given a workspace and an object of class \code{\link{GSAbstractDataStore}}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$updateDataStore(ws, dataStore)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{dataStore}}{datastore object of class \link{GSAbstractDataStore}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-deleteDataStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-deleteDataStore}{}}}
\subsection{Method \code{deleteDataStore()}}{
Deletes a datastore given workspace and datastore names.
   By defaut, the option \code{recurse} is set to FALSE, ie datastore layers are not removed.
   To remove all datastore layers, set this option to TRUE.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$deleteDataStore(ws, ds, recurse = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{recurse}}{recurse}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-getFeatureTypes"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-getFeatureTypes}{}}}
\subsection{Method \code{getFeatureTypes()}}{
Get the list of available feature types for given workspace and datastore.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$getFeatureTypes(ws, ds, list = "configured")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{list}}{list type value, among "configured", "available", "available_with_geom", "all"}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \code{list} giving items of class \code{\link{GSFeatureType}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-getFeatureTypeNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-getFeatureTypeNames}{}}}
\subsection{Method \code{getFeatureTypeNames()}}{
Get the list of available feature type names for given workspace and datastore.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$getFeatureTypeNames(ws, ds)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
a vector of class\code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-getFeatureType"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-getFeatureType}{}}}
\subsection{Method \code{getFeatureType()}}{
Get an object of class \code{\link{GSFeatureType}} given a workspace, datastore and feature type names.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$getFeatureType(ws, ds, ft)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{ft}}{feature type name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
an object of class \link{GSFeatureType}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-createFeatureType"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-createFeatureType}{}}}
\subsection{Method \code{createFeatureType()}}{
Creates a new featureType given a workspace, datastore names and an object of class \code{\link{GSFeatureType}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$createFeatureType(ws, ds, featureType)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{featureType}}{feature type}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-updateFeatureType"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-updateFeatureType}{}}}
\subsection{Method \code{updateFeatureType()}}{
Updates a featureType given a workspace, datastore names and an object of class \code{\link{GSFeatureType}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$updateFeatureType(ws, ds, featureType)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{featureType}}{feature type}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-deleteFeatureType"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-deleteFeatureType}{}}}
\subsection{Method \code{deleteFeatureType()}}{
Deletes a featureType given a workspace, datastore names, and an object of 
   class \code{\link{GSFeatureType}}. By defaut, the option \code{recurse} is 
   set to FALSE, ie datastore layers are not removed.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$deleteFeatureType(ws, ds, ft, recurse = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{ft}}{feature type name}

\item{\code{recurse}}{recurse}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-publishLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-publishLayer}{}}}
\subsection{Method \code{publishLayer()}}{
Publish a feature type/layer pair given a workspace and datastore. The name 'layer' here 
encompasses both \link{GSFeatureType} and \link{GSLayer} resources.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$publishLayer(ws, ds, featureType, layer)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{featureType}}{object of class \link{GSFeatureType}}

\item{\code{layer}}{object of class \link{GSLayer}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if published, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-unpublishLayer"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-unpublishLayer}{}}}
\subsection{Method \code{unpublishLayer()}}{
Unpublish a feature type/layer pair given a workspace and datastore. The name 'layer' here 
encompasses both \link{GSFeatureType} and \link{GSLayer} resources.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$unpublishLayer(ws, ds, lyr)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{lyr}}{layer name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if published, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadData"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadData}{}}}
\subsection{Method \code{uploadData()}}{
Uploads features data. The \code{extension} corresponds to the format/type of features to be uploaded among "shp", "spatialite", "h2", "gpkg". 
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadData(
  ws,
  ds,
  endpoint = "file",
  extension,
  configure = "first",
  update = "append",
  filename,
  charset,
  contentType
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{extension}}{extension}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}

\item{\code{contentType}}{content type}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadShapefile"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadShapefile}{}}}
\subsection{Method \code{uploadShapefile()}}{
Uploads zipped shapefile.
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadShapefile(
  ws,
  ds,
  endpoint = "file",
  configure = "first",
  update = "append",
  filename,
  charset = "UTF-8"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadProperties"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadProperties}{}}}
\subsection{Method \code{uploadProperties()}}{
Uploads properties.
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadProperties(
  ws,
  ds,
  endpoint = "file",
  configure = "first",
  update = "append",
  filename,
  charset = "UTF-8"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadH2"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadH2}{}}}
\subsection{Method \code{uploadH2()}}{
Uploads H2 database.
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadH2(
  ws,
  ds,
  endpoint = "file",
  configure = "first",
  update = "append",
  filename,
  charset = "UTF-8"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadSpatialite"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadSpatialite}{}}}
\subsection{Method \code{uploadSpatialite()}}{
Uploads spatialite file.
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadSpatialite(
  ws,
  ds,
  endpoint = "file",
  configure = "first",
  update = "append",
  filename,
  charset = "UTF-8"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadAppschema"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadAppschema}{}}}
\subsection{Method \code{uploadAppschema()}}{
Uploads App schema.
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadAppschema(
  ws,
  ds,
  endpoint = "file",
  configure = "first",
  update = "append",
  filename,
  charset = "UTF-8"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-uploadGeoPackage"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-uploadGeoPackage}{}}}
\subsection{Method \code{uploadGeoPackage()}}{
Uploads GeoPackage.
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$uploadGeoPackage(
  ws,
  ds,
  endpoint = "file",
  configure = "first",
  update = "append",
  filename,
  charset = "UTF-8"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{ds}}{datastore name}

\item{\code{endpoint}}{endpoint}

\item{\code{configure}}{configure strategy among values: "first" or "none"}

\item{\code{update}}{update strategy, among values: "append", "overwrite"}

\item{\code{filename}}{file name of the resource to upload}

\item{\code{charset}}{charset}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSDataStoreManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSDataStoreManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSDataStoreManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GSCoverageStoreManager.R
\docType{class}
\name{GSCoverageStoreManager}
\alias{GSCoverageStoreManager}
\title{Geoserver REST API CoverageStore Manager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} with methods for managing GeoServer
CoverageStores (i.e. stores of coverage data)
}
\description{
Geoserver REST API CoverageStore Manager

Geoserver REST API CoverageStore Manager
}
\examples{
\dontrun{
   GSCoverageStoreManager$new("http://localhost:8080/geoserver", "admin", "geoserver")
 }

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{CoverageStore}
\keyword{api}
\keyword{geoserver}
\keyword{rest}
\section{Super class}{
\code{\link[geosapi:GSManager]{geosapi::GSManager}} -> \code{GSCoverageStoreManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-GSCoverageStoreManager-getCoverageStores}{\code{GSCoverageStoreManager$getCoverageStores()}}
\item \href{#method-GSCoverageStoreManager-getCoverageStoreNames}{\code{GSCoverageStoreManager$getCoverageStoreNames()}}
\item \href{#method-GSCoverageStoreManager-getCoverageStore}{\code{GSCoverageStoreManager$getCoverageStore()}}
\item \href{#method-GSCoverageStoreManager-createCoverageStore}{\code{GSCoverageStoreManager$createCoverageStore()}}
\item \href{#method-GSCoverageStoreManager-updateCoverageStore}{\code{GSCoverageStoreManager$updateCoverageStore()}}
\item \href{#method-GSCoverageStoreManager-deleteCoverageStore}{\code{GSCoverageStoreManager$deleteCoverageStore()}}
\item \href{#method-GSCoverageStoreManager-getCoverages}{\code{GSCoverageStoreManager$getCoverages()}}
\item \href{#method-GSCoverageStoreManager-getCoverageNames}{\code{GSCoverageStoreManager$getCoverageNames()}}
\item \href{#method-GSCoverageStoreManager-getCoverage}{\code{GSCoverageStoreManager$getCoverage()}}
\item \href{#method-GSCoverageStoreManager-createCoverage}{\code{GSCoverageStoreManager$createCoverage()}}
\item \href{#method-GSCoverageStoreManager-updateCoverage}{\code{GSCoverageStoreManager$updateCoverage()}}
\item \href{#method-GSCoverageStoreManager-deleteCoverage}{\code{GSCoverageStoreManager$deleteCoverage()}}
\item \href{#method-GSCoverageStoreManager-uploadCoverage}{\code{GSCoverageStoreManager$uploadCoverage()}}
\item \href{#method-GSCoverageStoreManager-uploadGeoTIFF}{\code{GSCoverageStoreManager$uploadGeoTIFF()}}
\item \href{#method-GSCoverageStoreManager-uploadWorldImage}{\code{GSCoverageStoreManager$uploadWorldImage()}}
\item \href{#method-GSCoverageStoreManager-uploadArcGrid}{\code{GSCoverageStoreManager$uploadArcGrid()}}
\item \href{#method-GSCoverageStoreManager-uploadImageMosaic}{\code{GSCoverageStoreManager$uploadImageMosaic()}}
\item \href{#method-GSCoverageStoreManager-clone}{\code{GSCoverageStoreManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="ERROR"><a href='../../geosapi/html/GSManager.html#method-GSManager-ERROR'><code>geosapi::GSManager$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="INFO"><a href='../../geosapi/html/GSManager.html#method-GSManager-INFO'><code>geosapi::GSManager$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="WARN"><a href='../../geosapi/html/GSManager.html#method-GSManager-WARN'><code>geosapi::GSManager$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="connect"><a href='../../geosapi/html/GSManager.html#method-GSManager-connect'><code>geosapi::GSManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getClassName"><a href='../../geosapi/html/GSManager.html#method-GSManager-getClassName'><code>geosapi::GSManager$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getCoverageStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getCoverageStoreManager'><code>geosapi::GSManager$getCoverageStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getDataStoreManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getDataStoreManager'><code>geosapi::GSManager$getDataStoreManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getNamespaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getNamespaceManager'><code>geosapi::GSManager$getNamespaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getServiceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getServiceManager'><code>geosapi::GSManager$getServiceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getStyleManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getStyleManager'><code>geosapi::GSManager$getStyleManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getSystemStatus"><a href='../../geosapi/html/GSManager.html#method-GSManager-getSystemStatus'><code>geosapi::GSManager$getSystemStatus()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getUrl"><a href='../../geosapi/html/GSManager.html#method-GSManager-getUrl'><code>geosapi::GSManager$getUrl()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="getWorkspaceManager"><a href='../../geosapi/html/GSManager.html#method-GSManager-getWorkspaceManager'><code>geosapi::GSManager$getWorkspaceManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="initialize"><a href='../../geosapi/html/GSManager.html#method-GSManager-initialize'><code>geosapi::GSManager$initialize()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="logger"><a href='../../geosapi/html/GSManager.html#method-GSManager-logger'><code>geosapi::GSManager$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="monitor"><a href='../../geosapi/html/GSManager.html#method-GSManager-monitor'><code>geosapi::GSManager$monitor()</code></a></span></li>
<li><span class="pkg-link" data-pkg="geosapi" data-topic="GSManager" data-id="reload"><a href='../../geosapi/html/GSManager.html#method-GSManager-reload'><code>geosapi::GSManager$reload()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-getCoverageStores"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-getCoverageStores}{}}}
\subsection{Method \code{getCoverageStores()}}{
Get the list of available coverage stores. Returns an object of class \code{list}
   giving items of class \code{\link{GSAbstractCoverageStore}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$getCoverageStores(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the list of coverage stores
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-getCoverageStoreNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-getCoverageStoreNames}{}}}
\subsection{Method \code{getCoverageStoreNames()}}{
Get the list of available coverage store names. Returns an vector of class \code{character}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$getCoverageStoreNames(ws)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the list of coverage store names, as \code{character}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-getCoverageStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-getCoverageStore}{}}}
\subsection{Method \code{getCoverageStore()}}{
Get an object of class \code{\link{GSAbstractDataStore}} given a workspace and coverage store names.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$getCoverageStore(ws, cs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the coverage store
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-createCoverageStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-createCoverageStore}{}}}
\subsection{Method \code{createCoverageStore()}}{
Creates a new coverage store given a workspace, coverage store name. Abstract method used in below format-specific
   methods to create coverage stores.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$createCoverageStore(ws, coverageStore)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{coverageStore}}{coverage store object}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-updateCoverageStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-updateCoverageStore}{}}}
\subsection{Method \code{updateCoverageStore()}}{
Updates a coverage store given a workspace, coverage store name. Abstract method used in below format-specific
   methods to create coverage stores.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$updateCoverageStore(ws, coverageStore)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{coverageStore}}{coverage store object}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-deleteCoverageStore"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-deleteCoverageStore}{}}}
\subsection{Method \code{deleteCoverageStore()}}{
Deletes a coverage store given a workspace and an object of class \code{\link{GSAbstractCoverageStore}}.
   By defaut, the option \code{recurse} is set to FALSE, ie datastore layers are not removed.
   To remove all coverage store layers, set this option to TRUE. The \code{purge} parameter is used 
   to customize the delete of files on disk (in case the underlying reader implements a delete method). 
   It can take one of the three values: none, metadata, all. For more details see \url{https://docs.geoserver.org/main/en/user/rest/api/coveragestores/#rest_api_coveragestores_purge}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$deleteCoverageStore(
  ws,
  cs,
  recurse = FALSE,
  purge = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{recurse}}{recurse}

\item{\code{purge}}{purge}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-getCoverages"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-getCoverages}{}}}
\subsection{Method \code{getCoverages()}}{
Get the list of available coverages for given workspace and coverage store.
   Returns an object of class \code{list} giving items of class \code{\link{GSCoverage}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$getCoverages(ws, cs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the list of \link{GSCoverage}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-getCoverageNames"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-getCoverageNames}{}}}
\subsection{Method \code{getCoverageNames()}}{
Get the list of available coverage names for given workspace and coverage store.
   Returns an object of class \code{list} giving items of class \code{\link{GSCoverage}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$getCoverageNames(ws, cs)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the list of coverage names
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-getCoverage"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-getCoverage}{}}}
\subsection{Method \code{getCoverage()}}{
Get coverage
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$getCoverage(ws, cs, cv)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{cv}}{coverage name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-createCoverage"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-createCoverage}{}}}
\subsection{Method \code{createCoverage()}}{
Creates a new coverage given a workspace, coverage store names and an object of class \code{\link{GSCoverage}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$createCoverage(ws, cs, coverage)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{coverage}}{object of class \link{GSCoverage}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if created, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-updateCoverage"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-updateCoverage}{}}}
\subsection{Method \code{updateCoverage()}}{
Updates a coverage given a workspace, coverage store names and an object of class \code{\link{GSCoverage}}
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$updateCoverage(ws, cs, coverage)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{coverage}}{object of class \link{GSCoverage}}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if updated, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-deleteCoverage"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-deleteCoverage}{}}}
\subsection{Method \code{deleteCoverage()}}{
Deletes a coverage given a workspace, coverage store names, and an object of 
   class \code{\link{GSCoverage}}. By defaut, the option \code{recurse} is 
   set to FALSE, ie coverage layers are not removed.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$deleteCoverage(ws, cs, cv, recurse = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{cv}}{coverage name}

\item{\code{recurse}}{recurse}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-uploadCoverage"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-uploadCoverage}{}}}
\subsection{Method \code{uploadCoverage()}}{
Abstract method to upload a coverage file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{extension}
   corresponds to the format/type of coverage to be uploaded (among values 'geotiff', 'worldimage', 'arcgrid', or 'imagemosaic'). 
   The \code{endpoint} takes a value among \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name 
   of the coverage file to upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$uploadCoverage(
  ws,
  cs,
  endpoint = "file",
  extension,
  filename,
  configure = "first",
  update = "append",
  contentType
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{endpoint}}{endpoint. Default is "file"}

\item{\code{extension}}{extension}

\item{\code{filename}}{filename}

\item{\code{configure}}{configure. Default is "first"}

\item{\code{update}}{update. Default is "append"}

\item{\code{contentType}}{content type}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-uploadGeoTIFF"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-uploadGeoTIFF}{}}}
\subsection{Method \code{uploadGeoTIFF()}}{
Uploads a GeoTIFF file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
   \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the GeoTIFF file to
   upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$uploadGeoTIFF(
  ws,
  cs,
  endpoint = "file",
  filename,
  configure = "first",
  update = "append"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{endpoint}}{endpoint. Default is "file"}

\item{\code{filename}}{filename}

\item{\code{configure}}{configure. Default is "first"}

\item{\code{update}}{update. Default is "append"}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-uploadWorldImage"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-uploadWorldImage}{}}}
\subsection{Method \code{uploadWorldImage()}}{
Uploads a WorldImage file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
   \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the zipped file to
   upload and set for the newly created datastore. It is assumed the zip archive contains the .prj file to set the SRS. 
   The \code{configure} parameter can take a value among values  \code{"none"} (indicates to configure only the datastore 
   but no layer configuration) or \code{"first"} (configure both datastore and layer). The \code{update} defines the strategy
   for the upload: \code{"append"} (default value) for the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$uploadWorldImage(
  ws,
  cs,
  endpoint = "file",
  filename,
  configure = "first",
  update = "append"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{endpoint}}{endpoint. Default is "file"}

\item{\code{filename}}{filename}

\item{\code{configure}}{configure. Default is "first"}

\item{\code{update}}{update. Default is "append"}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-uploadArcGrid"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-uploadArcGrid}{}}}
\subsection{Method \code{uploadArcGrid()}}{
Uploads an ArcGrid file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
   \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the ArcGrid file to
   upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$uploadArcGrid(
  ws,
  cs,
  endpoint = "file",
  filename,
  configure = "first",
  update = "append"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{endpoint}}{endpoint. Default is "file"}

\item{\code{filename}}{filename}

\item{\code{configure}}{configure. Default is "first"}

\item{\code{update}}{update. Default is "append"}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-uploadImageMosaic"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-uploadImageMosaic}{}}}
\subsection{Method \code{uploadImageMosaic()}}{
Uploads an ImageMosaic file targeting a workspace (\code{ws}) and datastore (\code{cs}). The \code{endpoint} takes a value among
   \code{"file"} (default), \code{"url"} or \code{"external"}. The \code{filename} is the name of the ImageMosaic file to
   upload and set for the newly created datastore. The \code{configure} parameter can take a value among values 
   \code{"none"} (indicates to configure only the datastore but no layer configuration) or \code{"first"} (configure 
   both datastore and layer). The \code{update} defines the strategy for the upload: \code{"append"} (default value) for 
   the first upload, \code{"overwrite"} in case the file should be overwriten.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$uploadImageMosaic(
  ws,
  cs,
  endpoint = "file",
  filename,
  configure = "first",
  update = "append"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ws}}{workspace name}

\item{\code{cs}}{coverage store name}

\item{\code{endpoint}}{endpoint. Default is "file"}

\item{\code{filename}}{filename}

\item{\code{configure}}{configure. Default is "first"}

\item{\code{update}}{update. Default is "append"}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if uploaded, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-GSCoverageStoreManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-GSCoverageStoreManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{GSCoverageStoreManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
